/* Generated by Cython 0.18 on Mon Dec  9 16:00:33 2013 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02040000
    #error Cython requires Python 2.4+.
#else
#include <stddef.h> /* For offsetof */
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if PY_VERSION_HEX < 0x02050000
  typedef int Py_ssize_t;
  #define PY_SSIZE_T_MAX INT_MAX
  #define PY_SSIZE_T_MIN INT_MIN
  #define PY_FORMAT_SIZE_T ""
  #define CYTHON_FORMAT_SSIZE_T ""
  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
  #define PyInt_AsSsize_t(o)   __Pyx_PyInt_AsInt(o)
  #define PyNumber_Index(o)    ((PyNumber_Check(o) && !PyFloat_Check(o)) ? PyNumber_Int(o) : \
                                (PyErr_Format(PyExc_TypeError, \
                                              "expected index value, got %.200s", Py_TYPE(o)->tp_name), \
                                 (PyObject*)0))
  #define __Pyx_PyIndex_Check(o) (PyNumber_Check(o) && !PyFloat_Check(o) && \
                                  !PyComplex_Check(o))
  #define PyIndex_Check __Pyx_PyIndex_Check
  #define PyErr_WarnEx(category, message, stacklevel) PyErr_Warn(category, message)
  #define __PYX_BUILD_PY_SSIZE_T "i"
#else
  #define __PYX_BUILD_PY_SSIZE_T "n"
  #define CYTHON_FORMAT_SSIZE_T "z"
  #define __Pyx_PyIndex_Check PyIndex_Check
#endif
#if PY_VERSION_HEX < 0x02060000
  #define Py_REFCNT(ob) (((PyObject*)(ob))->ob_refcnt)
  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
  #define Py_SIZE(ob)   (((PyVarObject*)(ob))->ob_size)
  #define PyVarObject_HEAD_INIT(type, size) \
          PyObject_HEAD_INIT(type) size,
  #define PyType_Modified(t)
  typedef struct {
     void *buf;
     PyObject *obj;
     Py_ssize_t len;
     Py_ssize_t itemsize;
     int readonly;
     int ndim;
     char *format;
     Py_ssize_t *shape;
     Py_ssize_t *strides;
     Py_ssize_t *suboffsets;
     void *internal;
  } Py_buffer;
  #define PyBUF_SIMPLE 0
  #define PyBUF_WRITABLE 0x0001
  #define PyBUF_FORMAT 0x0004
  #define PyBUF_ND 0x0008
  #define PyBUF_STRIDES (0x0010 | PyBUF_ND)
  #define PyBUF_C_CONTIGUOUS (0x0020 | PyBUF_STRIDES)
  #define PyBUF_F_CONTIGUOUS (0x0040 | PyBUF_STRIDES)
  #define PyBUF_ANY_CONTIGUOUS (0x0080 | PyBUF_STRIDES)
  #define PyBUF_INDIRECT (0x0100 | PyBUF_STRIDES)
  #define PyBUF_RECORDS (PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_WRITABLE)
  #define PyBUF_FULL (PyBUF_INDIRECT | PyBUF_FORMAT | PyBUF_WRITABLE)
  typedef int (*getbufferproc)(PyObject *, Py_buffer *, int);
  typedef void (*releasebufferproc)(PyObject *, Py_buffer *);
#endif
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
  #define PyUnicode_FromString(s) PyUnicode_Decode(s, strlen(s), "UTF-8", "strict")
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#if (PY_VERSION_HEX < 0x02060000) || (PY_MAJOR_VERSION >= 3)
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((k=k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PyBytesObject                PyStringObject
  #define PyBytes_Type                 PyString_Type
  #define PyBytes_Check                PyString_Check
  #define PyBytes_CheckExact           PyString_CheckExact
  #define PyBytes_FromString           PyString_FromString
  #define PyBytes_FromStringAndSize    PyString_FromStringAndSize
  #define PyBytes_FromFormat           PyString_FromFormat
  #define PyBytes_DecodeEscape         PyString_DecodeEscape
  #define PyBytes_AsString             PyString_AsString
  #define PyBytes_AsStringAndSize      PyString_AsStringAndSize
  #define PyBytes_Size                 PyString_Size
  #define PyBytes_AS_STRING            PyString_AS_STRING
  #define PyBytes_GET_SIZE             PyString_GET_SIZE
  #define PyBytes_Repr                 PyString_Repr
  #define PyBytes_Concat               PyString_Concat
  #define PyBytes_ConcatAndDel         PyString_ConcatAndDel
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PySet_Check(obj)             PyObject_TypeCheck(obj, &PySet_Type)
  #define PyFrozenSet_Check(obj)       PyObject_TypeCheck(obj, &PyFrozenSet_Type)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_VERSION_HEX < 0x03020000
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if (PY_MAJOR_VERSION < 3) || (PY_VERSION_HEX >= 0x03010300)
  #define __Pyx_PySequence_GetSlice(obj, a, b) PySequence_GetSlice(obj, a, b)
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) PySequence_SetSlice(obj, a, b, value)
  #define __Pyx_PySequence_DelSlice(obj, a, b) PySequence_DelSlice(obj, a, b)
#else
  #define __Pyx_PySequence_GetSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), (PyObject*)0) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_GetSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object is unsliceable", (obj)->ob_type->tp_name), (PyObject*)0)))
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_SetSlice(obj, a, b, value)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice assignment", (obj)->ob_type->tp_name), -1)))
  #define __Pyx_PySequence_DelSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_DelSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice deletion", (obj)->ob_type->tp_name), -1)))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),((char *)(n)))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),((char *)(n)),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),((char *)(n)))
#else
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),(n))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),(n),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),(n))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_NAMESTR(n) ((char *)(n))
  #define __Pyx_DOCSTR(n)  ((char *)(n))
#else
  #define __Pyx_NAMESTR(n) (n)
  #define __Pyx_DOCSTR(n)  (n)
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__h5py__h5p
#define __PYX_HAVE_API__h5py__h5p
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "unistd.h"
#include "stdint.h"
#include "api_compat.h"
#include "lzf_filter.h"
#include "hdf5.h"
#include "H5Cpublic.h"
#include "hdf5_hl.h"
#include "numpy/arrayobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const long n; const char* encoding; const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry; /*proto*/

#define __Pyx_PyBytes_FromUString(s) PyBytes_FromString((char*)s)
#define __Pyx_PyBytes_AsUString(s)   ((unsigned char*) PyBytes_AsString(s))
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject*);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))


#ifdef __GNUC__
  /* Test for GCC > 2.95 */
  #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95))
    #define likely(x)   __builtin_expect(!!(x), 1)
    #define unlikely(x) __builtin_expect(!!(x), 0)
  #else /* __GNUC__ > 2 ... */
    #define likely(x)   (x)
    #define unlikely(x) (x)
  #endif /* __GNUC__ > 2 ... */
#else /* __GNUC__ */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
    
static PyObject *__pyx_m;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "h5p.pyx",
  "h5p.pxd",
  "numpy.pxd",
  "h5t.pxd",
  "h5ac.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_4h5py_8_objects_ObjectID;
struct __pyx_obj_4h5py_3h5p_PropID;
struct __pyx_obj_4h5py_3h5p_PropInstanceID;
struct __pyx_obj_4h5py_3h5p_PropFAID;
struct __pyx_obj_4h5py_4h5ac_CacheConfig;
struct __pyx_obj_4h5py_3h5t_TypeID;
struct __pyx_obj_4h5py_3h5t_TypeBitfieldID;
struct __pyx_obj_4h5py_3h5p_PropCreateID;
struct __pyx_obj_4h5py_3h5p_PropFCID;
struct __pyx_obj_4h5py_3h5p_PropClassID;
struct __pyx_obj_4h5py_3h5t_TypeCompositeID;
struct __pyx_obj_4h5py_3h5t_TypeCompoundID;
struct __pyx_obj_4h5py_3h5t_TypeVlenID;
struct __pyx_obj_4h5py_3h5p_PropOCID;
struct __pyx_obj_4h5py_3h5p_PropLCID;
struct __pyx_obj_4h5py_3h5p_PropGCID;
struct __pyx_obj_4h5py_3h5p_PropDCID;
struct __pyx_obj_4h5py_3h5t_TypeTimeID;
struct __pyx_obj_4h5py_3h5t_TypeArrayID;
struct __pyx_obj_4h5py_3h5t_TypeAtomicID;
struct __pyx_obj_4h5py_3h5t_TypeFloatID;
struct __pyx_obj_4h5py_3h5p_PropDAID;
struct __pyx_obj_4h5py_3h5t_TypeStringID;
struct __pyx_obj_4h5py_3h5t_TypeOpaqueID;
struct __pyx_obj_4h5py_3h5t_TypeEnumID;
struct __pyx_obj_4h5py_3h5p_PropCopyID;
struct __pyx_obj_4h5py_3h5t_TypeIntegerID;
struct __pyx_obj_4h5py_3h5t_TypeReferenceID;
struct __pyx_obj_4h5py_3h5p_PropDXID;
struct __pyx_obj_4h5py_3h5p_PropLAID;
struct __pyx_opt_args_4h5py_5utils_check_numpy_read;
struct __pyx_opt_args_4h5py_5utils_check_numpy_write;

/* "utils.pxd":17
 * cdef void efree(void* ptr)
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1
 * 
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_read {
  int __pyx_n;
  hid_t space_id;
};

/* "utils.pxd":18
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * 
 * cdef int convert_tuple(object tuple, hsize_t *dims, hsize_t rank) except -1
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_write {
  int __pyx_n;
  hid_t space_id;
};
struct __pyx_opt_args_4h5py_3h5t_py_create;

/* "h5t.pxd":69
 * 
 * cpdef TypeID typewrap(hid_t id_)
 * cpdef TypeID py_create(object dtype, bint logical=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_4h5py_3h5t_py_create {
  int __pyx_n;
  int logical;
};

/* "_objects.pxd":12
 * from defs cimport *
 * 
 * cdef class ObjectID:             # <<<<<<<<<<<<<<
 * 
 *     cdef object __weakref__
 */
struct __pyx_obj_4h5py_8_objects_ObjectID {
  PyObject_HEAD
  PyObject *__weakref__;
  hid_t id;
  int locked;
  PyObject *_hash;
};


/* "h5py/h5p.pxd":16
 * # --- Base classes ---
 * 
 * cdef class PropID(ObjectID):             # <<<<<<<<<<<<<<
 *     """ Base class for all property lists """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
};


/* "h5py/h5p.pxd":26
 *     pass
 * 
 * cdef class PropInstanceID(PropID):             # <<<<<<<<<<<<<<
 *     """ Represents an instance of a property list class (i.e. an actual list
 *         which can be passed on to other API functions).
 */
struct __pyx_obj_4h5py_3h5p_PropInstanceID {
  struct __pyx_obj_4h5py_3h5p_PropID __pyx_base;
};


/* "h5py/h5p.pxd":59
 * # --- Object access ---
 * 
 * cdef class PropFAID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ File access property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropFAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5ac.pxd":12
 * from defs cimport *
 * 
 * cdef class CacheConfig:             # <<<<<<<<<<<<<<
 *     cdef H5AC_cache_config_t cache_config
 */
struct __pyx_obj_4h5py_4h5ac_CacheConfig {
  PyObject_HEAD
  H5AC_cache_config_t cache_config;
};


/* "h5t.pxd":14
 * from _objects cimport class ObjectID
 * 
 * cdef class TypeID(ObjectID):             # <<<<<<<<<<<<<<
 * 
 *     cdef object py_dtype(self)
 */
struct __pyx_obj_4h5py_3h5t_TypeID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID *__pyx_vtab;
};


/* "h5t.pxd":37
 *     pass
 * 
 * cdef class TypeBitfieldID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeBitfieldID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5py/h5p.pxd":32
 *     pass
 * 
 * cdef class PropCreateID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Base class for all object creation lists.
 * 
 */
struct __pyx_obj_4h5py_3h5p_PropCreateID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5py/h5p.pxd":52
 *     pass
 * 
 * cdef class PropFCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ File creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropFCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5py/h5p.pxd":20
 *     pass
 * 
 * cdef class PropClassID(PropID):             # <<<<<<<<<<<<<<
 *     """ Represents an HDF5 property list class.  These can be either (locked)
 *         library-defined classes or user-created classes.
 */
struct __pyx_obj_4h5py_3h5p_PropClassID {
  struct __pyx_obj_4h5py_3h5p_PropID __pyx_base;
};


/* "h5t.pxd":56
 * # --- Enums & compound types ---
 * 
 * cdef class TypeCompositeID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeCompositeID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5t.pxd":63
 *     cdef int enum_convert(self, long long *buf, int reverse) except -1
 * 
 * cdef class TypeCompoundID(TypeCompositeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeCompoundID {
  struct __pyx_obj_4h5py_3h5t_TypeCompositeID __pyx_base;
};


/* "h5t.pxd":30
 *     pass
 * 
 * cdef class TypeVlenID(TypeID):             # <<<<<<<<<<<<<<
 *     # Non-string vlens
 *     pass
 */
struct __pyx_obj_4h5py_3h5t_TypeVlenID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5py/h5p.pxd":44
 * # --- Object creation ---
 * 
 * cdef class PropOCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ Object creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropOCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5py/h5p.pxd":70
 * # --- New in 1.8 ---
 * 
 * cdef class PropLCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ Link creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropLCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5py/h5p.pxd":78
 *     cdef char* _buf
 * 
 * cdef class PropGCID(PropOCID):             # <<<<<<<<<<<<<<
 *     """ Group creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropGCID {
  struct __pyx_obj_4h5py_3h5p_PropOCID __pyx_base;
};


/* "h5py/h5p.pxd":48
 *     pass
 * 
 * cdef class PropDCID(PropOCID):             # <<<<<<<<<<<<<<
 *     """ Dataset creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropDCID {
  struct __pyx_obj_4h5py_3h5p_PropOCID __pyx_base;
};


/* "h5t.pxd":34
 *     pass
 * 
 * cdef class TypeTimeID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeTimeID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5t.pxd":20
 * # --- Top-level classes ---
 * 
 * cdef class TypeArrayID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeArrayID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5t.pxd":45
 * # --- Numeric atomic types ---
 * 
 * cdef class TypeAtomicID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeAtomicID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5t.pxd":51
 *     pass
 * 
 * cdef class TypeFloatID(TypeAtomicID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeFloatID {
  struct __pyx_obj_4h5py_3h5t_TypeAtomicID __pyx_base;
};


/* "h5py/h5p.pyx":1165
 * 
 * # Dataset access
 * cdef class PropDAID(PropInstanceID):             # <<<<<<<<<<<<<<
 * 
 *     """ Dataset access property list """
 */
struct __pyx_obj_4h5py_3h5p_PropDAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5t.pxd":26
 *     pass
 * 
 * cdef class TypeStringID(TypeID):             # <<<<<<<<<<<<<<
 *     # Both vlen and fixed-len strings
 *     pass
 */
struct __pyx_obj_4h5py_3h5t_TypeStringID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5t.pxd":23
 *     pass
 * 
 * cdef class TypeOpaqueID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeOpaqueID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5t.pxd":59
 *     pass
 * 
 * cdef class TypeEnumID(TypeCompositeID):             # <<<<<<<<<<<<<<
 * 
 *     cdef int enum_convert(self, long long *buf, int reverse) except -1
 */
struct __pyx_obj_4h5py_3h5t_TypeEnumID {
  struct __pyx_obj_4h5py_3h5t_TypeCompositeID __pyx_base;
};


/* "h5py/h5p.pxd":39
 *     pass
 * 
 * cdef class PropCopyID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Property list for copying objects (as in h5o.copy) """
 * 
 */
struct __pyx_obj_4h5py_3h5p_PropCopyID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5t.pxd":48
 *     pass
 * 
 * cdef class TypeIntegerID(TypeAtomicID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeIntegerID {
  struct __pyx_obj_4h5py_3h5t_TypeAtomicID __pyx_base;
};


/* "h5t.pxd":40
 *     pass
 * 
 * cdef class TypeReferenceID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5t_TypeReferenceID {
  struct __pyx_obj_4h5py_3h5t_TypeID __pyx_base;
};


/* "h5py/h5p.pxd":63
 *     pass
 * 
 * cdef class PropDXID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Dataset transfer property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropDXID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5py/h5p.pxd":74
 *     pass
 * 
 * cdef class PropLAID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Link access property list """
 *     cdef char* _buf
 */
struct __pyx_obj_4h5py_3h5p_PropLAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
  char *_buf;
};



/* "h5t.pxd":14
 * from _objects cimport class ObjectID
 * 
 * cdef class TypeID(ObjectID):             # <<<<<<<<<<<<<<
 * 
 *     cdef object py_dtype(self)
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeID {
  PyObject *(*py_dtype)(struct __pyx_obj_4h5py_3h5t_TypeID *);
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeID *__pyx_vtabptr_4h5py_3h5t_TypeID;


/* "h5t.pxd":56
 * # --- Enums & compound types ---
 * 
 * cdef class TypeCompositeID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeCompositeID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeCompositeID *__pyx_vtabptr_4h5py_3h5t_TypeCompositeID;


/* "h5t.pxd":63
 *     cdef int enum_convert(self, long long *buf, int reverse) except -1
 * 
 * cdef class TypeCompoundID(TypeCompositeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeCompoundID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeCompositeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeCompoundID *__pyx_vtabptr_4h5py_3h5t_TypeCompoundID;


/* "h5t.pxd":23
 *     pass
 * 
 * cdef class TypeOpaqueID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeOpaqueID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeOpaqueID *__pyx_vtabptr_4h5py_3h5t_TypeOpaqueID;


/* "h5t.pxd":20
 * # --- Top-level classes ---
 * 
 * cdef class TypeArrayID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeArrayID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeArrayID *__pyx_vtabptr_4h5py_3h5t_TypeArrayID;


/* "h5t.pxd":45
 * # --- Numeric atomic types ---
 * 
 * cdef class TypeAtomicID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeAtomicID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeAtomicID *__pyx_vtabptr_4h5py_3h5t_TypeAtomicID;


/* "h5t.pxd":48
 *     pass
 * 
 * cdef class TypeIntegerID(TypeAtomicID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeIntegerID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeAtomicID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeIntegerID *__pyx_vtabptr_4h5py_3h5t_TypeIntegerID;


/* "h5t.pxd":40
 *     pass
 * 
 * cdef class TypeReferenceID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeReferenceID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeReferenceID *__pyx_vtabptr_4h5py_3h5t_TypeReferenceID;


/* "h5t.pxd":26
 *     pass
 * 
 * cdef class TypeStringID(TypeID):             # <<<<<<<<<<<<<<
 *     # Both vlen and fixed-len strings
 *     pass
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeStringID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeStringID *__pyx_vtabptr_4h5py_3h5t_TypeStringID;


/* "h5t.pxd":51
 *     pass
 * 
 * cdef class TypeFloatID(TypeAtomicID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeFloatID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeAtomicID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeFloatID *__pyx_vtabptr_4h5py_3h5t_TypeFloatID;


/* "h5t.pxd":34
 *     pass
 * 
 * cdef class TypeTimeID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeTimeID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeTimeID *__pyx_vtabptr_4h5py_3h5t_TypeTimeID;


/* "h5t.pxd":37
 *     pass
 * 
 * cdef class TypeBitfieldID(TypeID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeBitfieldID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeBitfieldID *__pyx_vtabptr_4h5py_3h5t_TypeBitfieldID;


/* "h5t.pxd":30
 *     pass
 * 
 * cdef class TypeVlenID(TypeID):             # <<<<<<<<<<<<<<
 *     # Non-string vlens
 *     pass
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeVlenID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeID __pyx_base;
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeVlenID *__pyx_vtabptr_4h5py_3h5t_TypeVlenID;


/* "h5t.pxd":59
 *     pass
 * 
 * cdef class TypeEnumID(TypeCompositeID):             # <<<<<<<<<<<<<<
 * 
 *     cdef int enum_convert(self, long long *buf, int reverse) except -1
 */

struct __pyx_vtabstruct_4h5py_3h5t_TypeEnumID {
  struct __pyx_vtabstruct_4h5py_3h5t_TypeCompositeID __pyx_base;
  int (*enum_convert)(struct __pyx_obj_4h5py_3h5t_TypeEnumID *, PY_LONG_LONG *, int);
};
static struct __pyx_vtabstruct_4h5py_3h5t_TypeEnumID *__pyx_vtabptr_4h5py_3h5t_TypeEnumID;
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname); /*proto*/
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif /* CYTHON_REFNANNY */
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name); /*proto*/

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb); /*proto*/
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause); /*proto*/

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type); /*proto*/

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact); /*proto*/

#define __Pyx_DelItemInt(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_DelItemInt_Fast(o, i) : \
                                                    __Pyx_DelItem_Generic(o, to_py_func(i)))
static CYTHON_INLINE int __Pyx_DelItem_Generic(PyObject *o, PyObject *j) {
    int r;
    if (!j) return -1;
    r = PyObject_DelItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_PYPY
    if (PySequence_Check(o)) {
        return PySequence_DelItem(o, i);
    }
#else
    PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
    if (likely(m && m->sq_ass_item)) {
        if (unlikely(i < 0) && likely(m->sq_length)) {
            Py_ssize_t l = m->sq_length(o);
            if (unlikely(l < 0)) return -1;
            i += l;
        }
        return m->sq_ass_item(o, i, (PyObject *)NULL);
    }
#endif
    return __Pyx_DelItem_Generic(o, PyInt_FromSsize_t(i));
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found); /*proto*/

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name); /*proto*/

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name); /*proto*/

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
#define __Pyx_GetItemInt_List(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_GetItemInt_List_Fast(o, i) : \
                                                    __Pyx_GetItemInt_Generic(o, to_py_func(i)))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    else if ((-PyList_GET_SIZE(o) <= i) & (i < 0)) {
        PyObject *r = PyList_GET_ITEM(o, PyList_GET_SIZE(o) + i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
#define __Pyx_GetItemInt_Tuple(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_GetItemInt_Tuple_Fast(o, i) : \
                                                    __Pyx_GetItemInt_Generic(o, to_py_func(i)))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    else if ((-PyTuple_GET_SIZE(o) <= i) & (i < 0)) {
        PyObject *r = PyTuple_GET_ITEM(o, PyTuple_GET_SIZE(o) + i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
#define __Pyx_GetItemInt(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_GetItemInt_Fast(o, i) : \
                                                    __Pyx_GetItemInt_Generic(o, to_py_func(i)))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (PyList_CheckExact(o)) {
        Py_ssize_t n = (likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if (likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = (likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if (likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {  /* inlined PySequence_GetItem() */
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (unlikely(l < 0)) return NULL;
                i += l;
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed); /*proto*/

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb); /*proto*/
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb); /*proto*/

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level); /*proto*/

static CYTHON_INLINE void __Pyx_RaiseImportError(PyObject *name);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t);

static CYTHON_INLINE hsize_t __Pyx_PyInt_from_py_hsize_t(PyObject *);

static CYTHON_INLINE hbool_t __Pyx_PyInt_from_py_hbool_t(PyObject *);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hbool_t(hbool_t);

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject *);

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject *);

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject *);

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject *);

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject *);

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject *);

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject *);

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject *);

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject *);

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject *);

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject *);

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject *);

static int __Pyx_check_binary_version(void);

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig); /*proto*/

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name); /*proto*/

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);  /*proto*/

static void* __Pyx_GetVtable(PyObject *dict); /*proto*/

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig); /*proto*/

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename); /*proto*/

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t); /*proto*/


/* Module declarations from 'h5py.api_types_ext' */

/* Module declarations from 'h5py.api_types_hdf5' */

/* Module declarations from 'h5py.defs' */
static herr_t (*__pyx_f_4h5py_4defs_H5Fget_mdc_config)(hid_t, H5AC_cache_config_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Fset_mdc_config)(hid_t, H5AC_cache_config_t *); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Idec_ref)(hid_t); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Iget_ref)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Pcreate)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Pcopy)(hid_t); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Pget_class)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pclose)(hid_t); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Pequal)(hid_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pclose_class)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_version)(hid_t, unsigned int *, unsigned int *, unsigned int *, unsigned int *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_userblock)(hid_t, hsize_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_userblock)(hid_t, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_sizes)(hid_t, size_t, size_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_sizes)(hid_t, size_t *, size_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fclose_degree)(hid_t, enum H5F_close_degree_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_fclose_degree)(hid_t, enum H5F_close_degree_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fapl_core)(hid_t, size_t, hbool_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_fapl_core)(hid_t, size_t *, hbool_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fapl_family)(hid_t, hsize_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_fapl_family)(hid_t, hsize_t *, hid_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fapl_log)(hid_t, char *, unsigned int, size_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_cache)(hid_t, int, int, size_t, double); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_cache)(hid_t, int *, int *, size_t *, double *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fapl_sec2)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fapl_stdio)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Pget_driver)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_layout)(hid_t, int); /*proto*/
static H5D_layout_t (*__pyx_f_4h5py_4defs_H5Pget_layout)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_chunk)(hid_t, int, hsize_t *); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Pget_chunk)(hid_t, int, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_deflate)(hid_t, int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fill_value)(hid_t, hid_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_fill_value)(hid_t, hid_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pfill_value_defined)(hid_t, H5D_fill_value_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fill_time)(hid_t, H5D_fill_time_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_fill_time)(hid_t, H5D_fill_time_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_alloc_time)(hid_t, H5D_alloc_time_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_alloc_time)(hid_t, H5D_alloc_time_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_filter)(hid_t, H5Z_filter_t, unsigned int, size_t, unsigned int *); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Pall_filters_avail)(hid_t); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Pget_nfilters)(hid_t); /*proto*/
static H5Z_filter_t (*__pyx_f_4h5py_4defs_H5Pget_filter)(hid_t, unsigned int, unsigned int *, size_t *, unsigned int *, size_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_filter_by_id)(hid_t, H5Z_filter_t, unsigned int *, size_t *, unsigned int *, size_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Premove_filter)(hid_t, H5Z_filter_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_fletcher32)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_shuffle)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_szip)(hid_t, unsigned int, unsigned int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_scaleoffset)(hid_t, enum H5Z_SO_scale_type_t, int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_chunk_cache)(hid_t, size_t, size_t, double); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_chunk_cache)(hid_t, size_t *, size_t *, double *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_sieve_buf_size)(hid_t, size_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_sieve_buf_size)(hid_t, size_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_nlinks)(hid_t, size_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_nlinks)(hid_t, size_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_elink_prefix)(hid_t, char *); /*proto*/
static Py_ssize_t (*__pyx_f_4h5py_4defs_H5Pget_elink_prefix)(hid_t, char *, size_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Pget_elink_fapl)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_elink_fapl)(hid_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_create_intermediate_group)(hid_t, unsigned int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_create_intermediate_group)(hid_t, unsigned int *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_copy_object)(hid_t, unsigned int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_copy_object)(hid_t, unsigned int *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_char_encoding)(hid_t, H5T_cset_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_char_encoding)(hid_t, H5T_cset_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_obj_track_times)(hid_t, hbool_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_obj_track_times)(hid_t, hbool_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_link_creation_order)(hid_t, unsigned int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_link_creation_order)(hid_t, unsigned int *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pset_libver_bounds)(hid_t, H5F_libver_t, H5F_libver_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Pget_libver_bounds)(hid_t, H5F_libver_t *, H5F_libver_t *); /*proto*/

/* Module declarations from 'h5py._objects' */
static PyTypeObject *__pyx_ptype_4h5py_8_objects_ObjectID = 0;

/* Module declarations from 'numpy' */

/* Module declarations from 'h5py.numpy' */
static PyTypeObject *__pyx_ptype_4h5py_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_4h5py_5numpy_ndarray = 0;

/* Module declarations from 'h5py.utils' */
static void *(*__pyx_f_4h5py_5utils_emalloc)(size_t); /*proto*/
static void (*__pyx_f_4h5py_5utils_efree)(void *); /*proto*/
static int (*__pyx_f_4h5py_5utils_check_numpy_read)(PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4h5py_5utils_check_numpy_read *__pyx_optional_args); /*proto*/
static int (*__pyx_f_4h5py_5utils_check_numpy_write)(PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4h5py_5utils_check_numpy_write *__pyx_optional_args); /*proto*/
static int (*__pyx_f_4h5py_5utils_convert_tuple)(PyObject *, hsize_t *, hsize_t); /*proto*/
static PyObject *(*__pyx_f_4h5py_5utils_convert_dims)(hsize_t *, hsize_t); /*proto*/
static int (*__pyx_f_4h5py_5utils_require_tuple)(PyObject *, int, int, char *); /*proto*/

/* Module declarations from 'h5py.h5t' */
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeArrayID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeOpaqueID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeStringID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeVlenID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeTimeID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeBitfieldID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeReferenceID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeAtomicID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeIntegerID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeFloatID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeCompositeID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeEnumID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5t_TypeCompoundID = 0;
static struct __pyx_obj_4h5py_3h5t_TypeID *(*__pyx_f_4h5py_3h5t_py_create)(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4h5py_3h5t_py_create *__pyx_optional_args); /*proto*/

/* Module declarations from 'h5py.h5ac' */
static PyTypeObject *__pyx_ptype_4h5py_4h5ac_CacheConfig = 0;

/* Module declarations from 'h5py.h5p' */
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropClassID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropInstanceID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropCreateID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropCopyID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropOCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropFCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropFAID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDXID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropLCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropLAID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropGCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDAID = 0;
static hid_t __pyx_f_4h5py_3h5p_pdefault(struct __pyx_obj_4h5py_3h5p_PropID *); /*proto*/
static PyObject *__pyx_f_4h5py_3h5p_propwrap(hid_t); /*proto*/
static PyObject *__pyx_f_4h5py_3h5p_lockcls(hid_t); /*proto*/
#define __Pyx_MODULE_NAME "h5py.h5p"
int __pyx_module_is_main_h5py__h5p = 0;

/* Implementation of 'h5py.h5p' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_NotImplemented;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_pf_4h5py_3h5p_create(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_3h5p_PropClassID *__pyx_v_cls); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_6PropID_equal(struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_self, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_plist); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_6PropID_2__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how); /* proto */
static Py_hash_t __pyx_pf_4h5py_3h5p_6PropID_4__hash__(CYTHON_UNUSED struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_11PropClassID___richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how); /* proto */
static Py_hash_t __pyx_pf_4h5py_3h5p_11PropClassID_2__hash__(struct __pyx_obj_4h5py_3h5p_PropClassID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_14PropInstanceID_copy(struct __pyx_obj_4h5py_3h5p_PropInstanceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_14PropInstanceID_2_close(struct __pyx_obj_4h5py_3h5p_PropInstanceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_14PropInstanceID_4get_class(struct __pyx_obj_4h5py_3h5p_PropInstanceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_10PropCopyID_set_copy_object(struct __pyx_obj_4h5py_3h5p_PropCopyID *__pyx_v_self, unsigned int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_10PropCopyID_2get_copy_object(struct __pyx_obj_4h5py_3h5p_PropCopyID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_get_version(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_2set_userblock(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self, hsize_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_4get_userblock(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_6set_sizes(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self, size_t __pyx_v_addr, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_8get_sizes(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_10set_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self, unsigned int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_12get_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_set_layout(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_layout_code); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_2get_layout(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_4set_chunk(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, PyObject *__pyx_v_chunksize); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_6get_chunk(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_8set_fill_value(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, PyArrayObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_10get_fill_value(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, PyArrayObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_12fill_value_defined(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_14set_fill_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_fill_time); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_16get_fill_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_18set_alloc_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_alloc_time); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_20get_alloc_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_22set_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_code, unsigned int __pyx_v_flags, PyObject *__pyx_v_values); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_24all_filters_avail(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_26get_nfilters(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_28get_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_idx); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_30_has_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_code); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_32get_filter_by_id(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_code); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_34remove_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_class); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_36set_deflate(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, unsigned int __pyx_v_level); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_38set_fletcher32(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_40set_shuffle(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_42set_szip(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, unsigned int __pyx_v_options, unsigned int __pyx_v_pixels_per_block); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_44set_scaleoffset(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, enum H5Z_SO_scale_type_t __pyx_v_scale_type, int __pyx_v_scale_factor); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_set_fclose_degree(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, int __pyx_v_close_degree); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_2get_fclose_degree(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_4set_fapl_core(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, size_t __pyx_v_block_size, hbool_t __pyx_v_backing_store); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_6get_fapl_core(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_8set_fapl_family(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, hsize_t __pyx_v_memb_size, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_memb_fapl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_10get_fapl_family(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_12set_fapl_log(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, char *__pyx_v_logfile, unsigned int __pyx_v_flags, size_t __pyx_v_buf_size); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_14set_fapl_sec2(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_16set_fapl_stdio(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_18get_driver(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_20set_cache(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, int __pyx_v_mdc, int __pyx_v_rdcc, size_t __pyx_v_rdcc_nbytes, double __pyx_v_rdcc_w0); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_22get_cache(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_24set_sieve_buf_size(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_26get_sieve_buf_size(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_28set_libver_bounds(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, int __pyx_v_low, int __pyx_v_high); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_30get_libver_bounds(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_32get_mdc_config(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_34set_mdc_config(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, struct __pyx_obj_4h5py_4h5ac_CacheConfig *__pyx_v_config); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_set_char_encoding(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self, int __pyx_v_encoding); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_2get_char_encoding(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_4set_create_intermediate_group(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self, int __pyx_v_create); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_6get_create_intermediate_group(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5p_8PropLAID___cinit__(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args); /* proto */
static void __pyx_pf_4h5py_3h5p_8PropLAID_2__dealloc__(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_4set_nlinks(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, size_t __pyx_v_nlinks); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_6get_nlinks(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_8set_elink_prefix(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, char *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_10get_elink_prefix(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_12set_elink_fapl(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_fapl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_14get_elink_fapl(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropGCID_set_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropGCID *__pyx_v_self, unsigned int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropGCID_2get_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropGCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropOCID_set_obj_track_times(struct __pyx_obj_4h5py_3h5p_PropOCID *__pyx_v_self, PyObject *__pyx_v_track_times); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropOCID_2get_obj_track_times(struct __pyx_obj_4h5py_3h5p_PropOCID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDAID_set_chunk_cache(struct __pyx_obj_4h5py_3h5p_PropDAID *__pyx_v_self, size_t __pyx_v_rdcc_nslots, size_t __pyx_v_rdcc_nbytes, double __pyx_v_rdcc_w0); /* proto */
static PyObject *__pyx_pf_4h5py_3h5p_8PropDAID_2get_chunk_cache(struct __pyx_obj_4h5py_3h5p_PropDAID *__pyx_v_self); /* proto */
static char __pyx_k_1[] = "No class found for ID %d";
static char __pyx_k_2[] = "Property lists are unhashable";
static char __pyx_k_5[] = "Filter index must be a non-negative integer";
static char __pyx_k_7[] = "Failed to get hit rate";
static char __pyx_k_10[] = "\n    HDF5 property list interface.\n";
static char __pyx_k_13[] = "/home/computer/h5py/h5py/h5p.pyx";
static char __pyx_k_14[] = "h5py.h5p";
static char __pyx_k__cls[] = "cls";
static char __pyx_k__low[] = "low";
static char __pyx_k__mdc[] = "mdc";
static char __pyx_k__addr[] = "addr";
static char __pyx_k__h5py[] = "h5py";
static char __pyx_k__high[] = "high";
static char __pyx_k__lock[] = "lock";
static char __pyx_k__open[] = "open";
static char __pyx_k__rdcc[] = "rdcc";
static char __pyx_k__size[] = "size";
static char __pyx_k__dtype[] = "dtype";
static char __pyx_k__equal[] = "equal";
static char __pyx_k__flags[] = "flags";
static char __pyx_k__level[] = "level";
static char __pyx_k__newid[] = "newid";
static char __pyx_k__valid[] = "valid";
static char __pyx_k__create[] = "create";
static char __pyx_k__values[] = "values";
static char __pyx_k__DEFAULT[] = "DEFAULT";
static char __pyx_k__logfile[] = "logfile";
static char __pyx_k__options[] = "options";
static char __pyx_k__rdcc_w0[] = "rdcc_w0";
static char __pyx_k__NO_CLASS[] = "NO_CLASS";
static char __pyx_k____exit__[] = "__exit__";
static char __pyx_k____main__[] = "__main__";
static char __pyx_k____test__[] = "__test__";
static char __pyx_k___objects[] = "_objects";
static char __pyx_k__buf_size[] = "buf_size";
static char __pyx_k__registry[] = "registry";
static char __pyx_k__TypeError[] = "TypeError";
static char __pyx_k____enter__[] = "__enter__";
static char __pyx_k__chunksize[] = "chunksize";
static char __pyx_k__memb_fapl[] = "memb_fapl";
static char __pyx_k__memb_size[] = "memb_size";
static char __pyx_k__ValueError[] = "ValueError";
static char __pyx_k__block_size[] = "block_size";
static char __pyx_k__get_filter[] = "get_filter";
static char __pyx_k__scale_type[] = "scale_type";
static char __pyx_k__FILE_ACCESS[] = "FILE_ACCESS";
static char __pyx_k__FILE_CREATE[] = "FILE_CREATE";
static char __pyx_k__LINK_ACCESS[] = "LINK_ACCESS";
static char __pyx_k__LINK_CREATE[] = "LINK_CREATE";
static char __pyx_k__OBJECT_COPY[] = "OBJECT_COPY";
static char __pyx_k____richcmp__[] = "__richcmp__";
static char __pyx_k___has_filter[] = "_has_filter";
static char __pyx_k__filter_code[] = "filter_code";
static char __pyx_k__rdcc_nbytes[] = "rdcc_nbytes";
static char __pyx_k__rdcc_nslots[] = "rdcc_nslots";
static char __pyx_k__DATASET_XFER[] = "DATASET_XFER";
static char __pyx_k__GROUP_CREATE[] = "GROUP_CREATE";
static char __pyx_k__RuntimeError[] = "RuntimeError";
static char __pyx_k__get_nfilters[] = "get_nfilters";
static char __pyx_k__scale_factor[] = "scale_factor";
static char __pyx_k__OBJECT_CREATE[] = "OBJECT_CREATE";
static char __pyx_k__backing_store[] = "backing_store";
static char __pyx_k__DATASET_ACCESS[] = "DATASET_ACCESS";
static char __pyx_k__DATASET_CREATE[] = "DATASET_CREATE";
static char __pyx_k__NotImplemented[] = "NotImplemented";
static char __pyx_k__pixels_per_block[] = "pixels_per_block";
static char __pyx_k__CRT_ORDER_INDEXED[] = "CRT_ORDER_INDEXED";
static char __pyx_k__CRT_ORDER_TRACKED[] = "CRT_ORDER_TRACKED";
static PyObject *__pyx_kp_s_1;
static PyObject *__pyx_kp_s_13;
static PyObject *__pyx_n_s_14;
static PyObject *__pyx_kp_s_2;
static PyObject *__pyx_kp_s_5;
static PyObject *__pyx_kp_s_7;
static PyObject *__pyx_n_s__CRT_ORDER_INDEXED;
static PyObject *__pyx_n_s__CRT_ORDER_TRACKED;
static PyObject *__pyx_n_s__DATASET_ACCESS;
static PyObject *__pyx_n_s__DATASET_CREATE;
static PyObject *__pyx_n_s__DATASET_XFER;
static PyObject *__pyx_n_s__DEFAULT;
static PyObject *__pyx_n_s__FILE_ACCESS;
static PyObject *__pyx_n_s__FILE_CREATE;
static PyObject *__pyx_n_s__GROUP_CREATE;
static PyObject *__pyx_n_s__LINK_ACCESS;
static PyObject *__pyx_n_s__LINK_CREATE;
static PyObject *__pyx_n_s__NO_CLASS;
static PyObject *__pyx_n_s__NotImplemented;
static PyObject *__pyx_n_s__OBJECT_COPY;
static PyObject *__pyx_n_s__OBJECT_CREATE;
static PyObject *__pyx_n_s__RuntimeError;
static PyObject *__pyx_n_s__TypeError;
static PyObject *__pyx_n_s__ValueError;
static PyObject *__pyx_n_s____enter__;
static PyObject *__pyx_n_s____exit__;
static PyObject *__pyx_n_s____main__;
static PyObject *__pyx_n_s____richcmp__;
static PyObject *__pyx_n_s____test__;
static PyObject *__pyx_n_s___has_filter;
static PyObject *__pyx_n_s___objects;
static PyObject *__pyx_n_s__addr;
static PyObject *__pyx_n_s__backing_store;
static PyObject *__pyx_n_s__block_size;
static PyObject *__pyx_n_s__buf_size;
static PyObject *__pyx_n_s__cls;
static PyObject *__pyx_n_s__create;
static PyObject *__pyx_n_s__dtype;
static PyObject *__pyx_n_s__equal;
static PyObject *__pyx_n_s__filter_code;
static PyObject *__pyx_n_s__flags;
static PyObject *__pyx_n_s__get_filter;
static PyObject *__pyx_n_s__get_nfilters;
static PyObject *__pyx_n_s__h5py;
static PyObject *__pyx_n_s__high;
static PyObject *__pyx_n_s__level;
static PyObject *__pyx_n_s__lock;
static PyObject *__pyx_n_s__logfile;
static PyObject *__pyx_n_s__low;
static PyObject *__pyx_n_s__mdc;
static PyObject *__pyx_n_s__memb_fapl;
static PyObject *__pyx_n_s__memb_size;
static PyObject *__pyx_n_s__newid;
static PyObject *__pyx_n_s__open;
static PyObject *__pyx_n_s__options;
static PyObject *__pyx_n_s__pixels_per_block;
static PyObject *__pyx_n_s__rdcc;
static PyObject *__pyx_n_s__rdcc_nbytes;
static PyObject *__pyx_n_s__rdcc_nslots;
static PyObject *__pyx_n_s__rdcc_w0;
static PyObject *__pyx_n_s__registry;
static PyObject *__pyx_n_s__scale_factor;
static PyObject *__pyx_n_s__scale_type;
static PyObject *__pyx_n_s__size;
static PyObject *__pyx_n_s__valid;
static PyObject *__pyx_n_s__values;
static PyObject *__pyx_k_tuple_3;
static PyObject *__pyx_k_tuple_4;
static PyObject *__pyx_k_tuple_6;
static PyObject *__pyx_k_tuple_8;
static PyObject *__pyx_k_tuple_9;
static PyObject *__pyx_k_tuple_11;
static PyObject *__pyx_k_codeobj_12;

/* "h5py/h5p.pyx":30
 * # === C API ===================================================================
 * 
 * cdef hid_t pdefault(PropID pid):             # <<<<<<<<<<<<<<
 * 
 *     if pid is None:
 */

static hid_t __pyx_f_4h5py_3h5p_pdefault(struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_pid) {
  hid_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("pdefault", 0);

  /* "h5py/h5p.pyx":32
 * cdef hid_t pdefault(PropID pid):
 * 
 *     if pid is None:             # <<<<<<<<<<<<<<
 *         return <hid_t>H5P_DEFAULT
 *     return pid.id
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_pid) == Py_None);
  if (__pyx_t_1) {

    /* "h5py/h5p.pyx":33
 * 
 *     if pid is None:
 *         return <hid_t>H5P_DEFAULT             # <<<<<<<<<<<<<<
 *     return pid.id
 * 
 */
    __pyx_r = ((hid_t)H5P_DEFAULT);
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":34
 *     if pid is None:
 *         return <hid_t>H5P_DEFAULT
 *     return pid.id             # <<<<<<<<<<<<<<
 * 
 * cdef object propwrap(hid_t id_in):
 */
  __pyx_r = __pyx_v_pid->__pyx_base.id;
  goto __pyx_L0;

  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":36
 *     return pid.id
 * 
 * cdef object propwrap(hid_t id_in):             # <<<<<<<<<<<<<<
 * 
 *     clsid = H5Pget_class(id_in)
 */

static PyObject *__pyx_f_4h5py_3h5p_propwrap(hid_t __pyx_v_id_in) {
  int __pyx_v_clsid;
  PyObject *__pyx_v_pcls = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  htri_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  herr_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("propwrap", 0);

  /* "h5py/h5p.pyx":38
 * cdef object propwrap(hid_t id_in):
 * 
 *     clsid = H5Pget_class(id_in)             # <<<<<<<<<<<<<<
 *     try:
 *         if H5Pequal(clsid, H5P_FILE_CREATE):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_class(__pyx_v_id_in); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 38; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_clsid = __pyx_t_1;

  /* "h5py/h5p.pyx":39
 * 
 *     clsid = H5Pget_class(id_in)
 *     try:             # <<<<<<<<<<<<<<
 *         if H5Pequal(clsid, H5P_FILE_CREATE):
 *             pcls = PropFCID
 */
  /*try:*/ {

    /* "h5py/h5p.pyx":40
 *     clsid = H5Pget_class(id_in)
 *     try:
 *         if H5Pequal(clsid, H5P_FILE_CREATE):             # <<<<<<<<<<<<<<
 *             pcls = PropFCID
 *         elif H5Pequal(clsid, H5P_FILE_ACCESS):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_FILE_CREATE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":41
 *     try:
 *         if H5Pequal(clsid, H5P_FILE_CREATE):
 *             pcls = PropFCID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_FILE_ACCESS):
 *             pcls = PropFAID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropFCID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropFCID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":42
 *         if H5Pequal(clsid, H5P_FILE_CREATE):
 *             pcls = PropFCID
 *         elif H5Pequal(clsid, H5P_FILE_ACCESS):             # <<<<<<<<<<<<<<
 *             pcls = PropFAID
 *         elif H5Pequal(clsid, H5P_DATASET_CREATE):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_FILE_ACCESS); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":43
 *             pcls = PropFCID
 *         elif H5Pequal(clsid, H5P_FILE_ACCESS):
 *             pcls = PropFAID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_DATASET_CREATE):
 *             pcls = PropDCID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropFAID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropFAID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":44
 *         elif H5Pequal(clsid, H5P_FILE_ACCESS):
 *             pcls = PropFAID
 *         elif H5Pequal(clsid, H5P_DATASET_CREATE):             # <<<<<<<<<<<<<<
 *             pcls = PropDCID
 *         elif H5Pequal(clsid, H5P_DATASET_XFER):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_DATASET_CREATE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":45
 *             pcls = PropFAID
 *         elif H5Pequal(clsid, H5P_DATASET_CREATE):
 *             pcls = PropDCID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_DATASET_XFER):
 *             pcls = PropDXID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropDCID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropDCID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":46
 *         elif H5Pequal(clsid, H5P_DATASET_CREATE):
 *             pcls = PropDCID
 *         elif H5Pequal(clsid, H5P_DATASET_XFER):             # <<<<<<<<<<<<<<
 *             pcls = PropDXID
 *         elif H5Pequal(clsid, H5P_OBJECT_COPY):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_DATASET_XFER); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 46; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":47
 *             pcls = PropDCID
 *         elif H5Pequal(clsid, H5P_DATASET_XFER):
 *             pcls = PropDXID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_OBJECT_COPY):
 *             pcls = PropCopyID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropDXID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropDXID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":48
 *         elif H5Pequal(clsid, H5P_DATASET_XFER):
 *             pcls = PropDXID
 *         elif H5Pequal(clsid, H5P_OBJECT_COPY):             # <<<<<<<<<<<<<<
 *             pcls = PropCopyID
 *         elif H5Pequal(clsid, H5P_LINK_CREATE):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_OBJECT_COPY); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":49
 *             pcls = PropDXID
 *         elif H5Pequal(clsid, H5P_OBJECT_COPY):
 *             pcls = PropCopyID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_LINK_CREATE):
 *             pcls = PropLCID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropCopyID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropCopyID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":50
 *         elif H5Pequal(clsid, H5P_OBJECT_COPY):
 *             pcls = PropCopyID
 *         elif H5Pequal(clsid, H5P_LINK_CREATE):             # <<<<<<<<<<<<<<
 *             pcls = PropLCID
 *         elif H5Pequal(clsid, H5P_LINK_ACCESS):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_LINK_CREATE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":51
 *             pcls = PropCopyID
 *         elif H5Pequal(clsid, H5P_LINK_CREATE):
 *             pcls = PropLCID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_LINK_ACCESS):
 *             pcls = PropLAID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropLCID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropLCID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":52
 *         elif H5Pequal(clsid, H5P_LINK_CREATE):
 *             pcls = PropLCID
 *         elif H5Pequal(clsid, H5P_LINK_ACCESS):             # <<<<<<<<<<<<<<
 *             pcls = PropLAID
 *         elif H5Pequal(clsid, H5P_GROUP_CREATE):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_LINK_ACCESS); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":53
 *             pcls = PropLCID
 *         elif H5Pequal(clsid, H5P_LINK_ACCESS):
 *             pcls = PropLAID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_GROUP_CREATE):
 *             pcls = PropGCID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropLAID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropLAID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":54
 *         elif H5Pequal(clsid, H5P_LINK_ACCESS):
 *             pcls = PropLAID
 *         elif H5Pequal(clsid, H5P_GROUP_CREATE):             # <<<<<<<<<<<<<<
 *             pcls = PropGCID
 *         elif H5Pequal(clsid, H5P_DATASET_ACCESS):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_GROUP_CREATE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":55
 *             pcls = PropLAID
 *         elif H5Pequal(clsid, H5P_GROUP_CREATE):
 *             pcls = PropGCID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_DATASET_ACCESS):
 *             pcls = PropDAID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropGCID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropGCID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":56
 *         elif H5Pequal(clsid, H5P_GROUP_CREATE):
 *             pcls = PropGCID
 *         elif H5Pequal(clsid, H5P_DATASET_ACCESS):             # <<<<<<<<<<<<<<
 *             pcls = PropDAID
 *         elif H5Pequal(clsid, H5P_OBJECT_CREATE):
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_DATASET_ACCESS); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":57
 *             pcls = PropGCID
 *         elif H5Pequal(clsid, H5P_DATASET_ACCESS):
 *             pcls = PropDAID             # <<<<<<<<<<<<<<
 *         elif H5Pequal(clsid, H5P_OBJECT_CREATE):
 *             pcls = PropOCID
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropDAID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropDAID);
      goto __pyx_L6;
    }

    /* "h5py/h5p.pyx":58
 *         elif H5Pequal(clsid, H5P_DATASET_ACCESS):
 *             pcls = PropDAID
 *         elif H5Pequal(clsid, H5P_OBJECT_CREATE):             # <<<<<<<<<<<<<<
 *             pcls = PropOCID
 * 
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_clsid, H5P_OBJECT_CREATE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (__pyx_t_2) {

      /* "h5py/h5p.pyx":59
 *             pcls = PropDAID
 *         elif H5Pequal(clsid, H5P_OBJECT_CREATE):
 *             pcls = PropOCID             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
      __Pyx_INCREF(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropOCID)));
      __pyx_v_pcls = ((PyObject*)__pyx_ptype_4h5py_3h5p_PropOCID);
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/h5p.pyx":62
 * 
 *         else:
 *             raise ValueError("No class found for ID %d" % id_in)             # <<<<<<<<<<<<<<
 * 
 *         return pcls.open(id_in)
 */
      __pyx_t_3 = __Pyx_PyInt_to_py_hid_t(__pyx_v_id_in); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyNumber_Remainder(((PyObject *)__pyx_kp_s_1), __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(((PyObject *)__pyx_t_4));
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_t_4));
      __Pyx_GIVEREF(((PyObject *)__pyx_t_4));
      __pyx_t_4 = 0;
      __pyx_t_4 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L4;}
    }
    __pyx_L6:;

    /* "h5py/h5p.pyx":64
 *             raise ValueError("No class found for ID %d" % id_in)
 * 
 *         return pcls.open(id_in)             # <<<<<<<<<<<<<<
 *     finally:
 *         H5Pclose_class(clsid)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = PyObject_GetAttr(((PyObject *)__pyx_v_pcls), __pyx_n_s__open); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyInt_to_py_hid_t(__pyx_v_id_in); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_Call(__pyx_t_4, ((PyObject *)__pyx_t_5), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3;
  }

  /* "h5py/h5p.pyx":66
 *         return pcls.open(id_in)
 *     finally:
 *         H5Pclose_class(clsid)             # <<<<<<<<<<<<<<
 * 
 * cdef object lockcls(hid_t id_in):
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_t_6 = __pyx_f_4h5py_4defs_H5Pclose_class(__pyx_v_clsid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
    goto __pyx_L8;
    __pyx_L7_error:;
    if (__pyx_why == 4) {
      Py_XDECREF(__pyx_exc_type);
      Py_XDECREF(__pyx_exc_value);
      Py_XDECREF(__pyx_exc_tb);
    }
    goto __pyx_L1_error;
    __pyx_L8:;
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("h5py.h5p.propwrap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pcls);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":68
 *         H5Pclose_class(clsid)
 * 
 * cdef object lockcls(hid_t id_in):             # <<<<<<<<<<<<<<
 *     cdef PropClassID pid
 *     pid = PropClassID.open(id_in)
 */

static PyObject *__pyx_f_4h5py_3h5p_lockcls(hid_t __pyx_v_id_in) {
  struct __pyx_obj_4h5py_3h5p_PropClassID *__pyx_v_pid = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lockcls", 0);

  /* "h5py/h5p.pyx":70
 * cdef object lockcls(hid_t id_in):
 *     cdef PropClassID pid
 *     pid = PropClassID.open(id_in)             # <<<<<<<<<<<<<<
 *     pid.locked = 1
 *     return pid
 */
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropClassID)), __pyx_n_s__open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_to_py_hid_t(__pyx_v_id_in); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_4h5py_3h5p_PropClassID))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_pid = ((struct __pyx_obj_4h5py_3h5p_PropClassID *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "h5py/h5p.pyx":71
 *     cdef PropClassID pid
 *     pid = PropClassID.open(id_in)
 *     pid.locked = 1             # <<<<<<<<<<<<<<
 *     return pid
 * 
 */
  __pyx_v_pid->__pyx_base.__pyx_base.locked = 1;

  /* "h5py/h5p.pyx":72
 *     pid = PropClassID.open(id_in)
 *     pid.locked = 1
 *     return pid             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_pid));
  __pyx_r = ((PyObject *)__pyx_v_pid);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5p.lockcls", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pid);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_1create(PyObject *__pyx_self, PyObject *__pyx_v_cls); /*proto*/
static char __pyx_doc_4h5py_3h5p_create[] = "(PropClassID cls) => PropID\n\n    Create a new property list as an instance of a class; classes are:\n\n    - FILE_CREATE\n    - FILE_ACCESS\n    - DATASET_CREATE\n    - DATASET_XFER\n    - DATASET_ACCESS\n    - LINK_CREATE\n    - LINK_ACCESS\n    - GROUP_CREATE\n    - OBJECT_COPY\n    - OBJECT_CREATE\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5p_1create = {__Pyx_NAMESTR("create"), (PyCFunction)__pyx_pw_4h5py_3h5p_1create, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_create)};
static PyObject *__pyx_pw_4h5py_3h5p_1create(PyObject *__pyx_self, PyObject *__pyx_v_cls) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_cls), __pyx_ptype_4h5py_3h5p_PropClassID, 0, "cls", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_create(__pyx_self, ((struct __pyx_obj_4h5py_3h5p_PropClassID *)__pyx_v_cls));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":106
 * # === Property list functional API ============================================
 * 
 * def create(PropClassID cls not None):             # <<<<<<<<<<<<<<
 *     """(PropClassID cls) => PropID
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_create(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_3h5p_PropClassID *__pyx_v_cls) {
  hid_t __pyx_v_newid;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create", 0);

  /* "h5py/h5p.pyx":123
 *     """
 *     cdef hid_t newid
 *     newid = H5Pcreate(cls.id)             # <<<<<<<<<<<<<<
 *     return propwrap(newid)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pcreate(__pyx_v_cls->__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_newid = __pyx_t_1;

  /* "h5py/h5p.pyx":124
 *     cdef hid_t newid
 *     newid = H5Pcreate(cls.id)
 *     return propwrap(newid)             # <<<<<<<<<<<<<<
 * 
 * # === Class API ===============================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_4h5py_3h5p_propwrap(__pyx_v_newid); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_6PropID_1equal(PyObject *__pyx_v_self, PyObject *__pyx_v_plist); /*proto*/
static char __pyx_doc_4h5py_3h5p_6PropID_equal[] = "(PropID plist) => BOOL\n\n        Compare this property list (or class) to another for equality.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_6PropID_1equal(PyObject *__pyx_v_self, PyObject *__pyx_v_plist) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("equal (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_plist), __pyx_ptype_4h5py_3h5p_PropID, 0, "plist", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_6PropID_equal(((struct __pyx_obj_4h5py_3h5p_PropID *)__pyx_v_self), ((struct __pyx_obj_4h5py_3h5p_PropID *)__pyx_v_plist));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":135
 * 
 * 
 *     def equal(self, PropID plist not None):             # <<<<<<<<<<<<<<
 *         """(PropID plist) => BOOL
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_6PropID_equal(struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_self, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_plist) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("equal", 0);

  /* "h5py/h5p.pyx":140
 *         Compare this property list (or class) to another for equality.
 *         """
 *         return <bint>(H5Pequal(self.id, plist.id))             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(self, object other, int how):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pequal(__pyx_v_self->__pyx_base.id, __pyx_v_plist->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyBool_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropID.equal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_6PropID_3__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5p_6PropID_3__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_6PropID_2__richcmp__(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_how));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":142
 *         return <bint>(H5Pequal(self.id, plist.id))
 * 
 *     def __richcmp__(self, object other, int how):             # <<<<<<<<<<<<<<
 *         cdef bint truthval = 0
 *         if how != 2 and how != 3:
 */

static PyObject *__pyx_pf_4h5py_3h5p_6PropID_2__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how) {
  int __pyx_v_truthval;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "h5py/h5p.pyx":143
 * 
 *     def __richcmp__(self, object other, int how):
 *         cdef bint truthval = 0             # <<<<<<<<<<<<<<
 *         if how != 2 and how != 3:
 *             return NotImplemented
 */
  __pyx_v_truthval = 0;

  /* "h5py/h5p.pyx":144
 *     def __richcmp__(self, object other, int how):
 *         cdef bint truthval = 0
 *         if how != 2 and how != 3:             # <<<<<<<<<<<<<<
 *             return NotImplemented
 *         if type(self) == type(other):
 */
  switch (__pyx_v_how) {
    case 2:
    case 3:
    __pyx_t_1 = 0;
    break;
    default:
    __pyx_t_1 = 1;
    break;
  }
  if (__pyx_t_1) {

    /* "h5py/h5p.pyx":145
 *         cdef bint truthval = 0
 *         if how != 2 and how != 3:
 *             return NotImplemented             # <<<<<<<<<<<<<<
 *         if type(self) == type(other):
 *             truthval = self.equal(other)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_builtin_NotImplemented);
    __pyx_r = __pyx_builtin_NotImplemented;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":146
 *         if how != 2 and how != 3:
 *             return NotImplemented
 *         if type(self) == type(other):             # <<<<<<<<<<<<<<
 *             truthval = self.equal(other)
 * 
 */
  __pyx_t_2 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_self)), ((PyObject *)Py_TYPE(__pyx_v_other)), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "h5py/h5p.pyx":147
 *             return NotImplemented
 *         if type(self) == type(other):
 *             truthval = self.equal(other)             # <<<<<<<<<<<<<<
 * 
 *         if how == 2:
 */
    __pyx_t_2 = PyObject_GetAttr(__pyx_v_self, __pyx_n_s__equal); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_other);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_other);
    __Pyx_GIVEREF(__pyx_v_other);
    __pyx_t_4 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_truthval = __pyx_t_1;
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "h5py/h5p.pyx":149
 *             truthval = self.equal(other)
 * 
 *         if how == 2:             # <<<<<<<<<<<<<<
 *             return truthval
 *         return not truthval
 */
  __pyx_t_1 = (__pyx_v_how == 2);
  if (__pyx_t_1) {

    /* "h5py/h5p.pyx":150
 * 
 *         if how == 2:
 *             return truthval             # <<<<<<<<<<<<<<
 *         return not truthval
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_truthval); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
    goto __pyx_L5;
  }
  __pyx_L5:;

  /* "h5py/h5p.pyx":151
 *         if how == 2:
 *             return truthval
 *         return not truthval             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBool_FromLong((!__pyx_v_truthval)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropID.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static Py_hash_t __pyx_pw_4h5py_3h5p_6PropID_5__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_4h5py_3h5p_6PropID_5__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_6PropID_4__hash__(((struct __pyx_obj_4h5py_3h5p_PropID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":153
 *         return not truthval
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         raise TypeError("Property lists are unhashable")
 * 
 */

static Py_hash_t __pyx_pf_4h5py_3h5p_6PropID_4__hash__(CYTHON_UNUSED struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "h5py/h5p.pyx":154
 * 
 *     def __hash__(self):
 *         raise TypeError("Property lists are unhashable")             # <<<<<<<<<<<<<<
 * 
 * cdef class PropClassID(PropID):
 */
  __pyx_t_1 = PyObject_Call(__pyx_builtin_TypeError, ((PyObject *)__pyx_k_tuple_3), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5p.PropID.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_11PropClassID_1__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5p_11PropClassID_1__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_11PropClassID___richcmp__(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_other), ((int)__pyx_v_how));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":165
 *     """
 * 
 *     def __richcmp__(self, object other, int how):             # <<<<<<<<<<<<<<
 *         return PropID.__richcmp__(self, other, how)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_11PropClassID___richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_how) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "h5py/h5p.pyx":166
 * 
 *     def __richcmp__(self, object other, int how):
 *         return PropID.__richcmp__(self, other, how)             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropID)), __pyx_n_s____richcmp__); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromLong(__pyx_v_how); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self);
  __Pyx_GIVEREF(__pyx_v_self);
  __Pyx_INCREF(__pyx_v_other);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_other);
  __Pyx_GIVEREF(__pyx_v_other);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5p.PropClassID.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static Py_hash_t __pyx_pw_4h5py_3h5p_11PropClassID_3__hash__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_4h5py_3h5p_11PropClassID_2__hash__[] = " Since classes are library-created and immutable, they are uniquely\n            identified by their HDF5 identifiers.\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_4h5py_3h5p_11PropClassID_2__hash__;
#endif
static Py_hash_t __pyx_pw_4h5py_3h5p_11PropClassID_3__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_11PropClassID_2__hash__(((struct __pyx_obj_4h5py_3h5p_PropClassID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":168
 *         return PropID.__richcmp__(self, other, how)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         """ Since classes are library-created and immutable, they are uniquely
 *             identified by their HDF5 identifiers.
 */

static Py_hash_t __pyx_pf_4h5py_3h5p_11PropClassID_2__hash__(struct __pyx_obj_4h5py_3h5p_PropClassID *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_hash_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "h5py/h5p.pyx":172
 *             identified by their HDF5 identifiers.
 *         """
 *         return hash(self.id)             # <<<<<<<<<<<<<<
 * 
 * cdef class PropInstanceID(PropID):
 */
  __pyx_t_1 = __Pyx_PyInt_to_py_hid_t(__pyx_v_self->__pyx_base.__pyx_base.id); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Hash(__pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5p.PropClassID.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_14PropInstanceID_1copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_14PropInstanceID_copy[] = "() => PropList newid\n\n         Create a new copy of an existing property list object.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_14PropInstanceID_1copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_14PropInstanceID_copy(((struct __pyx_obj_4h5py_3h5p_PropInstanceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":185
 * 
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         """() => PropList newid
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_14PropInstanceID_copy(struct __pyx_obj_4h5py_3h5p_PropInstanceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "h5py/h5p.pyx":190
 *          Create a new copy of an existing property list object.
 *         """
 *         return type(self)(H5Pcopy(self.id))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pcopy(__pyx_v_self->__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_to_py_hid_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))), ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5p.PropInstanceID.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_14PropInstanceID_3_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_14PropInstanceID_2_close[] = "()\n\n        Terminate access through this identifier.  You shouldn't have to\n        do this manually, as propery lists are automatically deleted when\n        their Python wrappers are freed.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_14PropInstanceID_3_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_close (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_14PropInstanceID_2_close(((struct __pyx_obj_4h5py_3h5p_PropInstanceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":193
 * 
 * 
 *     def _close(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_14PropInstanceID_2_close(struct __pyx_obj_4h5py_3h5p_PropInstanceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  herr_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_close", 0);

  /* "h5py/h5p.pyx":200
 *         their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Pclose(self.id)
 *             if not self.valid:
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_GetName(__pyx_m, __pyx_n_s___objects); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s__registry); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_GetAttr(__pyx_t_2, __pyx_n_s__lock); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s____exit__); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s____enter__); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "h5py/h5p.pyx":201
 *         """
 *         with _objects.registry.lock:
 *             H5Pclose(self.id)             # <<<<<<<<<<<<<<
 *             if not self.valid:
 *                 del _objects.registry[self.id]
 */
          __pyx_t_8 = __pyx_f_4h5py_4defs_H5Pclose(__pyx_v_self->__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L7_error;}

          /* "h5py/h5p.pyx":202
 *         with _objects.registry.lock:
 *             H5Pclose(self.id)
 *             if not self.valid:             # <<<<<<<<<<<<<<
 *                 del _objects.registry[self.id]
 * 
 */
          __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__valid); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_10 = (!__pyx_t_9);
          if (__pyx_t_10) {

            /* "h5py/h5p.pyx":203
 *             H5Pclose(self.id)
 *             if not self.valid:
 *                 del _objects.registry[self.id]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_1 = __Pyx_GetName(__pyx_m, __pyx_n_s___objects); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_4 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s__registry); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (__Pyx_DelItemInt(__pyx_t_4, __pyx_v_self->__pyx_base.__pyx_base.id, sizeof(hid_t), __Pyx_PyInt_to_py_hid_t) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            goto __pyx_L15;
          }
          __pyx_L15:;
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L14_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "h5py/h5p.pyx":200
 *         their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Pclose(self.id)
 *             if not self.valid:
 */
        /*except:*/ {
          __Pyx_AddTraceback("h5py.h5p.PropInstanceID._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_2);
          __Pyx_GIVEREF(__pyx_t_2);
          __pyx_t_12 = PyObject_Call(__pyx_t_3, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __pyx_t_9 = (!__pyx_t_10);
          if (__pyx_t_9) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_ErrRestore(__pyx_t_4, __pyx_t_1, __pyx_t_2);
            __pyx_t_4 = 0; __pyx_t_1 = 0; __pyx_t_2 = 0; 
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
            goto __pyx_L18;
          }
          __pyx_L18:;
          __Pyx_DECREF(((PyObject *)__pyx_t_11)); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L14_try_end:;
      }
    }
    /*finally:*/ {
      if (__pyx_t_3) {
        __pyx_t_7 = PyObject_Call(__pyx_t_3, __pyx_k_tuple_4, NULL);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    goto __pyx_L19;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L1_error;
    __pyx_L19:;
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("h5py.h5p.PropInstanceID._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_14PropInstanceID_5get_class(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_14PropInstanceID_4get_class[] = "() => PropClassID\n\n        Determine the class of a property list object.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_14PropInstanceID_5get_class(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_class (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_14PropInstanceID_4get_class(((struct __pyx_obj_4h5py_3h5p_PropInstanceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":206
 * 
 * 
 *     def get_class(self):             # <<<<<<<<<<<<<<
 *         """() => PropClassID
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_14PropInstanceID_4get_class(struct __pyx_obj_4h5py_3h5p_PropInstanceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_class", 0);

  /* "h5py/h5p.pyx":211
 *         Determine the class of a property list object.
 *         """
 *         return PropClassID.open(H5Pget_class(self.id))             # <<<<<<<<<<<<<<
 * 
 * cdef class PropCreateID(PropInstanceID):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropClassID)), __pyx_n_s__open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pget_class(__pyx_v_self->__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = PyInt_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_4), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropInstanceID.get_class", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_10PropCopyID_1set_copy_object(PyObject *__pyx_v_self, PyObject *__pyx_arg_flags); /*proto*/
static char __pyx_doc_4h5py_3h5p_10PropCopyID_set_copy_object[] = "(UINT flags)\n\n        Set flags for object copying process.  Legal flags are\n        from the h5o.COPY* family:\n\n        h5o.COPY_SHALLOW_HIERARCHY_FLAG\n            Copy only immediate members of a group.\n\n        h5o.COPY_EXPAND_SOFT_LINK_FLAG\n            Expand soft links into new objects.\n\n        h5o.COPY_EXPAND_EXT_LINK_FLAG\n            Expand external link into new objects.\n\n        h5o.COPY_EXPAND_REFERENCE_FLAG\n            Copy objects that are pointed to by references.\n\n        h5o.COPY_WITHOUT_ATTR_FLAG\n            Copy object without copying attributes.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_10PropCopyID_1set_copy_object(PyObject *__pyx_v_self, PyObject *__pyx_arg_flags) {
  unsigned int __pyx_v_flags;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_copy_object (wrapper)", 0);
  assert(__pyx_arg_flags); {
    __pyx_v_flags = __Pyx_PyInt_AsUnsignedInt(__pyx_arg_flags); if (unlikely((__pyx_v_flags == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropCopyID.set_copy_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_10PropCopyID_set_copy_object(((struct __pyx_obj_4h5py_3h5p_PropCopyID *)__pyx_v_self), ((unsigned int)__pyx_v_flags));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":227
 * 
 * 
 *     def set_copy_object(self, unsigned int flags):             # <<<<<<<<<<<<<<
 *         """(UINT flags)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_10PropCopyID_set_copy_object(struct __pyx_obj_4h5py_3h5p_PropCopyID *__pyx_v_self, unsigned int __pyx_v_flags) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_copy_object", 0);

  /* "h5py/h5p.pyx":248
 *             Copy object without copying attributes.
 *         """
 *         H5Pset_copy_object(self.id, flags)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_copy_object(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_flags); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropCopyID.set_copy_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_10PropCopyID_3get_copy_object(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_10PropCopyID_2get_copy_object[] = "() => UINT flags\n\n        Get copy process flags. Legal flags are h5o.COPY*.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_10PropCopyID_3get_copy_object(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_copy_object (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_10PropCopyID_2get_copy_object(((struct __pyx_obj_4h5py_3h5p_PropCopyID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":251
 * 
 * 
 *     def get_copy_object(self):             # <<<<<<<<<<<<<<
 *         """() => UINT flags
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_10PropCopyID_2get_copy_object(struct __pyx_obj_4h5py_3h5p_PropCopyID *__pyx_v_self) {
  unsigned int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_copy_object", 0);

  /* "h5py/h5p.pyx":257
 *         """
 *         cdef unsigned int flags
 *         H5Pget_copy_object(self.id, &flags)             # <<<<<<<<<<<<<<
 *         return flags
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_copy_object(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_flags)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":258
 *         cdef unsigned int flags
 *         H5Pget_copy_object(self.id, &flags)
 *         return flags             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyLong_FromUnsignedLong(__pyx_v_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropCopyID.get_copy_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_1get_version(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_get_version[] = "() => TUPLE version_info\n\n        Determine version information of various file attributes.\n        Elements are:\n\n        0.  UINT Super block version number\n        1.  UINT Freelist version number\n        2.  UINT Symbol table version number\n        3.  UINT Shared object header version number\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_1get_version(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_version (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_get_version(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":272
 * 
 * 
 *     def get_version(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE version_info
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_get_version(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self) {
  unsigned int __pyx_v_super_;
  unsigned int __pyx_v_freelist;
  unsigned int __pyx_v_stab;
  unsigned int __pyx_v_shhdr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_version", 0);

  /* "h5py/h5p.pyx":289
 *         cdef unsigned int shhdr
 * 
 *         H5Pget_version(self.id, &super_, &freelist, &stab, &shhdr)             # <<<<<<<<<<<<<<
 * 
 *         return (super_, freelist, stab, shhdr)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_version(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_super_), (&__pyx_v_freelist), (&__pyx_v_stab), (&__pyx_v_shhdr)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":291
 *         H5Pget_version(self.id, &super_, &freelist, &stab, &shhdr)
 * 
 *         return (super_, freelist, stab, shhdr)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyLong_FromUnsignedLong(__pyx_v_super_); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyLong_FromUnsignedLong(__pyx_v_freelist); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyLong_FromUnsignedLong(__pyx_v_stab); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyLong_FromUnsignedLong(__pyx_v_shhdr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_r = ((PyObject *)__pyx_t_6);
  __pyx_t_6 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("h5py.h5p.PropFCID.get_version", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_3set_userblock(PyObject *__pyx_v_self, PyObject *__pyx_arg_size); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_2set_userblock[] = "(INT/LONG size)\n\n        Set the file user block size, in bytes.\n        Must be a power of 2, and at least 512.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_3set_userblock(PyObject *__pyx_v_self, PyObject *__pyx_arg_size) {
  hsize_t __pyx_v_size;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_userblock (wrapper)", 0);
  assert(__pyx_arg_size); {
    __pyx_v_size = __Pyx_PyInt_from_py_hsize_t(__pyx_arg_size); if (unlikely((__pyx_v_size == (hsize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 294; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFCID.set_userblock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_2set_userblock(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self), ((hsize_t)__pyx_v_size));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":294
 * 
 * 
 *     def set_userblock(self, hsize_t size):             # <<<<<<<<<<<<<<
 *         """(INT/LONG size)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_2set_userblock(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self, hsize_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_userblock", 0);

  /* "h5py/h5p.pyx":300
 *         Must be a power of 2, and at least 512.
 *         """
 *         H5Pset_userblock(self.id, size)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_userblock(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_size); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFCID.set_userblock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_5get_userblock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_4get_userblock[] = "() => LONG size\n\n        Determine the user block size, in bytes.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_5get_userblock(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_userblock (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_4get_userblock(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":303
 * 
 * 
 *     def get_userblock(self):             # <<<<<<<<<<<<<<
 *         """() => LONG size
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_4get_userblock(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self) {
  hsize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_userblock", 0);

  /* "h5py/h5p.pyx":309
 *         """
 *         cdef hsize_t size
 *         H5Pget_userblock(self.id, &size)             # <<<<<<<<<<<<<<
 *         return size
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_userblock(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_size)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 309; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":310
 *         cdef hsize_t size
 *         H5Pget_userblock(self.id, &size)
 *         return size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_to_py_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropFCID.get_userblock", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_7set_sizes(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_6set_sizes[] = "(UINT addr, UINT size)\n\n        Set the addressing offsets and lengths for objects\n        in an HDF5 file, in bytes.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_7set_sizes(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_addr;
  size_t __pyx_v_size;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_sizes (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__addr,&__pyx_n_s__size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__addr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_sizes", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_sizes") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_addr = __Pyx_PyInt_AsSize_t(values[0]); if (unlikely((__pyx_v_addr == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_size = __Pyx_PyInt_AsSize_t(values[1]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_sizes", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFCID.set_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_6set_sizes(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self), __pyx_v_addr, __pyx_v_size);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":313
 * 
 * 
 *     def set_sizes(self, size_t addr, size_t size):             # <<<<<<<<<<<<<<
 *         """(UINT addr, UINT size)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_6set_sizes(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self, size_t __pyx_v_addr, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_sizes", 0);

  /* "h5py/h5p.pyx":319
 *         in an HDF5 file, in bytes.
 *         """
 *         H5Pset_sizes(self.id, addr, size)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_sizes(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_addr, __pyx_v_size); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFCID.set_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_9get_sizes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_8get_sizes[] = "() => TUPLE sizes\n\n        Determine addressing offsets and lengths for objects in an\n        HDF5 file, in bytes.  Return value is a 2-tuple with values:\n\n        0.  UINT Address offsets\n        1.  UINT Lengths\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_9get_sizes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sizes (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_8get_sizes(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":322
 * 
 * 
 *     def get_sizes(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE sizes
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_8get_sizes(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self) {
  size_t __pyx_v_addr;
  size_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sizes", 0);

  /* "h5py/h5p.pyx":333
 *         cdef size_t addr
 *         cdef size_t size
 *         H5Pget_sizes(self.id, &addr, &size)             # <<<<<<<<<<<<<<
 *         return (addr, size)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_sizes(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_addr), (&__pyx_v_size)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":334
 *         cdef size_t size
 *         H5Pget_sizes(self.id, &addr, &size)
 *         return (addr, size)             # <<<<<<<<<<<<<<
 * 
 *     def set_link_creation_order(self, unsigned int flags):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_addr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = ((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropFCID.get_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_11set_link_creation_order(PyObject *__pyx_v_self, PyObject *__pyx_arg_flags); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_10set_link_creation_order[] = " (UINT flags)\n\n        Set tracking and indexing of creation order for links added to this group\n\n        flags -- h5p.CRT_ORDER_TRACKED, h5p.CRT_ORDER_INDEXED\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_11set_link_creation_order(PyObject *__pyx_v_self, PyObject *__pyx_arg_flags) {
  unsigned int __pyx_v_flags;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_link_creation_order (wrapper)", 0);
  assert(__pyx_arg_flags); {
    __pyx_v_flags = __Pyx_PyInt_AsUnsignedInt(__pyx_arg_flags); if (unlikely((__pyx_v_flags == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFCID.set_link_creation_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_10set_link_creation_order(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self), ((unsigned int)__pyx_v_flags));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":336
 *         return (addr, size)
 * 
 *     def set_link_creation_order(self, unsigned int flags):             # <<<<<<<<<<<<<<
 *         """ (UINT flags)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_10set_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self, unsigned int __pyx_v_flags) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_link_creation_order", 0);

  /* "h5py/h5p.pyx":343
 *         flags -- h5p.CRT_ORDER_TRACKED, h5p.CRT_ORDER_INDEXED
 *         """
 *         H5Pset_link_creation_order(self.id, flags)             # <<<<<<<<<<<<<<
 * 
 *     def get_link_creation_order(self):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_link_creation_order(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_flags); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 343; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFCID.set_link_creation_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_13get_link_creation_order(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFCID_12get_link_creation_order[] = " () -> UINT flags\n\n        Get tracking and indexing of creation order for links added to this group\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFCID_13get_link_creation_order(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_link_creation_order (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFCID_12get_link_creation_order(((struct __pyx_obj_4h5py_3h5p_PropFCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":345
 *         H5Pset_link_creation_order(self.id, flags)
 * 
 *     def get_link_creation_order(self):             # <<<<<<<<<<<<<<
 *         """ () -> UINT flags
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFCID_12get_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropFCID *__pyx_v_self) {
  unsigned int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_link_creation_order", 0);

  /* "h5py/h5p.pyx":351
 *         """
 *         cdef unsigned int flags
 *         H5Pget_link_creation_order(self.id, &flags)             # <<<<<<<<<<<<<<
 *         return flags
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_link_creation_order(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_flags)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 351; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":352
 *         cdef unsigned int flags
 *         H5Pget_link_creation_order(self.id, &flags)
 *         return flags             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyLong_FromUnsignedLong(__pyx_v_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropFCID.get_link_creation_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_1set_layout(PyObject *__pyx_v_self, PyObject *__pyx_arg_layout_code); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_set_layout[] = "(INT layout_code)\n\n        Set dataset storage strategy; legal values are:\n\n        - h5d.COMPACT\n        - h5d.CONTIGUOUS\n        - h5d.CHUNKED\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_1set_layout(PyObject *__pyx_v_self, PyObject *__pyx_arg_layout_code) {
  int __pyx_v_layout_code;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_layout (wrapper)", 0);
  assert(__pyx_arg_layout_code); {
    __pyx_v_layout_code = __Pyx_PyInt_AsInt(__pyx_arg_layout_code); if (unlikely((__pyx_v_layout_code == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 363; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_layout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_set_layout(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_layout_code));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":363
 * 
 * 
 *     def set_layout(self, int layout_code):             # <<<<<<<<<<<<<<
 *         """(INT layout_code)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_set_layout(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_layout_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_layout", 0);

  /* "h5py/h5p.pyx":372
 *         - h5d.CHUNKED
 *         """
 *         H5Pset_layout(self.id, layout_code)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_layout(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_layout_code); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_layout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_3get_layout(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_2get_layout[] = "() => INT layout_code\n\n        Determine the storage strategy of a dataset; legal values are:\n\n        - h5d.COMPACT\n        - h5d.CONTIGUOUS\n        - h5d.CHUNKED\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_3get_layout(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_layout (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_2get_layout(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":375
 * 
 * 
 *     def get_layout(self):             # <<<<<<<<<<<<<<
 *         """() => INT layout_code
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_2get_layout(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  H5D_layout_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_layout", 0);

  /* "h5py/h5p.pyx":384
 *         - h5d.CHUNKED
 *         """
 *         return <int>H5Pget_layout(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_layout(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 384; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_layout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_5set_chunk(PyObject *__pyx_v_self, PyObject *__pyx_v_chunksize); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_4set_chunk[] = "(TUPLE chunksize)\n\n        Set the dataset chunk size.  It's up to you to provide\n        values which are compatible with your dataset.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_5set_chunk(PyObject *__pyx_v_self, PyObject *__pyx_v_chunksize) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_chunk (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_4set_chunk(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((PyObject *)__pyx_v_chunksize));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":387
 * 
 * 
 *     def set_chunk(self, object chunksize):             # <<<<<<<<<<<<<<
 *         """(TUPLE chunksize)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_4set_chunk(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, PyObject *__pyx_v_chunksize) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_dims;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  void *__pyx_t_3;
  herr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_chunk", 0);

  /* "h5py/h5p.pyx":395
 *         cdef int rank
 *         cdef hsize_t* dims
 *         dims = NULL             # <<<<<<<<<<<<<<
 * 
 *         require_tuple(chunksize, 0, -1, "chunksize")
 */
  __pyx_v_dims = NULL;

  /* "h5py/h5p.pyx":397
 *         dims = NULL
 * 
 *         require_tuple(chunksize, 0, -1, "chunksize")             # <<<<<<<<<<<<<<
 *         rank = len(chunksize)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_chunksize, 0, -1, __pyx_k__chunksize); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":398
 * 
 *         require_tuple(chunksize, 0, -1, "chunksize")
 *         rank = len(chunksize)             # <<<<<<<<<<<<<<
 * 
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_chunksize); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_2;

  /* "h5py/h5p.pyx":400
 *         rank = len(chunksize)
 * 
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *         try:
 *             convert_tuple(chunksize, dims, rank)
 */
  __pyx_t_3 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_3 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dims = ((hsize_t *)__pyx_t_3);

  /* "h5py/h5p.pyx":401
 * 
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         try:             # <<<<<<<<<<<<<<
 *             convert_tuple(chunksize, dims, rank)
 *             H5Pset_chunk(self.id, rank, dims)
 */
  /*try:*/ {

    /* "h5py/h5p.pyx":402
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         try:
 *             convert_tuple(chunksize, dims, rank)             # <<<<<<<<<<<<<<
 *             H5Pset_chunk(self.id, rank, dims)
 *         finally:
 */
    __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_chunksize, __pyx_v_dims, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 402; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5p.pyx":403
 *         try:
 *             convert_tuple(chunksize, dims, rank)
 *             H5Pset_chunk(self.id, rank, dims)             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(dims)
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Pset_chunk(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_rank, __pyx_v_dims); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 403; __pyx_clineno = __LINE__; goto __pyx_L4;}
  }

  /* "h5py/h5p.pyx":405
 *             H5Pset_chunk(self.id, rank, dims)
 *         finally:
 *             efree(dims)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_dims);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_7get_chunk(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_6get_chunk[] = "() => TUPLE chunk_dimensions\n\n        Obtain the dataset chunk size, as a tuple.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_7get_chunk(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_chunk (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_6get_chunk(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":408
 * 
 * 
 *     def get_chunk(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE chunk_dimensions
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_6get_chunk(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_dims;
  PyObject *__pyx_v_tpl = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_chunk", 0);

  /* "h5py/h5p.pyx":416
 *         cdef hsize_t *dims
 * 
 *         rank = H5Pget_chunk(self.id, 0, NULL)             # <<<<<<<<<<<<<<
 *         assert rank >= 0
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_chunk(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, 0, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_1;

  /* "h5py/h5p.pyx":417
 * 
 *         rank = H5Pget_chunk(self.id, 0, NULL)
 *         assert rank >= 0             # <<<<<<<<<<<<<<
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!(__pyx_v_rank >= 0))) {
    PyErr_SetNone(PyExc_AssertionError);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #endif

  /* "h5py/h5p.pyx":418
 *         rank = H5Pget_chunk(self.id, 0, NULL)
 *         assert rank >= 0
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dims = ((hsize_t *)__pyx_t_2);

  /* "h5py/h5p.pyx":420
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             H5Pget_chunk(self.id, rank, dims)
 *             tpl = convert_dims(dims, rank)
 */
  /*try:*/ {

    /* "h5py/h5p.pyx":421
 * 
 *         try:
 *             H5Pget_chunk(self.id, rank, dims)             # <<<<<<<<<<<<<<
 *             tpl = convert_dims(dims, rank)
 *             return tpl
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_chunk(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_rank, __pyx_v_dims); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 421; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5p.pyx":422
 *         try:
 *             H5Pget_chunk(self.id, rank, dims)
 *             tpl = convert_dims(dims, rank)             # <<<<<<<<<<<<<<
 *             return tpl
 *         finally:
 */
    __pyx_t_3 = __pyx_f_4h5py_5utils_convert_dims(__pyx_v_dims, __pyx_v_rank); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 422; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_tpl = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "h5py/h5p.pyx":423
 *             H5Pget_chunk(self.id, rank, dims)
 *             tpl = convert_dims(dims, rank)
 *             return tpl             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(dims)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_tpl);
    __pyx_r = __pyx_v_tpl;
    goto __pyx_L3;
  }

  /* "h5py/h5p.pyx":425
 *             return tpl
 *         finally:
 *             efree(dims)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_dims);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tpl);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_9set_fill_value(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_8set_fill_value[] = "(NDARRAY value)\n\n        Set the dataset fill value.  The object provided should be an\n        0-dimensional NumPy array; otherwise, the value will be read from\n        the first element.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_9set_fill_value(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fill_value (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_4h5py_5numpy_ndarray, 0, "value", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 428; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_8set_fill_value(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((PyArrayObject *)__pyx_v_value));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":428
 * 
 * 
 *     def set_fill_value(self, ndarray value not None):             # <<<<<<<<<<<<<<
 *         """(NDARRAY value)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_8set_fill_value(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, PyArrayObject *__pyx_v_value) {
  struct __pyx_obj_4h5py_3h5t_TypeID *__pyx_v_tid = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_opt_args_4h5py_5utils_check_numpy_read __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  herr_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fill_value", 0);

  /* "h5py/h5p.pyx":437
 *         cdef TypeID tid
 * 
 *         check_numpy_read(value, -1)             # <<<<<<<<<<<<<<
 *         tid = py_create(value.dtype)
 *         H5Pset_fill_value(self.id, tid.id, value.data)
 */
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.space_id = -1;
  __pyx_t_1 = __pyx_f_4h5py_5utils_check_numpy_read(__pyx_v_value, 0, &__pyx_t_2); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 437; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":438
 * 
 *         check_numpy_read(value, -1)
 *         tid = py_create(value.dtype)             # <<<<<<<<<<<<<<
 *         H5Pset_fill_value(self.id, tid.id, value.data)
 * 
 */
  __pyx_t_3 = PyObject_GetAttr(((PyObject *)__pyx_v_value), __pyx_n_s__dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = ((PyObject *)__pyx_f_4h5py_3h5t_py_create(__pyx_t_3, 0, NULL)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tid = ((struct __pyx_obj_4h5py_3h5t_TypeID *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "h5py/h5p.pyx":439
 *         check_numpy_read(value, -1)
 *         tid = py_create(value.dtype)
 *         H5Pset_fill_value(self.id, tid.id, value.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_f_4h5py_4defs_H5Pset_fill_value(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_tid->__pyx_base.id, __pyx_v_value->data); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 439; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_fill_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tid);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_11get_fill_value(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_10get_fill_value[] = "(NDARRAY value)\n\n        Read the dataset fill value into a NumPy array.  It will be\n        converted to match the array dtype.  If the array has nonzero\n        rank, only the first element will contain the value.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_11get_fill_value(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fill_value (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_4h5py_5numpy_ndarray, 0, "value", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_10get_fill_value(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((PyArrayObject *)__pyx_v_value));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":442
 * 
 * 
 *     def get_fill_value(self, ndarray value not None):             # <<<<<<<<<<<<<<
 *         """(NDARRAY value)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_10get_fill_value(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, PyArrayObject *__pyx_v_value) {
  struct __pyx_obj_4h5py_3h5t_TypeID *__pyx_v_tid = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_opt_args_4h5py_5utils_check_numpy_write __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  herr_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fill_value", 0);

  /* "h5py/h5p.pyx":451
 *         cdef TypeID tid
 * 
 *         check_numpy_write(value, -1)             # <<<<<<<<<<<<<<
 *         tid = py_create(value.dtype)
 *         H5Pget_fill_value(self.id, tid.id, value.data)
 */
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.space_id = -1;
  __pyx_t_1 = __pyx_f_4h5py_5utils_check_numpy_write(__pyx_v_value, 0, &__pyx_t_2); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 451; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":452
 * 
 *         check_numpy_write(value, -1)
 *         tid = py_create(value.dtype)             # <<<<<<<<<<<<<<
 *         H5Pget_fill_value(self.id, tid.id, value.data)
 * 
 */
  __pyx_t_3 = PyObject_GetAttr(((PyObject *)__pyx_v_value), __pyx_n_s__dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = ((PyObject *)__pyx_f_4h5py_3h5t_py_create(__pyx_t_3, 0, NULL)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 452; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tid = ((struct __pyx_obj_4h5py_3h5t_TypeID *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "h5py/h5p.pyx":453
 *         check_numpy_write(value, -1)
 *         tid = py_create(value.dtype)
 *         H5Pget_fill_value(self.id, tid.id, value.data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_f_4h5py_4defs_H5Pget_fill_value(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_tid->__pyx_base.id, __pyx_v_value->data); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_fill_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tid);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_13fill_value_defined(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_12fill_value_defined[] = "() => INT fill_status\n\n        Determine the status of the dataset fill value.  Return values are:\n\n        - h5d.FILL_VALUE_UNDEFINED\n        - h5d.FILL_VALUE_DEFAULT\n        - h5d.FILL_VALUE_USER_DEFINED\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_13fill_value_defined(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fill_value_defined (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_12fill_value_defined(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":456
 * 
 * 
 *     def fill_value_defined(self):             # <<<<<<<<<<<<<<
 *         """() => INT fill_status
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_12fill_value_defined(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  H5D_fill_value_t __pyx_v_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fill_value_defined", 0);

  /* "h5py/h5p.pyx":466
 *         """
 *         cdef H5D_fill_value_t val
 *         H5Pfill_value_defined(self.id, &val)             # <<<<<<<<<<<<<<
 *         return <int>val
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pfill_value_defined(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_val)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":467
 *         cdef H5D_fill_value_t val
 *         H5Pfill_value_defined(self.id, &val)
 *         return <int>val             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_v_val)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 467; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.fill_value_defined", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_15set_fill_time(PyObject *__pyx_v_self, PyObject *__pyx_arg_fill_time); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_14set_fill_time[] = "(INT fill_time)\n\n        Define when fill values are written to the dataset.  Legal\n        values (defined in module h5d) are:\n\n        - h5d.FILL_TIME_ALLOC\n        - h5d.FILL_TIME_NEVER\n        - h5d.FILL_TIME_IFSET\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_15set_fill_time(PyObject *__pyx_v_self, PyObject *__pyx_arg_fill_time) {
  int __pyx_v_fill_time;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fill_time (wrapper)", 0);
  assert(__pyx_arg_fill_time); {
    __pyx_v_fill_time = __Pyx_PyInt_AsInt(__pyx_arg_fill_time); if (unlikely((__pyx_v_fill_time == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 470; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_fill_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_14set_fill_time(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_fill_time));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":470
 * 
 * 
 *     def set_fill_time(self, int fill_time):             # <<<<<<<<<<<<<<
 *         """(INT fill_time)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_14set_fill_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_fill_time) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fill_time", 0);

  /* "h5py/h5p.pyx":480
 *         - h5d.FILL_TIME_IFSET
 *         """
 *         H5Pset_fill_time(self.id, <H5D_fill_time_t>fill_time)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fill_time(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, ((H5D_fill_time_t)__pyx_v_fill_time)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 480; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_fill_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_17get_fill_time(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_16get_fill_time[] = " () => INT\n\n        Determine when fill values are written to the dataset.  Legal\n        values (defined in module h5d) are:\n\n        - h5d.FILL_TIME_ALLOC\n        - h5d.FILL_TIME_NEVER\n        - h5d.FILL_TIME_IFSET\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_17get_fill_time(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fill_time (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_16get_fill_time(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":483
 * 
 * 
 *     def get_fill_time(self):             # <<<<<<<<<<<<<<
 *         """ () => INT
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_16get_fill_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  H5D_fill_time_t __pyx_v_fill_time;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fill_time", 0);

  /* "h5py/h5p.pyx":494
 *         """
 *         cdef H5D_fill_time_t fill_time
 *         H5Pget_fill_time(self.id, &fill_time)             # <<<<<<<<<<<<<<
 *         return <int>fill_time
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_fill_time(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_fill_time)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 494; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":495
 *         cdef H5D_fill_time_t fill_time
 *         H5Pget_fill_time(self.id, &fill_time)
 *         return <int>fill_time             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_v_fill_time)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 495; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_fill_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_19set_alloc_time(PyObject *__pyx_v_self, PyObject *__pyx_arg_alloc_time); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_18set_alloc_time[] = "(INT alloc_time)\n\n        Set the storage space allocation time.  One of h5d.ALLOC_TIME*.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_19set_alloc_time(PyObject *__pyx_v_self, PyObject *__pyx_arg_alloc_time) {
  int __pyx_v_alloc_time;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_alloc_time (wrapper)", 0);
  assert(__pyx_arg_alloc_time); {
    __pyx_v_alloc_time = __Pyx_PyInt_AsInt(__pyx_arg_alloc_time); if (unlikely((__pyx_v_alloc_time == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 498; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_alloc_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_18set_alloc_time(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_alloc_time));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":498
 * 
 * 
 *     def set_alloc_time(self, int alloc_time):             # <<<<<<<<<<<<<<
 *         """(INT alloc_time)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_18set_alloc_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_alloc_time) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_alloc_time", 0);

  /* "h5py/h5p.pyx":503
 *         Set the storage space allocation time.  One of h5d.ALLOC_TIME*.
 *         """
 *         H5Pset_alloc_time(self.id, <H5D_alloc_time_t>alloc_time)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_alloc_time(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, ((H5D_alloc_time_t)__pyx_v_alloc_time)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 503; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_alloc_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_21get_alloc_time(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_20get_alloc_time[] = "() => INT alloc_time\n\n        Get the storage space allocation time.  One of h5d.ALLOC_TIME*.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_21get_alloc_time(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_alloc_time (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_20get_alloc_time(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":506
 * 
 * 
 *     def get_alloc_time(self):             # <<<<<<<<<<<<<<
 *         """() => INT alloc_time
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_20get_alloc_time(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  H5D_alloc_time_t __pyx_v_alloc_time;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_alloc_time", 0);

  /* "h5py/h5p.pyx":512
 *         """
 *         cdef H5D_alloc_time_t alloc_time
 *         H5Pget_alloc_time(self.id, &alloc_time)             # <<<<<<<<<<<<<<
 *         return <int>alloc_time
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_alloc_time(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_alloc_time)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 512; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":513
 *         cdef H5D_alloc_time_t alloc_time
 *         H5Pget_alloc_time(self.id, &alloc_time)
 *         return <int>alloc_time             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_v_alloc_time)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 513; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_alloc_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_23set_filter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_22set_filter[] = "(INT filter_code, UINT flags=0, TUPLE values=None)\n\n        Set a filter in the pipeline.  Params are:\n\n        filter_code\n            One of the following:\n\n            - h5z.FILTER_DEFLATE\n            - h5z.FILTER_SHUFFLE\n            - h5z.FILTER_FLETCHER32\n            - h5z.FILTER_SZIP\n\n        flags\n            Bit flags (h5z.FLAG*) setting filter properties\n\n        values\n            TUPLE of UINTs giving auxiliary data for the filter\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_23set_filter(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_filter_code;
  unsigned int __pyx_v_flags;
  PyObject *__pyx_v_values = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_filter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__filter_code,&__pyx_n_s__flags,&__pyx_n_s__values,0};
    PyObject* values[3] = {0,0,0};

    /* "h5py/h5p.pyx":519
 * 
 * 
 *     def set_filter(self, int filter_code, unsigned int flags=0, object values=None):             # <<<<<<<<<<<<<<
 *         """(INT filter_code, UINT flags=0, TUPLE values=None)
 * 
 */
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__filter_code)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__flags);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__values);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_filter") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_filter_code = __Pyx_PyInt_AsInt(values[0]); if (unlikely((__pyx_v_filter_code == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[1]) {
      __pyx_v_flags = __Pyx_PyInt_AsUnsignedInt(values[1]); if (unlikely((__pyx_v_flags == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_flags = ((unsigned int)0);
    }
    __pyx_v_values = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_filter", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_22set_filter(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), __pyx_v_filter_code, __pyx_v_flags, __pyx_v_values);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_22set_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_code, unsigned int __pyx_v_flags, PyObject *__pyx_v_values) {
  size_t __pyx_v_nelements;
  unsigned int *__pyx_v_cd_values;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  void *__pyx_t_6;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  herr_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_filter", 0);

  /* "h5py/h5p.pyx":541
 *         cdef unsigned int *cd_values
 *         cdef int i
 *         cd_values = NULL             # <<<<<<<<<<<<<<
 * 
 *         require_tuple(values, 1, -1, "values")
 */
  __pyx_v_cd_values = NULL;

  /* "h5py/h5p.pyx":543
 *         cd_values = NULL
 * 
 *         require_tuple(values, 1, -1, "values")             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_values, 1, -1, __pyx_k__values); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":545
 *         require_tuple(values, 1, -1, "values")
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if values is None or len(values) == 0:
 *                 nelements = 0
 */
  /*try:*/ {

    /* "h5py/h5p.pyx":546
 * 
 *         try:
 *             if values is None or len(values) == 0:             # <<<<<<<<<<<<<<
 *                 nelements = 0
 *                 cd_values = NULL
 */
    __pyx_t_2 = (__pyx_v_values == Py_None);
    if (!__pyx_t_2) {
      __pyx_t_3 = PyObject_Length(__pyx_v_values); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 546; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_t_4 = (__pyx_t_3 == 0);
      __pyx_t_5 = __pyx_t_4;
    } else {
      __pyx_t_5 = __pyx_t_2;
    }
    if (__pyx_t_5) {

      /* "h5py/h5p.pyx":547
 *         try:
 *             if values is None or len(values) == 0:
 *                 nelements = 0             # <<<<<<<<<<<<<<
 *                 cd_values = NULL
 *             else:
 */
      __pyx_v_nelements = 0;

      /* "h5py/h5p.pyx":548
 *             if values is None or len(values) == 0:
 *                 nelements = 0
 *                 cd_values = NULL             # <<<<<<<<<<<<<<
 *             else:
 *                 nelements = len(values)
 */
      __pyx_v_cd_values = NULL;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/h5p.pyx":550
 *                 cd_values = NULL
 *             else:
 *                 nelements = len(values)             # <<<<<<<<<<<<<<
 *                 cd_values = <unsigned int*>emalloc(sizeof(unsigned int)*nelements)
 * 
 */
      __pyx_t_3 = PyObject_Length(__pyx_v_values); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 550; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_nelements = __pyx_t_3;

      /* "h5py/h5p.pyx":551
 *             else:
 *                 nelements = len(values)
 *                 cd_values = <unsigned int*>emalloc(sizeof(unsigned int)*nelements)             # <<<<<<<<<<<<<<
 * 
 *                 for i from 0<=i<nelements:
 */
      __pyx_t_6 = __pyx_f_4h5py_5utils_emalloc(((sizeof(unsigned int)) * __pyx_v_nelements)); if (unlikely(__pyx_t_6 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 551; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_cd_values = ((unsigned int *)__pyx_t_6);

      /* "h5py/h5p.pyx":553
 *                 cd_values = <unsigned int*>emalloc(sizeof(unsigned int)*nelements)
 * 
 *                 for i from 0<=i<nelements:             # <<<<<<<<<<<<<<
 *                     cd_values[i] = int(values[i])
 * 
 */
      __pyx_t_7 = __pyx_v_nelements;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

        /* "h5py/h5p.pyx":554
 * 
 *                 for i from 0<=i<nelements:
 *                     cd_values[i] = int(values[i])             # <<<<<<<<<<<<<<
 * 
 *             H5Pset_filter(self.id, <H5Z_filter_t>filter_code, flags, nelements, cd_values)
 */
        __pyx_t_8 = __Pyx_GetItemInt(__pyx_v_values, __pyx_v_i, sizeof(int), PyInt_FromLong); if (!__pyx_t_8) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __Pyx_GOTREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_8 = PyObject_Call(((PyObject *)((PyObject*)(&PyInt_Type))), ((PyObject *)__pyx_t_9), NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(((PyObject *)__pyx_t_9)); __pyx_t_9 = 0;
        __pyx_t_10 = __Pyx_PyInt_AsUnsignedInt(__pyx_t_8); if (unlikely((__pyx_t_10 == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 554; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        (__pyx_v_cd_values[__pyx_v_i]) = __pyx_t_10;
      }
    }
    __pyx_L6:;

    /* "h5py/h5p.pyx":556
 *                     cd_values[i] = int(values[i])
 * 
 *             H5Pset_filter(self.id, <H5Z_filter_t>filter_code, flags, nelements, cd_values)             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(cd_values)
 */
    __pyx_t_11 = __pyx_f_4h5py_4defs_H5Pset_filter(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, ((H5Z_filter_t)__pyx_v_filter_code), __pyx_v_flags, __pyx_v_nelements, __pyx_v_cd_values); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 556; __pyx_clineno = __LINE__; goto __pyx_L4;}
  }

  /* "h5py/h5p.pyx":558
 *             H5Pset_filter(self.id, <H5Z_filter_t>filter_code, flags, nelements, cd_values)
 *         finally:
 *             efree(cd_values)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_cd_values);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_25all_filters_avail(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_24all_filters_avail[] = "() => BOOL\n\n        Determine if all the filters in the pipelist are available to\n        the library.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_25all_filters_avail(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("all_filters_avail (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_24all_filters_avail(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":561
 * 
 * 
 *     def all_filters_avail(self):             # <<<<<<<<<<<<<<
 *         """() => BOOL
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_24all_filters_avail(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("all_filters_avail", 0);

  /* "h5py/h5p.pyx":567
 *         the library.
 *         """
 *         return <bint>(H5Pall_filters_avail(self.id))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pall_filters_avail(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyBool_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.all_filters_avail", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_27get_nfilters(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_26get_nfilters[] = "() => INT\n\n        Determine the number of filters in the pipeline.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_27get_nfilters(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_nfilters (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_26get_nfilters(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":570
 * 
 * 
 *     def get_nfilters(self):             # <<<<<<<<<<<<<<
 *         """() => INT
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_26get_nfilters(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_nfilters", 0);

  /* "h5py/h5p.pyx":575
 *         Determine the number of filters in the pipeline.
 *         """
 *         return H5Pget_nfilters(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_nfilters(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 575; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 575; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_nfilters", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_29get_filter(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_idx); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_28get_filter[] = "(UINT filter_idx) => TUPLE filter_info\n\n        Get information about a filter, identified by its index.  Tuple\n        elements are:\n\n        0. INT filter code (h5z.FILTER*)\n        1. UINT flags (h5z.FLAG*)\n        2. TUPLE of UINT values; filter aux data (16 values max)\n        3. STRING name of filter (256 chars max)\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_29get_filter(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_idx) {
  int __pyx_v_filter_idx;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_filter (wrapper)", 0);
  assert(__pyx_arg_filter_idx); {
    __pyx_v_filter_idx = __Pyx_PyInt_AsInt(__pyx_arg_filter_idx); if (unlikely((__pyx_v_filter_idx == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 578; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_28get_filter(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_filter_idx));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":578
 * 
 * 
 *     def get_filter(self, int filter_idx):             # <<<<<<<<<<<<<<
 *         """(UINT filter_idx) => TUPLE filter_info
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_28get_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_idx) {
  PyObject *__pyx_v_vlist = 0;
  int __pyx_v_filter_code;
  unsigned int __pyx_v_flags;
  size_t __pyx_v_nelements;
  unsigned int __pyx_v_cd_values[16];
  char __pyx_v_name[257];
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  H5Z_filter_t __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_filter", 0);

  /* "h5py/h5p.pyx":596
 *         cdef char name[257]
 *         cdef int i
 *         nelements = 16 # HDF5 library actually complains if this is too big.             # <<<<<<<<<<<<<<
 * 
 *         if filter_idx < 0:
 */
  __pyx_v_nelements = 16;

  /* "h5py/h5p.pyx":598
 *         nelements = 16 # HDF5 library actually complains if this is too big.
 * 
 *         if filter_idx < 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Filter index must be a non-negative integer")
 * 
 */
  __pyx_t_1 = (__pyx_v_filter_idx < 0);
  if (__pyx_t_1) {

    /* "h5py/h5p.pyx":599
 * 
 *         if filter_idx < 0:
 *             raise ValueError("Filter index must be a non-negative integer")             # <<<<<<<<<<<<<<
 * 
 *         filter_code = <int>H5Pget_filter(self.id, filter_idx, &flags,
 */
    __pyx_t_2 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_k_tuple_6), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":602
 * 
 *         filter_code = <int>H5Pget_filter(self.id, filter_idx, &flags,
 *                                          &nelements, cd_values, 256, name)             # <<<<<<<<<<<<<<
 *         name[256] = c'\0'  # in case it's > 256 chars
 * 
 */
  __pyx_t_3 = __pyx_f_4h5py_4defs_H5Pget_filter(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_filter_idx, (&__pyx_v_flags), (&__pyx_v_nelements), __pyx_v_cd_values, 256, __pyx_v_name); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_filter_code = ((int)__pyx_t_3);

  /* "h5py/h5p.pyx":603
 *         filter_code = <int>H5Pget_filter(self.id, filter_idx, &flags,
 *                                          &nelements, cd_values, 256, name)
 *         name[256] = c'\0'  # in case it's > 256 chars             # <<<<<<<<<<<<<<
 * 
 *         vlist = []
 */
  (__pyx_v_name[256]) = '\x00';

  /* "h5py/h5p.pyx":605
 *         name[256] = c'\0'  # in case it's > 256 chars
 * 
 *         vlist = []             # <<<<<<<<<<<<<<
 *         for i from 0<=i<nelements:
 *             vlist.append(cd_values[i])
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_vlist = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "h5py/h5p.pyx":606
 * 
 *         vlist = []
 *         for i from 0<=i<nelements:             # <<<<<<<<<<<<<<
 *             vlist.append(cd_values[i])
 * 
 */
  __pyx_t_4 = __pyx_v_nelements;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_4; __pyx_v_i++) {

    /* "h5py/h5p.pyx":607
 *         vlist = []
 *         for i from 0<=i<nelements:
 *             vlist.append(cd_values[i])             # <<<<<<<<<<<<<<
 * 
 *         return (filter_code, flags, tuple(vlist), name)
 */
    __pyx_t_2 = PyLong_FromUnsignedLong((__pyx_v_cd_values[__pyx_v_i])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyList_Append(__pyx_v_vlist, __pyx_t_2); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "h5py/h5p.pyx":609
 *             vlist.append(cd_values[i])
 * 
 *         return (filter_code, flags, tuple(vlist), name)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(__pyx_v_filter_code); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = PyLong_FromUnsignedLong(__pyx_v_flags); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = ((PyObject *)PyList_AsTuple(__pyx_v_vlist)); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_7));
  __pyx_t_8 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_8));
  __pyx_t_9 = PyTuple_New(4); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 609; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_9, 2, ((PyObject *)__pyx_t_7));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_7));
  PyTuple_SET_ITEM(__pyx_t_9, 3, ((PyObject *)__pyx_t_8));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_8));
  __pyx_t_2 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_r = ((PyObject *)__pyx_t_9);
  __pyx_t_9 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_vlist);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_31_has_filter(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_code); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_30_has_filter[] = "(INT filter_code)\n\n        Slow & stupid method to determine if a filter is used in this\n        property list.  Used because the HDF5 function H5Pget_filter_by_id\n        is broken.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_31_has_filter(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_code) {
  int __pyx_v_filter_code;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_has_filter (wrapper)", 0);
  assert(__pyx_arg_filter_code); {
    __pyx_v_filter_code = __Pyx_PyInt_AsInt(__pyx_arg_filter_code); if (unlikely((__pyx_v_filter_code == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 612; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID._has_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_30_has_filter(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_filter_code));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":612
 * 
 * 
 *     def _has_filter(self, int filter_code):             # <<<<<<<<<<<<<<
 *         """(INT filter_code)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_30_has_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_code) {
  int __pyx_v_nfilters;
  long __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_has_filter", 0);

  /* "h5py/h5p.pyx":620
 *         """
 *         cdef int nfilters
 *         nfilters = self.get_nfilters()             # <<<<<<<<<<<<<<
 *         for i from 0<=i<nfilters:
 *             if self.get_filter(i)[0] == filter_code:
 */
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__get_nfilters); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_AsInt(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_nfilters = __pyx_t_3;

  /* "h5py/h5p.pyx":621
 *         cdef int nfilters
 *         nfilters = self.get_nfilters()
 *         for i from 0<=i<nfilters:             # <<<<<<<<<<<<<<
 *             if self.get_filter(i)[0] == filter_code:
 *                 return True
 */
  __pyx_t_3 = __pyx_v_nfilters;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_3; __pyx_v_i++) {

    /* "h5py/h5p.pyx":622
 *         nfilters = self.get_nfilters()
 *         for i from 0<=i<nfilters:
 *             if self.get_filter(i)[0] == filter_code:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
 */
    __pyx_t_2 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__get_filter); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyInt_FromLong(__pyx_v_i); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_t_4), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_1, 0, sizeof(long), PyInt_FromLong); if (!__pyx_t_4) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyInt_FromLong(__pyx_v_filter_code); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_4, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_5) {

      /* "h5py/h5p.pyx":623
 *         for i from 0<=i<nfilters:
 *             if self.get_filter(i)[0] == filter_code:
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_2 = __Pyx_PyBool_FromLong(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;
      goto __pyx_L5;
    }
    __pyx_L5:;
  }

  /* "h5py/h5p.pyx":624
 *             if self.get_filter(i)[0] == filter_code:
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropDCID._has_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_33get_filter_by_id(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_code); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_32get_filter_by_id[] = "(INT filter_code) => TUPLE filter_info or None\n\n        Get information about a filter, identified by its code (one\n        of h5z.FILTER*).  If the filter doesn't exist, returns None.\n        Tuple elements are:\n\n        0. UINT flags (h5z.FLAG*)\n        1. TUPLE of UINT values; filter aux data (16 values max)\n        2. STRING name of filter (256 chars max)\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_33get_filter_by_id(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_code) {
  int __pyx_v_filter_code;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_filter_by_id (wrapper)", 0);
  assert(__pyx_arg_filter_code); {
    __pyx_v_filter_code = __Pyx_PyInt_AsInt(__pyx_arg_filter_code); if (unlikely((__pyx_v_filter_code == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_filter_by_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_32get_filter_by_id(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_filter_code));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":627
 * 
 * 
 *     def get_filter_by_id(self, int filter_code):             # <<<<<<<<<<<<<<
 *         """(INT filter_code) => TUPLE filter_info or None
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_32get_filter_by_id(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_code) {
  PyObject *__pyx_v_vlist = 0;
  unsigned int __pyx_v_flags;
  size_t __pyx_v_nelements;
  unsigned int __pyx_v_cd_values[16];
  char __pyx_v_name[257];
  CYTHON_UNUSED herr_t __pyx_v_retval;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  herr_t __pyx_t_6;
  size_t __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_filter_by_id", 0);

  /* "h5py/h5p.pyx":645
 *         cdef herr_t retval
 *         cdef int i
 *         nelements = 16 # HDF5 library actually complains if this is too big.             # <<<<<<<<<<<<<<
 * 
 *         if not self._has_filter(filter_code):
 */
  __pyx_v_nelements = 16;

  /* "h5py/h5p.pyx":647
 *         nelements = 16 # HDF5 library actually complains if this is too big.
 * 
 *         if not self._has_filter(filter_code):             # <<<<<<<<<<<<<<
 *             # Avoid library segfault
 *             return None
 */
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s___has_filter); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromLong(__pyx_v_filter_code); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = (!__pyx_t_4);
  if (__pyx_t_5) {

    /* "h5py/h5p.pyx":649
 *         if not self._has_filter(filter_code):
 *             # Avoid library segfault
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         retval = H5Pget_filter_by_id(self.id, <H5Z_filter_t>filter_code,
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":652
 * 
 *         retval = H5Pget_filter_by_id(self.id, <H5Z_filter_t>filter_code,
 *                                      &flags, &nelements, cd_values, 256, name)             # <<<<<<<<<<<<<<
 *         assert nelements <= 16
 * 
 */
  __pyx_t_6 = __pyx_f_4h5py_4defs_H5Pget_filter_by_id(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, ((H5Z_filter_t)__pyx_v_filter_code), (&__pyx_v_flags), (&__pyx_v_nelements), __pyx_v_cd_values, 256, __pyx_v_name); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_retval = __pyx_t_6;

  /* "h5py/h5p.pyx":653
 *         retval = H5Pget_filter_by_id(self.id, <H5Z_filter_t>filter_code,
 *                                      &flags, &nelements, cd_values, 256, name)
 *         assert nelements <= 16             # <<<<<<<<<<<<<<
 * 
 *         name[256] = c'\0'  # In case HDF5 doesn't terminate it properly
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!(__pyx_v_nelements <= 16))) {
    PyErr_SetNone(PyExc_AssertionError);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 653; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #endif

  /* "h5py/h5p.pyx":655
 *         assert nelements <= 16
 * 
 *         name[256] = c'\0'  # In case HDF5 doesn't terminate it properly             # <<<<<<<<<<<<<<
 * 
 *         vlist = []
 */
  (__pyx_v_name[256]) = '\x00';

  /* "h5py/h5p.pyx":657
 *         name[256] = c'\0'  # In case HDF5 doesn't terminate it properly
 * 
 *         vlist = []             # <<<<<<<<<<<<<<
 *         for i from 0<=i<nelements:
 *             vlist.append(cd_values[i])
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 657; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_vlist = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "h5py/h5p.pyx":658
 * 
 *         vlist = []
 *         for i from 0<=i<nelements:             # <<<<<<<<<<<<<<
 *             vlist.append(cd_values[i])
 * 
 */
  __pyx_t_7 = __pyx_v_nelements;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

    /* "h5py/h5p.pyx":659
 *         vlist = []
 *         for i from 0<=i<nelements:
 *             vlist.append(cd_values[i])             # <<<<<<<<<<<<<<
 * 
 *         return (flags, tuple(vlist), name)
 */
    __pyx_t_2 = PyLong_FromUnsignedLong((__pyx_v_cd_values[__pyx_v_i])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = PyList_Append(__pyx_v_vlist, __pyx_t_2); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "h5py/h5p.pyx":661
 *             vlist.append(cd_values[i])
 * 
 *         return (flags, tuple(vlist), name)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyLong_FromUnsignedLong(__pyx_v_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = ((PyObject *)PyList_AsTuple(__pyx_v_vlist)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_3));
  __pyx_t_1 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_9, 1, ((PyObject *)__pyx_t_3));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_3));
  PyTuple_SET_ITEM(__pyx_t_9, 2, ((PyObject *)__pyx_t_1));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_1));
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_r = ((PyObject *)__pyx_t_9);
  __pyx_t_9 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("h5py.h5p.PropDCID.get_filter_by_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_vlist);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_35remove_filter(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_class); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_34remove_filter[] = "(INT filter_class)\n\n        Remove a filter from the pipeline.  The class code is one of\n        h5z.FILTER*.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_35remove_filter(PyObject *__pyx_v_self, PyObject *__pyx_arg_filter_class) {
  int __pyx_v_filter_class;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("remove_filter (wrapper)", 0);
  assert(__pyx_arg_filter_class); {
    __pyx_v_filter_class = __Pyx_PyInt_AsInt(__pyx_arg_filter_class); if (unlikely((__pyx_v_filter_class == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 664; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.remove_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_34remove_filter(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), ((int)__pyx_v_filter_class));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":664
 * 
 * 
 *     def remove_filter(self, int filter_class):             # <<<<<<<<<<<<<<
 *         """(INT filter_class)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_34remove_filter(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, int __pyx_v_filter_class) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("remove_filter", 0);

  /* "h5py/h5p.pyx":670
 *         h5z.FILTER*.
 *         """
 *         H5Premove_filter(self.id, <H5Z_filter_t>filter_class)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Premove_filter(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, ((H5Z_filter_t)__pyx_v_filter_class)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 670; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.remove_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_37set_deflate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_36set_deflate[] = "(UINT level=5)\n\n        Enable deflate (gzip) compression, at the given level.\n        Valid levels are 0-9, default is 5.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_37set_deflate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  unsigned int __pyx_v_level;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_deflate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__level,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__level);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_deflate") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 673; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_level = __Pyx_PyInt_AsUnsignedInt(values[0]); if (unlikely((__pyx_v_level == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 673; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_level = ((unsigned int)5);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_deflate", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 673; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_deflate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_36set_deflate(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), __pyx_v_level);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":673
 * 
 * 
 *     def set_deflate(self, unsigned int level=5):             # <<<<<<<<<<<<<<
 *         """(UINT level=5)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_36set_deflate(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, unsigned int __pyx_v_level) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_deflate", 0);

  /* "h5py/h5p.pyx":679
 *         Valid levels are 0-9, default is 5.
 *         """
 *         H5Pset_deflate(self.id, level)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_deflate(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_level); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 679; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_deflate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_39set_fletcher32(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_38set_fletcher32[] = "()\n\n        Enable Fletcher32 error correction on this list.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_39set_fletcher32(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fletcher32 (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_38set_fletcher32(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":682
 * 
 * 
 *     def set_fletcher32(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_38set_fletcher32(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fletcher32", 0);

  /* "h5py/h5p.pyx":687
 *         Enable Fletcher32 error correction on this list.
 *         """
 *         H5Pset_fletcher32(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fletcher32(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 687; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_fletcher32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_41set_shuffle(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_40set_shuffle[] = "()\n\n        Enable to use of the shuffle filter.  Use this immediately before\n        the deflate filter to increase the compression ratio.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_41set_shuffle(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_shuffle (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_40set_shuffle(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":690
 * 
 * 
 *     def set_shuffle(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_40set_shuffle(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_shuffle", 0);

  /* "h5py/h5p.pyx":696
 *         the deflate filter to increase the compression ratio.
 *         """
 *         H5Pset_shuffle(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_shuffle(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_shuffle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_43set_szip(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_42set_szip[] = "(UINT options, UINT pixels_per_block)\n\n        Enable SZIP compression.  See the HDF5 docs for argument meanings,\n        and general restrictions on use of the SZIP format.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_43set_szip(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  unsigned int __pyx_v_options;
  unsigned int __pyx_v_pixels_per_block;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_szip (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__options,&__pyx_n_s__pixels_per_block,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__options)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__pixels_per_block)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_szip", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_szip") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_options = __Pyx_PyInt_AsUnsignedInt(values[0]); if (unlikely((__pyx_v_options == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_pixels_per_block = __Pyx_PyInt_AsUnsignedInt(values[1]); if (unlikely((__pyx_v_pixels_per_block == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_szip", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 699; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_szip", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_42set_szip(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), __pyx_v_options, __pyx_v_pixels_per_block);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":699
 * 
 * 
 *     def set_szip(self, unsigned int options, unsigned int pixels_per_block):             # <<<<<<<<<<<<<<
 *         """(UINT options, UINT pixels_per_block)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_42set_szip(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, unsigned int __pyx_v_options, unsigned int __pyx_v_pixels_per_block) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_szip", 0);

  /* "h5py/h5p.pyx":705
 *         and general restrictions on use of the SZIP format.
 *         """
 *         H5Pset_szip(self.id, options, pixels_per_block)             # <<<<<<<<<<<<<<
 * 
 *     def set_scaleoffset(self, H5Z_SO_scale_type_t scale_type, int scale_factor):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_szip(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_options, __pyx_v_pixels_per_block); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 705; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_szip", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_45set_scaleoffset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDCID_44set_scaleoffset[] = "(H5Z_SO_scale_type_t scale_type, INT scale_factor)\n\n        Enable scale/offset (usually lossy) compression; lossless (e.g. gzip)\n        compression and other filters may be applied on top of this.\n\n        Note that error detection (i.e. fletcher32) cannot precede this in\n        the filter chain, or else all reads on lossily-compressed data will\n        fail.";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDCID_45set_scaleoffset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  enum H5Z_SO_scale_type_t __pyx_v_scale_type;
  int __pyx_v_scale_factor;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_scaleoffset (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__scale_type,&__pyx_n_s__scale_factor,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__scale_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__scale_factor)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_scaleoffset", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_scaleoffset") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_scale_type = ((enum H5Z_SO_scale_type_t)PyInt_AsLong(values[0])); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_scale_factor = __Pyx_PyInt_AsInt(values[1]); if (unlikely((__pyx_v_scale_factor == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_scaleoffset", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 707; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_scaleoffset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDCID_44set_scaleoffset(((struct __pyx_obj_4h5py_3h5p_PropDCID *)__pyx_v_self), __pyx_v_scale_type, __pyx_v_scale_factor);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":707
 *         H5Pset_szip(self.id, options, pixels_per_block)
 * 
 *     def set_scaleoffset(self, H5Z_SO_scale_type_t scale_type, int scale_factor):             # <<<<<<<<<<<<<<
 *         '''(H5Z_SO_scale_type_t scale_type, INT scale_factor)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDCID_44set_scaleoffset(struct __pyx_obj_4h5py_3h5p_PropDCID *__pyx_v_self, enum H5Z_SO_scale_type_t __pyx_v_scale_type, int __pyx_v_scale_factor) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_scaleoffset", 0);

  /* "h5py/h5p.pyx":716
 *         the filter chain, or else all reads on lossily-compressed data will
 *         fail.'''
 *         H5Pset_scaleoffset(self.id, scale_type, scale_factor)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_scaleoffset(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_scale_type, __pyx_v_scale_factor); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 716; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDCID.set_scaleoffset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_1set_fclose_degree(PyObject *__pyx_v_self, PyObject *__pyx_arg_close_degree); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_set_fclose_degree[] = "(INT close_degree)\n\n        Set the file-close degree, which determines library behavior when\n        a file is closed when objects are still open.  Legal values:\n\n        * h5f.CLOSE_WEAK\n        * h5f.CLOSE_SEMI\n        * h5f.CLOSE_STRONG\n        * h5f.CLOSE_DEFAULT\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_1set_fclose_degree(PyObject *__pyx_v_self, PyObject *__pyx_arg_close_degree) {
  int __pyx_v_close_degree;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fclose_degree (wrapper)", 0);
  assert(__pyx_arg_close_degree); {
    __pyx_v_close_degree = __Pyx_PyInt_AsInt(__pyx_arg_close_degree); if (unlikely((__pyx_v_close_degree == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 727; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fclose_degree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_set_fclose_degree(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), ((int)__pyx_v_close_degree));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":727
 * 
 * 
 *     def set_fclose_degree(self, int close_degree):             # <<<<<<<<<<<<<<
 *         """(INT close_degree)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_set_fclose_degree(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, int __pyx_v_close_degree) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fclose_degree", 0);

  /* "h5py/h5p.pyx":738
 *         * h5f.CLOSE_DEFAULT
 *         """
 *         H5Pset_fclose_degree(self.id, <H5F_close_degree_t>close_degree)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fclose_degree(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, ((enum H5F_close_degree_t)__pyx_v_close_degree)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 738; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fclose_degree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_3get_fclose_degree(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_2get_fclose_degree[] = "() => INT close_degree\n        - h5fd.\n        Get the file-close degree, which determines library behavior when\n        a file is closed when objects are still open.  Legal values:\n\n        * h5f.CLOSE_WEAK\n        * h5f.CLOSE_SEMI\n        * h5f.CLOSE_STRONG\n        * h5f.CLOSE_DEFAULT\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_3get_fclose_degree(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fclose_degree (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_2get_fclose_degree(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":741
 * 
 * 
 *     def get_fclose_degree(self):             # <<<<<<<<<<<<<<
 *         """() => INT close_degree
 *         - h5fd.
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_2get_fclose_degree(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  enum H5F_close_degree_t __pyx_v_deg;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fclose_degree", 0);

  /* "h5py/h5p.pyx":753
 *         """
 *         cdef H5F_close_degree_t deg
 *         H5Pget_fclose_degree(self.id, &deg)             # <<<<<<<<<<<<<<
 *         return deg
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_fclose_degree(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_deg)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":754
 *         cdef H5F_close_degree_t deg
 *         H5Pget_fclose_degree(self.id, &deg)
 *         return deg             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(__pyx_v_deg); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 754; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_fclose_degree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_5set_fapl_core(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_4set_fapl_core[] = "(UINT increment=64k, BOOL backing_store=True)\n\n        Use the h5fd.CORE (memory-resident) file driver.\n\n        increment\n            Chunk size for new memory requests (default 1 meg)\n\n        backing_store\n            If True (default), memory contents are associated with an\n            on-disk file, which is updated when the file is closed.\n            Set to False for a purely in-memory file.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_5set_fapl_core(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_block_size;
  hbool_t __pyx_v_backing_store;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fapl_core (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__block_size,&__pyx_n_s__backing_store,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__block_size);
          if (value) { values[0] = value; kw_args--; }
        }
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__backing_store);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_fapl_core") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 757; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_block_size = __Pyx_PyInt_AsSize_t(values[0]); if (unlikely((__pyx_v_block_size == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 757; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_block_size = ((size_t)65536);
    }
    if (values[1]) {
      __pyx_v_backing_store = __Pyx_PyInt_from_py_hbool_t(values[1]); if (unlikely((__pyx_v_backing_store == (hbool_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 757; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_backing_store = ((hbool_t)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_fapl_core", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 757; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_core", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_4set_fapl_core(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), __pyx_v_block_size, __pyx_v_backing_store);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":757
 * 
 * 
 *     def set_fapl_core(self, size_t block_size=64*1024, hbool_t backing_store=1):             # <<<<<<<<<<<<<<
 *         """(UINT increment=64k, BOOL backing_store=True)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_4set_fapl_core(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, size_t __pyx_v_block_size, hbool_t __pyx_v_backing_store) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fapl_core", 0);

  /* "h5py/h5p.pyx":770
 *             Set to False for a purely in-memory file.
 *         """
 *         H5Pset_fapl_core(self.id, block_size, backing_store)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fapl_core(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_block_size, __pyx_v_backing_store); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 770; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_core", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_7get_fapl_core(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_6get_fapl_core[] = "() => TUPLE core_settings\n\n        Determine settings for the h5fd.CORE (memory-resident) file driver.\n        Tuple elements are:\n\n        0. UINT \"increment\": Chunk size for new memory requests\n        1. BOOL \"backing_store\": If True, write the memory contents to\n           disk when the file is closed.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_7get_fapl_core(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fapl_core (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_6get_fapl_core(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":773
 * 
 * 
 *     def get_fapl_core(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE core_settings
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_6get_fapl_core(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  size_t __pyx_v_increment;
  hbool_t __pyx_v_backing_store;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fapl_core", 0);

  /* "h5py/h5p.pyx":785
 *         cdef size_t increment
 *         cdef hbool_t backing_store
 *         H5Pget_fapl_core(self.id, &increment, &backing_store)             # <<<<<<<<<<<<<<
 *         return (increment, <bint>(backing_store))
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_fapl_core(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_increment), (&__pyx_v_backing_store)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 785; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":786
 *         cdef hbool_t backing_store
 *         H5Pget_fapl_core(self.id, &increment, &backing_store)
 *         return (increment, <bint>(backing_store))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_increment); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 786; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(((int)__pyx_v_backing_store)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 786; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 786; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = ((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_fapl_core", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_9set_fapl_family(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_8set_fapl_family[] = "(UINT memb_size=2**31-1, PropFAID memb_fapl=None)\n\n        Set up the family driver.\n\n        memb_size\n            Member file size\n\n        memb_fapl\n            File access property list for each member access\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_9set_fapl_family(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hsize_t __pyx_v_memb_size;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_memb_fapl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fapl_family (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__memb_size,&__pyx_n_s__memb_fapl,0};
    PyObject* values[2] = {0,0};

    /* "h5py/h5p.pyx":789
 * 
 * 
 *     def set_fapl_family(self, hsize_t memb_size=2147483647, PropID memb_fapl=None):             # <<<<<<<<<<<<<<
 *         """(UINT memb_size=2**31-1, PropFAID memb_fapl=None)
 * 
 */
    values[1] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__memb_size);
          if (value) { values[0] = value; kw_args--; }
        }
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__memb_fapl);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_fapl_family") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_memb_size = __Pyx_PyInt_from_py_hsize_t(values[0]); if (unlikely((__pyx_v_memb_size == (hsize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_memb_size = ((hsize_t)2147483647);
    }
    __pyx_v_memb_fapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_fapl_family", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_family", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_memb_fapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "memb_fapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 789; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_8set_fapl_family(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), __pyx_v_memb_size, __pyx_v_memb_fapl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_8set_fapl_family(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, hsize_t __pyx_v_memb_size, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_memb_fapl) {
  hid_t __pyx_v_plist_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fapl_family", 0);

  /* "h5py/h5p.pyx":801
 *         """
 *         cdef hid_t plist_id
 *         plist_id = pdefault(memb_fapl)             # <<<<<<<<<<<<<<
 *         H5Pset_fapl_family(self.id, memb_size, plist_id)
 * 
 */
  __pyx_v_plist_id = __pyx_f_4h5py_3h5p_pdefault(__pyx_v_memb_fapl);

  /* "h5py/h5p.pyx":802
 *         cdef hid_t plist_id
 *         plist_id = pdefault(memb_fapl)
 *         H5Pset_fapl_family(self.id, memb_size, plist_id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fapl_family(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_memb_size, __pyx_v_plist_id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_family", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_11get_fapl_family(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_10get_fapl_family[] = "() => TUPLE info\n\n        Determine family driver settings. Tuple values are:\n\n        0. UINT memb_size\n        1. PropFAID memb_fapl or None\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_11get_fapl_family(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fapl_family (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_10get_fapl_family(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":805
 * 
 * 
 *     def get_fapl_family(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE info
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_10get_fapl_family(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  hid_t __pyx_v_mfapl_id;
  hsize_t __pyx_v_msize;
  struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_plist = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fapl_family", 0);

  /* "h5py/h5p.pyx":816
 *         cdef hsize_t msize
 *         cdef PropFAID plist
 *         plist = None             # <<<<<<<<<<<<<<
 * 
 *         H5Pget_fapl_family(self.id, &msize, &mfapl_id)
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_plist = ((struct __pyx_obj_4h5py_3h5p_PropFAID *)Py_None);

  /* "h5py/h5p.pyx":818
 *         plist = None
 * 
 *         H5Pget_fapl_family(self.id, &msize, &mfapl_id)             # <<<<<<<<<<<<<<
 * 
 *         if mfapl_id > 0:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_fapl_family(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_msize), (&__pyx_v_mfapl_id)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 818; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":820
 *         H5Pget_fapl_family(self.id, &msize, &mfapl_id)
 * 
 *         if mfapl_id > 0:             # <<<<<<<<<<<<<<
 *             plist = PropFAID.open(mfapl_id)
 * 
 */
  __pyx_t_2 = (__pyx_v_mfapl_id > 0);
  if (__pyx_t_2) {

    /* "h5py/h5p.pyx":821
 * 
 *         if mfapl_id > 0:
 *             plist = PropFAID.open(mfapl_id)             # <<<<<<<<<<<<<<
 * 
 *         return (msize, plist)
 */
    __pyx_t_3 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5p_PropFAID)), __pyx_n_s__open); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_to_py_hid_t(__pyx_v_mfapl_id); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_Call(__pyx_t_3, ((PyObject *)__pyx_t_5), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_4h5py_3h5p_PropFAID))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(((PyObject *)__pyx_v_plist));
    __pyx_v_plist = ((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":823
 *             plist = PropFAID.open(mfapl_id)
 * 
 *         return (msize, plist)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyInt_to_py_hsize_t(__pyx_v_msize); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_plist));
  PyTuple_SET_ITEM(__pyx_t_5, 1, ((PyObject *)__pyx_v_plist));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_plist));
  __pyx_t_4 = 0;
  __pyx_r = ((PyObject *)__pyx_t_5);
  __pyx_t_5 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_fapl_family", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_plist);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_13set_fapl_log(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_12set_fapl_log[] = "(STRING logfile, UINT flags, UINT buf_size)\n\n        Enable the use of the logging driver.  See the HDF5 documentation\n        for details.  Flag constants are stored in module h5fd.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_13set_fapl_log(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  char *__pyx_v_logfile;
  unsigned int __pyx_v_flags;
  size_t __pyx_v_buf_size;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fapl_log (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__logfile,&__pyx_n_s__flags,&__pyx_n_s__buf_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__logfile)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_fapl_log", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__buf_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_fapl_log", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_fapl_log") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_logfile = PyBytes_AsString(values[0]); if (unlikely((!__pyx_v_logfile) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_flags = __Pyx_PyInt_AsUnsignedInt(values[1]); if (unlikely((__pyx_v_flags == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_buf_size = __Pyx_PyInt_AsSize_t(values[2]); if (unlikely((__pyx_v_buf_size == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_fapl_log", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_log", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_12set_fapl_log(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), __pyx_v_logfile, __pyx_v_flags, __pyx_v_buf_size);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":826
 * 
 * 
 *     def set_fapl_log(self, char* logfile, unsigned int flags, size_t buf_size):             # <<<<<<<<<<<<<<
 *         """(STRING logfile, UINT flags, UINT buf_size)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_12set_fapl_log(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, char *__pyx_v_logfile, unsigned int __pyx_v_flags, size_t __pyx_v_buf_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fapl_log", 0);

  /* "h5py/h5p.pyx":832
 *         for details.  Flag constants are stored in module h5fd.
 *         """
 *         H5Pset_fapl_log(self.id, logfile, flags, buf_size)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fapl_log(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_logfile, __pyx_v_flags, __pyx_v_buf_size); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_log", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_15set_fapl_sec2(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_14set_fapl_sec2[] = "()\n\n        Select the \"section-2\" driver (h5fd.SEC2).\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_15set_fapl_sec2(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fapl_sec2 (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_14set_fapl_sec2(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":835
 * 
 * 
 *     def set_fapl_sec2(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_14set_fapl_sec2(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fapl_sec2", 0);

  /* "h5py/h5p.pyx":840
 *         Select the "section-2" driver (h5fd.SEC2).
 *         """
 *         H5Pset_fapl_sec2(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fapl_sec2(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_sec2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_17set_fapl_stdio(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_16set_fapl_stdio[] = "()\n\n        Select the \"stdio\" driver (h5fd.STDIO)\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_17set_fapl_stdio(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_fapl_stdio (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_16set_fapl_stdio(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":843
 * 
 * 
 *     def set_fapl_stdio(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_16set_fapl_stdio(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_fapl_stdio", 0);

  /* "h5py/h5p.pyx":848
 *         Select the "stdio" driver (h5fd.STDIO)
 *         """
 *         H5Pset_fapl_stdio(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_fapl_stdio(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 848; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_fapl_stdio", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_19get_driver(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_18get_driver[] = "() => INT driver code\n\n        Return an integer identifier for the driver used by this list.\n        Although HDF5 implements these as full-fledged objects, they are\n        treated as integers by Python.  Built-in drivers identifiers are\n        listed in module h5fd; they are:\n\n        - h5fd.CORE\n        - h5fd.FAMILY\n        - h5fd.LOG\n        - h5fd.MPIO\n        - h5fd.MULTI\n        - h5fd.SEC2\n        - h5fd.STDIO\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_19get_driver(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_driver (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_18get_driver(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":851
 * 
 * 
 *     def get_driver(self):             # <<<<<<<<<<<<<<
 *         """() => INT driver code
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_18get_driver(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_driver", 0);

  /* "h5py/h5p.pyx":867
 *         - h5fd.STDIO
 *         """
 *         return H5Pget_driver(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_driver(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 867; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_to_py_hid_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 867; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_driver", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_21set_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_20set_cache[] = "(INT mdc, INT rdcc, UINT rdcc_nbytes, DOUBLE rdcc_w0)\n\n        Set the metadata (mdc) and raw data chunk (rdcc) cache properties.\n        See the HDF5 docs for a full explanation.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_21set_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_mdc;
  int __pyx_v_rdcc;
  size_t __pyx_v_rdcc_nbytes;
  double __pyx_v_rdcc_w0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_cache (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__mdc,&__pyx_n_s__rdcc,&__pyx_n_s__rdcc_nbytes,&__pyx_n_s__rdcc_w0,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__mdc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__rdcc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_cache", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__rdcc_nbytes)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_cache", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__rdcc_w0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_cache", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_cache") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_mdc = __Pyx_PyInt_AsInt(values[0]); if (unlikely((__pyx_v_mdc == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_rdcc = __Pyx_PyInt_AsInt(values[1]); if (unlikely((__pyx_v_rdcc == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_rdcc_nbytes = __Pyx_PyInt_AsSize_t(values[2]); if (unlikely((__pyx_v_rdcc_nbytes == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_rdcc_w0 = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_rdcc_w0 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_cache", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 870; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_20set_cache(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), __pyx_v_mdc, __pyx_v_rdcc, __pyx_v_rdcc_nbytes, __pyx_v_rdcc_w0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":870
 * 
 * 
 *     def set_cache(self, int mdc, int rdcc, size_t rdcc_nbytes, double rdcc_w0):             # <<<<<<<<<<<<<<
 *         """(INT mdc, INT rdcc, UINT rdcc_nbytes, DOUBLE rdcc_w0)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_20set_cache(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, int __pyx_v_mdc, int __pyx_v_rdcc, size_t __pyx_v_rdcc_nbytes, double __pyx_v_rdcc_w0) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_cache", 0);

  /* "h5py/h5p.pyx":876
 *         See the HDF5 docs for a full explanation.
 *         """
 *         H5Pset_cache(self.id, mdc, rdcc, rdcc_nbytes, rdcc_w0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_cache(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_mdc, __pyx_v_rdcc, __pyx_v_rdcc_nbytes, __pyx_v_rdcc_w0); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 876; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_23get_cache(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_22get_cache[] = "() => TUPLE cache info\n\n        Get the metadata and raw data chunk cache settings.  See the HDF5\n        docs for element definitions.  Return is a 4-tuple with entries:\n\n        1. INT mdc:              Number of metadata objects\n        2. INT rdcc:             Number of raw data chunks\n        3. UINT rdcc_nbytes:     Size of raw data cache\n        4. DOUBLE rdcc_w0:       Preemption policy for data cache.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_23get_cache(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_cache (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_22get_cache(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":879
 * 
 * 
 *     def get_cache(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE cache info
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_22get_cache(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  int __pyx_v_mdc;
  int __pyx_v_rdcc;
  size_t __pyx_v_rdcc_nbytes;
  double __pyx_v_w0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_cache", 0);

  /* "h5py/h5p.pyx":894
 *         cdef double w0
 * 
 *         H5Pget_cache(self.id, &mdc, &rdcc, &rdcc_nbytes, &w0)             # <<<<<<<<<<<<<<
 *         return (mdc, rdcc, rdcc_nbytes, w0)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_cache(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_mdc), (&__pyx_v_rdcc), (&__pyx_v_rdcc_nbytes), (&__pyx_v_w0)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 894; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":895
 * 
 *         H5Pget_cache(self.id, &mdc, &rdcc, &rdcc_nbytes, &w0)
 *         return (mdc, rdcc, rdcc_nbytes, w0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(__pyx_v_mdc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 895; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyInt_FromLong(__pyx_v_rdcc); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 895; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_rdcc_nbytes); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 895; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_w0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 895; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 895; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_r = ((PyObject *)__pyx_t_6);
  __pyx_t_6 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_25set_sieve_buf_size(PyObject *__pyx_v_self, PyObject *__pyx_arg_size); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_24set_sieve_buf_size[] = " (UINT size)\n\n        Set the maximum size of the data sieve buffer (in bytes).  This\n        buffer can improve I/O performance for hyperslab I/O, by combining\n        reads and writes into blocks of the given size.  The default is 64k.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_25set_sieve_buf_size(PyObject *__pyx_v_self, PyObject *__pyx_arg_size) {
  size_t __pyx_v_size;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_sieve_buf_size (wrapper)", 0);
  assert(__pyx_arg_size); {
    __pyx_v_size = __Pyx_PyInt_AsSize_t(__pyx_arg_size); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 898; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_sieve_buf_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_24set_sieve_buf_size(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), ((size_t)__pyx_v_size));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":898
 * 
 * 
 *     def set_sieve_buf_size(self, size_t size):             # <<<<<<<<<<<<<<
 *         """ (UINT size)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_24set_sieve_buf_size(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, size_t __pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_sieve_buf_size", 0);

  /* "h5py/h5p.pyx":905
 *         reads and writes into blocks of the given size.  The default is 64k.
 *         """
 *         H5Pset_sieve_buf_size(self.id, size)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_sieve_buf_size(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_size); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_sieve_buf_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_27get_sieve_buf_size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_26get_sieve_buf_size[] = " () => UINT size\n\n        Get the current maximum size of the data sieve buffer (in bytes).\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_27get_sieve_buf_size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sieve_buf_size (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_26get_sieve_buf_size(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":908
 * 
 * 
 *     def get_sieve_buf_size(self):             # <<<<<<<<<<<<<<
 *         """ () => UINT size
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_26get_sieve_buf_size(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  size_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sieve_buf_size", 0);

  /* "h5py/h5p.pyx":914
 *         """
 *         cdef size_t size
 *         H5Pget_sieve_buf_size(self.id, &size)             # <<<<<<<<<<<<<<
 *         return size
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_sieve_buf_size(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_size)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":915
 *         cdef size_t size
 *         H5Pget_sieve_buf_size(self.id, &size)
 *         return size             # <<<<<<<<<<<<<<
 * 
 *     def set_libver_bounds(self, int low, int high):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_sieve_buf_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_29set_libver_bounds(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_28set_libver_bounds[] = " (INT low, INT high)\n\n        Set the compatibility level for file format.  Legal values are:\n\n        - h5f.LIBVER_EARLIEST\n        - h5f.LIBVER_LATEST\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_29set_libver_bounds(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_low;
  int __pyx_v_high;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_libver_bounds (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__low,&__pyx_n_s__high,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__low)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__high)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_libver_bounds", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 917; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_libver_bounds") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 917; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_low = __Pyx_PyInt_AsInt(values[0]); if (unlikely((__pyx_v_low == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 917; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_high = __Pyx_PyInt_AsInt(values[1]); if (unlikely((__pyx_v_high == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 917; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_libver_bounds", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 917; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_libver_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_28set_libver_bounds(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), __pyx_v_low, __pyx_v_high);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":917
 *         return size
 * 
 *     def set_libver_bounds(self, int low, int high):             # <<<<<<<<<<<<<<
 *         """ (INT low, INT high)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_28set_libver_bounds(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, int __pyx_v_low, int __pyx_v_high) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_libver_bounds", 0);

  /* "h5py/h5p.pyx":925
 *         - h5f.LIBVER_LATEST
 *         """
 *         H5Pset_libver_bounds(self.id, <H5F_libver_t>low, <H5F_libver_t>high)             # <<<<<<<<<<<<<<
 * 
 *     def get_libver_bounds(self):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_libver_bounds(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, ((H5F_libver_t)__pyx_v_low), ((H5F_libver_t)__pyx_v_high)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 925; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_libver_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_31get_libver_bounds(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_30get_libver_bounds[] = " () => (INT low, INT high)\n\n        Get the compatibility level for file format. Returned values are from:\n\n        - h5f.LIBVER_EARLIEST\n        - h5f.LIBVER_LATEST\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_31get_libver_bounds(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_libver_bounds (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_30get_libver_bounds(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":927
 *         H5Pset_libver_bounds(self.id, <H5F_libver_t>low, <H5F_libver_t>high)
 * 
 *     def get_libver_bounds(self):             # <<<<<<<<<<<<<<
 *         """ () => (INT low, INT high)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_30get_libver_bounds(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  H5F_libver_t __pyx_v_low;
  H5F_libver_t __pyx_v_high;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_libver_bounds", 0);

  /* "h5py/h5p.pyx":937
 *         cdef H5F_libver_t low
 *         cdef H5F_libver_t high
 *         H5Pget_libver_bounds(self.id, &low, &high)             # <<<<<<<<<<<<<<
 * 
 *         return (<int>low, <int>high)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_libver_bounds(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_low), (&__pyx_v_high)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":939
 *         H5Pget_libver_bounds(self.id, &low, &high)
 * 
 *         return (<int>low, <int>high)             # <<<<<<<<<<<<<<
 * 
 *     IF MPI:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_v_low)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 939; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyInt_FromLong(((int)__pyx_v_high)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 939; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 939; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = ((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_libver_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_33get_mdc_config(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_32get_mdc_config[] = "() => CacheConfig\n        Returns an object that stores all the information about the meta-data cache\n        configuration\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_33get_mdc_config(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_mdc_config (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_32get_mdc_config(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":963
 *             H5Pset_fapl_mpiposix(self.id, comm.ob_mpi, use_gpfs_hints)
 * 
 *     def get_mdc_config(self):             # <<<<<<<<<<<<<<
 *         """() => CacheConfig
 *         Returns an object that stores all the information about the meta-data cache
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_32get_mdc_config(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self) {
  struct __pyx_obj_4h5py_4h5ac_CacheConfig *__pyx_v_config = 0;
  herr_t __pyx_v_err;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  herr_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_mdc_config", 0);

  /* "h5py/h5p.pyx":969
 *         """
 * 
 *         cdef CacheConfig config = CacheConfig()             # <<<<<<<<<<<<<<
 * 
 *         cdef herr_t  err
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_4h5ac_CacheConfig)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 969; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_config = ((struct __pyx_obj_4h5py_4h5ac_CacheConfig *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5p.pyx":972
 * 
 *         cdef herr_t  err
 *         err = H5Fget_mdc_config(self.id, &config.cache_config)             # <<<<<<<<<<<<<<
 *         if err < 0:
 *             raise RuntimeError("Failed to get hit rate")
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Fget_mdc_config(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_config->cache_config)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_err = __pyx_t_2;

  /* "h5py/h5p.pyx":973
 *         cdef herr_t  err
 *         err = H5Fget_mdc_config(self.id, &config.cache_config)
 *         if err < 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Failed to get hit rate")
 * 
 */
  __pyx_t_3 = (__pyx_v_err < 0);
  if (__pyx_t_3) {

    /* "h5py/h5p.pyx":974
 *         err = H5Fget_mdc_config(self.id, &config.cache_config)
 *         if err < 0:
 *             raise RuntimeError("Failed to get hit rate")             # <<<<<<<<<<<<<<
 * 
 *         return config
 */
    __pyx_t_1 = PyObject_Call(__pyx_builtin_RuntimeError, ((PyObject *)__pyx_k_tuple_8), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 974; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 974; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":976
 *             raise RuntimeError("Failed to get hit rate")
 * 
 *         return config             # <<<<<<<<<<<<<<
 * 
 *     def set_mdc_config(self, CacheConfig config not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_config));
  __pyx_r = ((PyObject *)__pyx_v_config);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.get_mdc_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_config);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_35set_mdc_config(PyObject *__pyx_v_self, PyObject *__pyx_v_config); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropFAID_34set_mdc_config[] = "(CacheConfig) => None\n        Returns an object that stores all the information about the meta-data cache\n        configuration\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropFAID_35set_mdc_config(PyObject *__pyx_v_self, PyObject *__pyx_v_config) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_mdc_config (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_config), __pyx_ptype_4h5py_4h5ac_CacheConfig, 0, "config", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 978; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropFAID_34set_mdc_config(((struct __pyx_obj_4h5py_3h5p_PropFAID *)__pyx_v_self), ((struct __pyx_obj_4h5py_4h5ac_CacheConfig *)__pyx_v_config));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":978
 *         return config
 * 
 *     def set_mdc_config(self, CacheConfig config not None):             # <<<<<<<<<<<<<<
 *         """(CacheConfig) => None
 *         Returns an object that stores all the information about the meta-data cache
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropFAID_34set_mdc_config(struct __pyx_obj_4h5py_3h5p_PropFAID *__pyx_v_self, struct __pyx_obj_4h5py_4h5ac_CacheConfig *__pyx_v_config) {
  herr_t __pyx_v_err;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_mdc_config", 0);

  /* "h5py/h5p.pyx":985
 *         # I feel this should have some sanity checking to make sure that
 *         cdef herr_t  err
 *         err = H5Fset_mdc_config(self.id, &config.cache_config)             # <<<<<<<<<<<<<<
 *         if err < 0:
 *             raise RuntimeError("Failed to get hit rate")
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Fset_mdc_config(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_config->cache_config)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 985; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_err = __pyx_t_1;

  /* "h5py/h5p.pyx":986
 *         cdef herr_t  err
 *         err = H5Fset_mdc_config(self.id, &config.cache_config)
 *         if err < 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Failed to get hit rate")
 * 
 */
  __pyx_t_2 = (__pyx_v_err < 0);
  if (__pyx_t_2) {

    /* "h5py/h5p.pyx":987
 *         err = H5Fset_mdc_config(self.id, &config.cache_config)
 *         if err < 0:
 *             raise RuntimeError("Failed to get hit rate")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = PyObject_Call(__pyx_builtin_RuntimeError, ((PyObject *)__pyx_k_tuple_9), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5p.PropFAID.set_mdc_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_1set_char_encoding(PyObject *__pyx_v_self, PyObject *__pyx_arg_encoding); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLCID_set_char_encoding[] = " (INT encoding)\n\n        Set the character encoding for link names.  Legal values are:\n\n        - h5t.CSET_ASCII\n        - h5t.CSET_UTF8\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_1set_char_encoding(PyObject *__pyx_v_self, PyObject *__pyx_arg_encoding) {
  int __pyx_v_encoding;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_char_encoding (wrapper)", 0);
  assert(__pyx_arg_encoding); {
    __pyx_v_encoding = __Pyx_PyInt_AsInt(__pyx_arg_encoding); if (unlikely((__pyx_v_encoding == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 995; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLCID.set_char_encoding", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLCID_set_char_encoding(((struct __pyx_obj_4h5py_3h5p_PropLCID *)__pyx_v_self), ((int)__pyx_v_encoding));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":995
 *     """ Link creation property list """
 * 
 *     def set_char_encoding(self, int encoding):             # <<<<<<<<<<<<<<
 *         """ (INT encoding)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_set_char_encoding(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self, int __pyx_v_encoding) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_char_encoding", 0);

  /* "h5py/h5p.pyx":1003
 *         - h5t.CSET_UTF8
 *         """
 *         H5Pset_char_encoding(self.id, <H5T_cset_t>encoding)             # <<<<<<<<<<<<<<
 * 
 *     def get_char_encoding(self):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_char_encoding(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, ((H5T_cset_t)__pyx_v_encoding)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1003; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLCID.set_char_encoding", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_3get_char_encoding(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLCID_2get_char_encoding[] = " () => INT encoding\n\n        Get the character encoding for link names.  Legal values are:\n\n        - h5t.CSET_ASCII\n        - h5t.CSET_UTF8\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_3get_char_encoding(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_char_encoding (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLCID_2get_char_encoding(((struct __pyx_obj_4h5py_3h5p_PropLCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1005
 *         H5Pset_char_encoding(self.id, <H5T_cset_t>encoding)
 * 
 *     def get_char_encoding(self):             # <<<<<<<<<<<<<<
 *         """ () => INT encoding
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_2get_char_encoding(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self) {
  H5T_cset_t __pyx_v_encoding;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_char_encoding", 0);

  /* "h5py/h5p.pyx":1014
 *         """
 *         cdef H5T_cset_t encoding
 *         H5Pget_char_encoding(self.id, &encoding)             # <<<<<<<<<<<<<<
 *         return <int>encoding
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_char_encoding(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_encoding)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1014; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":1015
 *         cdef H5T_cset_t encoding
 *         H5Pget_char_encoding(self.id, &encoding)
 *         return <int>encoding             # <<<<<<<<<<<<<<
 * 
 *     def set_create_intermediate_group(self, bint create):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_v_encoding)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1015; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropLCID.get_char_encoding", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_5set_create_intermediate_group(PyObject *__pyx_v_self, PyObject *__pyx_arg_create); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLCID_4set_create_intermediate_group[] = "(BOOL create)\n\n        Set whether missing intermediate groups are automatically created.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_5set_create_intermediate_group(PyObject *__pyx_v_self, PyObject *__pyx_arg_create) {
  int __pyx_v_create;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_create_intermediate_group (wrapper)", 0);
  assert(__pyx_arg_create); {
    __pyx_v_create = __Pyx_PyObject_IsTrue(__pyx_arg_create); if (unlikely((__pyx_v_create == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1017; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLCID.set_create_intermediate_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLCID_4set_create_intermediate_group(((struct __pyx_obj_4h5py_3h5p_PropLCID *)__pyx_v_self), ((int)__pyx_v_create));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1017
 *         return <int>encoding
 * 
 *     def set_create_intermediate_group(self, bint create):             # <<<<<<<<<<<<<<
 *         """(BOOL create)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_4set_create_intermediate_group(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self, int __pyx_v_create) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_create_intermediate_group", 0);

  /* "h5py/h5p.pyx":1022
 *         Set whether missing intermediate groups are automatically created.
 *         """
 *         H5Pset_create_intermediate_group(self.id, create)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_create_intermediate_group(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_create); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1022; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLCID.set_create_intermediate_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_7get_create_intermediate_group(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLCID_6get_create_intermediate_group[] = "() => BOOL\n\n        Determine if missing intermediate groups are automatically created.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLCID_7get_create_intermediate_group(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_create_intermediate_group (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLCID_6get_create_intermediate_group(((struct __pyx_obj_4h5py_3h5p_PropLCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1025
 * 
 * 
 *     def get_create_intermediate_group(self):             # <<<<<<<<<<<<<<
 *         """() => BOOL
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLCID_6get_create_intermediate_group(struct __pyx_obj_4h5py_3h5p_PropLCID *__pyx_v_self) {
  unsigned int __pyx_v_create;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_create_intermediate_group", 0);

  /* "h5py/h5p.pyx":1031
 *         """
 *         cdef unsigned int create
 *         H5Pget_create_intermediate_group(self.id, &create)             # <<<<<<<<<<<<<<
 *         return <bint>create
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_create_intermediate_group(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_create)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1031; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":1032
 *         cdef unsigned int create
 *         H5Pget_create_intermediate_group(self.id, &create)
 *         return <bint>create             # <<<<<<<<<<<<<<
 * 
 * # Link access
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(((int)__pyx_v_create)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1032; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropLCID.get_create_intermediate_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5p_8PropLAID_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5p_8PropLAID_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID___cinit__(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self), __pyx_v_args);
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1039
 *     """ Link access property list """
 * 
 *     def __cinit__(self, *args):             # <<<<<<<<<<<<<<
 *         self._buf = NULL
 * 
 */

static int __pyx_pf_4h5py_3h5p_8PropLAID___cinit__(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "h5py/h5p.pyx":1040
 * 
 *     def __cinit__(self, *args):
 *         self._buf = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->_buf = NULL;

  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static void __pyx_pw_4h5py_3h5p_8PropLAID_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4h5py_3h5p_8PropLAID_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4h5py_3h5p_8PropLAID_2__dealloc__(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
}

/* "h5py/h5p.pyx":1042
 *         self._buf = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         efree(self._buf)
 * 
 */

static void __pyx_pf_4h5py_3h5p_8PropLAID_2__dealloc__(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "h5py/h5p.pyx":1043
 * 
 *     def __dealloc__(self):
 *         efree(self._buf)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_f_4h5py_5utils_efree(__pyx_v_self->_buf);

  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_5set_nlinks(PyObject *__pyx_v_self, PyObject *__pyx_arg_nlinks); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLAID_4set_nlinks[] = "(UINT nlinks)\n\n        Set the maximum traversal depth for soft links\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_5set_nlinks(PyObject *__pyx_v_self, PyObject *__pyx_arg_nlinks) {
  size_t __pyx_v_nlinks;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_nlinks (wrapper)", 0);
  assert(__pyx_arg_nlinks); {
    __pyx_v_nlinks = __Pyx_PyInt_AsSize_t(__pyx_arg_nlinks); if (unlikely((__pyx_v_nlinks == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1046; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLAID.set_nlinks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID_4set_nlinks(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self), ((size_t)__pyx_v_nlinks));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1046
 * 
 * 
 *     def set_nlinks(self, size_t nlinks):             # <<<<<<<<<<<<<<
 *         """(UINT nlinks)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_4set_nlinks(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, size_t __pyx_v_nlinks) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_nlinks", 0);

  /* "h5py/h5p.pyx":1051
 *         Set the maximum traversal depth for soft links
 *         """
 *         H5Pset_nlinks(self.id, nlinks)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_nlinks(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_nlinks); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLAID.set_nlinks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_7get_nlinks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLAID_6get_nlinks[] = "() => UINT\n\n        Get the maximum traversal depth for soft links\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_7get_nlinks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_nlinks (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID_6get_nlinks(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1054
 * 
 * 
 *     def get_nlinks(self):             # <<<<<<<<<<<<<<
 *         """() => UINT
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_6get_nlinks(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self) {
  size_t __pyx_v_nlinks;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_nlinks", 0);

  /* "h5py/h5p.pyx":1060
 *         """
 *         cdef size_t nlinks
 *         H5Pget_nlinks(self.id, &nlinks)             # <<<<<<<<<<<<<<
 *         return nlinks
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_nlinks(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_nlinks)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1060; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":1061
 *         cdef size_t nlinks
 *         H5Pget_nlinks(self.id, &nlinks)
 *         return nlinks             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_nlinks); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1061; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropLAID.get_nlinks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_9set_elink_prefix(PyObject *__pyx_v_self, PyObject *__pyx_arg_prefix); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLAID_8set_elink_prefix[] = "(STRING prefix)\n\n        Set the external link prefix.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_9set_elink_prefix(PyObject *__pyx_v_self, PyObject *__pyx_arg_prefix) {
  char *__pyx_v_prefix;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_elink_prefix (wrapper)", 0);
  assert(__pyx_arg_prefix); {
    __pyx_v_prefix = PyBytes_AsString(__pyx_arg_prefix); if (unlikely((!__pyx_v_prefix) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1064; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLAID.set_elink_prefix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID_8set_elink_prefix(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self), ((char *)__pyx_v_prefix));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1064
 * 
 * 
 *     def set_elink_prefix(self, char* prefix):             # <<<<<<<<<<<<<<
 *         """(STRING prefix)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_8set_elink_prefix(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, char *__pyx_v_prefix) {
  size_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  void *__pyx_t_1;
  herr_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_elink_prefix", 0);

  /* "h5py/h5p.pyx":1072
 * 
 *         # HDF5 requires that we hang on to this buffer
 *         efree(self._buf)             # <<<<<<<<<<<<<<
 *         size = strlen(prefix)
 *         self._buf = <char*>emalloc(size+1)
 */
  __pyx_f_4h5py_5utils_efree(__pyx_v_self->_buf);

  /* "h5py/h5p.pyx":1073
 *         # HDF5 requires that we hang on to this buffer
 *         efree(self._buf)
 *         size = strlen(prefix)             # <<<<<<<<<<<<<<
 *         self._buf = <char*>emalloc(size+1)
 *         strcpy(self._buf, prefix)
 */
  __pyx_v_size = strlen(__pyx_v_prefix);

  /* "h5py/h5p.pyx":1074
 *         efree(self._buf)
 *         size = strlen(prefix)
 *         self._buf = <char*>emalloc(size+1)             # <<<<<<<<<<<<<<
 *         strcpy(self._buf, prefix)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_emalloc((__pyx_v_size + 1)); if (unlikely(__pyx_t_1 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1074; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->_buf = ((char *)__pyx_t_1);

  /* "h5py/h5p.pyx":1075
 *         size = strlen(prefix)
 *         self._buf = <char*>emalloc(size+1)
 *         strcpy(self._buf, prefix)             # <<<<<<<<<<<<<<
 * 
 *         H5Pset_elink_prefix(self.id, self._buf)
 */
  strcpy(__pyx_v_self->_buf, __pyx_v_prefix);

  /* "h5py/h5p.pyx":1077
 *         strcpy(self._buf, prefix)
 * 
 *         H5Pset_elink_prefix(self.id, self._buf)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pset_elink_prefix(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_self->_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1077; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLAID.set_elink_prefix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_11get_elink_prefix(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLAID_10get_elink_prefix[] = "() => STRING prefix\n\n        Get the external link prefix\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_11get_elink_prefix(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_elink_prefix (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID_10get_elink_prefix(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1080
 * 
 * 
 *     def get_elink_prefix(self):             # <<<<<<<<<<<<<<
 *         """() => STRING prefix
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_10get_elink_prefix(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self) {
  char *__pyx_v_buf;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_pstr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_elink_prefix", 0);

  /* "h5py/h5p.pyx":1085
 *         Get the external link prefix
 *         """
 *         cdef char* buf = NULL             # <<<<<<<<<<<<<<
 *         cdef ssize_t size
 * 
 */
  __pyx_v_buf = NULL;

  /* "h5py/h5p.pyx":1088
 *         cdef ssize_t size
 * 
 *         size = H5Pget_elink_prefix(self.id, NULL, 0)             # <<<<<<<<<<<<<<
 *         buf = <char*>emalloc(size+1)
 *         try:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_elink_prefix(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, NULL, 0); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1088; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_size = __pyx_t_1;

  /* "h5py/h5p.pyx":1089
 * 
 *         size = H5Pget_elink_prefix(self.id, NULL, 0)
 *         buf = <char*>emalloc(size+1)             # <<<<<<<<<<<<<<
 *         try:
 *             H5Pget_elink_prefix(self.id, buf, size+1)
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc((__pyx_v_size + 1)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1089; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = ((char *)__pyx_t_2);

  /* "h5py/h5p.pyx":1090
 *         size = H5Pget_elink_prefix(self.id, NULL, 0)
 *         buf = <char*>emalloc(size+1)
 *         try:             # <<<<<<<<<<<<<<
 *             H5Pget_elink_prefix(self.id, buf, size+1)
 *             pstr = buf
 */
  /*try:*/ {

    /* "h5py/h5p.pyx":1091
 *         buf = <char*>emalloc(size+1)
 *         try:
 *             H5Pget_elink_prefix(self.id, buf, size+1)             # <<<<<<<<<<<<<<
 *             pstr = buf
 *         finally:
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_elink_prefix(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_buf, (__pyx_v_size + 1)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1091; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5p.pyx":1092
 *         try:
 *             H5Pget_elink_prefix(self.id, buf, size+1)
 *             pstr = buf             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(buf)
 */
    __pyx_v_pstr = __pyx_v_buf;
  }

  /* "h5py/h5p.pyx":1094
 *             pstr = buf
 *         finally:
 *             efree(buf)             # <<<<<<<<<<<<<<
 * 
 *         return pstr
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_buf);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  /* "h5py/h5p.pyx":1096
 *             efree(buf)
 * 
 *         return pstr             # <<<<<<<<<<<<<<
 * 
 *     def set_elink_fapl(self, PropID fapl not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyBytes_FromString(__pyx_v_pstr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1096; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_3));
  __pyx_r = ((PyObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5p.PropLAID.get_elink_prefix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_13set_elink_fapl(PyObject *__pyx_v_self, PyObject *__pyx_v_fapl); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLAID_12set_elink_fapl[] = " (PropFAID fapl)\n\n        Set the file access property list used when opening external files.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_13set_elink_fapl(PyObject *__pyx_v_self, PyObject *__pyx_v_fapl) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_elink_fapl (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fapl), __pyx_ptype_4h5py_3h5p_PropID, 0, "fapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1098; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID_12set_elink_fapl(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self), ((struct __pyx_obj_4h5py_3h5p_PropID *)__pyx_v_fapl));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1098
 *         return pstr
 * 
 *     def set_elink_fapl(self, PropID fapl not None):             # <<<<<<<<<<<<<<
 *         """ (PropFAID fapl)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_12set_elink_fapl(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_fapl) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_elink_fapl", 0);

  /* "h5py/h5p.pyx":1103
 *         Set the file access property list used when opening external files.
 *         """
 *         H5Pset_elink_fapl(self.id, fapl.id)             # <<<<<<<<<<<<<<
 * 
 *     def get_elink_fapl(self):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_elink_fapl(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_fapl->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropLAID.set_elink_fapl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_15get_elink_fapl(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropLAID_14get_elink_fapl[] = " () => PropFAID fapl\n\n        Get the file access property list used when opening external files.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropLAID_15get_elink_fapl(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_elink_fapl (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropLAID_14get_elink_fapl(((struct __pyx_obj_4h5py_3h5p_PropLAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1105
 *         H5Pset_elink_fapl(self.id, fapl.id)
 * 
 *     def get_elink_fapl(self):             # <<<<<<<<<<<<<<
 *         """ () => PropFAID fapl
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropLAID_14get_elink_fapl(struct __pyx_obj_4h5py_3h5p_PropLAID *__pyx_v_self) {
  hid_t __pyx_v_fid;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_elink_fapl", 0);

  /* "h5py/h5p.pyx":1111
 *         """
 *         cdef hid_t fid
 *         fid = H5Pget_elink_fapl(self.id)             # <<<<<<<<<<<<<<
 *         if H5Iget_ref(fid) > 1:
 *             H5Idec_ref(fid)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_elink_fapl(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_fid = __pyx_t_1;

  /* "h5py/h5p.pyx":1112
 *         cdef hid_t fid
 *         fid = H5Pget_elink_fapl(self.id)
 *         if H5Iget_ref(fid) > 1:             # <<<<<<<<<<<<<<
 *             H5Idec_ref(fid)
 *         return propwrap(fid)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Iget_ref(__pyx_v_fid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = (__pyx_t_2 > 1);
  if (__pyx_t_3) {

    /* "h5py/h5p.pyx":1113
 *         fid = H5Pget_elink_fapl(self.id)
 *         if H5Iget_ref(fid) > 1:
 *             H5Idec_ref(fid)             # <<<<<<<<<<<<<<
 *         return propwrap(fid)
 * 
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Idec_ref(__pyx_v_fid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5p.pyx":1114
 *         if H5Iget_ref(fid) > 1:
 *             H5Idec_ref(fid)
 *         return propwrap(fid)             # <<<<<<<<<<<<<<
 * 
 * # Group creation
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_f_4h5py_3h5p_propwrap(__pyx_v_fid); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5p.PropLAID.get_elink_fapl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropGCID_1set_link_creation_order(PyObject *__pyx_v_self, PyObject *__pyx_arg_flags); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropGCID_set_link_creation_order[] = " (UINT flags)\n\n        Set tracking and indexing of creation order for links added to this group\n\n        flags -- h5p.CRT_ORDER_TRACKED, h5p.CRT_ORDER_INDEXED\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropGCID_1set_link_creation_order(PyObject *__pyx_v_self, PyObject *__pyx_arg_flags) {
  unsigned int __pyx_v_flags;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_link_creation_order (wrapper)", 0);
  assert(__pyx_arg_flags); {
    __pyx_v_flags = __Pyx_PyInt_AsUnsignedInt(__pyx_arg_flags); if (unlikely((__pyx_v_flags == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropGCID.set_link_creation_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropGCID_set_link_creation_order(((struct __pyx_obj_4h5py_3h5p_PropGCID *)__pyx_v_self), ((unsigned int)__pyx_v_flags));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1120
 *     """ Group creation property list """
 * 
 *     def set_link_creation_order(self, unsigned int flags):             # <<<<<<<<<<<<<<
 *         """ (UINT flags)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropGCID_set_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropGCID *__pyx_v_self, unsigned int __pyx_v_flags) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_link_creation_order", 0);

  /* "h5py/h5p.pyx":1127
 *         flags -- h5p.CRT_ORDER_TRACKED, h5p.CRT_ORDER_INDEXED
 *         """
 *         H5Pset_link_creation_order(self.id, flags)             # <<<<<<<<<<<<<<
 * 
 *     def get_link_creation_order(self):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_link_creation_order(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_flags); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropGCID.set_link_creation_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropGCID_3get_link_creation_order(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropGCID_2get_link_creation_order[] = " () -> UINT flags\n\n        Get tracking and indexing of creation order for links added to this group\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropGCID_3get_link_creation_order(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_link_creation_order (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropGCID_2get_link_creation_order(((struct __pyx_obj_4h5py_3h5p_PropGCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1129
 *         H5Pset_link_creation_order(self.id, flags)
 * 
 *     def get_link_creation_order(self):             # <<<<<<<<<<<<<<
 *         """ () -> UINT flags
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropGCID_2get_link_creation_order(struct __pyx_obj_4h5py_3h5p_PropGCID *__pyx_v_self) {
  unsigned int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_link_creation_order", 0);

  /* "h5py/h5p.pyx":1135
 *         """
 *         cdef unsigned int flags
 *         H5Pget_link_creation_order(self.id, &flags)             # <<<<<<<<<<<<<<
 *         return flags
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_link_creation_order(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_flags)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":1136
 *         cdef unsigned int flags
 *         H5Pget_link_creation_order(self.id, &flags)
 *         return flags             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyLong_FromUnsignedLong(__pyx_v_flags); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropGCID.get_link_creation_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropOCID_1set_obj_track_times(PyObject *__pyx_v_self, PyObject *__pyx_v_track_times); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropOCID_set_obj_track_times[] = "Sets the recording of times associated with an object.";
static PyObject *__pyx_pw_4h5py_3h5p_8PropOCID_1set_obj_track_times(PyObject *__pyx_v_self, PyObject *__pyx_v_track_times) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_obj_track_times (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropOCID_set_obj_track_times(((struct __pyx_obj_4h5py_3h5p_PropOCID *)__pyx_v_self), ((PyObject *)__pyx_v_track_times));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1148
 *     The documentation is somewhat hazy
 *     """
 *     def set_obj_track_times(self,track_times):             # <<<<<<<<<<<<<<
 *         """Sets the recording of times associated with an object."""
 *         H5Pset_obj_track_times(self.id,track_times)
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropOCID_set_obj_track_times(struct __pyx_obj_4h5py_3h5p_PropOCID *__pyx_v_self, PyObject *__pyx_v_track_times) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hbool_t __pyx_t_1;
  herr_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_obj_track_times", 0);

  /* "h5py/h5p.pyx":1150
 *     def set_obj_track_times(self,track_times):
 *         """Sets the recording of times associated with an object."""
 *         H5Pset_obj_track_times(self.id,track_times)             # <<<<<<<<<<<<<<
 * 
 *     def get_obj_track_times(self):
 */
  __pyx_t_1 = __Pyx_PyInt_from_py_hbool_t(__pyx_v_track_times); if (unlikely((__pyx_t_1 == (hbool_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Pset_obj_track_times(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, __pyx_t_1); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropOCID.set_obj_track_times", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropOCID_3get_obj_track_times(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropOCID_2get_obj_track_times[] = "\n        Determines whether times associated with an object are being recorded.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropOCID_3get_obj_track_times(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_obj_track_times (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropOCID_2get_obj_track_times(((struct __pyx_obj_4h5py_3h5p_PropOCID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1152
 *         H5Pset_obj_track_times(self.id,track_times)
 * 
 *     def get_obj_track_times(self):             # <<<<<<<<<<<<<<
 *         """
 *         Determines whether times associated with an object are being recorded.
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropOCID_2get_obj_track_times(struct __pyx_obj_4h5py_3h5p_PropOCID *__pyx_v_self) {
  hbool_t __pyx_v_track_times;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_obj_track_times", 0);

  /* "h5py/h5p.pyx":1159
 *         cdef hbool_t track_times
 * 
 *         H5Pget_obj_track_times(self.id,&track_times)             # <<<<<<<<<<<<<<
 * 
 *         return track_times
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_obj_track_times(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_track_times)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1159; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":1161
 *         H5Pget_obj_track_times(self.id,&track_times)
 * 
 *         return track_times             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_to_py_hbool_t(__pyx_v_track_times); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5p.PropOCID.get_obj_track_times", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDAID_1set_chunk_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDAID_set_chunk_cache[] = "(size_t rdcc_nslots,size_t rdcc_nbytes, double rdcc_w0)\n\n        Sets the raw data chunk cache parameters.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDAID_1set_chunk_cache(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_rdcc_nslots;
  size_t __pyx_v_rdcc_nbytes;
  double __pyx_v_rdcc_w0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_chunk_cache (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__rdcc_nslots,&__pyx_n_s__rdcc_nbytes,&__pyx_n_s__rdcc_w0,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__rdcc_nslots)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__rdcc_nbytes)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_chunk_cache", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__rdcc_w0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_chunk_cache", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_chunk_cache") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_rdcc_nslots = __Pyx_PyInt_AsSize_t(values[0]); if (unlikely((__pyx_v_rdcc_nslots == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_rdcc_nbytes = __Pyx_PyInt_AsSize_t(values[1]); if (unlikely((__pyx_v_rdcc_nbytes == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_rdcc_w0 = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_rdcc_w0 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_chunk_cache", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDAID.set_chunk_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDAID_set_chunk_cache(((struct __pyx_obj_4h5py_3h5p_PropDAID *)__pyx_v_self), __pyx_v_rdcc_nslots, __pyx_v_rdcc_nbytes, __pyx_v_rdcc_w0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1169
 *     """ Dataset access property list """
 * 
 *     def set_chunk_cache(self, size_t rdcc_nslots,size_t rdcc_nbytes, double rdcc_w0):             # <<<<<<<<<<<<<<
 *         """(size_t rdcc_nslots,size_t rdcc_nbytes, double rdcc_w0)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDAID_set_chunk_cache(struct __pyx_obj_4h5py_3h5p_PropDAID *__pyx_v_self, size_t __pyx_v_rdcc_nslots, size_t __pyx_v_rdcc_nbytes, double __pyx_v_rdcc_w0) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_chunk_cache", 0);

  /* "h5py/h5p.pyx":1174
 *         Sets the raw data chunk cache parameters.
 *         """
 *         H5Pset_chunk_cache(self.id,rdcc_nslots,rdcc_nbytes,rdcc_w0)             # <<<<<<<<<<<<<<
 * 
 *     def get_chunk_cache(self):
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pset_chunk_cache(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, __pyx_v_rdcc_nslots, __pyx_v_rdcc_nbytes, __pyx_v_rdcc_w0); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5p.PropDAID.set_chunk_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5p_8PropDAID_3get_chunk_cache(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5p_8PropDAID_2get_chunk_cache[] = "() => TUPLE chunk cache info\n\n        Get the metadata and raw data chunk cache settings.  See the HDF5\n        docs for element definitions.  Return is a 3-tuple with entries:\n\n        0. size_t rdcc_nslots: Number of chunk slots in the raw data chunk cache hash table.\n        1. size_t rdcc_nbytes: Total size of the raw data chunk cache, in bytes.\n        2. DOUBLE rdcc_w0:     Preemption policy.\n        ";
static PyObject *__pyx_pw_4h5py_3h5p_8PropDAID_3get_chunk_cache(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_chunk_cache (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5p_8PropDAID_2get_chunk_cache(((struct __pyx_obj_4h5py_3h5p_PropDAID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5p.pyx":1176
 *         H5Pset_chunk_cache(self.id,rdcc_nslots,rdcc_nbytes,rdcc_w0)
 * 
 *     def get_chunk_cache(self):             # <<<<<<<<<<<<<<
 *         """() => TUPLE chunk cache info
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5p_8PropDAID_2get_chunk_cache(struct __pyx_obj_4h5py_3h5p_PropDAID *__pyx_v_self) {
  size_t __pyx_v_rdcc_nslots;
  size_t __pyx_v_rdcc_nbytes;
  double __pyx_v_rdcc_w0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_chunk_cache", 0);

  /* "h5py/h5p.pyx":1190
 *         cdef double rdcc_w0
 * 
 *         H5Pget_chunk_cache(self.id, &rdcc_nslots, &rdcc_nbytes, &rdcc_w0 )             # <<<<<<<<<<<<<<
 *         return (rdcc_nslots,rdcc_nbytes,rdcc_w0)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Pget_chunk_cache(__pyx_v_self->__pyx_base.__pyx_base.__pyx_base.id, (&__pyx_v_rdcc_nslots), (&__pyx_v_rdcc_nbytes), (&__pyx_v_rdcc_w0)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":1191
 * 
 *         H5Pget_chunk_cache(self.id, &rdcc_nslots, &rdcc_nbytes, &rdcc_w0 )
 *         return (rdcc_nslots,rdcc_nbytes,rdcc_w0)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_v_rdcc_nslots); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_rdcc_nbytes); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_rdcc_w0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = ((PyObject *)__pyx_t_5);
  __pyx_t_5 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("h5py.h5p.PropDAID.get_chunk_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_4h5py_3h5p_PropID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_ptype_4h5py_8_objects_ObjectID->tp_new(t, a, k);
  if (!o) return 0;
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5p_PropID(PyObject *o) {
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_4h5py_8_objects_ObjectID)) __pyx_ptype_4h5py_8_objects_ObjectID->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_4h5py_3h5p_PropID);
}

static int __pyx_tp_traverse_4h5py_3h5p_PropID(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_4h5py_8_objects_ObjectID)) ? ((__pyx_ptype_4h5py_8_objects_ObjectID->tp_traverse) ? __pyx_ptype_4h5py_8_objects_ObjectID->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4h5py_3h5p_PropID)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5p_PropID(PyObject *o) {
  if (likely(__pyx_ptype_4h5py_8_objects_ObjectID)) { if (__pyx_ptype_4h5py_8_objects_ObjectID->tp_clear) __pyx_ptype_4h5py_8_objects_ObjectID->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4h5py_3h5p_PropID);
  return 0;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropID[] = {
  {__Pyx_NAMESTR("equal"), (PyCFunction)__pyx_pw_4h5py_3h5p_6PropID_1equal, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_6PropID_equal)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropID, /*tp_as_mapping*/
  __pyx_pw_4h5py_3h5p_6PropID_5__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Base class for all property lists and classes\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  __pyx_pw_4h5py_3h5p_6PropID_3__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropClassID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropClassID[] = {
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropClassID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropClassID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropClassID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropClassID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropClassID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropClassID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropClassID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropClassID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropClassID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropClassID, /*tp_as_mapping*/
  __pyx_pw_4h5py_3h5p_11PropClassID_3__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropClassID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        An HDF5 property list class.\n\n        * Hashable: Yes, by identifier\n        * Equality: Logical H5P comparison\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  __pyx_pw_4h5py_3h5p_11PropClassID_1__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropClassID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropClassID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropInstanceID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropInstanceID[] = {
  {__Pyx_NAMESTR("copy"), (PyCFunction)__pyx_pw_4h5py_3h5p_14PropInstanceID_1copy, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_14PropInstanceID_copy)},
  {__Pyx_NAMESTR("_close"), (PyCFunction)__pyx_pw_4h5py_3h5p_14PropInstanceID_3_close, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_14PropInstanceID_2_close)},
  {__Pyx_NAMESTR("get_class"), (PyCFunction)__pyx_pw_4h5py_3h5p_14PropInstanceID_5get_class, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_14PropInstanceID_4get_class)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropInstanceID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropInstanceID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropInstanceID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropInstanceID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropInstanceID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropInstanceID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropInstanceID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropInstanceID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropInstanceID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropInstanceID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropInstanceID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Base class for property list instance objects.  Provides methods which\n        are common across all HDF5 property list classes.\n\n        * Hashable: No\n        * Equality: Logical H5P comparison\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropInstanceID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropInstanceID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropCreateID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropCreateID[] = {
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropCreateID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropCreateID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropCreateID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropCreateID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropCreateID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropCreateID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropCreateID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropCreateID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropCreateID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropCreateID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropCreateID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Generic object creation property list.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropCreateID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropCreateID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropCopyID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropCopyID[] = {
  {__Pyx_NAMESTR("set_copy_object"), (PyCFunction)__pyx_pw_4h5py_3h5p_10PropCopyID_1set_copy_object, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_10PropCopyID_set_copy_object)},
  {__Pyx_NAMESTR("get_copy_object"), (PyCFunction)__pyx_pw_4h5py_3h5p_10PropCopyID_3get_copy_object, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_10PropCopyID_2get_copy_object)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropCopyID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropCopyID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropCopyID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropCopyID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropCopyID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropCopyID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropCopyID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropCopyID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropCopyID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropCopyID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropCopyID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Generic object copy property list\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropCopyID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropCopyID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropOCID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropOCID[] = {
  {__Pyx_NAMESTR("set_obj_track_times"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropOCID_1set_obj_track_times, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropOCID_set_obj_track_times)},
  {__Pyx_NAMESTR("get_obj_track_times"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropOCID_3get_obj_track_times, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropOCID_2get_obj_track_times)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropOCID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropOCID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropOCID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropOCID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropOCID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropOCID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropOCID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropOCID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropOCID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropOCID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropOCID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR(" Object creation property list\n\n    This seems to be a super class for dataset creation property list\n    and group creation property list.\n\n    The documentation is somewhat hazy\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropOCID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropOCID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropDCID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropDCID[] = {
  {__Pyx_NAMESTR("set_layout"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_1set_layout, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_set_layout)},
  {__Pyx_NAMESTR("get_layout"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_3get_layout, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_2get_layout)},
  {__Pyx_NAMESTR("set_chunk"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_5set_chunk, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_4set_chunk)},
  {__Pyx_NAMESTR("get_chunk"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_7get_chunk, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_6get_chunk)},
  {__Pyx_NAMESTR("set_fill_value"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_9set_fill_value, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_8set_fill_value)},
  {__Pyx_NAMESTR("get_fill_value"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_11get_fill_value, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_10get_fill_value)},
  {__Pyx_NAMESTR("fill_value_defined"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_13fill_value_defined, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_12fill_value_defined)},
  {__Pyx_NAMESTR("set_fill_time"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_15set_fill_time, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_14set_fill_time)},
  {__Pyx_NAMESTR("get_fill_time"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_17get_fill_time, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_16get_fill_time)},
  {__Pyx_NAMESTR("set_alloc_time"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_19set_alloc_time, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_18set_alloc_time)},
  {__Pyx_NAMESTR("get_alloc_time"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_21get_alloc_time, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_20get_alloc_time)},
  {__Pyx_NAMESTR("set_filter"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_23set_filter, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_22set_filter)},
  {__Pyx_NAMESTR("all_filters_avail"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_25all_filters_avail, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_24all_filters_avail)},
  {__Pyx_NAMESTR("get_nfilters"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_27get_nfilters, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_26get_nfilters)},
  {__Pyx_NAMESTR("get_filter"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_29get_filter, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_28get_filter)},
  {__Pyx_NAMESTR("_has_filter"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_31_has_filter, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_30_has_filter)},
  {__Pyx_NAMESTR("get_filter_by_id"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_33get_filter_by_id, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_32get_filter_by_id)},
  {__Pyx_NAMESTR("remove_filter"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_35remove_filter, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_34remove_filter)},
  {__Pyx_NAMESTR("set_deflate"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_37set_deflate, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_36set_deflate)},
  {__Pyx_NAMESTR("set_fletcher32"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_39set_fletcher32, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_38set_fletcher32)},
  {__Pyx_NAMESTR("set_shuffle"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_41set_shuffle, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_40set_shuffle)},
  {__Pyx_NAMESTR("set_szip"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_43set_szip, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_42set_szip)},
  {__Pyx_NAMESTR("set_scaleoffset"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDCID_45set_scaleoffset, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDCID_44set_scaleoffset)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropDCID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropDCID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropDCID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropDCID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropDCID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropDCID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropDCID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropDCID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropDCID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropDCID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropDCID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Dataset creation property list.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropDCID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropDCID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropFCID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropFCID[] = {
  {__Pyx_NAMESTR("get_version"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_1get_version, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_get_version)},
  {__Pyx_NAMESTR("set_userblock"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_3set_userblock, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_2set_userblock)},
  {__Pyx_NAMESTR("get_userblock"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_5get_userblock, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_4get_userblock)},
  {__Pyx_NAMESTR("set_sizes"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_7set_sizes, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_6set_sizes)},
  {__Pyx_NAMESTR("get_sizes"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_9get_sizes, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_8get_sizes)},
  {__Pyx_NAMESTR("set_link_creation_order"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_11set_link_creation_order, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_10set_link_creation_order)},
  {__Pyx_NAMESTR("get_link_creation_order"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFCID_13get_link_creation_order, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFCID_12get_link_creation_order)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropFCID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropFCID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropFCID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropFCID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropFCID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropFCID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropFCID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropFCID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropFCID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropFCID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropFCID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        File creation property list.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropFCID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropFCID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropFAID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropFAID[] = {
  {__Pyx_NAMESTR("set_fclose_degree"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_1set_fclose_degree, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_set_fclose_degree)},
  {__Pyx_NAMESTR("get_fclose_degree"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_3get_fclose_degree, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_2get_fclose_degree)},
  {__Pyx_NAMESTR("set_fapl_core"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_5set_fapl_core, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_4set_fapl_core)},
  {__Pyx_NAMESTR("get_fapl_core"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_7get_fapl_core, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_6get_fapl_core)},
  {__Pyx_NAMESTR("set_fapl_family"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_9set_fapl_family, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_8set_fapl_family)},
  {__Pyx_NAMESTR("get_fapl_family"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_11get_fapl_family, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_10get_fapl_family)},
  {__Pyx_NAMESTR("set_fapl_log"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_13set_fapl_log, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_12set_fapl_log)},
  {__Pyx_NAMESTR("set_fapl_sec2"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_15set_fapl_sec2, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_14set_fapl_sec2)},
  {__Pyx_NAMESTR("set_fapl_stdio"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_17set_fapl_stdio, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_16set_fapl_stdio)},
  {__Pyx_NAMESTR("get_driver"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_19get_driver, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_18get_driver)},
  {__Pyx_NAMESTR("set_cache"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_21set_cache, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_20set_cache)},
  {__Pyx_NAMESTR("get_cache"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_23get_cache, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_22get_cache)},
  {__Pyx_NAMESTR("set_sieve_buf_size"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_25set_sieve_buf_size, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_24set_sieve_buf_size)},
  {__Pyx_NAMESTR("get_sieve_buf_size"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_27get_sieve_buf_size, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_26get_sieve_buf_size)},
  {__Pyx_NAMESTR("set_libver_bounds"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_29set_libver_bounds, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_28set_libver_bounds)},
  {__Pyx_NAMESTR("get_libver_bounds"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_31get_libver_bounds, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_30get_libver_bounds)},
  {__Pyx_NAMESTR("get_mdc_config"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_33get_mdc_config, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_32get_mdc_config)},
  {__Pyx_NAMESTR("set_mdc_config"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropFAID_35set_mdc_config, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropFAID_34set_mdc_config)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropFAID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropFAID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropFAID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropFAID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropFAID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropFAID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropFAID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropFAID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropFAID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropFAID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropFAID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        File access property list\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropFAID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropFAID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropDXID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropDXID[] = {
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropDXID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropDXID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropDXID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropDXID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropDXID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropDXID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropDXID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropDXID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropDXID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropDXID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropDXID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR(" Dataset transfer property list "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropDXID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropDXID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropLCID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropLCID[] = {
  {__Pyx_NAMESTR("set_char_encoding"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLCID_1set_char_encoding, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLCID_set_char_encoding)},
  {__Pyx_NAMESTR("get_char_encoding"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLCID_3get_char_encoding, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLCID_2get_char_encoding)},
  {__Pyx_NAMESTR("set_create_intermediate_group"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLCID_5set_create_intermediate_group, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLCID_4set_create_intermediate_group)},
  {__Pyx_NAMESTR("get_create_intermediate_group"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLCID_7get_create_intermediate_group, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLCID_6get_create_intermediate_group)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropLCID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropLCID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropLCID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropLCID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropLCID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropLCID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropLCID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropLCID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropLCID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropLCID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropLCID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR(" Link creation property list "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropLCID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropLCID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropLAID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  if (__pyx_pw_4h5py_3h5p_8PropLAID_1__cinit__(o, a, k) < 0) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5p_PropLAID(PyObject *o) {
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4h5py_3h5p_8PropLAID_3__dealloc__(o);
    if (PyErr_Occurred()) PyErr_WriteUnraisable(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4h5py_3h5p_PropID(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropLAID[] = {
  {__Pyx_NAMESTR("set_nlinks"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLAID_5set_nlinks, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLAID_4set_nlinks)},
  {__Pyx_NAMESTR("get_nlinks"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLAID_7get_nlinks, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLAID_6get_nlinks)},
  {__Pyx_NAMESTR("set_elink_prefix"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLAID_9set_elink_prefix, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLAID_8set_elink_prefix)},
  {__Pyx_NAMESTR("get_elink_prefix"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLAID_11get_elink_prefix, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLAID_10get_elink_prefix)},
  {__Pyx_NAMESTR("set_elink_fapl"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLAID_13set_elink_fapl, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLAID_12set_elink_fapl)},
  {__Pyx_NAMESTR("get_elink_fapl"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropLAID_15get_elink_fapl, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropLAID_14get_elink_fapl)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropLAID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropLAID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropLAID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropLAID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropLAID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropLAID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropLAID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropLAID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropLAID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropLAID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropLAID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropLAID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR(" Link access property list "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropLAID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropLAID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropGCID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropGCID[] = {
  {__Pyx_NAMESTR("set_link_creation_order"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropGCID_1set_link_creation_order, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropGCID_set_link_creation_order)},
  {__Pyx_NAMESTR("get_link_creation_order"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropGCID_3get_link_creation_order, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropGCID_2get_link_creation_order)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropGCID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropGCID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropGCID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropGCID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropGCID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropGCID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropGCID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropGCID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropGCID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropGCID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropGCID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR(" Group creation property list "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropGCID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropGCID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5p_PropDAID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5p_PropID(t, a, k);
  if (!o) return 0;
  return o;
}

static PyMethodDef __pyx_methods_4h5py_3h5p_PropDAID[] = {
  {__Pyx_NAMESTR("set_chunk_cache"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDAID_1set_chunk_cache, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDAID_set_chunk_cache)},
  {__Pyx_NAMESTR("get_chunk_cache"), (PyCFunction)__pyx_pw_4h5py_3h5p_8PropDAID_3get_chunk_cache, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5p_8PropDAID_2get_chunk_cache)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_PropDAID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_PropDAID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_PropDAID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_PropDAID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5p_PropDAID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5p.PropDAID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5p_PropDAID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5p_PropID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_PropDAID, /*tp_as_number*/
  &__pyx_tp_as_sequence_PropDAID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_PropDAID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_PropDAID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR(" Dataset access property list "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5p_PropID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5p_PropID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5p_PropDAID, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5p_PropDAID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    __Pyx_NAMESTR("h5p"),
    __Pyx_DOCSTR(__pyx_k_10), /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 1, 0},
  {&__pyx_kp_s_13, __pyx_k_13, sizeof(__pyx_k_13), 0, 0, 1, 0},
  {&__pyx_n_s_14, __pyx_k_14, sizeof(__pyx_k_14), 0, 0, 1, 1},
  {&__pyx_kp_s_2, __pyx_k_2, sizeof(__pyx_k_2), 0, 0, 1, 0},
  {&__pyx_kp_s_5, __pyx_k_5, sizeof(__pyx_k_5), 0, 0, 1, 0},
  {&__pyx_kp_s_7, __pyx_k_7, sizeof(__pyx_k_7), 0, 0, 1, 0},
  {&__pyx_n_s__CRT_ORDER_INDEXED, __pyx_k__CRT_ORDER_INDEXED, sizeof(__pyx_k__CRT_ORDER_INDEXED), 0, 0, 1, 1},
  {&__pyx_n_s__CRT_ORDER_TRACKED, __pyx_k__CRT_ORDER_TRACKED, sizeof(__pyx_k__CRT_ORDER_TRACKED), 0, 0, 1, 1},
  {&__pyx_n_s__DATASET_ACCESS, __pyx_k__DATASET_ACCESS, sizeof(__pyx_k__DATASET_ACCESS), 0, 0, 1, 1},
  {&__pyx_n_s__DATASET_CREATE, __pyx_k__DATASET_CREATE, sizeof(__pyx_k__DATASET_CREATE), 0, 0, 1, 1},
  {&__pyx_n_s__DATASET_XFER, __pyx_k__DATASET_XFER, sizeof(__pyx_k__DATASET_XFER), 0, 0, 1, 1},
  {&__pyx_n_s__DEFAULT, __pyx_k__DEFAULT, sizeof(__pyx_k__DEFAULT), 0, 0, 1, 1},
  {&__pyx_n_s__FILE_ACCESS, __pyx_k__FILE_ACCESS, sizeof(__pyx_k__FILE_ACCESS), 0, 0, 1, 1},
  {&__pyx_n_s__FILE_CREATE, __pyx_k__FILE_CREATE, sizeof(__pyx_k__FILE_CREATE), 0, 0, 1, 1},
  {&__pyx_n_s__GROUP_CREATE, __pyx_k__GROUP_CREATE, sizeof(__pyx_k__GROUP_CREATE), 0, 0, 1, 1},
  {&__pyx_n_s__LINK_ACCESS, __pyx_k__LINK_ACCESS, sizeof(__pyx_k__LINK_ACCESS), 0, 0, 1, 1},
  {&__pyx_n_s__LINK_CREATE, __pyx_k__LINK_CREATE, sizeof(__pyx_k__LINK_CREATE), 0, 0, 1, 1},
  {&__pyx_n_s__NO_CLASS, __pyx_k__NO_CLASS, sizeof(__pyx_k__NO_CLASS), 0, 0, 1, 1},
  {&__pyx_n_s__NotImplemented, __pyx_k__NotImplemented, sizeof(__pyx_k__NotImplemented), 0, 0, 1, 1},
  {&__pyx_n_s__OBJECT_COPY, __pyx_k__OBJECT_COPY, sizeof(__pyx_k__OBJECT_COPY), 0, 0, 1, 1},
  {&__pyx_n_s__OBJECT_CREATE, __pyx_k__OBJECT_CREATE, sizeof(__pyx_k__OBJECT_CREATE), 0, 0, 1, 1},
  {&__pyx_n_s__RuntimeError, __pyx_k__RuntimeError, sizeof(__pyx_k__RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s__TypeError, __pyx_k__TypeError, sizeof(__pyx_k__TypeError), 0, 0, 1, 1},
  {&__pyx_n_s__ValueError, __pyx_k__ValueError, sizeof(__pyx_k__ValueError), 0, 0, 1, 1},
  {&__pyx_n_s____enter__, __pyx_k____enter__, sizeof(__pyx_k____enter__), 0, 0, 1, 1},
  {&__pyx_n_s____exit__, __pyx_k____exit__, sizeof(__pyx_k____exit__), 0, 0, 1, 1},
  {&__pyx_n_s____main__, __pyx_k____main__, sizeof(__pyx_k____main__), 0, 0, 1, 1},
  {&__pyx_n_s____richcmp__, __pyx_k____richcmp__, sizeof(__pyx_k____richcmp__), 0, 0, 1, 1},
  {&__pyx_n_s____test__, __pyx_k____test__, sizeof(__pyx_k____test__), 0, 0, 1, 1},
  {&__pyx_n_s___has_filter, __pyx_k___has_filter, sizeof(__pyx_k___has_filter), 0, 0, 1, 1},
  {&__pyx_n_s___objects, __pyx_k___objects, sizeof(__pyx_k___objects), 0, 0, 1, 1},
  {&__pyx_n_s__addr, __pyx_k__addr, sizeof(__pyx_k__addr), 0, 0, 1, 1},
  {&__pyx_n_s__backing_store, __pyx_k__backing_store, sizeof(__pyx_k__backing_store), 0, 0, 1, 1},
  {&__pyx_n_s__block_size, __pyx_k__block_size, sizeof(__pyx_k__block_size), 0, 0, 1, 1},
  {&__pyx_n_s__buf_size, __pyx_k__buf_size, sizeof(__pyx_k__buf_size), 0, 0, 1, 1},
  {&__pyx_n_s__cls, __pyx_k__cls, sizeof(__pyx_k__cls), 0, 0, 1, 1},
  {&__pyx_n_s__create, __pyx_k__create, sizeof(__pyx_k__create), 0, 0, 1, 1},
  {&__pyx_n_s__dtype, __pyx_k__dtype, sizeof(__pyx_k__dtype), 0, 0, 1, 1},
  {&__pyx_n_s__equal, __pyx_k__equal, sizeof(__pyx_k__equal), 0, 0, 1, 1},
  {&__pyx_n_s__filter_code, __pyx_k__filter_code, sizeof(__pyx_k__filter_code), 0, 0, 1, 1},
  {&__pyx_n_s__flags, __pyx_k__flags, sizeof(__pyx_k__flags), 0, 0, 1, 1},
  {&__pyx_n_s__get_filter, __pyx_k__get_filter, sizeof(__pyx_k__get_filter), 0, 0, 1, 1},
  {&__pyx_n_s__get_nfilters, __pyx_k__get_nfilters, sizeof(__pyx_k__get_nfilters), 0, 0, 1, 1},
  {&__pyx_n_s__h5py, __pyx_k__h5py, sizeof(__pyx_k__h5py), 0, 0, 1, 1},
  {&__pyx_n_s__high, __pyx_k__high, sizeof(__pyx_k__high), 0, 0, 1, 1},
  {&__pyx_n_s__level, __pyx_k__level, sizeof(__pyx_k__level), 0, 0, 1, 1},
  {&__pyx_n_s__lock, __pyx_k__lock, sizeof(__pyx_k__lock), 0, 0, 1, 1},
  {&__pyx_n_s__logfile, __pyx_k__logfile, sizeof(__pyx_k__logfile), 0, 0, 1, 1},
  {&__pyx_n_s__low, __pyx_k__low, sizeof(__pyx_k__low), 0, 0, 1, 1},
  {&__pyx_n_s__mdc, __pyx_k__mdc, sizeof(__pyx_k__mdc), 0, 0, 1, 1},
  {&__pyx_n_s__memb_fapl, __pyx_k__memb_fapl, sizeof(__pyx_k__memb_fapl), 0, 0, 1, 1},
  {&__pyx_n_s__memb_size, __pyx_k__memb_size, sizeof(__pyx_k__memb_size), 0, 0, 1, 1},
  {&__pyx_n_s__newid, __pyx_k__newid, sizeof(__pyx_k__newid), 0, 0, 1, 1},
  {&__pyx_n_s__open, __pyx_k__open, sizeof(__pyx_k__open), 0, 0, 1, 1},
  {&__pyx_n_s__options, __pyx_k__options, sizeof(__pyx_k__options), 0, 0, 1, 1},
  {&__pyx_n_s__pixels_per_block, __pyx_k__pixels_per_block, sizeof(__pyx_k__pixels_per_block), 0, 0, 1, 1},
  {&__pyx_n_s__rdcc, __pyx_k__rdcc, sizeof(__pyx_k__rdcc), 0, 0, 1, 1},
  {&__pyx_n_s__rdcc_nbytes, __pyx_k__rdcc_nbytes, sizeof(__pyx_k__rdcc_nbytes), 0, 0, 1, 1},
  {&__pyx_n_s__rdcc_nslots, __pyx_k__rdcc_nslots, sizeof(__pyx_k__rdcc_nslots), 0, 0, 1, 1},
  {&__pyx_n_s__rdcc_w0, __pyx_k__rdcc_w0, sizeof(__pyx_k__rdcc_w0), 0, 0, 1, 1},
  {&__pyx_n_s__registry, __pyx_k__registry, sizeof(__pyx_k__registry), 0, 0, 1, 1},
  {&__pyx_n_s__scale_factor, __pyx_k__scale_factor, sizeof(__pyx_k__scale_factor), 0, 0, 1, 1},
  {&__pyx_n_s__scale_type, __pyx_k__scale_type, sizeof(__pyx_k__scale_type), 0, 0, 1, 1},
  {&__pyx_n_s__size, __pyx_k__size, sizeof(__pyx_k__size), 0, 0, 1, 1},
  {&__pyx_n_s__valid, __pyx_k__valid, sizeof(__pyx_k__valid), 0, 0, 1, 1},
  {&__pyx_n_s__values, __pyx_k__values, sizeof(__pyx_k__values), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetName(__pyx_b, __pyx_n_s__ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_NotImplemented = __Pyx_GetName(__pyx_b, __pyx_n_s__NotImplemented); if (!__pyx_builtin_NotImplemented) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_TypeError = __Pyx_GetName(__pyx_b, __pyx_n_s__TypeError); if (!__pyx_builtin_TypeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetName(__pyx_b, __pyx_n_s__RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 974; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "h5py/h5p.pyx":154
 * 
 *     def __hash__(self):
 *         raise TypeError("Property lists are unhashable")             # <<<<<<<<<<<<<<
 * 
 * cdef class PropClassID(PropID):
 */
  __pyx_k_tuple_3 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_2)); if (unlikely(!__pyx_k_tuple_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_3);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_3));

  /* "h5py/h5p.pyx":200
 *         their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Pclose(self.id)
 *             if not self.valid:
 */
  __pyx_k_tuple_4 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_k_tuple_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_4);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_4));

  /* "h5py/h5p.pyx":599
 * 
 *         if filter_idx < 0:
 *             raise ValueError("Filter index must be a non-negative integer")             # <<<<<<<<<<<<<<
 * 
 *         filter_code = <int>H5Pget_filter(self.id, filter_idx, &flags,
 */
  __pyx_k_tuple_6 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_5)); if (unlikely(!__pyx_k_tuple_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_6);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_6));

  /* "h5py/h5p.pyx":974
 *         err = H5Fget_mdc_config(self.id, &config.cache_config)
 *         if err < 0:
 *             raise RuntimeError("Failed to get hit rate")             # <<<<<<<<<<<<<<
 * 
 *         return config
 */
  __pyx_k_tuple_8 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_7)); if (unlikely(!__pyx_k_tuple_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 974; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_8);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_8));

  /* "h5py/h5p.pyx":987
 *         err = H5Fset_mdc_config(self.id, &config.cache_config)
 *         if err < 0:
 *             raise RuntimeError("Failed to get hit rate")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_k_tuple_9 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_7)); if (unlikely(!__pyx_k_tuple_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 987; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_9);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_9));

  /* "h5py/h5p.pyx":106
 * # === Property list functional API ============================================
 * 
 * def create(PropClassID cls not None):             # <<<<<<<<<<<<<<
 *     """(PropClassID cls) => PropID
 * 
 */
  __pyx_k_tuple_11 = PyTuple_Pack(2, ((PyObject *)__pyx_n_s__cls), ((PyObject *)__pyx_n_s__newid)); if (unlikely(!__pyx_k_tuple_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_11);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_11));
  __pyx_k_codeobj_12 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_13, __pyx_n_s__create, 106, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC inith5p(void); /*proto*/
PyMODINIT_FUNC inith5p(void)
#else
PyMODINIT_FUNC PyInit_h5p(void); /*proto*/
PyMODINIT_FUNC PyInit_h5p(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_h5p(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4(__Pyx_NAMESTR("h5p"), __pyx_methods, __Pyx_DOCSTR(__pyx_k_10), 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "h5py.h5p")) {
      if (unlikely(PyDict_SetItemString(modules, "h5py.h5p", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  __pyx_b = PyImport_AddModule(__Pyx_NAMESTR(__Pyx_BUILTIN_MODULE_NAME)); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_module_is_main_h5py__h5p) {
    if (__Pyx_SetAttrString(__pyx_m, "__name__", __pyx_n_s____main__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("pdefault", (void (*)(void))__pyx_f_4h5py_3h5p_pdefault, "hid_t (struct __pyx_obj_4h5py_3h5p_PropID *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("propwrap", (void (*)(void))__pyx_f_4h5py_3h5p_propwrap, "PyObject *(hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Type init code ---*/
  __pyx_ptype_4h5py_8_objects_ObjectID = __Pyx_ImportType("h5py._objects", "ObjectID", sizeof(struct __pyx_obj_4h5py_8_objects_ObjectID), 1); if (unlikely(!__pyx_ptype_4h5py_8_objects_ObjectID)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_4h5py_3h5p_PropID.tp_base = __pyx_ptype_4h5py_8_objects_ObjectID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropID", (PyObject *)&__pyx_type_4h5py_3h5p_PropID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropID, __pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropID = &__pyx_type_4h5py_3h5p_PropID;
  __pyx_type_4h5py_3h5p_PropClassID.tp_base = __pyx_ptype_4h5py_3h5p_PropID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropClassID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = __Pyx_GetAttrString((PyObject *)&__pyx_type_4h5py_3h5p_PropClassID, "__hash__"); if (unlikely(!wrapper)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_4h5py_3h5p_11PropClassID_2__hash__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4h5py_3h5p_11PropClassID_2__hash__.doc = __pyx_doc_4h5py_3h5p_11PropClassID_2__hash__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4h5py_3h5p_11PropClassID_2__hash__;
    }
  }
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "PropClassID", (PyObject *)&__pyx_type_4h5py_3h5p_PropClassID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropClassID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropClassID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropClassID, __pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropClassID = &__pyx_type_4h5py_3h5p_PropClassID;
  __pyx_type_4h5py_3h5p_PropInstanceID.tp_base = __pyx_ptype_4h5py_3h5p_PropID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropInstanceID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropInstanceID", (PyObject *)&__pyx_type_4h5py_3h5p_PropInstanceID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropInstanceID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropInstanceID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropInstanceID, __pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropInstanceID = &__pyx_type_4h5py_3h5p_PropInstanceID;
  __pyx_type_4h5py_3h5p_PropCreateID.tp_base = __pyx_ptype_4h5py_3h5p_PropInstanceID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropCreateID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 213; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropCreateID", (PyObject *)&__pyx_type_4h5py_3h5p_PropCreateID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 213; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropCreateID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropCreateID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropCreateID, __pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropCreateID = &__pyx_type_4h5py_3h5p_PropCreateID;
  __pyx_type_4h5py_3h5p_PropCopyID.tp_base = __pyx_ptype_4h5py_3h5p_PropInstanceID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropCopyID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropCopyID", (PyObject *)&__pyx_type_4h5py_3h5p_PropCopyID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropCopyID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropCopyID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropCopyID, __pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropCopyID = &__pyx_type_4h5py_3h5p_PropCopyID;
  __pyx_type_4h5py_3h5p_PropOCID.tp_base = __pyx_ptype_4h5py_3h5p_PropCreateID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropOCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropOCID", (PyObject *)&__pyx_type_4h5py_3h5p_PropOCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropOCID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropOCID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropOCID, __pyx_base.__pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropOCID = &__pyx_type_4h5py_3h5p_PropOCID;
  __pyx_type_4h5py_3h5p_PropDCID.tp_base = __pyx_ptype_4h5py_3h5p_PropOCID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropDCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropDCID", (PyObject *)&__pyx_type_4h5py_3h5p_PropDCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 356; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropDCID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropDCID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropDCID, __pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropDCID = &__pyx_type_4h5py_3h5p_PropDCID;
  __pyx_type_4h5py_3h5p_PropFCID.tp_base = __pyx_ptype_4h5py_3h5p_PropCreateID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropFCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropFCID", (PyObject *)&__pyx_type_4h5py_3h5p_PropFCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropFCID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropFCID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropFCID, __pyx_base.__pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropFCID = &__pyx_type_4h5py_3h5p_PropFCID;
  __pyx_type_4h5py_3h5p_PropFAID.tp_base = __pyx_ptype_4h5py_3h5p_PropInstanceID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropFAID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropFAID", (PyObject *)&__pyx_type_4h5py_3h5p_PropFAID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropFAID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropFAID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropFAID, __pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropFAID = &__pyx_type_4h5py_3h5p_PropFAID;
  __pyx_type_4h5py_3h5p_PropDXID.tp_base = __pyx_ptype_4h5py_3h5p_PropInstanceID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropDXID) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropDXID", (PyObject *)&__pyx_type_4h5py_3h5p_PropDXID) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropDXID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropDXID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropDXID, __pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropDXID = &__pyx_type_4h5py_3h5p_PropDXID;
  __pyx_type_4h5py_3h5p_PropLCID.tp_base = __pyx_ptype_4h5py_3h5p_PropCreateID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropLCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropLCID", (PyObject *)&__pyx_type_4h5py_3h5p_PropLCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 991; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropLCID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropLCID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropLCID, __pyx_base.__pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropLCID = &__pyx_type_4h5py_3h5p_PropLCID;
  __pyx_type_4h5py_3h5p_PropLAID.tp_base = __pyx_ptype_4h5py_3h5p_PropInstanceID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropLAID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropLAID", (PyObject *)&__pyx_type_4h5py_3h5p_PropLAID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1035; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropLAID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropLAID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropLAID, __pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropLAID = &__pyx_type_4h5py_3h5p_PropLAID;
  __pyx_type_4h5py_3h5p_PropGCID.tp_base = __pyx_ptype_4h5py_3h5p_PropOCID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropGCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropGCID", (PyObject *)&__pyx_type_4h5py_3h5p_PropGCID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropGCID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropGCID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropGCID, __pyx_base.__pyx_base.__pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropGCID = &__pyx_type_4h5py_3h5p_PropGCID;
  __pyx_type_4h5py_3h5p_PropDAID.tp_base = __pyx_ptype_4h5py_3h5p_PropInstanceID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5p_PropDAID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "PropDAID", (PyObject *)&__pyx_type_4h5py_3h5p_PropDAID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5p_PropDAID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5p_PropDAID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5p_PropDAID, __pyx_base.__pyx_base.__pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5p_PropDAID = &__pyx_type_4h5py_3h5p_PropDAID;
  /*--- Type import code ---*/
  __pyx_ptype_4h5py_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_dtype)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_ndarray)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeID = __Pyx_ImportType("h5py.h5t", "TypeID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeArrayID = __Pyx_ImportType("h5py.h5t", "TypeArrayID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeArrayID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeArrayID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeArrayID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeArrayID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeArrayID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeArrayID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeOpaqueID = __Pyx_ImportType("h5py.h5t", "TypeOpaqueID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeOpaqueID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeOpaqueID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeOpaqueID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeOpaqueID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeOpaqueID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeOpaqueID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeStringID = __Pyx_ImportType("h5py.h5t", "TypeStringID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeStringID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeStringID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeStringID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeStringID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeStringID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeStringID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeVlenID = __Pyx_ImportType("h5py.h5t", "TypeVlenID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeVlenID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeVlenID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeVlenID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeVlenID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeVlenID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeVlenID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeTimeID = __Pyx_ImportType("h5py.h5t", "TypeTimeID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeTimeID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeTimeID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeTimeID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeTimeID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeTimeID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeTimeID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeBitfieldID = __Pyx_ImportType("h5py.h5t", "TypeBitfieldID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeBitfieldID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeBitfieldID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeBitfieldID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeBitfieldID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeBitfieldID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeBitfieldID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeReferenceID = __Pyx_ImportType("h5py.h5t", "TypeReferenceID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeReferenceID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeReferenceID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeReferenceID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeReferenceID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeReferenceID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeReferenceID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeAtomicID = __Pyx_ImportType("h5py.h5t", "TypeAtomicID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeAtomicID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeAtomicID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeAtomicID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeAtomicID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeAtomicID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeAtomicID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeIntegerID = __Pyx_ImportType("h5py.h5t", "TypeIntegerID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeIntegerID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeIntegerID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeIntegerID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeIntegerID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeIntegerID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeIntegerID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeFloatID = __Pyx_ImportType("h5py.h5t", "TypeFloatID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeFloatID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeFloatID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeFloatID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeFloatID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeFloatID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeFloatID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeCompositeID = __Pyx_ImportType("h5py.h5t", "TypeCompositeID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeCompositeID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeCompositeID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeCompositeID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeCompositeID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeCompositeID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeCompositeID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeEnumID = __Pyx_ImportType("h5py.h5t", "TypeEnumID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeEnumID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeEnumID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeEnumID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeEnumID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeEnumID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeEnumID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5t_TypeCompoundID = __Pyx_ImportType("h5py.h5t", "TypeCompoundID", sizeof(struct __pyx_obj_4h5py_3h5t_TypeCompoundID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5t_TypeCompoundID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_vtabptr_4h5py_3h5t_TypeCompoundID = (struct __pyx_vtabstruct_4h5py_3h5t_TypeCompoundID*)__Pyx_GetVtable(__pyx_ptype_4h5py_3h5t_TypeCompoundID->tp_dict); if (unlikely(!__pyx_vtabptr_4h5py_3h5t_TypeCompoundID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_4h5ac_CacheConfig = __Pyx_ImportType("h5py.h5ac", "CacheConfig", sizeof(struct __pyx_obj_4h5py_4h5ac_CacheConfig), 1); if (unlikely(!__pyx_ptype_4h5py_4h5ac_CacheConfig)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("h5py.defs"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Fget_mdc_config", (void (**)(void))&__pyx_f_4h5py_4defs_H5Fget_mdc_config, "herr_t (hid_t, H5AC_cache_config_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Fset_mdc_config", (void (**)(void))&__pyx_f_4h5py_4defs_H5Fset_mdc_config, "herr_t (hid_t, H5AC_cache_config_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Idec_ref", (void (**)(void))&__pyx_f_4h5py_4defs_H5Idec_ref, "int (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Iget_ref", (void (**)(void))&__pyx_f_4h5py_4defs_H5Iget_ref, "int (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pcreate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pcreate, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pcopy", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pcopy, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_class", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_class, "int (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pclose", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pclose, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pequal", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pequal, "htri_t (hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pclose_class", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pclose_class, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_version", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_version, "herr_t (hid_t, unsigned int *, unsigned int *, unsigned int *, unsigned int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_userblock", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_userblock, "herr_t (hid_t, hsize_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_userblock", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_userblock, "herr_t (hid_t, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_sizes", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_sizes, "herr_t (hid_t, size_t, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_sizes", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_sizes, "herr_t (hid_t, size_t *, size_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fclose_degree", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fclose_degree, "herr_t (hid_t, enum H5F_close_degree_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_fclose_degree", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_fclose_degree, "herr_t (hid_t, enum H5F_close_degree_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fapl_core", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fapl_core, "herr_t (hid_t, size_t, hbool_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_fapl_core", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_fapl_core, "herr_t (hid_t, size_t *, hbool_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fapl_family", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fapl_family, "herr_t (hid_t, hsize_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_fapl_family", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_fapl_family, "herr_t (hid_t, hsize_t *, hid_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fapl_log", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fapl_log, "herr_t (hid_t, char *, unsigned int, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_cache", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_cache, "herr_t (hid_t, int, int, size_t, double)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_cache", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_cache, "herr_t (hid_t, int *, int *, size_t *, double *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fapl_sec2", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fapl_sec2, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fapl_stdio", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fapl_stdio, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_driver", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_driver, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_layout", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_layout, "herr_t (hid_t, int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_layout", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_layout, "H5D_layout_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_chunk", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_chunk, "herr_t (hid_t, int, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_chunk", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_chunk, "int (hid_t, int, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_deflate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_deflate, "herr_t (hid_t, int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fill_value", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fill_value, "herr_t (hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_fill_value", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_fill_value, "herr_t (hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pfill_value_defined", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pfill_value_defined, "herr_t (hid_t, H5D_fill_value_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fill_time", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fill_time, "herr_t (hid_t, H5D_fill_time_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_fill_time", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_fill_time, "herr_t (hid_t, H5D_fill_time_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_alloc_time", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_alloc_time, "herr_t (hid_t, H5D_alloc_time_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_alloc_time", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_alloc_time, "herr_t (hid_t, H5D_alloc_time_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_filter", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_filter, "herr_t (hid_t, H5Z_filter_t, unsigned int, size_t, unsigned int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pall_filters_avail", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pall_filters_avail, "htri_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_nfilters", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_nfilters, "int (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_filter", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_filter, "H5Z_filter_t (hid_t, unsigned int, unsigned int *, size_t *, unsigned int *, size_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_filter_by_id", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_filter_by_id, "herr_t (hid_t, H5Z_filter_t, unsigned int *, size_t *, unsigned int *, size_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Premove_filter", (void (**)(void))&__pyx_f_4h5py_4defs_H5Premove_filter, "herr_t (hid_t, H5Z_filter_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_fletcher32", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_fletcher32, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_shuffle", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_shuffle, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_szip", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_szip, "herr_t (hid_t, unsigned int, unsigned int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_scaleoffset", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_scaleoffset, "herr_t (hid_t, enum H5Z_SO_scale_type_t, int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_chunk_cache", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_chunk_cache, "herr_t (hid_t, size_t, size_t, double)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_chunk_cache", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_chunk_cache, "herr_t (hid_t, size_t *, size_t *, double *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_sieve_buf_size", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_sieve_buf_size, "herr_t (hid_t, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_sieve_buf_size", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_sieve_buf_size, "herr_t (hid_t, size_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_nlinks", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_nlinks, "herr_t (hid_t, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_nlinks", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_nlinks, "herr_t (hid_t, size_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_elink_prefix", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_elink_prefix, "herr_t (hid_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_elink_prefix", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_elink_prefix, "Py_ssize_t (hid_t, char *, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_elink_fapl", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_elink_fapl, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_elink_fapl", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_elink_fapl, "herr_t (hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_create_intermediate_group", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_create_intermediate_group, "herr_t (hid_t, unsigned int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_create_intermediate_group", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_create_intermediate_group, "herr_t (hid_t, unsigned int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_copy_object", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_copy_object, "herr_t (hid_t, unsigned int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_copy_object", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_copy_object, "herr_t (hid_t, unsigned int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_char_encoding", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_char_encoding, "herr_t (hid_t, H5T_cset_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_char_encoding", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_char_encoding, "herr_t (hid_t, H5T_cset_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_obj_track_times", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_obj_track_times, "herr_t (hid_t, hbool_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_obj_track_times", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_obj_track_times, "herr_t (hid_t, hbool_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_link_creation_order", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_link_creation_order, "herr_t (hid_t, unsigned int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_link_creation_order", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_link_creation_order, "herr_t (hid_t, unsigned int *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pset_libver_bounds", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pset_libver_bounds, "herr_t (hid_t, H5F_libver_t, H5F_libver_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Pget_libver_bounds", (void (**)(void))&__pyx_f_4h5py_4defs_H5Pget_libver_bounds, "herr_t (hid_t, H5F_libver_t *, H5F_libver_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_ImportModule("h5py.utils"); if (!__pyx_t_2) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "emalloc", (void (**)(void))&__pyx_f_4h5py_5utils_emalloc, "void *(size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "efree", (void (**)(void))&__pyx_f_4h5py_5utils_efree, "void (void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "check_numpy_read", (void (**)(void))&__pyx_f_4h5py_5utils_check_numpy_read, "int (PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4h5py_5utils_check_numpy_read *__pyx_optional_args)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "check_numpy_write", (void (**)(void))&__pyx_f_4h5py_5utils_check_numpy_write, "int (PyArrayObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4h5py_5utils_check_numpy_write *__pyx_optional_args)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "convert_tuple", (void (**)(void))&__pyx_f_4h5py_5utils_convert_tuple, "int (PyObject *, hsize_t *, hsize_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "convert_dims", (void (**)(void))&__pyx_f_4h5py_5utils_convert_dims, "PyObject *(hsize_t *, hsize_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "require_tuple", (void (**)(void))&__pyx_f_4h5py_5utils_require_tuple, "int (PyObject *, int, int, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_ImportModule("h5py.h5t"); if (!__pyx_t_3) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_3, "py_create", (void (**)(void))&__pyx_f_4h5py_3h5t_py_create, "struct __pyx_obj_4h5py_3h5t_TypeID *(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4h5py_3h5t_py_create *__pyx_optional_args)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  /*--- Execution code ---*/

  /* "h5py/h5p.pyx":23
 * from h5t cimport TypeID, py_create
 * from h5ac cimport CacheConfig
 * from h5py import _objects             # <<<<<<<<<<<<<<
 * 
 * # Initialization
 */
  __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_n_s___objects));
  PyList_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_n_s___objects));
  __Pyx_GIVEREF(((PyObject *)__pyx_n_s___objects));
  __pyx_t_5 = __Pyx_Import(((PyObject *)__pyx_n_s__h5py), ((PyObject *)__pyx_t_4), -1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_GetAttr(__pyx_t_5, __pyx_n_s___objects);
  if (__pyx_t_4 == NULL) {
    if (PyErr_ExceptionMatches(PyExc_AttributeError)) __Pyx_RaiseImportError(__pyx_n_s___objects);
    if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s___objects, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":26
 * 
 * # Initialization
 * import_array()             # <<<<<<<<<<<<<<
 * 
 * # === C API ===================================================================
 */
  import_array();

  /* "h5py/h5p.pyx":81
 * 
 * 
 * NO_CLASS       = lockcls(H5P_NO_CLASS)             # <<<<<<<<<<<<<<
 * FILE_CREATE    = lockcls(H5P_FILE_CREATE)
 * FILE_ACCESS    = lockcls(H5P_FILE_ACCESS)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_NO_CLASS); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 81; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__NO_CLASS, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 81; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":82
 * 
 * NO_CLASS       = lockcls(H5P_NO_CLASS)
 * FILE_CREATE    = lockcls(H5P_FILE_CREATE)             # <<<<<<<<<<<<<<
 * FILE_ACCESS    = lockcls(H5P_FILE_ACCESS)
 * DATASET_CREATE = lockcls(H5P_DATASET_CREATE)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_FILE_CREATE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 82; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__FILE_CREATE, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 82; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":83
 * NO_CLASS       = lockcls(H5P_NO_CLASS)
 * FILE_CREATE    = lockcls(H5P_FILE_CREATE)
 * FILE_ACCESS    = lockcls(H5P_FILE_ACCESS)             # <<<<<<<<<<<<<<
 * DATASET_CREATE = lockcls(H5P_DATASET_CREATE)
 * DATASET_XFER   = lockcls(H5P_DATASET_XFER)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_FILE_ACCESS); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 83; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__FILE_ACCESS, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 83; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":84
 * FILE_CREATE    = lockcls(H5P_FILE_CREATE)
 * FILE_ACCESS    = lockcls(H5P_FILE_ACCESS)
 * DATASET_CREATE = lockcls(H5P_DATASET_CREATE)             # <<<<<<<<<<<<<<
 * DATASET_XFER   = lockcls(H5P_DATASET_XFER)
 * DATASET_ACCESS = lockcls(H5P_DATASET_ACCESS)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_DATASET_CREATE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__DATASET_CREATE, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":85
 * FILE_ACCESS    = lockcls(H5P_FILE_ACCESS)
 * DATASET_CREATE = lockcls(H5P_DATASET_CREATE)
 * DATASET_XFER   = lockcls(H5P_DATASET_XFER)             # <<<<<<<<<<<<<<
 * DATASET_ACCESS = lockcls(H5P_DATASET_ACCESS)
 * 
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_DATASET_XFER); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__DATASET_XFER, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":86
 * DATASET_CREATE = lockcls(H5P_DATASET_CREATE)
 * DATASET_XFER   = lockcls(H5P_DATASET_XFER)
 * DATASET_ACCESS = lockcls(H5P_DATASET_ACCESS)             # <<<<<<<<<<<<<<
 * 
 * OBJECT_COPY = lockcls(H5P_OBJECT_COPY)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_DATASET_ACCESS); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 86; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__DATASET_ACCESS, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 86; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":88
 * DATASET_ACCESS = lockcls(H5P_DATASET_ACCESS)
 * 
 * OBJECT_COPY = lockcls(H5P_OBJECT_COPY)             # <<<<<<<<<<<<<<
 * 
 * LINK_CREATE = lockcls(H5P_LINK_CREATE)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_OBJECT_COPY); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__OBJECT_COPY, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":90
 * OBJECT_COPY = lockcls(H5P_OBJECT_COPY)
 * 
 * LINK_CREATE = lockcls(H5P_LINK_CREATE)             # <<<<<<<<<<<<<<
 * LINK_ACCESS = lockcls(H5P_LINK_ACCESS)
 * GROUP_CREATE = lockcls(H5P_GROUP_CREATE)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_LINK_CREATE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__LINK_CREATE, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":91
 * 
 * LINK_CREATE = lockcls(H5P_LINK_CREATE)
 * LINK_ACCESS = lockcls(H5P_LINK_ACCESS)             # <<<<<<<<<<<<<<
 * GROUP_CREATE = lockcls(H5P_GROUP_CREATE)
 * OBJECT_CREATE = lockcls(H5P_OBJECT_CREATE)
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_LINK_ACCESS); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__LINK_ACCESS, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":92
 * LINK_CREATE = lockcls(H5P_LINK_CREATE)
 * LINK_ACCESS = lockcls(H5P_LINK_ACCESS)
 * GROUP_CREATE = lockcls(H5P_GROUP_CREATE)             # <<<<<<<<<<<<<<
 * OBJECT_CREATE = lockcls(H5P_OBJECT_CREATE)
 * 
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_GROUP_CREATE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 92; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__GROUP_CREATE, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 92; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":93
 * LINK_ACCESS = lockcls(H5P_LINK_ACCESS)
 * GROUP_CREATE = lockcls(H5P_GROUP_CREATE)
 * OBJECT_CREATE = lockcls(H5P_OBJECT_CREATE)             # <<<<<<<<<<<<<<
 * 
 * CRT_ORDER_TRACKED = H5P_CRT_ORDER_TRACKED
 */
  __pyx_t_5 = __pyx_f_4h5py_3h5p_lockcls(H5P_OBJECT_CREATE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__OBJECT_CREATE, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":95
 * OBJECT_CREATE = lockcls(H5P_OBJECT_CREATE)
 * 
 * CRT_ORDER_TRACKED = H5P_CRT_ORDER_TRACKED             # <<<<<<<<<<<<<<
 * CRT_ORDER_INDEXED = H5P_CRT_ORDER_INDEXED
 * 
 */
  __pyx_t_5 = __Pyx_PyInt_to_py_hid_t(H5P_CRT_ORDER_TRACKED); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__CRT_ORDER_TRACKED, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":96
 * 
 * CRT_ORDER_TRACKED = H5P_CRT_ORDER_TRACKED
 * CRT_ORDER_INDEXED = H5P_CRT_ORDER_INDEXED             # <<<<<<<<<<<<<<
 * 
 * DEFAULT = None   # In the HDF5 header files this is actually 0, which is an
 */
  __pyx_t_5 = __Pyx_PyInt_to_py_hid_t(H5P_CRT_ORDER_INDEXED); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__CRT_ORDER_INDEXED, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":98
 * CRT_ORDER_INDEXED = H5P_CRT_ORDER_INDEXED
 * 
 * DEFAULT = None   # In the HDF5 header files this is actually 0, which is an             # <<<<<<<<<<<<<<
 *                  # invalid identifier.  The new strategy for default options
 *                  # is to make them all None, to better match the Python style
 */
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__DEFAULT, Py_None) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 98; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5p.pyx":106
 * # === Property list functional API ============================================
 * 
 * def create(PropClassID cls not None):             # <<<<<<<<<<<<<<
 *     """(PropClassID cls) => PropID
 * 
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5p_1create, NULL, __pyx_n_s_14); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__create, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 106; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5p.pyx":1
 * # This file is part of h5py, a Python interface to the HDF5 library.             # <<<<<<<<<<<<<<
 * #
 * # http://www.h5py.org
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_5));
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s____test__, ((PyObject *)__pyx_t_5)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    __Pyx_AddTraceback("init h5py.h5p", __pyx_clineno, __pyx_lineno, __pyx_filename);
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init h5py.h5p");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif /* CYTHON_REFNANNY */

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name) {
    PyObject *result;
    result = PyObject_GetAttr(dict, name);
    if (!result) {
        if (dict != __pyx_b) {
            PyErr_Clear();
            result = PyObject_GetAttr(__pyx_b, name);
        }
        if (!result) {
            PyErr_SetObject(PyExc_NameError, name);
        }
    }
    return result;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    #if PY_VERSION_HEX < 0x02050000
    if (PyClass_Check(type)) {
    #else
    if (PyType_Check(type)) {
    #endif
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        #if PY_VERSION_HEX < 0x02050000
            if (PyInstance_Check(type)) {
                type = (PyObject*) ((PyInstanceObject*)type)->in_class;
                Py_INCREF(type);
            }
            else {
                type = 0;
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception must be an old-style class or instance");
                goto raise_error;
            }
        #else
            type = (PyObject*) Py_TYPE(type);
            Py_INCREF(type);
            if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception class must be a subclass of BaseException");
                goto raise_error;
            }
        #endif
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else /* Python 3+ */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *args;
        if (!value)
            args = PyTuple_New(0);
        else if (PyTuple_Check(value)) {
            Py_INCREF(value);
            args = value;
        }
        else
            args = PyTuple_Pack(1, value);
        if (!args)
            goto bad;
        owned_instance = PyEval_CallObject(type, args);
        Py_DECREF(args);
        if (!owned_instance)
            goto bad;
        value = owned_instance;
        if (!PyExceptionInstance_Check(value)) {
            PyErr_Format(PyExc_TypeError,
                         "calling %R should have returned an instance of "
                         "BaseException, not %R",
                         type, Py_TYPE(value));
            goto bad;
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause && cause != Py_None) {
        PyObject *fixed_cause;
        if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        }
        else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        }
        else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (!type) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (Py_TYPE(obj) == type) return 1;
    }
    else {
        if (PyObject_TypeCheck(obj, type)) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%s' has incorrect type (expected %s, got %s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
        goto bad;
    #endif
    Py_INCREF(local_type);
    Py_INCREF(local_value);
    Py_INCREF(local_tb);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    /* Make sure tstate is in a consistent state when we XDECREF
       these objects (DECREF may run arbitrary code). */
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%s() takes %s %" CYTHON_FORMAT_SSIZE_T "d positional argument%s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%s() got an unexpected keyword argument '%s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CPYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%s() got an unexpected keyword argument '%s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import = 0;
    py_import = __Pyx_GetAttrString(__pyx_b, "__import__");
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    #if PY_VERSION_HEX >= 0x02050000
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0; /* try absolute import on failure */
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
    #else
    if (level>0) {
        PyErr_SetString(PyExc_RuntimeError, "Relative import is not supported for Python <=2.4.");
        goto bad;
    }
    module = PyObject_CallFunctionObjArgs(py_import,
        name, global_dict, empty_dict, list, NULL);
    #endif
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static CYTHON_INLINE void __Pyx_RaiseImportError(PyObject *name) {
#if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_ImportError, "cannot import name %.230s",
                 PyString_AsString(name));
#else
    PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
#endif
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t val) {
    const hid_t neg_one = (hid_t)-1, const_zero = (hid_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hid_t) == sizeof(char))  ||
        (sizeof(hid_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hid_t) == sizeof(int)) ||
               (sizeof(hid_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hid_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE hsize_t __Pyx_PyInt_from_py_hsize_t(PyObject* x) {
    const hsize_t neg_one = (hsize_t)-1, const_zero = (hsize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if (sizeof(hsize_t) == sizeof(char)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedChar(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedChar(x);
    } else if (sizeof(hsize_t) == sizeof(short)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedShort(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedShort(x);
    } else if (sizeof(hsize_t) == sizeof(int)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedInt(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedInt(x);
    } else if (sizeof(hsize_t) == sizeof(long)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedLong(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedLong(x);
    } else if (sizeof(hsize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedLongLong(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedLongLong(x);
    }  else {
        #if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
        #else
        hsize_t val;
        PyObject *v = __Pyx_PyNumber_Int(x);
        #if PY_VERSION_HEX < 0x03000000
        if (likely(v) && !PyLong_Check(v)) {
            PyObject *tmp = v;
            v = PyNumber_Long(tmp);
            Py_DECREF(tmp);
        }
        #endif
        if (likely(v)) {
            int one = 1; int is_little = (int)*(unsigned char *)&one;
            unsigned char *bytes = (unsigned char *)&val;
            int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                          bytes, sizeof(val),
                                          is_little, !is_unsigned);
            Py_DECREF(v);
            if (likely(!ret))
                return val;
        }
        #endif
        return (hsize_t)-1;
    }
}

static CYTHON_INLINE hbool_t __Pyx_PyInt_from_py_hbool_t(PyObject* x) {
    const hbool_t neg_one = (hbool_t)-1, const_zero = (hbool_t)0;
    const int is_unsigned = const_zero < neg_one;
    if (sizeof(hbool_t) == sizeof(char)) {
        if (is_unsigned)
            return (hbool_t)__Pyx_PyInt_AsUnsignedChar(x);
        else
            return (hbool_t)__Pyx_PyInt_AsSignedChar(x);
    } else if (sizeof(hbool_t) == sizeof(short)) {
        if (is_unsigned)
            return (hbool_t)__Pyx_PyInt_AsUnsignedShort(x);
        else
            return (hbool_t)__Pyx_PyInt_AsSignedShort(x);
    } else if (sizeof(hbool_t) == sizeof(int)) {
        if (is_unsigned)
            return (hbool_t)__Pyx_PyInt_AsUnsignedInt(x);
        else
            return (hbool_t)__Pyx_PyInt_AsSignedInt(x);
    } else if (sizeof(hbool_t) == sizeof(long)) {
        if (is_unsigned)
            return (hbool_t)__Pyx_PyInt_AsUnsignedLong(x);
        else
            return (hbool_t)__Pyx_PyInt_AsSignedLong(x);
    } else if (sizeof(hbool_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return (hbool_t)__Pyx_PyInt_AsUnsignedLongLong(x);
        else
            return (hbool_t)__Pyx_PyInt_AsSignedLongLong(x);
    }  else {
        #if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
        #else
        hbool_t val;
        PyObject *v = __Pyx_PyNumber_Int(x);
        #if PY_VERSION_HEX < 0x03000000
        if (likely(v) && !PyLong_Check(v)) {
            PyObject *tmp = v;
            v = PyNumber_Long(tmp);
            Py_DECREF(tmp);
        }
        #endif
        if (likely(v)) {
            int one = 1; int is_little = (int)*(unsigned char *)&one;
            unsigned char *bytes = (unsigned char *)&val;
            int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                          bytes, sizeof(val),
                                          is_little, !is_unsigned);
            Py_DECREF(v);
            if (likely(!ret))
                return val;
        }
        #endif
        return (hbool_t)-1;
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t val) {
    const hsize_t neg_one = (hsize_t)-1, const_zero = (hsize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hsize_t) == sizeof(char))  ||
        (sizeof(hsize_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hsize_t) == sizeof(int)) ||
               (sizeof(hsize_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hsize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hsize_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hbool_t(hbool_t val) {
    const hbool_t neg_one = (hbool_t)-1, const_zero = (hbool_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hbool_t) == sizeof(char))  ||
        (sizeof(hbool_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hbool_t) == sizeof(int)) ||
               (sizeof(hbool_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hbool_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hbool_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject* x) {
    const unsigned char neg_one = (unsigned char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned char" :
                    "value too large to convert to unsigned char");
            }
            return (unsigned char)-1;
        }
        return (unsigned char)val;
    }
    return (unsigned char)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject* x) {
    const unsigned short neg_one = (unsigned short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned short" :
                    "value too large to convert to unsigned short");
            }
            return (unsigned short)-1;
        }
        return (unsigned short)val;
    }
    return (unsigned short)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject* x) {
    const unsigned int neg_one = (unsigned int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned int" :
                    "value too large to convert to unsigned int");
            }
            return (unsigned int)-1;
        }
        return (unsigned int)val;
    }
    return (unsigned int)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject* x) {
    const char neg_one = (char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to char" :
                    "value too large to convert to char");
            }
            return (char)-1;
        }
        return (char)val;
    }
    return (char)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject* x) {
    const short neg_one = (short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to short" :
                    "value too large to convert to short");
            }
            return (short)-1;
        }
        return (short)val;
    }
    return (short)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject* x) {
    const signed char neg_one = (signed char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed char" :
                    "value too large to convert to signed char");
            }
            return (signed char)-1;
        }
        return (signed char)val;
    }
    return (signed char)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject* x) {
    const signed short neg_one = (signed short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed short" :
                    "value too large to convert to signed short");
            }
            return (signed short)-1;
        }
        return (signed short)val;
    }
    return (signed short)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject* x) {
    const signed int neg_one = (signed int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed int" :
                    "value too large to convert to signed int");
            }
            return (signed int)-1;
        }
        return (signed int)val;
    }
    return (signed int)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject* x) {
    const unsigned long neg_one = (unsigned long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned long");
            return (unsigned long)-1;
        }
        return (unsigned long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned long");
                return (unsigned long)-1;
            }
            return (unsigned long)PyLong_AsUnsignedLong(x);
        } else {
            return (unsigned long)PyLong_AsLong(x);
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned long)-1;
        val = __Pyx_PyInt_AsUnsignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject* x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned PY_LONG_LONG");
            return (unsigned PY_LONG_LONG)-1;
        }
        return (unsigned PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned PY_LONG_LONG");
                return (unsigned PY_LONG_LONG)-1;
            }
            return (unsigned PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (unsigned PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsUnsignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject* x) {
    const long neg_one = (long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to long");
            return (long)-1;
        }
        return (long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to long");
                return (long)-1;
            }
            return (long)PyLong_AsUnsignedLong(x);
        } else {
            return (long)PyLong_AsLong(x);
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long)-1;
        val = __Pyx_PyInt_AsLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject* x) {
    const PY_LONG_LONG neg_one = (PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to PY_LONG_LONG");
            return (PY_LONG_LONG)-1;
        }
        return (PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to PY_LONG_LONG");
                return (PY_LONG_LONG)-1;
            }
            return (PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject* x) {
    const signed long neg_one = (signed long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed long");
            return (signed long)-1;
        }
        return (signed long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed long");
                return (signed long)-1;
            }
            return (signed long)PyLong_AsUnsignedLong(x);
        } else {
            return (signed long)PyLong_AsLong(x);
        }
    } else {
        signed long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed long)-1;
        val = __Pyx_PyInt_AsSignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject* x) {
    const signed PY_LONG_LONG neg_one = (signed PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed PY_LONG_LONG");
            return (signed PY_LONG_LONG)-1;
        }
        return (signed PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed PY_LONG_LONG");
                return (signed PY_LONG_LONG)-1;
            }
            return (signed PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (signed PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        signed PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsSignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        #if PY_VERSION_HEX < 0x02050000
        return PyErr_Warn(NULL, message);
        #else
        return PyErr_WarnEx(NULL, message, 1);
        #endif
    }
    return 0;
}

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3&&PY_MINOR_VERSION==0)
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%s.%s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        #if PY_VERSION_HEX < 0x02050000
        if (PyErr_Warn(NULL, warning) < 0) goto bad;
        #else
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
        #endif
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%s.%s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyMapping_GetItemString(dict, (char *)"__pyx_vtable__");
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3&&PY_MINOR_VERSION==0)
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%s does not export expected C function %s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3 && PY_MINOR_VERSION==0)
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,            /*int argcount,*/
        0,            /*int kwonlyargcount,*/
        0,            /*int nlocals,*/
        0,            /*int stacksize,*/
        0,            /*int flags,*/
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,      /*int firstlineno,*/
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_globals = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_globals = PyModule_GetDict(__pyx_m);
    if (!py_globals) goto bad;
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        py_globals,          /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else  /* Python 3+ has unicode identifiers */
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_VERSION_HEX < 0x03000000
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_VERSION_HEX < 0x03000000
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%s__ returned non-%s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject* x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
#if PY_VERSION_HEX < 0x02050000
   if (ival <= LONG_MAX)
       return PyInt_FromLong((long)ival);
   else {
       unsigned char *bytes = (unsigned char *) &ival;
       int one = 1; int little = (int)*(unsigned char*)&one;
       return _PyLong_FromByteArray(bytes, sizeof(size_t), little, 0);
   }
#else
   return PyInt_FromSize_t(ival);
#endif
}
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject* x) {
   unsigned PY_LONG_LONG val = __Pyx_PyInt_AsUnsignedLongLong(x);
   if (unlikely(val == (unsigned PY_LONG_LONG)-1 && PyErr_Occurred())) {
       return (size_t)-1;
   } else if (unlikely(val != (unsigned PY_LONG_LONG)(size_t)val)) {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to size_t");
       return (size_t)-1;
   }
   return (size_t)val;
}


#endif /* Py_PYTHON_H */
