/* Generated by Cython 0.18 on Mon Dec  9 16:00:33 2013 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02040000
    #error Cython requires Python 2.4+.
#else
#include <stddef.h> /* For offsetof */
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if PY_VERSION_HEX < 0x02050000
  typedef int Py_ssize_t;
  #define PY_SSIZE_T_MAX INT_MAX
  #define PY_SSIZE_T_MIN INT_MIN
  #define PY_FORMAT_SIZE_T ""
  #define CYTHON_FORMAT_SSIZE_T ""
  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
  #define PyInt_AsSsize_t(o)   __Pyx_PyInt_AsInt(o)
  #define PyNumber_Index(o)    ((PyNumber_Check(o) && !PyFloat_Check(o)) ? PyNumber_Int(o) : \
                                (PyErr_Format(PyExc_TypeError, \
                                              "expected index value, got %.200s", Py_TYPE(o)->tp_name), \
                                 (PyObject*)0))
  #define __Pyx_PyIndex_Check(o) (PyNumber_Check(o) && !PyFloat_Check(o) && \
                                  !PyComplex_Check(o))
  #define PyIndex_Check __Pyx_PyIndex_Check
  #define PyErr_WarnEx(category, message, stacklevel) PyErr_Warn(category, message)
  #define __PYX_BUILD_PY_SSIZE_T "i"
#else
  #define __PYX_BUILD_PY_SSIZE_T "n"
  #define CYTHON_FORMAT_SSIZE_T "z"
  #define __Pyx_PyIndex_Check PyIndex_Check
#endif
#if PY_VERSION_HEX < 0x02060000
  #define Py_REFCNT(ob) (((PyObject*)(ob))->ob_refcnt)
  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
  #define Py_SIZE(ob)   (((PyVarObject*)(ob))->ob_size)
  #define PyVarObject_HEAD_INIT(type, size) \
          PyObject_HEAD_INIT(type) size,
  #define PyType_Modified(t)
  typedef struct {
     void *buf;
     PyObject *obj;
     Py_ssize_t len;
     Py_ssize_t itemsize;
     int readonly;
     int ndim;
     char *format;
     Py_ssize_t *shape;
     Py_ssize_t *strides;
     Py_ssize_t *suboffsets;
     void *internal;
  } Py_buffer;
  #define PyBUF_SIMPLE 0
  #define PyBUF_WRITABLE 0x0001
  #define PyBUF_FORMAT 0x0004
  #define PyBUF_ND 0x0008
  #define PyBUF_STRIDES (0x0010 | PyBUF_ND)
  #define PyBUF_C_CONTIGUOUS (0x0020 | PyBUF_STRIDES)
  #define PyBUF_F_CONTIGUOUS (0x0040 | PyBUF_STRIDES)
  #define PyBUF_ANY_CONTIGUOUS (0x0080 | PyBUF_STRIDES)
  #define PyBUF_INDIRECT (0x0100 | PyBUF_STRIDES)
  #define PyBUF_RECORDS (PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_WRITABLE)
  #define PyBUF_FULL (PyBUF_INDIRECT | PyBUF_FORMAT | PyBUF_WRITABLE)
  typedef int (*getbufferproc)(PyObject *, Py_buffer *, int);
  typedef void (*releasebufferproc)(PyObject *, Py_buffer *);
#endif
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
  #define PyUnicode_FromString(s) PyUnicode_Decode(s, strlen(s), "UTF-8", "strict")
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#if (PY_VERSION_HEX < 0x02060000) || (PY_MAJOR_VERSION >= 3)
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((k=k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PyBytesObject                PyStringObject
  #define PyBytes_Type                 PyString_Type
  #define PyBytes_Check                PyString_Check
  #define PyBytes_CheckExact           PyString_CheckExact
  #define PyBytes_FromString           PyString_FromString
  #define PyBytes_FromStringAndSize    PyString_FromStringAndSize
  #define PyBytes_FromFormat           PyString_FromFormat
  #define PyBytes_DecodeEscape         PyString_DecodeEscape
  #define PyBytes_AsString             PyString_AsString
  #define PyBytes_AsStringAndSize      PyString_AsStringAndSize
  #define PyBytes_Size                 PyString_Size
  #define PyBytes_AS_STRING            PyString_AS_STRING
  #define PyBytes_GET_SIZE             PyString_GET_SIZE
  #define PyBytes_Repr                 PyString_Repr
  #define PyBytes_Concat               PyString_Concat
  #define PyBytes_ConcatAndDel         PyString_ConcatAndDel
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PySet_Check(obj)             PyObject_TypeCheck(obj, &PySet_Type)
  #define PyFrozenSet_Check(obj)       PyObject_TypeCheck(obj, &PyFrozenSet_Type)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_VERSION_HEX < 0x03020000
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if (PY_MAJOR_VERSION < 3) || (PY_VERSION_HEX >= 0x03010300)
  #define __Pyx_PySequence_GetSlice(obj, a, b) PySequence_GetSlice(obj, a, b)
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) PySequence_SetSlice(obj, a, b, value)
  #define __Pyx_PySequence_DelSlice(obj, a, b) PySequence_DelSlice(obj, a, b)
#else
  #define __Pyx_PySequence_GetSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), (PyObject*)0) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_GetSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object is unsliceable", (obj)->ob_type->tp_name), (PyObject*)0)))
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_SetSlice(obj, a, b, value)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice assignment", (obj)->ob_type->tp_name), -1)))
  #define __Pyx_PySequence_DelSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_DelSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice deletion", (obj)->ob_type->tp_name), -1)))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),((char *)(n)))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),((char *)(n)),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),((char *)(n)))
#else
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),(n))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),(n),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),(n))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_NAMESTR(n) ((char *)(n))
  #define __Pyx_DOCSTR(n)  ((char *)(n))
#else
  #define __Pyx_NAMESTR(n) (n)
  #define __Pyx_DOCSTR(n)  (n)
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__h5py___conv
#define __PYX_HAVE_API__h5py___conv
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "unistd.h"
#include "stdint.h"
#include "api_compat.h"
#include "lzf_filter.h"
#include "hdf5.h"
#include "H5Cpublic.h"
#include "hdf5_hl.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const long n; const char* encoding; const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry; /*proto*/

#define __Pyx_PyBytes_FromUString(s) PyBytes_FromString((char*)s)
#define __Pyx_PyBytes_AsUString(s)   ((unsigned char*) PyBytes_AsString(s))
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject*);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))


#ifdef __GNUC__
  /* Test for GCC > 2.95 */
  #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95))
    #define likely(x)   __builtin_expect(!!(x), 1)
    #define unlikely(x) __builtin_expect(!!(x), 0)
  #else /* __GNUC__ > 2 ... */
    #define likely(x)   (x)
    #define unlikely(x) (x)
  #endif /* __GNUC__ > 2 ... */
#else /* __GNUC__ */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
    
static PyObject *__pyx_m;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "_conv.pyx",
  "h5r.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_4h5py_3h5r_Reference;
struct __pyx_obj_4h5py_3h5r_RegionReference;
union __pyx_t_4h5py_3h5r_ref_u;

/* "h5r.pxd":17
 *   ctypedef unsigned char hdset_reg_ref_t[12]
 * 
 * cdef union ref_u:             # <<<<<<<<<<<<<<
 *     hobj_ref_t         obj_ref
 *     hdset_reg_ref_t    reg_ref
 */
union __pyx_t_4h5py_3h5r_ref_u {
  hobj_ref_t obj_ref;
  hdset_reg_ref_t reg_ref;
};
struct __pyx_t_4h5py_5_conv_conv_size_t;
typedef struct __pyx_t_4h5py_5_conv_conv_size_t __pyx_t_4h5py_5_conv_conv_size_t;
struct __pyx_t_4h5py_5_conv_conv_enum_t;

/* "h5py/_conv.pyx":60
 *     return H5PY_OBJ
 * 
 * ctypedef int (*conv_operator_t)(void* ipt, void* opt, void* bkg, void* priv) except -1             # <<<<<<<<<<<<<<
 * ctypedef herr_t (*init_operator_t)(hid_t src, hid_t dst, void** priv) except -1
 * 
 */
typedef int (*__pyx_t_4h5py_5_conv_conv_operator_t)(void *, void *, void *, void *);

/* "h5py/_conv.pyx":61
 * 
 * ctypedef int (*conv_operator_t)(void* ipt, void* opt, void* bkg, void* priv) except -1
 * ctypedef herr_t (*init_operator_t)(hid_t src, hid_t dst, void** priv) except -1             # <<<<<<<<<<<<<<
 * 
 * # Generic conversion callback
 */
typedef herr_t (*__pyx_t_4h5py_5_conv_init_operator_t)(hid_t, hid_t, void **);

/* "h5py/_conv.pyx":140
 * # Generic conversion
 * 
 * ctypedef struct conv_size_t:             # <<<<<<<<<<<<<<
 *     size_t src_size
 *     size_t dst_size
 */
struct __pyx_t_4h5py_5_conv_conv_size_t {
  size_t src_size;
  size_t dst_size;
  int cset;
};

/* "h5py/_conv.pyx":468
 * # Enum to integer converter
 * 
 * cdef struct conv_enum_t:             # <<<<<<<<<<<<<<
 *     size_t src_size
 *     size_t dst_size
 */
struct __pyx_t_4h5py_5_conv_conv_enum_t {
  size_t src_size;
  size_t dst_size;
  hid_t supertype;
  int identical;
};

/* "h5r.pxd":21
 *     hdset_reg_ref_t    reg_ref
 * 
 * cdef class Reference:             # <<<<<<<<<<<<<<
 * 
 *     cdef ref_u ref
 */
struct __pyx_obj_4h5py_3h5r_Reference {
  PyObject_HEAD
  union __pyx_t_4h5py_3h5r_ref_u ref;
  int typecode;
  size_t typesize;
};


/* "h5r.pxd":27
 *     cdef readonly size_t typesize
 * 
 * cdef class RegionReference(Reference):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5r_RegionReference {
  struct __pyx_obj_4h5py_3h5r_Reference __pyx_base;
};

#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname); /*proto*/
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif /* CYTHON_REFNANNY */
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name); /*proto*/

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb); /*proto*/
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause); /*proto*/

#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb); /*proto*/
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb); /*proto*/

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_htri_t(htri_t);

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject *);

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject *);

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject *);

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject *);

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject *);

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject *);

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject *);

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject *);

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject *);

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject *);

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject *);

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject *);

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename); /*proto*/

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name); /*proto*/

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);  /*proto*/

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig); /*proto*/

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename); /*proto*/

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t); /*proto*/


/* Module declarations from 'h5py.api_types_ext' */

/* Module declarations from 'h5py.api_types_hdf5' */

/* Module declarations from 'h5py.defs' */
static hid_t (*__pyx_f_4h5py_4defs_H5Tcreate)(enum H5T_class_t, size_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Tcopy)(hid_t); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Tequal)(hid_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tlock)(hid_t); /*proto*/
static size_t (*__pyx_f_4h5py_4defs_H5Tget_size)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Tget_super)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tclose)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tconvert)(hid_t, hid_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tset_size)(hid_t, size_t); /*proto*/
static H5T_cset_t (*__pyx_f_4h5py_4defs_H5Tget_cset)(hid_t); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Tis_variable_str)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Tenum_create)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tset_tag)(hid_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tregister)(H5T_pers_t, char *, hid_t, hid_t, H5T_conv_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tunregister)(H5T_pers_t, char *, hid_t, hid_t, H5T_conv_t); /*proto*/

/* Module declarations from 'h5py.h5r' */
static PyTypeObject *__pyx_ptype_4h5py_3h5r_Reference = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5r_RegionReference = 0;

/* Module declarations from 'h5py._conv' */
static hid_t __pyx_v_4h5py_5_conv_H5PY_OBJ;
static hid_t __pyx_f_4h5py_5_conv_get_python_obj(int __pyx_skip_dispatch); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_generic_converter(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t, __pyx_t_4h5py_5_conv_conv_operator_t, __pyx_t_4h5py_5_conv_init_operator_t, H5T_bkg_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_init_generic(hid_t, hid_t, void **); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_vlen2str(void *, void *, void *, void *); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_str2vlen(void *, void *, void *, void *); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_init_vlen2fixed(hid_t, hid_t, void **); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_init_fixed2vlen(hid_t, hid_t, void **); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_vlen2fixed(void *, void *, void *, void *); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_fixed2vlen(void *, void *, void *, void *); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_objref2pyref(void *, void *, void *, void *); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_pyref2objref(void *, void *, void *, void *); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_regref2pyref(void *, void *, void *, void *); /*proto*/
static int __pyx_f_4h5py_5_conv_conv_pyref2regref(void *, void *, void *, void *); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_vlen2str(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_str2vlen(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_vlen2fixed(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_fixed2vlen(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_objref2pyref(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_pyref2objref(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_regref2pyref(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_pyref2regref(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static int __pyx_f_4h5py_5_conv_enum_int_converter_init(hid_t, hid_t, H5T_cdata_t *, int); /*proto*/
static void __pyx_f_4h5py_5_conv_enum_int_converter_free(H5T_cdata_t *); /*proto*/
static int __pyx_f_4h5py_5_conv_enum_int_converter_conv(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t, int); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_enum_int_converter(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t, int); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_enum2int(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static herr_t __pyx_f_4h5py_5_conv_int2enum(hid_t, hid_t, H5T_cdata_t *, size_t, size_t, size_t, void *, void *, hid_t); /*proto*/
static int __pyx_f_4h5py_5_conv_register_converters(int __pyx_skip_dispatch); /*proto*/
static int __pyx_f_4h5py_5_conv_unregister_converters(int __pyx_skip_dispatch); /*proto*/
#define __Pyx_MODULE_NAME "h5py._conv"
int __pyx_module_is_main_h5py___conv = 0;

/* Implementation of 'h5py._conv' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_pf_4h5py_5_conv_get_python_obj(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4h5py_5_conv_2register_converters(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4h5py_5_conv_4unregister_converters(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static char __pyx_k_1[] = "PYTHON:OBJECT";
static char __pyx_k_2[] = "";
static char __pyx_k_3[] = "Byte string is not valid utf-8 and can't be stored in a utf-8 dataset";
static char __pyx_k_5[] = "Unrecognized dataset encoding";
static char __pyx_k_7[] = "VLEN strings do not support embedded NULLs";
static char __pyx_k_9[] = "Can't convert incompatible object to HDF5 object reference";
static char __pyx_k_11[] = "Can't convert incompatible object to HDF5 region reference";
static char __pyx_k_13[] = "\n    Low-level type-conversion routines.\n";
static char __pyx_k____main__[] = "__main__";
static char __pyx_k____test__[] = "__test__";
static char __pyx_k__enum2int[] = "enum2int";
static char __pyx_k__int2enum[] = "int2enum";
static char __pyx_k__str2vlen[] = "str2vlen";
static char __pyx_k__vlen2str[] = "vlen2str";
static char __pyx_k__TypeError[] = "TypeError";
static char __pyx_k__ValueError[] = "ValueError";
static char __pyx_k__fixed2vlen[] = "fixed2vlen";
static char __pyx_k__vlen2fixed[] = "vlen2fixed";
static char __pyx_k__MemoryError[] = "MemoryError";
static char __pyx_k__objref2pyref[] = "objref2pyref";
static char __pyx_k__pyref2objref[] = "pyref2objref";
static char __pyx_k__pyref2regref[] = "pyref2regref";
static char __pyx_k__regref2pyref[] = "regref2pyref";
static PyObject *__pyx_kp_s_11;
static PyObject *__pyx_kp_s_3;
static PyObject *__pyx_kp_s_5;
static PyObject *__pyx_kp_s_7;
static PyObject *__pyx_kp_s_9;
static PyObject *__pyx_n_s__MemoryError;
static PyObject *__pyx_n_s__TypeError;
static PyObject *__pyx_n_s__ValueError;
static PyObject *__pyx_n_s____main__;
static PyObject *__pyx_n_s____test__;
static PyObject *__pyx_k_tuple_4;
static PyObject *__pyx_k_tuple_6;
static PyObject *__pyx_k_tuple_8;
static PyObject *__pyx_k_tuple_10;
static PyObject *__pyx_k_tuple_12;

/* "h5py/_conv.pyx":45
 *     void Py_XDECREF(PyObject* obj)
 * 
 * cdef object objectify(PyObject* o):             # <<<<<<<<<<<<<<
 *     Py_INCREF(o)
 *     return <object>o
 */

static PyObject *__pyx_f_4h5py_5_conv_objectify(PyObject *__pyx_v_o) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("objectify", 0);

  /* "h5py/_conv.pyx":46
 * 
 * cdef object objectify(PyObject* o):
 *     Py_INCREF(o)             # <<<<<<<<<<<<<<
 *     return <object>o
 * 
 */
  Py_INCREF(__pyx_v_o);

  /* "h5py/_conv.pyx":47
 * cdef object objectify(PyObject* o):
 *     Py_INCREF(o)
 *     return <object>o             # <<<<<<<<<<<<<<
 * 
 * # Create Python object equivalents
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_o));
  __pyx_r = ((PyObject *)__pyx_v_o);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":52
 * cdef hid_t H5PY_OBJ = 0
 * 
 * cpdef hid_t get_python_obj():             # <<<<<<<<<<<<<<
 *     global H5PY_OBJ
 *     if H5PY_OBJ <= 0:
 */

static PyObject *__pyx_pw_4h5py_5_conv_1get_python_obj(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static hid_t __pyx_f_4h5py_5_conv_get_python_obj(CYTHON_UNUSED int __pyx_skip_dispatch) {
  hid_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hid_t __pyx_t_2;
  herr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_python_obj", 0);

  /* "h5py/_conv.pyx":54
 * cpdef hid_t get_python_obj():
 *     global H5PY_OBJ
 *     if H5PY_OBJ <= 0:             # <<<<<<<<<<<<<<
 *         H5PY_OBJ = H5Tcreate(H5T_OPAQUE, sizeof(PyObject*))
 *         H5Tset_tag(H5PY_OBJ, "PYTHON:OBJECT")
 */
  __pyx_t_1 = (__pyx_v_4h5py_5_conv_H5PY_OBJ <= 0);
  if (__pyx_t_1) {

    /* "h5py/_conv.pyx":55
 *     global H5PY_OBJ
 *     if H5PY_OBJ <= 0:
 *         H5PY_OBJ = H5Tcreate(H5T_OPAQUE, sizeof(PyObject*))             # <<<<<<<<<<<<<<
 *         H5Tset_tag(H5PY_OBJ, "PYTHON:OBJECT")
 *         H5Tlock(H5PY_OBJ)
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tcreate(H5T_OPAQUE, (sizeof(PyObject *))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 55; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_4h5py_5_conv_H5PY_OBJ = __pyx_t_2;

    /* "h5py/_conv.pyx":56
 *     if H5PY_OBJ <= 0:
 *         H5PY_OBJ = H5Tcreate(H5T_OPAQUE, sizeof(PyObject*))
 *         H5Tset_tag(H5PY_OBJ, "PYTHON:OBJECT")             # <<<<<<<<<<<<<<
 *         H5Tlock(H5PY_OBJ)
 *     return H5PY_OBJ
 */
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Tset_tag(__pyx_v_4h5py_5_conv_H5PY_OBJ, __pyx_k_1); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "h5py/_conv.pyx":57
 *         H5PY_OBJ = H5Tcreate(H5T_OPAQUE, sizeof(PyObject*))
 *         H5Tset_tag(H5PY_OBJ, "PYTHON:OBJECT")
 *         H5Tlock(H5PY_OBJ)             # <<<<<<<<<<<<<<
 *     return H5PY_OBJ
 * 
 */
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Tlock(__pyx_v_4h5py_5_conv_H5PY_OBJ); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 57; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":58
 *         H5Tset_tag(H5PY_OBJ, "PYTHON:OBJECT")
 *         H5Tlock(H5PY_OBJ)
 *     return H5PY_OBJ             # <<<<<<<<<<<<<<
 * 
 * ctypedef int (*conv_operator_t)(void* ipt, void* opt, void* bkg, void* priv) except -1
 */
  __pyx_r = __pyx_v_4h5py_5_conv_H5PY_OBJ;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("h5py._conv.get_python_obj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_5_conv_1get_python_obj(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_5_conv_1get_python_obj(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_python_obj (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_5_conv_get_python_obj(__pyx_self);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":52
 * cdef hid_t H5PY_OBJ = 0
 * 
 * cpdef hid_t get_python_obj():             # <<<<<<<<<<<<<<
 *     global H5PY_OBJ
 *     if H5PY_OBJ <= 0:
 */

static PyObject *__pyx_pf_4h5py_5_conv_get_python_obj(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_python_obj", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_hid_t(__pyx_f_4h5py_5_conv_get_python_obj(0)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py._conv.get_python_obj", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":72
 * # likely small compared to the cost of the Python-side API calls required to
 * # implement the conversions.
 * cdef herr_t generic_converter(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl, conv_operator_t op,
 */

static herr_t __pyx_f_4h5py_5_conv_generic_converter(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, CYTHON_UNUSED hid_t __pyx_v_dxpl, __pyx_t_4h5py_5_conv_conv_operator_t __pyx_v_op, __pyx_t_4h5py_5_conv_init_operator_t __pyx_v_initop, H5T_bkg_t __pyx_v_need_bkg) {
  int __pyx_v_command;
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  int __pyx_v_i;
  char *__pyx_v_buf;
  char *__pyx_v_bkg;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  H5T_cmd_t __pyx_t_1;
  int __pyx_t_2;
  herr_t __pyx_t_3;
  htri_t __pyx_t_4;
  H5T_cset_t __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("generic_converter", 0);

  /* "h5py/_conv.pyx":77
 *                     init_operator_t initop, H5T_bkg_t need_bkg) except -1:
 * 
 *     cdef int command = cdata[0].command             # <<<<<<<<<<<<<<
 *     cdef conv_size_t *sizes
 *     cdef int i
 */
  __pyx_t_1 = (__pyx_v_cdata[0]).command;
  __pyx_v_command = __pyx_t_1;

  /* "h5py/_conv.pyx":81
 *     cdef int i
 * 
 *     cdef char* buf = <char*>buf_i             # <<<<<<<<<<<<<<
 *     cdef char* bkg = <char*>bkg_i
 * 
 */
  __pyx_v_buf = ((char *)__pyx_v_buf_i);

  /* "h5py/_conv.pyx":82
 * 
 *     cdef char* buf = <char*>buf_i
 *     cdef char* bkg = <char*>bkg_i             # <<<<<<<<<<<<<<
 * 
 *     if command == H5T_CONV_INIT:
 */
  __pyx_v_bkg = ((char *)__pyx_v_bkg_i);

  /* "h5py/_conv.pyx":84
 *     cdef char* bkg = <char*>bkg_i
 * 
 *     if command == H5T_CONV_INIT:             # <<<<<<<<<<<<<<
 * 
 *         cdata[0].need_bkg = need_bkg
 */
  __pyx_t_2 = (__pyx_v_command == H5T_CONV_INIT);
  if (__pyx_t_2) {

    /* "h5py/_conv.pyx":86
 *     if command == H5T_CONV_INIT:
 * 
 *         cdata[0].need_bkg = need_bkg             # <<<<<<<<<<<<<<
 *         return initop(src_id, dst_id, &(cdata[0].priv))
 * 
 */
    (__pyx_v_cdata[0]).need_bkg = __pyx_v_need_bkg;

    /* "h5py/_conv.pyx":87
 * 
 *         cdata[0].need_bkg = need_bkg
 *         return initop(src_id, dst_id, &(cdata[0].priv))             # <<<<<<<<<<<<<<
 * 
 *     elif command == H5T_CONV_FREE:
 */
    __pyx_t_3 = __pyx_v_initop(__pyx_v_src_id, __pyx_v_dst_id, (&(__pyx_v_cdata[0]).priv)); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_3;
    goto __pyx_L0;
    goto __pyx_L3;
  }

  /* "h5py/_conv.pyx":89
 *         return initop(src_id, dst_id, &(cdata[0].priv))
 * 
 *     elif command == H5T_CONV_FREE:             # <<<<<<<<<<<<<<
 * 
 *         free(cdata[0].priv)
 */
  __pyx_t_2 = (__pyx_v_command == H5T_CONV_FREE);
  if (__pyx_t_2) {

    /* "h5py/_conv.pyx":91
 *     elif command == H5T_CONV_FREE:
 * 
 *         free(cdata[0].priv)             # <<<<<<<<<<<<<<
 *         cdata[0].priv = NULL
 * 
 */
    free((__pyx_v_cdata[0]).priv);

    /* "h5py/_conv.pyx":92
 * 
 *         free(cdata[0].priv)
 *         cdata[0].priv = NULL             # <<<<<<<<<<<<<<
 * 
 *     elif command == H5T_CONV_CONV:
 */
    (__pyx_v_cdata[0]).priv = NULL;
    goto __pyx_L3;
  }

  /* "h5py/_conv.pyx":94
 *         cdata[0].priv = NULL
 * 
 *     elif command == H5T_CONV_CONV:             # <<<<<<<<<<<<<<
 * 
 *         sizes = <conv_size_t*>cdata[0].priv
 */
  __pyx_t_2 = (__pyx_v_command == H5T_CONV_CONV);
  if (__pyx_t_2) {

    /* "h5py/_conv.pyx":96
 *     elif command == H5T_CONV_CONV:
 * 
 *         sizes = <conv_size_t*>cdata[0].priv             # <<<<<<<<<<<<<<
 * 
 *         if H5Tis_variable_str(src_id):
 */
    __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)(__pyx_v_cdata[0]).priv);

    /* "h5py/_conv.pyx":98
 *         sizes = <conv_size_t*>cdata[0].priv
 * 
 *         if H5Tis_variable_str(src_id):             # <<<<<<<<<<<<<<
 *             sizes.cset = H5Tget_cset(src_id)
 *         elif H5Tis_variable_str(dst_id):
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_src_id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 98; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_4) {

      /* "h5py/_conv.pyx":99
 * 
 *         if H5Tis_variable_str(src_id):
 *             sizes.cset = H5Tget_cset(src_id)             # <<<<<<<<<<<<<<
 *         elif H5Tis_variable_str(dst_id):
 *             sizes.cset = H5Tget_cset(dst_id)
 */
      __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tget_cset(__pyx_v_src_id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_sizes->cset = __pyx_t_5;
      goto __pyx_L4;
    }

    /* "h5py/_conv.pyx":100
 *         if H5Tis_variable_str(src_id):
 *             sizes.cset = H5Tget_cset(src_id)
 *         elif H5Tis_variable_str(dst_id):             # <<<<<<<<<<<<<<
 *             sizes.cset = H5Tget_cset(dst_id)
 * 
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_dst_id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_4) {

      /* "h5py/_conv.pyx":101
 *             sizes.cset = H5Tget_cset(src_id)
 *         elif H5Tis_variable_str(dst_id):
 *             sizes.cset = H5Tget_cset(dst_id)             # <<<<<<<<<<<<<<
 * 
 *         if bkg_stride==0: bkg_stride = sizes[0].dst_size;
 */
      __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tget_cset(__pyx_v_dst_id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_sizes->cset = __pyx_t_5;
      goto __pyx_L4;
    }
    __pyx_L4:;

    /* "h5py/_conv.pyx":103
 *             sizes.cset = H5Tget_cset(dst_id)
 * 
 *         if bkg_stride==0: bkg_stride = sizes[0].dst_size;             # <<<<<<<<<<<<<<
 * 
 *         if buf_stride == 0:
 */
    __pyx_t_2 = (__pyx_v_bkg_stride == 0);
    if (__pyx_t_2) {
      __pyx_t_6 = (__pyx_v_sizes[0]).dst_size;
      __pyx_v_bkg_stride = __pyx_t_6;
      goto __pyx_L5;
    }
    __pyx_L5:;

    /* "h5py/_conv.pyx":105
 *         if bkg_stride==0: bkg_stride = sizes[0].dst_size;
 * 
 *         if buf_stride == 0:             # <<<<<<<<<<<<<<
 *             # No explicit stride seems to mean that the elements are packed
 *             # contiguously in the buffer.  In this case we must be careful
 */
    __pyx_t_2 = (__pyx_v_buf_stride == 0);
    if (__pyx_t_2) {

      /* "h5py/_conv.pyx":111
 *             # of a larger size.
 * 
 *             if sizes[0].src_size >= sizes[0].dst_size:             # <<<<<<<<<<<<<<
 *                 for i from 0<=i<nl:
 *                     op( buf + (i*sizes[0].src_size),    # input pointer
 */
      __pyx_t_2 = ((__pyx_v_sizes[0]).src_size >= (__pyx_v_sizes[0]).dst_size);
      if (__pyx_t_2) {

        /* "h5py/_conv.pyx":112
 * 
 *             if sizes[0].src_size >= sizes[0].dst_size:
 *                 for i from 0<=i<nl:             # <<<<<<<<<<<<<<
 *                     op( buf + (i*sizes[0].src_size),    # input pointer
 *                         buf + (i*sizes[0].dst_size),    # output pointer
 */
        __pyx_t_6 = __pyx_v_nl;
        for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_6; __pyx_v_i++) {

          /* "h5py/_conv.pyx":116
 *                         buf + (i*sizes[0].dst_size),    # output pointer
 *                         bkg + (i*bkg_stride),           # backing buffer
 *                         cdata[0].priv)                  # conversion context             # <<<<<<<<<<<<<<
 *             else:
 *                 for i from nl>i>=0:
 */
          __pyx_t_7 = __pyx_v_op((__pyx_v_buf + (__pyx_v_i * (__pyx_v_sizes[0]).src_size)), (__pyx_v_buf + (__pyx_v_i * (__pyx_v_sizes[0]).dst_size)), (__pyx_v_bkg + (__pyx_v_i * __pyx_v_bkg_stride)), (__pyx_v_cdata[0]).priv); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        goto __pyx_L7;
      }
      /*else*/ {

        /* "h5py/_conv.pyx":118
 *                         cdata[0].priv)                  # conversion context
 *             else:
 *                 for i from nl>i>=0:             # <<<<<<<<<<<<<<
 *                     op( buf + (i*sizes[0].src_size),
 *                         buf + (i*sizes[0].dst_size),
 */
        for (__pyx_v_i = __pyx_v_nl-1; __pyx_v_i >= 0; __pyx_v_i--) {

          /* "h5py/_conv.pyx":122
 *                         buf + (i*sizes[0].dst_size),
 *                         bkg + (i*bkg_stride),
 *                         cdata[0].priv)             # <<<<<<<<<<<<<<
 *         else:
 *             # With explicit strides, we assume that the library knows the
 */
          __pyx_t_7 = __pyx_v_op((__pyx_v_buf + (__pyx_v_i * (__pyx_v_sizes[0]).src_size)), (__pyx_v_buf + (__pyx_v_i * (__pyx_v_sizes[0]).dst_size)), (__pyx_v_bkg + (__pyx_v_i * __pyx_v_bkg_stride)), (__pyx_v_cdata[0]).priv); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
      }
      __pyx_L7:;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":127
 *             # alignment better than us.  Therefore we use the given stride
 *             # offsets exclusively.
 *             for i from 0<=i<nl:             # <<<<<<<<<<<<<<
 *                 op( buf + (i*buf_stride),
 *                     buf + (i*buf_stride),   # note this is the same!
 */
      __pyx_t_6 = __pyx_v_nl;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_6; __pyx_v_i++) {

        /* "h5py/_conv.pyx":131
 *                     buf + (i*buf_stride),   # note this is the same!
 *                     bkg + (i*bkg_stride),
 *                     cdata[0].priv)             # <<<<<<<<<<<<<<
 *     else:
 *         return -2   # Unrecognized command.  Note this is NOT an exception.
 */
        __pyx_t_7 = __pyx_v_op((__pyx_v_buf + (__pyx_v_i * __pyx_v_buf_stride)), (__pyx_v_buf + (__pyx_v_i * __pyx_v_buf_stride)), (__pyx_v_bkg + (__pyx_v_i * __pyx_v_bkg_stride)), (__pyx_v_cdata[0]).priv); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    __pyx_L6:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":133
 *                     cdata[0].priv)
 *     else:
 *         return -2   # Unrecognized command.  Note this is NOT an exception.             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_r = -2;
    goto __pyx_L0;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":135
 *         return -2   # Unrecognized command.  Note this is NOT an exception.
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.generic_converter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":145
 *     int cset
 * 
 * cdef herr_t init_generic(hid_t src, hid_t dst, void** priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef conv_size_t *sizes
 */

static herr_t __pyx_f_4h5py_5_conv_init_generic(hid_t __pyx_v_src, hid_t __pyx_v_dst, void **__pyx_v_priv) {
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init_generic", 0);

  /* "h5py/_conv.pyx":148
 * 
 *     cdef conv_size_t *sizes
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))             # <<<<<<<<<<<<<<
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)malloc((sizeof(__pyx_t_4h5py_5_conv_conv_size_t))));

  /* "h5py/_conv.pyx":149
 *     cdef conv_size_t *sizes
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 *     priv[0] = sizes             # <<<<<<<<<<<<<<
 *     sizes[0].src_size = H5Tget_size(src)
 *     sizes[0].dst_size = H5Tget_size(dst)
 */
  (__pyx_v_priv[0]) = __pyx_v_sizes;

  /* "h5py/_conv.pyx":150
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)             # <<<<<<<<<<<<<<
 *     sizes[0].dst_size = H5Tget_size(dst)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_sizes[0]).src_size = __pyx_t_1;

  /* "h5py/_conv.pyx":151
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)
 *     sizes[0].dst_size = H5Tget_size(dst)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_sizes[0]).dst_size = __pyx_t_1;

  /* "h5py/_conv.pyx":153
 *     sizes[0].dst_size = H5Tget_size(dst)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.init_generic", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":158
 * # Vlen string conversion
 * 
 * cdef int conv_vlen2str(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt
 */

static int __pyx_f_4h5py_5_conv_conv_vlen2str(void *__pyx_v_ipt, void *__pyx_v_opt, void *__pyx_v_bkg, void *__pyx_v_priv) {
  PyObject **__pyx_v_buf_obj;
  PyObject **__pyx_v_bkg_obj;
  char **__pyx_v_buf_cstring;
  PyObject *__pyx_v_temp_obj;
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("conv_vlen2str", 0);

  /* "h5py/_conv.pyx":160
 * cdef int conv_vlen2str(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt             # <<<<<<<<<<<<<<
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef char** buf_cstring = <char**>ipt
 */
  __pyx_v_buf_obj = ((PyObject **)__pyx_v_opt);

  /* "h5py/_conv.pyx":161
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt
 *     cdef PyObject** bkg_obj = <PyObject**>bkg             # <<<<<<<<<<<<<<
 *     cdef char** buf_cstring = <char**>ipt
 *     cdef PyObject* temp_obj = NULL
 */
  __pyx_v_bkg_obj = ((PyObject **)__pyx_v_bkg);

  /* "h5py/_conv.pyx":162
 *     cdef PyObject** buf_obj = <PyObject**>opt
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef char** buf_cstring = <char**>ipt             # <<<<<<<<<<<<<<
 *     cdef PyObject* temp_obj = NULL
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 */
  __pyx_v_buf_cstring = ((char **)__pyx_v_ipt);

  /* "h5py/_conv.pyx":163
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef char** buf_cstring = <char**>ipt
 *     cdef PyObject* temp_obj = NULL             # <<<<<<<<<<<<<<
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 * 
 */
  __pyx_v_temp_obj = NULL;

  /* "h5py/_conv.pyx":164
 *     cdef char** buf_cstring = <char**>ipt
 *     cdef PyObject* temp_obj = NULL
 *     cdef conv_size_t *sizes = <conv_size_t*>priv             # <<<<<<<<<<<<<<
 * 
 *     # When reading we identify H5T_CSET_ASCII as a byte string and
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)__pyx_v_priv);

  /* "h5py/_conv.pyx":168
 *     # When reading we identify H5T_CSET_ASCII as a byte string and
 *     # H5T_CSET_UTF8 as a utf8-encoded unicode string
 *     if sizes.cset == H5T_CSET_ASCII:             # <<<<<<<<<<<<<<
 *         if buf_cstring[0] == NULL:
 *             temp_obj = PyBytes_FromString("")
 */
  __pyx_t_1 = (__pyx_v_sizes->cset == H5T_CSET_ASCII);
  if (__pyx_t_1) {

    /* "h5py/_conv.pyx":169
 *     # H5T_CSET_UTF8 as a utf8-encoded unicode string
 *     if sizes.cset == H5T_CSET_ASCII:
 *         if buf_cstring[0] == NULL:             # <<<<<<<<<<<<<<
 *             temp_obj = PyBytes_FromString("")
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_buf_cstring[0]) == NULL);
    if (__pyx_t_1) {

      /* "h5py/_conv.pyx":170
 *     if sizes.cset == H5T_CSET_ASCII:
 *         if buf_cstring[0] == NULL:
 *             temp_obj = PyBytes_FromString("")             # <<<<<<<<<<<<<<
 *         else:
 *             temp_obj = PyBytes_FromString(buf_cstring[0])
 */
      __pyx_t_2 = PyBytes_FromString(__pyx_k_2); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_temp_obj = __pyx_t_2;
      goto __pyx_L4;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":172
 *             temp_obj = PyBytes_FromString("")
 *         else:
 *             temp_obj = PyBytes_FromString(buf_cstring[0])             # <<<<<<<<<<<<<<
 *     elif sizes.cset == H5T_CSET_UTF8:
 *         if buf_cstring[0] == NULL:
 */
      __pyx_t_2 = PyBytes_FromString((__pyx_v_buf_cstring[0])); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_temp_obj = __pyx_t_2;
    }
    __pyx_L4:;
    goto __pyx_L3;
  }

  /* "h5py/_conv.pyx":173
 *         else:
 *             temp_obj = PyBytes_FromString(buf_cstring[0])
 *     elif sizes.cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *         if buf_cstring[0] == NULL:
 *             temp_obj = PyUnicode_DecodeUTF8("", 0, NULL)
 */
  __pyx_t_1 = (__pyx_v_sizes->cset == H5T_CSET_UTF8);
  if (__pyx_t_1) {

    /* "h5py/_conv.pyx":174
 *             temp_obj = PyBytes_FromString(buf_cstring[0])
 *     elif sizes.cset == H5T_CSET_UTF8:
 *         if buf_cstring[0] == NULL:             # <<<<<<<<<<<<<<
 *             temp_obj = PyUnicode_DecodeUTF8("", 0, NULL)
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_buf_cstring[0]) == NULL);
    if (__pyx_t_1) {

      /* "h5py/_conv.pyx":175
 *     elif sizes.cset == H5T_CSET_UTF8:
 *         if buf_cstring[0] == NULL:
 *             temp_obj = PyUnicode_DecodeUTF8("", 0, NULL)             # <<<<<<<<<<<<<<
 *         else:
 *             temp_obj = PyUnicode_DecodeUTF8(buf_cstring[0], strlen(buf_cstring[0]), NULL)
 */
      __pyx_t_2 = PyUnicode_DecodeUTF8(__pyx_k_2, 0, NULL); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_temp_obj = __pyx_t_2;
      goto __pyx_L5;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":177
 *             temp_obj = PyUnicode_DecodeUTF8("", 0, NULL)
 *         else:
 *             temp_obj = PyUnicode_DecodeUTF8(buf_cstring[0], strlen(buf_cstring[0]), NULL)             # <<<<<<<<<<<<<<
 * 
 *     # Since all data conversions are by definition in-place, it
 */
      __pyx_t_2 = PyUnicode_DecodeUTF8((__pyx_v_buf_cstring[0]), strlen((__pyx_v_buf_cstring[0])), NULL); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_temp_obj = __pyx_t_2;
    }
    __pyx_L5:;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":181
 *     # Since all data conversions are by definition in-place, it
 *     # is our responsibility to free the memory used by the vlens.
 *     free(buf_cstring[0])             # <<<<<<<<<<<<<<
 * 
 *     # HDF5 will eventuallly overwrite this target location, so we
 */
  free((__pyx_v_buf_cstring[0]));

  /* "h5py/_conv.pyx":185
 *     # HDF5 will eventuallly overwrite this target location, so we
 *     # make sure to decref the object there.
 *     Py_XDECREF(bkg_obj[0])             # <<<<<<<<<<<<<<
 * 
 *     # Write the new string object to the buffer in-place
 */
  Py_XDECREF((__pyx_v_bkg_obj[0]));

  /* "h5py/_conv.pyx":188
 * 
 *     # Write the new string object to the buffer in-place
 *     buf_obj[0] = temp_obj             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  (__pyx_v_buf_obj[0]) = __pyx_v_temp_obj;

  /* "h5py/_conv.pyx":190
 *     buf_obj[0] = temp_obj
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int conv_str2vlen(void* ipt, void* opt, void* bkg, void* priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.conv_vlen2str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":192
 *     return 0
 * 
 * cdef int conv_str2vlen(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 */

static int __pyx_f_4h5py_5_conv_conv_str2vlen(void *__pyx_v_ipt, void *__pyx_v_opt, CYTHON_UNUSED void *__pyx_v_bkg, void *__pyx_v_priv) {
  PyObject **__pyx_v_buf_obj;
  char **__pyx_v_buf_cstring;
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  PyObject *__pyx_v_temp_object;
  PyObject *__pyx_v_temp_encoded;
  char *__pyx_v_temp_string;
  size_t __pyx_v_temp_string_len;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  char *__pyx_t_12;
  PyObject *__pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("conv_str2vlen", 0);

  /* "h5py/_conv.pyx":194
 * cdef int conv_str2vlen(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt             # <<<<<<<<<<<<<<
 *     cdef char** buf_cstring = <char**>opt
 *     cdef conv_size_t* sizes = <conv_size_t*>priv
 */
  __pyx_v_buf_obj = ((PyObject **)__pyx_v_ipt);

  /* "h5py/_conv.pyx":195
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 *     cdef char** buf_cstring = <char**>opt             # <<<<<<<<<<<<<<
 *     cdef conv_size_t* sizes = <conv_size_t*>priv
 * 
 */
  __pyx_v_buf_cstring = ((char **)__pyx_v_opt);

  /* "h5py/_conv.pyx":196
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 *     cdef char** buf_cstring = <char**>opt
 *     cdef conv_size_t* sizes = <conv_size_t*>priv             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject* temp_object = NULL
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)__pyx_v_priv);

  /* "h5py/_conv.pyx":198
 *     cdef conv_size_t* sizes = <conv_size_t*>priv
 * 
 *     cdef PyObject* temp_object = NULL             # <<<<<<<<<<<<<<
 *     cdef PyObject* temp_encoded = NULL
 * 
 */
  __pyx_v_temp_object = NULL;

  /* "h5py/_conv.pyx":199
 * 
 *     cdef PyObject* temp_object = NULL
 *     cdef PyObject* temp_encoded = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef char* temp_string = NULL
 */
  __pyx_v_temp_encoded = NULL;

  /* "h5py/_conv.pyx":201
 *     cdef PyObject* temp_encoded = NULL
 * 
 *     cdef char* temp_string = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t temp_string_len = 0  # Not including null term
 * 
 */
  __pyx_v_temp_string = NULL;

  /* "h5py/_conv.pyx":202
 * 
 *     cdef char* temp_string = NULL
 *     cdef size_t temp_string_len = 0  # Not including null term             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
  __pyx_v_temp_string_len = 0;

  /* "h5py/_conv.pyx":204
 *     cdef size_t temp_string_len = 0  # Not including null term
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if buf_obj[0] == NULL or buf_obj[0] == Py_None:
 *             temp_string = ""
 */
  /*try:*/ {

    /* "h5py/_conv.pyx":205
 * 
 *     try:
 *         if buf_obj[0] == NULL or buf_obj[0] == Py_None:             # <<<<<<<<<<<<<<
 *             temp_string = ""
 *             temp_string_len = 0
 */
    __pyx_t_1 = ((__pyx_v_buf_obj[0]) == NULL);
    if (!__pyx_t_1) {
      __pyx_t_2 = ((__pyx_v_buf_obj[0]) == Py_None);
      __pyx_t_3 = __pyx_t_2;
    } else {
      __pyx_t_3 = __pyx_t_1;
    }
    if (__pyx_t_3) {

      /* "h5py/_conv.pyx":206
 *     try:
 *         if buf_obj[0] == NULL or buf_obj[0] == Py_None:
 *             temp_string = ""             # <<<<<<<<<<<<<<
 *             temp_string_len = 0
 *         else:
 */
      __pyx_v_temp_string = __pyx_k_2;

      /* "h5py/_conv.pyx":207
 *         if buf_obj[0] == NULL or buf_obj[0] == Py_None:
 *             temp_string = ""
 *             temp_string_len = 0             # <<<<<<<<<<<<<<
 *         else:
 *             if PyBytes_CheckExact(buf_obj[0]):
 */
      __pyx_v_temp_string_len = 0;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":209
 *             temp_string_len = 0
 *         else:
 *             if PyBytes_CheckExact(buf_obj[0]):             # <<<<<<<<<<<<<<
 * 
 *                 # Input is a byte string.  If we're using CSET_UTF8, make sure
 */
      __pyx_t_4 = PyBytes_CheckExact((__pyx_v_buf_obj[0])); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L4;}
      if (__pyx_t_4) {

        /* "h5py/_conv.pyx":213
 *                 # Input is a byte string.  If we're using CSET_UTF8, make sure
 *                 # it's valid UTF-8.  Otherwise just store it.
 *                 temp_object = buf_obj[0]             # <<<<<<<<<<<<<<
 *                 Py_INCREF(temp_object)
 *                 if sizes.cset == H5T_CSET_UTF8:
 */
        __pyx_v_temp_object = (__pyx_v_buf_obj[0]);

        /* "h5py/_conv.pyx":214
 *                 # it's valid UTF-8.  Otherwise just store it.
 *                 temp_object = buf_obj[0]
 *                 Py_INCREF(temp_object)             # <<<<<<<<<<<<<<
 *                 if sizes.cset == H5T_CSET_UTF8:
 *                     try:
 */
        Py_INCREF(__pyx_v_temp_object);

        /* "h5py/_conv.pyx":215
 *                 temp_object = buf_obj[0]
 *                 Py_INCREF(temp_object)
 *                 if sizes.cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *                     try:
 *                         pass # disabled for Python 3 compatibility
 */
        __pyx_t_3 = (__pyx_v_sizes->cset == H5T_CSET_UTF8);
        if (__pyx_t_3) {

          /* "h5py/_conv.pyx":216
 *                 Py_INCREF(temp_object)
 *                 if sizes.cset == H5T_CSET_UTF8:
 *                     try:             # <<<<<<<<<<<<<<
 *                         pass # disabled for Python 3 compatibility
 *                         #temp_encoded = PyString_AsDecodedObject(temp_object, "utf8", NULL)
 */
          {
            __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
            __Pyx_XGOTREF(__pyx_t_5);
            __Pyx_XGOTREF(__pyx_t_6);
            __Pyx_XGOTREF(__pyx_t_7);
            /*try:*/ {
            }
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            goto __pyx_L16_try_end;

            /* "h5py/_conv.pyx":219
 *                         pass # disabled for Python 3 compatibility
 *                         #temp_encoded = PyString_AsDecodedObject(temp_object, "utf8", NULL)
 *                     except:             # <<<<<<<<<<<<<<
 *                         raise ValueError("Byte string is not valid utf-8 and can't be stored in a utf-8 dataset")
 *                 temp_string = PyBytes_AsString(temp_object)
 */
            /*except:*/ {
              __Pyx_AddTraceback("h5py._conv.conv_str2vlen", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L11_except_error;}
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_GOTREF(__pyx_t_9);
              __Pyx_GOTREF(__pyx_t_10);

              /* "h5py/_conv.pyx":220
 *                         #temp_encoded = PyString_AsDecodedObject(temp_object, "utf8", NULL)
 *                     except:
 *                         raise ValueError("Byte string is not valid utf-8 and can't be stored in a utf-8 dataset")             # <<<<<<<<<<<<<<
 *                 temp_string = PyBytes_AsString(temp_object)
 *                 temp_string_len = PyBytes_Size(temp_object)
 */
              __pyx_t_11 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_k_tuple_4), NULL); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L11_except_error;}
              __Pyx_GOTREF(__pyx_t_11);
              __Pyx_Raise(__pyx_t_11, 0, 0, 0);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
              {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L11_except_error;}
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              goto __pyx_L10_exception_handled;
            }
            __pyx_L11_except_error:;
            __Pyx_XGIVEREF(__pyx_t_5);
            __Pyx_XGIVEREF(__pyx_t_6);
            __Pyx_XGIVEREF(__pyx_t_7);
            __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
            goto __pyx_L4;
            __pyx_L10_exception_handled:;
            __Pyx_XGIVEREF(__pyx_t_5);
            __Pyx_XGIVEREF(__pyx_t_6);
            __Pyx_XGIVEREF(__pyx_t_7);
            __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
            __pyx_L16_try_end:;
          }
          goto __pyx_L8;
        }
        __pyx_L8:;

        /* "h5py/_conv.pyx":221
 *                     except:
 *                         raise ValueError("Byte string is not valid utf-8 and can't be stored in a utf-8 dataset")
 *                 temp_string = PyBytes_AsString(temp_object)             # <<<<<<<<<<<<<<
 *                 temp_string_len = PyBytes_Size(temp_object)
 * 
 */
        __pyx_t_12 = PyBytes_AsString(__pyx_v_temp_object); if (unlikely(__pyx_t_12 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 221; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_temp_string = __pyx_t_12;

        /* "h5py/_conv.pyx":222
 *                         raise ValueError("Byte string is not valid utf-8 and can't be stored in a utf-8 dataset")
 *                 temp_string = PyBytes_AsString(temp_object)
 *                 temp_string_len = PyBytes_Size(temp_object)             # <<<<<<<<<<<<<<
 * 
 *             # We are given a Unicode object.  Encode it to utf-8 regardless of
 */
        __pyx_t_4 = PyBytes_Size(__pyx_v_temp_object); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_temp_string_len = __pyx_t_4;
        goto __pyx_L7;
      }

      /* "h5py/_conv.pyx":226
 *             # We are given a Unicode object.  Encode it to utf-8 regardless of
 *             # the HDF5 character set.
 *             elif PyUnicode_CheckExact(buf_obj[0]):             # <<<<<<<<<<<<<<
 *                 temp_object = buf_obj[0]
 *                 Py_INCREF(temp_object)
 */
      __pyx_t_4 = PyUnicode_CheckExact((__pyx_v_buf_obj[0])); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L4;}
      if (__pyx_t_4) {

        /* "h5py/_conv.pyx":227
 *             # the HDF5 character set.
 *             elif PyUnicode_CheckExact(buf_obj[0]):
 *                 temp_object = buf_obj[0]             # <<<<<<<<<<<<<<
 *                 Py_INCREF(temp_object)
 *                 temp_encoded = PyUnicode_AsUTF8String(temp_object)
 */
        __pyx_v_temp_object = (__pyx_v_buf_obj[0]);

        /* "h5py/_conv.pyx":228
 *             elif PyUnicode_CheckExact(buf_obj[0]):
 *                 temp_object = buf_obj[0]
 *                 Py_INCREF(temp_object)             # <<<<<<<<<<<<<<
 *                 temp_encoded = PyUnicode_AsUTF8String(temp_object)
 *                 temp_string = PyBytes_AsString(temp_encoded)
 */
        Py_INCREF(__pyx_v_temp_object);

        /* "h5py/_conv.pyx":229
 *                 temp_object = buf_obj[0]
 *                 Py_INCREF(temp_object)
 *                 temp_encoded = PyUnicode_AsUTF8String(temp_object)             # <<<<<<<<<<<<<<
 *                 temp_string = PyBytes_AsString(temp_encoded)
 *                 temp_string_len = PyBytes_Size(temp_encoded)
 */
        __pyx_t_13 = PyUnicode_AsUTF8String(__pyx_v_temp_object); if (unlikely(__pyx_t_13 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 229; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_temp_encoded = __pyx_t_13;

        /* "h5py/_conv.pyx":230
 *                 Py_INCREF(temp_object)
 *                 temp_encoded = PyUnicode_AsUTF8String(temp_object)
 *                 temp_string = PyBytes_AsString(temp_encoded)             # <<<<<<<<<<<<<<
 *                 temp_string_len = PyBytes_Size(temp_encoded)
 * 
 */
        __pyx_t_12 = PyBytes_AsString(__pyx_v_temp_encoded); if (unlikely(__pyx_t_12 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 230; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_temp_string = __pyx_t_12;

        /* "h5py/_conv.pyx":231
 *                 temp_encoded = PyUnicode_AsUTF8String(temp_object)
 *                 temp_string = PyBytes_AsString(temp_encoded)
 *                 temp_string_len = PyBytes_Size(temp_encoded)             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
        __pyx_t_4 = PyBytes_Size(__pyx_v_temp_encoded); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_temp_string_len = __pyx_t_4;
        goto __pyx_L7;
      }
      /*else*/ {

        /* "h5py/_conv.pyx":234
 * 
 *             else:
 *                 if sizes.cset == H5T_CSET_ASCII:             # <<<<<<<<<<<<<<
 *                     temp_object = PyObject_Str(buf_obj[0])
 *                     temp_string = PyBytes_AsString(temp_object)
 */
        __pyx_t_3 = (__pyx_v_sizes->cset == H5T_CSET_ASCII);
        if (__pyx_t_3) {

          /* "h5py/_conv.pyx":235
 *             else:
 *                 if sizes.cset == H5T_CSET_ASCII:
 *                     temp_object = PyObject_Str(buf_obj[0])             # <<<<<<<<<<<<<<
 *                     temp_string = PyBytes_AsString(temp_object)
 *                     temp_string_len = PyBytes_Size(temp_object)
 */
          __pyx_t_13 = PyObject_Str((__pyx_v_buf_obj[0])); if (unlikely(__pyx_t_13 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 235; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_object = __pyx_t_13;

          /* "h5py/_conv.pyx":236
 *                 if sizes.cset == H5T_CSET_ASCII:
 *                     temp_object = PyObject_Str(buf_obj[0])
 *                     temp_string = PyBytes_AsString(temp_object)             # <<<<<<<<<<<<<<
 *                     temp_string_len = PyBytes_Size(temp_object)
 *                 elif sizes.cset == H5T_CSET_UTF8:
 */
          __pyx_t_12 = PyBytes_AsString(__pyx_v_temp_object); if (unlikely(__pyx_t_12 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_string = __pyx_t_12;

          /* "h5py/_conv.pyx":237
 *                     temp_object = PyObject_Str(buf_obj[0])
 *                     temp_string = PyBytes_AsString(temp_object)
 *                     temp_string_len = PyBytes_Size(temp_object)             # <<<<<<<<<<<<<<
 *                 elif sizes.cset == H5T_CSET_UTF8:
 *                     temp_object = PyObject_Str(buf_obj[0])
 */
          __pyx_t_4 = PyBytes_Size(__pyx_v_temp_object); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_string_len = __pyx_t_4;
          goto __pyx_L19;
        }

        /* "h5py/_conv.pyx":238
 *                     temp_string = PyBytes_AsString(temp_object)
 *                     temp_string_len = PyBytes_Size(temp_object)
 *                 elif sizes.cset == H5T_CSET_UTF8:             # <<<<<<<<<<<<<<
 *                     temp_object = PyObject_Str(buf_obj[0])
 *                     Py_INCREF(temp_object)
 */
        __pyx_t_3 = (__pyx_v_sizes->cset == H5T_CSET_UTF8);
        if (__pyx_t_3) {

          /* "h5py/_conv.pyx":239
 *                     temp_string_len = PyBytes_Size(temp_object)
 *                 elif sizes.cset == H5T_CSET_UTF8:
 *                     temp_object = PyObject_Str(buf_obj[0])             # <<<<<<<<<<<<<<
 *                     Py_INCREF(temp_object)
 *                     temp_encoded = PyUnicode_AsUTF8String(temp_object)
 */
          __pyx_t_13 = PyObject_Str((__pyx_v_buf_obj[0])); if (unlikely(__pyx_t_13 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_object = __pyx_t_13;

          /* "h5py/_conv.pyx":240
 *                 elif sizes.cset == H5T_CSET_UTF8:
 *                     temp_object = PyObject_Str(buf_obj[0])
 *                     Py_INCREF(temp_object)             # <<<<<<<<<<<<<<
 *                     temp_encoded = PyUnicode_AsUTF8String(temp_object)
 *                     Py_INCREF(temp_encoded)
 */
          Py_INCREF(__pyx_v_temp_object);

          /* "h5py/_conv.pyx":241
 *                     temp_object = PyObject_Str(buf_obj[0])
 *                     Py_INCREF(temp_object)
 *                     temp_encoded = PyUnicode_AsUTF8String(temp_object)             # <<<<<<<<<<<<<<
 *                     Py_INCREF(temp_encoded)
 *                     temp_string = PyBytes_AsString(temp_encoded)
 */
          __pyx_t_13 = PyUnicode_AsUTF8String(__pyx_v_temp_object); if (unlikely(__pyx_t_13 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_encoded = __pyx_t_13;

          /* "h5py/_conv.pyx":242
 *                     Py_INCREF(temp_object)
 *                     temp_encoded = PyUnicode_AsUTF8String(temp_object)
 *                     Py_INCREF(temp_encoded)             # <<<<<<<<<<<<<<
 *                     temp_string = PyBytes_AsString(temp_encoded)
 *                     temp_string_len = PyBytes_Size(temp_encoded)
 */
          Py_INCREF(__pyx_v_temp_encoded);

          /* "h5py/_conv.pyx":243
 *                     temp_encoded = PyUnicode_AsUTF8String(temp_object)
 *                     Py_INCREF(temp_encoded)
 *                     temp_string = PyBytes_AsString(temp_encoded)             # <<<<<<<<<<<<<<
 *                     temp_string_len = PyBytes_Size(temp_encoded)
 *                 else:
 */
          __pyx_t_12 = PyBytes_AsString(__pyx_v_temp_encoded); if (unlikely(__pyx_t_12 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_string = __pyx_t_12;

          /* "h5py/_conv.pyx":244
 *                     Py_INCREF(temp_encoded)
 *                     temp_string = PyBytes_AsString(temp_encoded)
 *                     temp_string_len = PyBytes_Size(temp_encoded)             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise TypeError("Unrecognized dataset encoding")
 */
          __pyx_t_4 = PyBytes_Size(__pyx_v_temp_encoded); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __pyx_v_temp_string_len = __pyx_t_4;
          goto __pyx_L19;
        }
        /*else*/ {

          /* "h5py/_conv.pyx":246
 *                     temp_string_len = PyBytes_Size(temp_encoded)
 *                 else:
 *                     raise TypeError("Unrecognized dataset encoding")             # <<<<<<<<<<<<<<
 * 
 *         if strlen(temp_string) != temp_string_len:
 */
          __pyx_t_10 = PyObject_Call(__pyx_builtin_TypeError, ((PyObject *)__pyx_k_tuple_6), NULL); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L4;}
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_Raise(__pyx_t_10, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L4;}
        }
        __pyx_L19:;
      }
      __pyx_L7:;
    }
    __pyx_L6:;

    /* "h5py/_conv.pyx":248
 *                     raise TypeError("Unrecognized dataset encoding")
 * 
 *         if strlen(temp_string) != temp_string_len:             # <<<<<<<<<<<<<<
 *             raise ValueError("VLEN strings do not support embedded NULLs")
 * 
 */
    __pyx_t_3 = (strlen(__pyx_v_temp_string) != __pyx_v_temp_string_len);
    if (__pyx_t_3) {

      /* "h5py/_conv.pyx":249
 * 
 *         if strlen(temp_string) != temp_string_len:
 *             raise ValueError("VLEN strings do not support embedded NULLs")             # <<<<<<<<<<<<<<
 * 
 *         buf_cstring[0] = <char*>malloc(temp_string_len+1)
 */
      __pyx_t_10 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_k_tuple_8), NULL); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 249; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_Raise(__pyx_t_10, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 249; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L20;
    }
    __pyx_L20:;

    /* "h5py/_conv.pyx":251
 *             raise ValueError("VLEN strings do not support embedded NULLs")
 * 
 *         buf_cstring[0] = <char*>malloc(temp_string_len+1)             # <<<<<<<<<<<<<<
 *         memcpy(buf_cstring[0], temp_string, temp_string_len+1)
 * 
 */
    (__pyx_v_buf_cstring[0]) = ((char *)malloc((__pyx_v_temp_string_len + 1)));

    /* "h5py/_conv.pyx":252
 * 
 *         buf_cstring[0] = <char*>malloc(temp_string_len+1)
 *         memcpy(buf_cstring[0], temp_string, temp_string_len+1)             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    memcpy((__pyx_v_buf_cstring[0]), __pyx_v_temp_string, (__pyx_v_temp_string_len + 1));

    /* "h5py/_conv.pyx":254
 *         memcpy(buf_cstring[0], temp_string, temp_string_len+1)
 * 
 *         return 0             # <<<<<<<<<<<<<<
 *     finally:
 *         Py_XDECREF(temp_object)
 */
    __pyx_r = 0;
    goto __pyx_L3;
  }

  /* "h5py/_conv.pyx":256
 *         return 0
 *     finally:
 *         Py_XDECREF(temp_object)             # <<<<<<<<<<<<<<
 *         Py_XDECREF(temp_encoded)
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    Py_XDECREF(__pyx_v_temp_object);

    /* "h5py/_conv.pyx":257
 *     finally:
 *         Py_XDECREF(temp_object)
 *         Py_XDECREF(temp_encoded)             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
    Py_XDECREF(__pyx_v_temp_encoded);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("h5py._conv.conv_str2vlen", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":262
 * # VLEN to fixed-width strings
 * 
 * cdef herr_t init_vlen2fixed(hid_t src, hid_t dst, void** priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef conv_size_t *sizes
 */

static herr_t __pyx_f_4h5py_5_conv_init_vlen2fixed(hid_t __pyx_v_src, hid_t __pyx_v_dst, void **__pyx_v_priv) {
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  htri_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init_vlen2fixed", 0);

  /* "h5py/_conv.pyx":266
 *     cdef conv_size_t *sizes
 * 
 *     if not (H5Tis_variable_str(src) and (not H5Tis_variable_str(dst))):             # <<<<<<<<<<<<<<
 *         return -2
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = (!__pyx_t_2);
    __pyx_t_4 = __pyx_t_3;
  } else {
    __pyx_t_4 = __pyx_t_1;
  }
  __pyx_t_3 = (!__pyx_t_4);
  if (__pyx_t_3) {

    /* "h5py/_conv.pyx":267
 * 
 *     if not (H5Tis_variable_str(src) and (not H5Tis_variable_str(dst))):
 *         return -2             # <<<<<<<<<<<<<<
 * 
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 */
    __pyx_r = -2;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":269
 *         return -2
 * 
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))             # <<<<<<<<<<<<<<
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)malloc((sizeof(__pyx_t_4h5py_5_conv_conv_size_t))));

  /* "h5py/_conv.pyx":270
 * 
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 *     priv[0] = sizes             # <<<<<<<<<<<<<<
 *     sizes[0].src_size = H5Tget_size(src)
 *     sizes[0].dst_size = H5Tget_size(dst)
 */
  (__pyx_v_priv[0]) = __pyx_v_sizes;

  /* "h5py/_conv.pyx":271
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)             # <<<<<<<<<<<<<<
 *     sizes[0].dst_size = H5Tget_size(dst)
 * 
 */
  __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_sizes[0]).src_size = __pyx_t_5;

  /* "h5py/_conv.pyx":272
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)
 *     sizes[0].dst_size = H5Tget_size(dst)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 272; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_sizes[0]).dst_size = __pyx_t_5;

  /* "h5py/_conv.pyx":274
 *     sizes[0].dst_size = H5Tget_size(dst)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t init_fixed2vlen(hid_t src, hid_t dst, void** priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.init_vlen2fixed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":276
 *     return 0
 * 
 * cdef herr_t init_fixed2vlen(hid_t src, hid_t dst, void** priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef conv_size_t *sizes
 */

static herr_t __pyx_f_4h5py_5_conv_init_fixed2vlen(hid_t __pyx_v_src, hid_t __pyx_v_dst, void **__pyx_v_priv) {
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  htri_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("init_fixed2vlen", 0);

  /* "h5py/_conv.pyx":280
 *     cdef conv_size_t *sizes
 * 
 *     if not (H5Tis_variable_str(dst) and (not H5Tis_variable_str(src))):             # <<<<<<<<<<<<<<
 *         return -2
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_1) {
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = (!__pyx_t_2);
    __pyx_t_4 = __pyx_t_3;
  } else {
    __pyx_t_4 = __pyx_t_1;
  }
  __pyx_t_3 = (!__pyx_t_4);
  if (__pyx_t_3) {

    /* "h5py/_conv.pyx":281
 * 
 *     if not (H5Tis_variable_str(dst) and (not H5Tis_variable_str(src))):
 *         return -2             # <<<<<<<<<<<<<<
 * 
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 */
    __pyx_r = -2;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":283
 *         return -2
 * 
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))             # <<<<<<<<<<<<<<
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)malloc((sizeof(__pyx_t_4h5py_5_conv_conv_size_t))));

  /* "h5py/_conv.pyx":284
 * 
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 *     priv[0] = sizes             # <<<<<<<<<<<<<<
 *     sizes[0].src_size = H5Tget_size(src)
 *     sizes[0].dst_size = H5Tget_size(dst)
 */
  (__pyx_v_priv[0]) = __pyx_v_sizes;

  /* "h5py/_conv.pyx":285
 *     sizes = <conv_size_t*>malloc(sizeof(conv_size_t))
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)             # <<<<<<<<<<<<<<
 *     sizes[0].dst_size = H5Tget_size(dst)
 * 
 */
  __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_sizes[0]).src_size = __pyx_t_5;

  /* "h5py/_conv.pyx":286
 *     priv[0] = sizes
 *     sizes[0].src_size = H5Tget_size(src)
 *     sizes[0].dst_size = H5Tget_size(dst)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_sizes[0]).dst_size = __pyx_t_5;

  /* "h5py/_conv.pyx":288
 *     sizes[0].dst_size = H5Tget_size(dst)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int conv_vlen2fixed(void* ipt, void* opt, void* bkg, void* priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.init_fixed2vlen", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":290
 *     return 0
 * 
 * cdef int conv_vlen2fixed(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef char** buf_vlen = <char**>ipt
 */

static int __pyx_f_4h5py_5_conv_conv_vlen2fixed(void *__pyx_v_ipt, void *__pyx_v_opt, CYTHON_UNUSED void *__pyx_v_bkg, void *__pyx_v_priv) {
  char **__pyx_v_buf_vlen;
  char *__pyx_v_buf_fixed;
  char *__pyx_v_temp_string;
  size_t __pyx_v_temp_string_len;
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("conv_vlen2fixed", 0);

  /* "h5py/_conv.pyx":292
 * cdef int conv_vlen2fixed(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef char** buf_vlen = <char**>ipt             # <<<<<<<<<<<<<<
 *     cdef char* buf_fixed = <char*>opt
 *     cdef char* temp_string = NULL
 */
  __pyx_v_buf_vlen = ((char **)__pyx_v_ipt);

  /* "h5py/_conv.pyx":293
 * 
 *     cdef char** buf_vlen = <char**>ipt
 *     cdef char* buf_fixed = <char*>opt             # <<<<<<<<<<<<<<
 *     cdef char* temp_string = NULL
 *     cdef size_t temp_string_len = 0  # Without null term
 */
  __pyx_v_buf_fixed = ((char *)__pyx_v_opt);

  /* "h5py/_conv.pyx":294
 *     cdef char** buf_vlen = <char**>ipt
 *     cdef char* buf_fixed = <char*>opt
 *     cdef char* temp_string = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t temp_string_len = 0  # Without null term
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 */
  __pyx_v_temp_string = NULL;

  /* "h5py/_conv.pyx":295
 *     cdef char* buf_fixed = <char*>opt
 *     cdef char* temp_string = NULL
 *     cdef size_t temp_string_len = 0  # Without null term             # <<<<<<<<<<<<<<
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 * 
 */
  __pyx_v_temp_string_len = 0;

  /* "h5py/_conv.pyx":296
 *     cdef char* temp_string = NULL
 *     cdef size_t temp_string_len = 0  # Without null term
 *     cdef conv_size_t *sizes = <conv_size_t*>priv             # <<<<<<<<<<<<<<
 * 
 *     if buf_vlen[0] != NULL:
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)__pyx_v_priv);

  /* "h5py/_conv.pyx":298
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 * 
 *     if buf_vlen[0] != NULL:             # <<<<<<<<<<<<<<
 *         temp_string = buf_vlen[0]
 *         temp_string_len = strlen(temp_string)
 */
  __pyx_t_1 = ((__pyx_v_buf_vlen[0]) != NULL);
  if (__pyx_t_1) {

    /* "h5py/_conv.pyx":299
 * 
 *     if buf_vlen[0] != NULL:
 *         temp_string = buf_vlen[0]             # <<<<<<<<<<<<<<
 *         temp_string_len = strlen(temp_string)
 * 
 */
    __pyx_v_temp_string = (__pyx_v_buf_vlen[0]);

    /* "h5py/_conv.pyx":300
 *     if buf_vlen[0] != NULL:
 *         temp_string = buf_vlen[0]
 *         temp_string_len = strlen(temp_string)             # <<<<<<<<<<<<<<
 * 
 *         if temp_string_len <= sizes[0].dst_size:
 */
    __pyx_v_temp_string_len = strlen(__pyx_v_temp_string);

    /* "h5py/_conv.pyx":302
 *         temp_string_len = strlen(temp_string)
 * 
 *         if temp_string_len <= sizes[0].dst_size:             # <<<<<<<<<<<<<<
 *             # Pad with zeros
 *             memcpy(buf_fixed, temp_string, temp_string_len)
 */
    __pyx_t_1 = (__pyx_v_temp_string_len <= (__pyx_v_sizes[0]).dst_size);
    if (__pyx_t_1) {

      /* "h5py/_conv.pyx":304
 *         if temp_string_len <= sizes[0].dst_size:
 *             # Pad with zeros
 *             memcpy(buf_fixed, temp_string, temp_string_len)             # <<<<<<<<<<<<<<
 *             memset(buf_fixed + temp_string_len, c'\0', sizes[0].dst_size - temp_string_len)
 *         else:
 */
      memcpy(__pyx_v_buf_fixed, __pyx_v_temp_string, __pyx_v_temp_string_len);

      /* "h5py/_conv.pyx":305
 *             # Pad with zeros
 *             memcpy(buf_fixed, temp_string, temp_string_len)
 *             memset(buf_fixed + temp_string_len, c'\0', sizes[0].dst_size - temp_string_len)             # <<<<<<<<<<<<<<
 *         else:
 *             # Simply truncate the string
 */
      memset((__pyx_v_buf_fixed + __pyx_v_temp_string_len), '\x00', ((__pyx_v_sizes[0]).dst_size - __pyx_v_temp_string_len));
      goto __pyx_L4;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":308
 *         else:
 *             # Simply truncate the string
 *             memcpy(buf_fixed, temp_string, sizes[0].dst_size)             # <<<<<<<<<<<<<<
 *     else:
 *         memset(buf_fixed, c'\0', sizes[0].dst_size)
 */
      memcpy(__pyx_v_buf_fixed, __pyx_v_temp_string, (__pyx_v_sizes[0]).dst_size);
    }
    __pyx_L4:;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":310
 *             memcpy(buf_fixed, temp_string, sizes[0].dst_size)
 *     else:
 *         memset(buf_fixed, c'\0', sizes[0].dst_size)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    memset(__pyx_v_buf_fixed, '\x00', (__pyx_v_sizes[0]).dst_size);
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":312
 *         memset(buf_fixed, c'\0', sizes[0].dst_size)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int conv_fixed2vlen(void* ipt, void* opt, void* bkg, void* priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":314
 *     return 0
 * 
 * cdef int conv_fixed2vlen(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef char** buf_vlen = <char**>opt
 */

static int __pyx_f_4h5py_5_conv_conv_fixed2vlen(void *__pyx_v_ipt, void *__pyx_v_opt, CYTHON_UNUSED void *__pyx_v_bkg, void *__pyx_v_priv) {
  char **__pyx_v_buf_vlen;
  char *__pyx_v_buf_fixed;
  char *__pyx_v_temp_string;
  __pyx_t_4h5py_5_conv_conv_size_t *__pyx_v_sizes;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("conv_fixed2vlen", 0);

  /* "h5py/_conv.pyx":316
 * cdef int conv_fixed2vlen(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef char** buf_vlen = <char**>opt             # <<<<<<<<<<<<<<
 *     cdef char* buf_fixed = <char*>ipt
 *     cdef char* temp_string = NULL
 */
  __pyx_v_buf_vlen = ((char **)__pyx_v_opt);

  /* "h5py/_conv.pyx":317
 * 
 *     cdef char** buf_vlen = <char**>opt
 *     cdef char* buf_fixed = <char*>ipt             # <<<<<<<<<<<<<<
 *     cdef char* temp_string = NULL
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 */
  __pyx_v_buf_fixed = ((char *)__pyx_v_ipt);

  /* "h5py/_conv.pyx":318
 *     cdef char** buf_vlen = <char**>opt
 *     cdef char* buf_fixed = <char*>ipt
 *     cdef char* temp_string = NULL             # <<<<<<<<<<<<<<
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 * 
 */
  __pyx_v_temp_string = NULL;

  /* "h5py/_conv.pyx":319
 *     cdef char* buf_fixed = <char*>ipt
 *     cdef char* temp_string = NULL
 *     cdef conv_size_t *sizes = <conv_size_t*>priv             # <<<<<<<<<<<<<<
 * 
 *     temp_string = <char*>malloc(sizes[0].src_size+1)
 */
  __pyx_v_sizes = ((__pyx_t_4h5py_5_conv_conv_size_t *)__pyx_v_priv);

  /* "h5py/_conv.pyx":321
 *     cdef conv_size_t *sizes = <conv_size_t*>priv
 * 
 *     temp_string = <char*>malloc(sizes[0].src_size+1)             # <<<<<<<<<<<<<<
 *     memcpy(temp_string, buf_fixed, sizes[0].src_size)
 *     temp_string[sizes[0].src_size] = c'\0'
 */
  __pyx_v_temp_string = ((char *)malloc(((__pyx_v_sizes[0]).src_size + 1)));

  /* "h5py/_conv.pyx":322
 * 
 *     temp_string = <char*>malloc(sizes[0].src_size+1)
 *     memcpy(temp_string, buf_fixed, sizes[0].src_size)             # <<<<<<<<<<<<<<
 *     temp_string[sizes[0].src_size] = c'\0'
 * 
 */
  memcpy(__pyx_v_temp_string, __pyx_v_buf_fixed, (__pyx_v_sizes[0]).src_size);

  /* "h5py/_conv.pyx":323
 *     temp_string = <char*>malloc(sizes[0].src_size+1)
 *     memcpy(temp_string, buf_fixed, sizes[0].src_size)
 *     temp_string[sizes[0].src_size] = c'\0'             # <<<<<<<<<<<<<<
 * 
 *     buf_vlen[0] = temp_string
 */
  (__pyx_v_temp_string[(__pyx_v_sizes[0]).src_size]) = '\x00';

  /* "h5py/_conv.pyx":325
 *     temp_string[sizes[0].src_size] = c'\0'
 * 
 *     buf_vlen[0] = temp_string             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  (__pyx_v_buf_vlen[0]) = __pyx_v_temp_string;

  /* "h5py/_conv.pyx":327
 *     buf_vlen[0] = temp_string
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":332
 * # HDF5 references to Python instances of h5r.Reference
 * 
 * cdef int conv_objref2pyref(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt
 */

static int __pyx_f_4h5py_5_conv_conv_objref2pyref(void *__pyx_v_ipt, void *__pyx_v_opt, void *__pyx_v_bkg, CYTHON_UNUSED void *__pyx_v_priv) {
  PyObject **__pyx_v_buf_obj;
  PyObject **__pyx_v_bkg_obj;
  hobj_ref_t *__pyx_v_buf_ref;
  struct __pyx_obj_4h5py_3h5r_Reference *__pyx_v_ref = 0;
  PyObject *__pyx_v_ref_ptr;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("conv_objref2pyref", 0);

  /* "h5py/_conv.pyx":334
 * cdef int conv_objref2pyref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt             # <<<<<<<<<<<<<<
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef hobj_ref_t* buf_ref = <hobj_ref_t*>ipt
 */
  __pyx_v_buf_obj = ((PyObject **)__pyx_v_opt);

  /* "h5py/_conv.pyx":335
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt
 *     cdef PyObject** bkg_obj = <PyObject**>bkg             # <<<<<<<<<<<<<<
 *     cdef hobj_ref_t* buf_ref = <hobj_ref_t*>ipt
 * 
 */
  __pyx_v_bkg_obj = ((PyObject **)__pyx_v_bkg);

  /* "h5py/_conv.pyx":336
 *     cdef PyObject** buf_obj = <PyObject**>opt
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef hobj_ref_t* buf_ref = <hobj_ref_t*>ipt             # <<<<<<<<<<<<<<
 * 
 *     cdef Reference ref = Reference()
 */
  __pyx_v_buf_ref = ((hobj_ref_t *)__pyx_v_ipt);

  /* "h5py/_conv.pyx":338
 *     cdef hobj_ref_t* buf_ref = <hobj_ref_t*>ipt
 * 
 *     cdef Reference ref = Reference()             # <<<<<<<<<<<<<<
 *     cdef PyObject* ref_ptr = NULL
 * 
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5r_Reference)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ref = ((struct __pyx_obj_4h5py_3h5r_Reference *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/_conv.pyx":339
 * 
 *     cdef Reference ref = Reference()
 *     cdef PyObject* ref_ptr = NULL             # <<<<<<<<<<<<<<
 * 
 *     ref.ref.obj_ref = buf_ref[0]
 */
  __pyx_v_ref_ptr = NULL;

  /* "h5py/_conv.pyx":341
 *     cdef PyObject* ref_ptr = NULL
 * 
 *     ref.ref.obj_ref = buf_ref[0]             # <<<<<<<<<<<<<<
 *     ref.typecode = H5R_OBJECT
 * 
 */
  __pyx_v_ref->ref.obj_ref = (__pyx_v_buf_ref[0]);

  /* "h5py/_conv.pyx":342
 * 
 *     ref.ref.obj_ref = buf_ref[0]
 *     ref.typecode = H5R_OBJECT             # <<<<<<<<<<<<<<
 * 
 *     ref_ptr = <PyObject*>ref
 */
  __pyx_v_ref->typecode = H5R_OBJECT;

  /* "h5py/_conv.pyx":344
 *     ref.typecode = H5R_OBJECT
 * 
 *     ref_ptr = <PyObject*>ref             # <<<<<<<<<<<<<<
 *     Py_INCREF(ref_ptr)  # because Cython discards its reference when the
 *                         # function exits
 */
  __pyx_v_ref_ptr = ((PyObject *)__pyx_v_ref);

  /* "h5py/_conv.pyx":345
 * 
 *     ref_ptr = <PyObject*>ref
 *     Py_INCREF(ref_ptr)  # because Cython discards its reference when the             # <<<<<<<<<<<<<<
 *                         # function exits
 * 
 */
  Py_INCREF(__pyx_v_ref_ptr);

  /* "h5py/_conv.pyx":348
 *                         # function exits
 * 
 *     Py_XDECREF(bkg_obj[0])             # <<<<<<<<<<<<<<
 *     buf_obj[0] = ref_ptr
 * 
 */
  Py_XDECREF((__pyx_v_bkg_obj[0]));

  /* "h5py/_conv.pyx":349
 * 
 *     Py_XDECREF(bkg_obj[0])
 *     buf_obj[0] = ref_ptr             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  (__pyx_v_buf_obj[0]) = __pyx_v_ref_ptr;

  /* "h5py/_conv.pyx":351
 *     buf_obj[0] = ref_ptr
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int conv_pyref2objref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py._conv.conv_objref2pyref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ref);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":353
 *     return 0
 * 
 * cdef int conv_pyref2objref(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 */

static int __pyx_f_4h5py_5_conv_conv_pyref2objref(void *__pyx_v_ipt, void *__pyx_v_opt, CYTHON_UNUSED void *__pyx_v_bkg, CYTHON_UNUSED void *__pyx_v_priv) {
  PyObject **__pyx_v_buf_obj;
  hobj_ref_t *__pyx_v_buf_ref;
  PyObject *__pyx_v_obj = 0;
  struct __pyx_obj_4h5py_3h5r_Reference *__pyx_v_ref = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  hobj_ref_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("conv_pyref2objref", 0);

  /* "h5py/_conv.pyx":355
 * cdef int conv_pyref2objref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt             # <<<<<<<<<<<<<<
 *     cdef hobj_ref_t* buf_ref = <hobj_ref_t*>opt
 * 
 */
  __pyx_v_buf_obj = ((PyObject **)__pyx_v_ipt);

  /* "h5py/_conv.pyx":356
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 *     cdef hobj_ref_t* buf_ref = <hobj_ref_t*>opt             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */
  __pyx_v_buf_ref = ((hobj_ref_t *)__pyx_v_opt);

  /* "h5py/_conv.pyx":361
 *     cdef Reference ref
 * 
 *     if buf_obj[0] != NULL and buf_obj[0] != Py_None:             # <<<<<<<<<<<<<<
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, Reference):
 */
  __pyx_t_1 = ((__pyx_v_buf_obj[0]) != NULL);
  if (__pyx_t_1) {
    __pyx_t_2 = ((__pyx_v_buf_obj[0]) != Py_None);
    __pyx_t_3 = __pyx_t_2;
  } else {
    __pyx_t_3 = __pyx_t_1;
  }
  if (__pyx_t_3) {

    /* "h5py/_conv.pyx":362
 * 
 *     if buf_obj[0] != NULL and buf_obj[0] != Py_None:
 *         obj = <object>(buf_obj[0])             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, Reference):
 *             raise TypeError("Can't convert incompatible object to HDF5 object reference")
 */
    __Pyx_INCREF(((PyObject *)(__pyx_v_buf_obj[0])));
    __pyx_v_obj = ((PyObject *)(__pyx_v_buf_obj[0]));

    /* "h5py/_conv.pyx":363
 *     if buf_obj[0] != NULL and buf_obj[0] != Py_None:
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, Reference):             # <<<<<<<<<<<<<<
 *             raise TypeError("Can't convert incompatible object to HDF5 object reference")
 *         ref = <Reference>(buf_obj[0])
 */
    __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_obj, ((PyObject*)__pyx_ptype_4h5py_3h5r_Reference)); 
    __pyx_t_1 = (!__pyx_t_3);
    if (__pyx_t_1) {

      /* "h5py/_conv.pyx":364
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, Reference):
 *             raise TypeError("Can't convert incompatible object to HDF5 object reference")             # <<<<<<<<<<<<<<
 *         ref = <Reference>(buf_obj[0])
 *         buf_ref[0] = ref.ref.obj_ref
 */
      __pyx_t_4 = PyObject_Call(__pyx_builtin_TypeError, ((PyObject *)__pyx_k_tuple_10), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 364; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 364; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L4;
    }
    __pyx_L4:;

    /* "h5py/_conv.pyx":365
 *         if not isinstance(obj, Reference):
 *             raise TypeError("Can't convert incompatible object to HDF5 object reference")
 *         ref = <Reference>(buf_obj[0])             # <<<<<<<<<<<<<<
 *         buf_ref[0] = ref.ref.obj_ref
 *     else:
 */
    __Pyx_INCREF(((PyObject *)((struct __pyx_obj_4h5py_3h5r_Reference *)(__pyx_v_buf_obj[0]))));
    __pyx_v_ref = ((struct __pyx_obj_4h5py_3h5r_Reference *)(__pyx_v_buf_obj[0]));

    /* "h5py/_conv.pyx":366
 *             raise TypeError("Can't convert incompatible object to HDF5 object reference")
 *         ref = <Reference>(buf_obj[0])
 *         buf_ref[0] = ref.ref.obj_ref             # <<<<<<<<<<<<<<
 *     else:
 *         memset(buf_ref, c'\0', sizeof(hobj_ref_t))
 */
    __pyx_t_5 = __pyx_v_ref->ref.obj_ref;
    (__pyx_v_buf_ref[0]) = __pyx_t_5;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":368
 *         buf_ref[0] = ref.ref.obj_ref
 *     else:
 *         memset(buf_ref, c'\0', sizeof(hobj_ref_t))             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    memset(__pyx_v_buf_ref, '\x00', (sizeof(hobj_ref_t)));
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":370
 *         memset(buf_ref, c'\0', sizeof(hobj_ref_t))
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int conv_regref2pyref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py._conv.conv_pyref2objref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF((PyObject *)__pyx_v_ref);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":372
 *     return 0
 * 
 * cdef int conv_regref2pyref(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt
 */

static int __pyx_f_4h5py_5_conv_conv_regref2pyref(void *__pyx_v_ipt, void *__pyx_v_opt, void *__pyx_v_bkg, CYTHON_UNUSED void *__pyx_v_priv) {
  PyObject **__pyx_v_buf_obj;
  PyObject **__pyx_v_bkg_obj;
  hdset_reg_ref_t *__pyx_v_buf_ref;
  struct __pyx_obj_4h5py_3h5r_RegionReference *__pyx_v_ref = 0;
  PyObject *__pyx_v_ref_ptr;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("conv_regref2pyref", 0);

  /* "h5py/_conv.pyx":374
 * cdef int conv_regref2pyref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt             # <<<<<<<<<<<<<<
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef hdset_reg_ref_t* buf_ref = <hdset_reg_ref_t*>ipt
 */
  __pyx_v_buf_obj = ((PyObject **)__pyx_v_opt);

  /* "h5py/_conv.pyx":375
 * 
 *     cdef PyObject** buf_obj = <PyObject**>opt
 *     cdef PyObject** bkg_obj = <PyObject**>bkg             # <<<<<<<<<<<<<<
 *     cdef hdset_reg_ref_t* buf_ref = <hdset_reg_ref_t*>ipt
 * 
 */
  __pyx_v_bkg_obj = ((PyObject **)__pyx_v_bkg);

  /* "h5py/_conv.pyx":376
 *     cdef PyObject** buf_obj = <PyObject**>opt
 *     cdef PyObject** bkg_obj = <PyObject**>bkg
 *     cdef hdset_reg_ref_t* buf_ref = <hdset_reg_ref_t*>ipt             # <<<<<<<<<<<<<<
 * 
 *     cdef RegionReference ref = RegionReference()
 */
  __pyx_v_buf_ref = ((hdset_reg_ref_t *)__pyx_v_ipt);

  /* "h5py/_conv.pyx":378
 *     cdef hdset_reg_ref_t* buf_ref = <hdset_reg_ref_t*>ipt
 * 
 *     cdef RegionReference ref = RegionReference()             # <<<<<<<<<<<<<<
 *     cdef PyObject* ref_ptr = NULL
 * 
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5r_RegionReference)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 378; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ref = ((struct __pyx_obj_4h5py_3h5r_RegionReference *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/_conv.pyx":379
 * 
 *     cdef RegionReference ref = RegionReference()
 *     cdef PyObject* ref_ptr = NULL             # <<<<<<<<<<<<<<
 * 
 *     memcpy(ref.ref.reg_ref, buf_ref, sizeof(hdset_reg_ref_t))
 */
  __pyx_v_ref_ptr = NULL;

  /* "h5py/_conv.pyx":381
 *     cdef PyObject* ref_ptr = NULL
 * 
 *     memcpy(ref.ref.reg_ref, buf_ref, sizeof(hdset_reg_ref_t))             # <<<<<<<<<<<<<<
 * 
 *     ref.typecode = H5R_DATASET_REGION
 */
  memcpy(__pyx_v_ref->__pyx_base.ref.reg_ref, __pyx_v_buf_ref, (sizeof(hdset_reg_ref_t)));

  /* "h5py/_conv.pyx":383
 *     memcpy(ref.ref.reg_ref, buf_ref, sizeof(hdset_reg_ref_t))
 * 
 *     ref.typecode = H5R_DATASET_REGION             # <<<<<<<<<<<<<<
 * 
 *     ref_ptr = <PyObject*>ref
 */
  __pyx_v_ref->__pyx_base.typecode = H5R_DATASET_REGION;

  /* "h5py/_conv.pyx":385
 *     ref.typecode = H5R_DATASET_REGION
 * 
 *     ref_ptr = <PyObject*>ref             # <<<<<<<<<<<<<<
 *     Py_INCREF(ref_ptr)  # because Cython discards its reference when the
 *                         # function exits
 */
  __pyx_v_ref_ptr = ((PyObject *)__pyx_v_ref);

  /* "h5py/_conv.pyx":386
 * 
 *     ref_ptr = <PyObject*>ref
 *     Py_INCREF(ref_ptr)  # because Cython discards its reference when the             # <<<<<<<<<<<<<<
 *                         # function exits
 * 
 */
  Py_INCREF(__pyx_v_ref_ptr);

  /* "h5py/_conv.pyx":389
 *                         # function exits
 * 
 *     Py_XDECREF(bkg_obj[0])             # <<<<<<<<<<<<<<
 *     buf_obj[0] = ref_ptr
 * 
 */
  Py_XDECREF((__pyx_v_bkg_obj[0]));

  /* "h5py/_conv.pyx":390
 * 
 *     Py_XDECREF(bkg_obj[0])
 *     buf_obj[0] = ref_ptr             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  (__pyx_v_buf_obj[0]) = __pyx_v_ref_ptr;

  /* "h5py/_conv.pyx":392
 *     buf_obj[0] = ref_ptr
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int conv_pyref2regref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py._conv.conv_regref2pyref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ref);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":394
 *     return 0
 * 
 * cdef int conv_pyref2regref(void* ipt, void* opt, void* bkg, void* priv) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 */

static int __pyx_f_4h5py_5_conv_conv_pyref2regref(void *__pyx_v_ipt, void *__pyx_v_opt, CYTHON_UNUSED void *__pyx_v_bkg, CYTHON_UNUSED void *__pyx_v_priv) {
  PyObject **__pyx_v_buf_obj;
  hdset_reg_ref_t *__pyx_v_buf_ref;
  PyObject *__pyx_v_obj = 0;
  struct __pyx_obj_4h5py_3h5r_RegionReference *__pyx_v_ref = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("conv_pyref2regref", 0);

  /* "h5py/_conv.pyx":396
 * cdef int conv_pyref2regref(void* ipt, void* opt, void* bkg, void* priv) except -1:
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt             # <<<<<<<<<<<<<<
 *     cdef hdset_reg_ref_t* buf_ref = <hdset_reg_ref_t*>opt
 * 
 */
  __pyx_v_buf_obj = ((PyObject **)__pyx_v_ipt);

  /* "h5py/_conv.pyx":397
 * 
 *     cdef PyObject** buf_obj = <PyObject**>ipt
 *     cdef hdset_reg_ref_t* buf_ref = <hdset_reg_ref_t*>opt             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */
  __pyx_v_buf_ref = ((hdset_reg_ref_t *)__pyx_v_opt);

  /* "h5py/_conv.pyx":402
 *     cdef RegionReference ref
 * 
 *     if buf_obj[0] != NULL and buf_obj[0] != Py_None:             # <<<<<<<<<<<<<<
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, RegionReference):
 */
  __pyx_t_1 = ((__pyx_v_buf_obj[0]) != NULL);
  if (__pyx_t_1) {
    __pyx_t_2 = ((__pyx_v_buf_obj[0]) != Py_None);
    __pyx_t_3 = __pyx_t_2;
  } else {
    __pyx_t_3 = __pyx_t_1;
  }
  if (__pyx_t_3) {

    /* "h5py/_conv.pyx":403
 * 
 *     if buf_obj[0] != NULL and buf_obj[0] != Py_None:
 *         obj = <object>(buf_obj[0])             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, RegionReference):
 *             raise TypeError("Can't convert incompatible object to HDF5 region reference")
 */
    __Pyx_INCREF(((PyObject *)(__pyx_v_buf_obj[0])));
    __pyx_v_obj = ((PyObject *)(__pyx_v_buf_obj[0]));

    /* "h5py/_conv.pyx":404
 *     if buf_obj[0] != NULL and buf_obj[0] != Py_None:
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, RegionReference):             # <<<<<<<<<<<<<<
 *             raise TypeError("Can't convert incompatible object to HDF5 region reference")
 *         ref = <RegionReference>(buf_obj[0])
 */
    __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_obj, ((PyObject*)__pyx_ptype_4h5py_3h5r_RegionReference)); 
    __pyx_t_1 = (!__pyx_t_3);
    if (__pyx_t_1) {

      /* "h5py/_conv.pyx":405
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, RegionReference):
 *             raise TypeError("Can't convert incompatible object to HDF5 region reference")             # <<<<<<<<<<<<<<
 *         ref = <RegionReference>(buf_obj[0])
 *         memcpy(buf_ref, ref.ref.reg_ref, sizeof(hdset_reg_ref_t))
 */
      __pyx_t_4 = PyObject_Call(__pyx_builtin_TypeError, ((PyObject *)__pyx_k_tuple_12), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L4;
    }
    __pyx_L4:;

    /* "h5py/_conv.pyx":406
 *         if not isinstance(obj, RegionReference):
 *             raise TypeError("Can't convert incompatible object to HDF5 region reference")
 *         ref = <RegionReference>(buf_obj[0])             # <<<<<<<<<<<<<<
 *         memcpy(buf_ref, ref.ref.reg_ref, sizeof(hdset_reg_ref_t))
 *     else:
 */
    __Pyx_INCREF(((PyObject *)((struct __pyx_obj_4h5py_3h5r_RegionReference *)(__pyx_v_buf_obj[0]))));
    __pyx_v_ref = ((struct __pyx_obj_4h5py_3h5r_RegionReference *)(__pyx_v_buf_obj[0]));

    /* "h5py/_conv.pyx":407
 *             raise TypeError("Can't convert incompatible object to HDF5 region reference")
 *         ref = <RegionReference>(buf_obj[0])
 *         memcpy(buf_ref, ref.ref.reg_ref, sizeof(hdset_reg_ref_t))             # <<<<<<<<<<<<<<
 *     else:
 *         memset(buf_ref, c'\0', sizeof(hdset_reg_ref_t))
 */
    memcpy(__pyx_v_buf_ref, __pyx_v_ref->__pyx_base.ref.reg_ref, (sizeof(hdset_reg_ref_t)));
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":409
 *         memcpy(buf_ref, ref.ref.reg_ref, sizeof(hdset_reg_ref_t))
 *     else:
 *         memset(buf_ref, c'\0', sizeof(hdset_reg_ref_t))             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    memset(__pyx_v_buf_ref, '\x00', (sizeof(hdset_reg_ref_t)));
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":411
 *         memset(buf_ref, c'\0', sizeof(hdset_reg_ref_t))
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py._conv.conv_pyref2regref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF((PyObject *)__pyx_v_ref);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":417
 * 
 * 
 * cdef herr_t vlen2str(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_vlen2str(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vlen2str", 0);

  /* "h5py/_conv.pyx":421
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl,  conv_vlen2str, init_generic, H5T_BKG_YES)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t str2vlen(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_vlen2str, __pyx_f_4h5py_5_conv_init_generic, H5T_BKG_YES); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.vlen2str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":423
 *              buf_i, bkg_i, dxpl,  conv_vlen2str, init_generic, H5T_BKG_YES)
 * 
 * cdef herr_t str2vlen(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_str2vlen(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("str2vlen", 0);

  /* "h5py/_conv.pyx":427
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_str2vlen, init_generic, H5T_BKG_NO)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t vlen2fixed(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_str2vlen, __pyx_f_4h5py_5_conv_init_generic, H5T_BKG_NO); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 426; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.str2vlen", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":429
 *              buf_i, bkg_i, dxpl, conv_str2vlen, init_generic, H5T_BKG_NO)
 * 
 * cdef herr_t vlen2fixed(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_vlen2fixed(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("vlen2fixed", 0);

  /* "h5py/_conv.pyx":433
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_vlen2fixed, init_vlen2fixed, H5T_BKG_NO)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t fixed2vlen(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_vlen2fixed, __pyx_f_4h5py_5_conv_init_vlen2fixed, H5T_BKG_NO); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 432; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.vlen2fixed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":435
 *              buf_i, bkg_i, dxpl, conv_vlen2fixed, init_vlen2fixed, H5T_BKG_NO)
 * 
 * cdef herr_t fixed2vlen(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_fixed2vlen(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fixed2vlen", 0);

  /* "h5py/_conv.pyx":439
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_fixed2vlen, init_fixed2vlen, H5T_BKG_NO)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t objref2pyref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_fixed2vlen, __pyx_f_4h5py_5_conv_init_fixed2vlen, H5T_BKG_NO); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 438; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.fixed2vlen", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":441
 *              buf_i, bkg_i, dxpl, conv_fixed2vlen, init_fixed2vlen, H5T_BKG_NO)
 * 
 * cdef herr_t objref2pyref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_objref2pyref(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("objref2pyref", 0);

  /* "h5py/_conv.pyx":445
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_objref2pyref, init_generic, H5T_BKG_YES)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t pyref2objref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_objref2pyref, __pyx_f_4h5py_5_conv_init_generic, H5T_BKG_YES); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.objref2pyref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":447
 *              buf_i, bkg_i, dxpl, conv_objref2pyref, init_generic, H5T_BKG_YES)
 * 
 * cdef herr_t pyref2objref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_pyref2objref(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pyref2objref", 0);

  /* "h5py/_conv.pyx":451
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_pyref2objref, init_generic, H5T_BKG_NO)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t regref2pyref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_pyref2objref, __pyx_f_4h5py_5_conv_init_generic, H5T_BKG_NO); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.pyref2objref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":453
 *              buf_i, bkg_i, dxpl, conv_pyref2objref, init_generic, H5T_BKG_NO)
 * 
 * cdef herr_t regref2pyref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_regref2pyref(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("regref2pyref", 0);

  /* "h5py/_conv.pyx":457
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_regref2pyref, init_generic, H5T_BKG_YES)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t pyref2regref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_regref2pyref, __pyx_f_4h5py_5_conv_init_generic, H5T_BKG_YES); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.regref2pyref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":459
 *              buf_i, bkg_i, dxpl, conv_regref2pyref, init_generic, H5T_BKG_YES)
 * 
 * cdef herr_t pyref2regref(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_pyref2regref(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pyref2regref", 0);

  /* "h5py/_conv.pyx":463
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return generic_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, conv_pyref2regref, init_generic, H5T_BKG_NO)             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_generic_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_f_4h5py_5_conv_conv_pyref2regref, __pyx_f_4h5py_5_conv_init_generic, H5T_BKG_NO); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.pyref2regref", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":475
 * 
 * 
 * cdef int enum_int_converter_init(hid_t src, hid_t dst,             # <<<<<<<<<<<<<<
 *                                  H5T_cdata_t *cdata, int forward) except -1 with gil:
 *     cdef conv_enum_t *info
 */

static int __pyx_f_4h5py_5_conv_enum_int_converter_init(hid_t __pyx_v_src, hid_t __pyx_v_dst, H5T_cdata_t *__pyx_v_cdata, CYTHON_UNUSED int __pyx_v_forward) {
  struct __pyx_t_4h5py_5_conv_conv_enum_t *__pyx_v_info;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  struct __pyx_t_4h5py_5_conv_conv_enum_t *__pyx_t_1;
  size_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("enum_int_converter_init", 0);

  /* "h5py/_conv.pyx":479
 *     cdef conv_enum_t *info
 * 
 *     cdata[0].need_bkg = H5T_BKG_NO             # <<<<<<<<<<<<<<
 *     cdata[0].priv = info = <conv_enum_t*>malloc(sizeof(conv_enum_t))
 *     info[0].src_size = H5Tget_size(src)
 */
  (__pyx_v_cdata[0]).need_bkg = H5T_BKG_NO;

  /* "h5py/_conv.pyx":480
 * 
 *     cdata[0].need_bkg = H5T_BKG_NO
 *     cdata[0].priv = info = <conv_enum_t*>malloc(sizeof(conv_enum_t))             # <<<<<<<<<<<<<<
 *     info[0].src_size = H5Tget_size(src)
 *     info[0].dst_size = H5Tget_size(dst)
 */
  __pyx_t_1 = ((struct __pyx_t_4h5py_5_conv_conv_enum_t *)malloc((sizeof(struct __pyx_t_4h5py_5_conv_conv_enum_t))));
  (__pyx_v_cdata[0]).priv = __pyx_t_1;
  __pyx_v_info = __pyx_t_1;

  /* "h5py/_conv.pyx":481
 *     cdata[0].need_bkg = H5T_BKG_NO
 *     cdata[0].priv = info = <conv_enum_t*>malloc(sizeof(conv_enum_t))
 *     info[0].src_size = H5Tget_size(src)             # <<<<<<<<<<<<<<
 *     info[0].dst_size = H5Tget_size(dst)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 481; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_info[0]).src_size = __pyx_t_2;

  /* "h5py/_conv.pyx":482
 *     cdata[0].priv = info = <conv_enum_t*>malloc(sizeof(conv_enum_t))
 *     info[0].src_size = H5Tget_size(src)
 *     info[0].dst_size = H5Tget_size(dst)             # <<<<<<<<<<<<<<
 * 
 * cdef void enum_int_converter_free(H5T_cdata_t *cdata):
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 482; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_info[0]).dst_size = __pyx_t_2;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.enum_int_converter_init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "h5py/_conv.pyx":484
 *     info[0].dst_size = H5Tget_size(dst)
 * 
 * cdef void enum_int_converter_free(H5T_cdata_t *cdata):             # <<<<<<<<<<<<<<
 *     cdef conv_enum_t *info
 * 
 */

static void __pyx_f_4h5py_5_conv_enum_int_converter_free(H5T_cdata_t *__pyx_v_cdata) {
  struct __pyx_t_4h5py_5_conv_conv_enum_t *__pyx_v_info;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enum_int_converter_free", 0);

  /* "h5py/_conv.pyx":487
 *     cdef conv_enum_t *info
 * 
 *     info = <conv_enum_t*>cdata[0].priv             # <<<<<<<<<<<<<<
 *     free(info)
 *     cdata[0].priv = NULL
 */
  __pyx_v_info = ((struct __pyx_t_4h5py_5_conv_conv_enum_t *)(__pyx_v_cdata[0]).priv);

  /* "h5py/_conv.pyx":488
 * 
 *     info = <conv_enum_t*>cdata[0].priv
 *     free(info)             # <<<<<<<<<<<<<<
 *     cdata[0].priv = NULL
 * 
 */
  free(__pyx_v_info);

  /* "h5py/_conv.pyx":489
 *     info = <conv_enum_t*>cdata[0].priv
 *     free(info)
 *     cdata[0].priv = NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
  (__pyx_v_cdata[0]).priv = NULL;

  __Pyx_RefNannyFinishContext();
}

/* "h5py/_conv.pyx":492
 * 
 * 
 * cdef int enum_int_converter_conv(hid_t src, hid_t dst, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                                   size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                                  void *bkg_i, hid_t dxpl, int forward) except -1 with gil:
 */

static int __pyx_f_4h5py_5_conv_enum_int_converter_conv(hid_t __pyx_v_src, hid_t __pyx_v_dst, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, CYTHON_UNUSED size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, CYTHON_UNUSED void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl, int __pyx_v_forward) {
  struct __pyx_t_4h5py_5_conv_conv_enum_t *__pyx_v_info;
  size_t __pyx_v_nalloc;
  int __pyx_v_i;
  char *__pyx_v_cbuf;
  char *__pyx_v_buf;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  htri_t __pyx_t_2;
  int __pyx_t_3;
  herr_t __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("enum_int_converter_conv", 0);

  /* "h5py/_conv.pyx":498
 *     cdef size_t nalloc
 *     cdef int i
 *     cdef char* cbuf = NULL             # <<<<<<<<<<<<<<
 *     cdef char* buf = <char*>buf_i
 * 
 */
  __pyx_v_cbuf = NULL;

  /* "h5py/_conv.pyx":499
 *     cdef int i
 *     cdef char* cbuf = NULL
 *     cdef char* buf = <char*>buf_i             # <<<<<<<<<<<<<<
 * 
 *     info = <conv_enum_t*>cdata[0].priv
 */
  __pyx_v_buf = ((char *)__pyx_v_buf_i);

  /* "h5py/_conv.pyx":501
 *     cdef char* buf = <char*>buf_i
 * 
 *     info = <conv_enum_t*>cdata[0].priv             # <<<<<<<<<<<<<<
 * 
 *     if forward:
 */
  __pyx_v_info = ((struct __pyx_t_4h5py_5_conv_conv_enum_t *)(__pyx_v_cdata[0]).priv);

  /* "h5py/_conv.pyx":503
 *     info = <conv_enum_t*>cdata[0].priv
 * 
 *     if forward:             # <<<<<<<<<<<<<<
 *         info[0].supertype = H5Tget_super(src)
 *         info[0].identical = H5Tequal(info[0].supertype, dst)
 */
  if (__pyx_v_forward) {

    /* "h5py/_conv.pyx":504
 * 
 *     if forward:
 *         info[0].supertype = H5Tget_super(src)             # <<<<<<<<<<<<<<
 *         info[0].identical = H5Tequal(info[0].supertype, dst)
 *     else:
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_super(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 504; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    (__pyx_v_info[0]).supertype = __pyx_t_1;

    /* "h5py/_conv.pyx":505
 *     if forward:
 *         info[0].supertype = H5Tget_super(src)
 *         info[0].identical = H5Tequal(info[0].supertype, dst)             # <<<<<<<<<<<<<<
 *     else:
 *         info[0].supertype = H5Tget_super(dst)
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tequal((__pyx_v_info[0]).supertype, __pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    (__pyx_v_info[0]).identical = __pyx_t_2;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":507
 *         info[0].identical = H5Tequal(info[0].supertype, dst)
 *     else:
 *         info[0].supertype = H5Tget_super(dst)             # <<<<<<<<<<<<<<
 *         info[0].identical = H5Tequal(info[0].supertype, src)
 * 
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_super(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 507; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    (__pyx_v_info[0]).supertype = __pyx_t_1;

    /* "h5py/_conv.pyx":508
 *     else:
 *         info[0].supertype = H5Tget_super(dst)
 *         info[0].identical = H5Tequal(info[0].supertype, src)             # <<<<<<<<<<<<<<
 * 
 *     # Short-circuit success
 */
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tequal((__pyx_v_info[0]).supertype, __pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 508; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    (__pyx_v_info[0]).identical = __pyx_t_2;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":511
 * 
 *     # Short-circuit success
 *     if info[0].identical:             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  if ((__pyx_v_info[0]).identical) {

    /* "h5py/_conv.pyx":512
 *     # Short-circuit success
 *     if info[0].identical:
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     if buf_stride == 0:
 */
    __pyx_r = 0;
    goto __pyx_L0;
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "h5py/_conv.pyx":514
 *         return 0
 * 
 *     if buf_stride == 0:             # <<<<<<<<<<<<<<
 *         # Contiguous case: call H5Tconvert directly
 *         if forward:
 */
  __pyx_t_3 = (__pyx_v_buf_stride == 0);
  if (__pyx_t_3) {

    /* "h5py/_conv.pyx":516
 *     if buf_stride == 0:
 *         # Contiguous case: call H5Tconvert directly
 *         if forward:             # <<<<<<<<<<<<<<
 *             H5Tconvert(info[0].supertype, dst, nl, buf, NULL, dxpl)
 *         else:
 */
    if (__pyx_v_forward) {

      /* "h5py/_conv.pyx":517
 *         # Contiguous case: call H5Tconvert directly
 *         if forward:
 *             H5Tconvert(info[0].supertype, dst, nl, buf, NULL, dxpl)             # <<<<<<<<<<<<<<
 *         else:
 *             H5Tconvert(src, info[0].supertype, nl, buf, NULL, dxpl)
 */
      __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tconvert((__pyx_v_info[0]).supertype, __pyx_v_dst, __pyx_v_nl, __pyx_v_buf, NULL, __pyx_v_dxpl); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 517; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":519
 *             H5Tconvert(info[0].supertype, dst, nl, buf, NULL, dxpl)
 *         else:
 *             H5Tconvert(src, info[0].supertype, nl, buf, NULL, dxpl)             # <<<<<<<<<<<<<<
 *     else:
 *         # Non-contiguous: gather, convert and then scatter
 */
      __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tconvert(__pyx_v_src, (__pyx_v_info[0]).supertype, __pyx_v_nl, __pyx_v_buf, NULL, __pyx_v_dxpl); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 519; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L6:;
    goto __pyx_L5;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":522
 *     else:
 *         # Non-contiguous: gather, convert and then scatter
 *         if info[0].src_size > info[0].dst_size:             # <<<<<<<<<<<<<<
 *             nalloc = info[0].src_size*nl
 *         else:
 */
    __pyx_t_3 = ((__pyx_v_info[0]).src_size > (__pyx_v_info[0]).dst_size);
    if (__pyx_t_3) {

      /* "h5py/_conv.pyx":523
 *         # Non-contiguous: gather, convert and then scatter
 *         if info[0].src_size > info[0].dst_size:
 *             nalloc = info[0].src_size*nl             # <<<<<<<<<<<<<<
 *         else:
 *             nalloc = info[0].dst_size*nl
 */
      __pyx_v_nalloc = ((__pyx_v_info[0]).src_size * __pyx_v_nl);
      goto __pyx_L7;
    }
    /*else*/ {

      /* "h5py/_conv.pyx":525
 *             nalloc = info[0].src_size*nl
 *         else:
 *             nalloc = info[0].dst_size*nl             # <<<<<<<<<<<<<<
 * 
 *         cbuf = <char*>malloc(nalloc)
 */
      __pyx_v_nalloc = ((__pyx_v_info[0]).dst_size * __pyx_v_nl);
    }
    __pyx_L7:;

    /* "h5py/_conv.pyx":527
 *             nalloc = info[0].dst_size*nl
 * 
 *         cbuf = <char*>malloc(nalloc)             # <<<<<<<<<<<<<<
 *         if cbuf == NULL:
 *             raise MemoryError()
 */
    __pyx_v_cbuf = ((char *)malloc(__pyx_v_nalloc));

    /* "h5py/_conv.pyx":528
 * 
 *         cbuf = <char*>malloc(nalloc)
 *         if cbuf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError()
 *         try:
 */
    __pyx_t_3 = (__pyx_v_cbuf == NULL);
    if (__pyx_t_3) {

      /* "h5py/_conv.pyx":529
 *         cbuf = <char*>malloc(nalloc)
 *         if cbuf == NULL:
 *             raise MemoryError()             # <<<<<<<<<<<<<<
 *         try:
 *             for i from 0<=i<nl:
 */
      PyErr_NoMemory(); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      goto __pyx_L8;
    }
    __pyx_L8:;

    /* "h5py/_conv.pyx":530
 *         if cbuf == NULL:
 *             raise MemoryError()
 *         try:             # <<<<<<<<<<<<<<
 *             for i from 0<=i<nl:
 *                 memcpy(cbuf + (i*info[0].src_size), buf + (i*buf_stride),
 */
    /*try:*/ {

      /* "h5py/_conv.pyx":531
 *             raise MemoryError()
 *         try:
 *             for i from 0<=i<nl:             # <<<<<<<<<<<<<<
 *                 memcpy(cbuf + (i*info[0].src_size), buf + (i*buf_stride),
 *                         info[0].src_size)
 */
      __pyx_t_5 = __pyx_v_nl;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_5; __pyx_v_i++) {

        /* "h5py/_conv.pyx":533
 *             for i from 0<=i<nl:
 *                 memcpy(cbuf + (i*info[0].src_size), buf + (i*buf_stride),
 *                         info[0].src_size)             # <<<<<<<<<<<<<<
 * 
 *             if forward:
 */
        memcpy((__pyx_v_cbuf + (__pyx_v_i * (__pyx_v_info[0]).src_size)), (__pyx_v_buf + (__pyx_v_i * __pyx_v_buf_stride)), (__pyx_v_info[0]).src_size);
      }

      /* "h5py/_conv.pyx":535
 *                         info[0].src_size)
 * 
 *             if forward:             # <<<<<<<<<<<<<<
 *                 H5Tconvert(info[0].supertype, dst, nl, cbuf, NULL, dxpl)
 *             else:
 */
      if (__pyx_v_forward) {

        /* "h5py/_conv.pyx":536
 * 
 *             if forward:
 *                 H5Tconvert(info[0].supertype, dst, nl, cbuf, NULL, dxpl)             # <<<<<<<<<<<<<<
 *             else:
 *                 H5Tconvert(src, info[0].supertype, nl, cbuf, NULL, dxpl)
 */
        __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tconvert((__pyx_v_info[0]).supertype, __pyx_v_dst, __pyx_v_nl, __pyx_v_cbuf, NULL, __pyx_v_dxpl); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L10;}
        goto __pyx_L14;
      }
      /*else*/ {

        /* "h5py/_conv.pyx":538
 *                 H5Tconvert(info[0].supertype, dst, nl, cbuf, NULL, dxpl)
 *             else:
 *                 H5Tconvert(src, info[0].supertype, nl, cbuf, NULL, dxpl)             # <<<<<<<<<<<<<<
 * 
 *             for i from 0<=i<nl:
 */
        __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tconvert(__pyx_v_src, (__pyx_v_info[0]).supertype, __pyx_v_nl, __pyx_v_cbuf, NULL, __pyx_v_dxpl); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 538; __pyx_clineno = __LINE__; goto __pyx_L10;}
      }
      __pyx_L14:;

      /* "h5py/_conv.pyx":540
 *                 H5Tconvert(src, info[0].supertype, nl, cbuf, NULL, dxpl)
 * 
 *             for i from 0<=i<nl:             # <<<<<<<<<<<<<<
 *                 memcpy(buf + (i*buf_stride), cbuf + (i*info[0].dst_size),
 *                         info[0].dst_size)
 */
      __pyx_t_5 = __pyx_v_nl;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_5; __pyx_v_i++) {

        /* "h5py/_conv.pyx":542
 *             for i from 0<=i<nl:
 *                 memcpy(buf + (i*buf_stride), cbuf + (i*info[0].dst_size),
 *                         info[0].dst_size)             # <<<<<<<<<<<<<<
 *         finally:
 *             free(cbuf)
 */
        memcpy((__pyx_v_buf + (__pyx_v_i * __pyx_v_buf_stride)), (__pyx_v_cbuf + (__pyx_v_i * (__pyx_v_info[0]).dst_size)), (__pyx_v_info[0]).dst_size);
      }
    }

    /* "h5py/_conv.pyx":544
 *                         info[0].dst_size)
 *         finally:
 *             free(cbuf)             # <<<<<<<<<<<<<<
 *             cbuf = NULL
 *     return 0
 */
    /*finally:*/ {
      int __pyx_why;
      PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
      int __pyx_exc_lineno;
      __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
      __pyx_why = 0; goto __pyx_L11;
      __pyx_L10: {
        __pyx_why = 4;
        __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
        __pyx_exc_lineno = __pyx_lineno;
        goto __pyx_L11;
      }
      __pyx_L11:;
      free(__pyx_v_cbuf);

      /* "h5py/_conv.pyx":545
 *         finally:
 *             free(cbuf)
 *             cbuf = NULL             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
      __pyx_v_cbuf = NULL;
      switch (__pyx_why) {
        case 4: {
          __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
          __pyx_lineno = __pyx_exc_lineno;
          __pyx_exc_type = 0;
          __pyx_exc_value = 0;
          __pyx_exc_tb = 0;
          goto __pyx_L1_error;
        }
      }
    }
  }
  __pyx_L5:;

  /* "h5py/_conv.pyx":546
 *             free(cbuf)
 *             cbuf = NULL
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.enum_int_converter_conv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "h5py/_conv.pyx":550
 * 
 * # Direction ("forward"): 1 = enum to int, 0 = int to enum
 * cdef herr_t enum_int_converter(hid_t src, hid_t dst, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                                void *bkg_i, hid_t dxpl, int forward) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_enum_int_converter(hid_t __pyx_v_src, hid_t __pyx_v_dst, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl, int __pyx_v_forward) {
  int __pyx_v_command;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  H5T_cmd_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enum_int_converter", 0);

  /* "h5py/_conv.pyx":554
 *                                void *bkg_i, hid_t dxpl, int forward) except -1:
 * 
 *     cdef int command = cdata[0].command             # <<<<<<<<<<<<<<
 * 
 *     if command == H5T_CONV_INIT:
 */
  __pyx_t_1 = (__pyx_v_cdata[0]).command;
  __pyx_v_command = __pyx_t_1;

  /* "h5py/_conv.pyx":556
 *     cdef int command = cdata[0].command
 * 
 *     if command == H5T_CONV_INIT:             # <<<<<<<<<<<<<<
 *         enum_int_converter_init(src, dst, cdata, forward)
 *     elif command == H5T_CONV_FREE:
 */
  __pyx_t_2 = (__pyx_v_command == H5T_CONV_INIT);
  if (__pyx_t_2) {

    /* "h5py/_conv.pyx":557
 * 
 *     if command == H5T_CONV_INIT:
 *         enum_int_converter_init(src, dst, cdata, forward)             # <<<<<<<<<<<<<<
 *     elif command == H5T_CONV_FREE:
 *         enum_int_converter_free(cdata)
 */
    __pyx_t_3 = __pyx_f_4h5py_5_conv_enum_int_converter_init(__pyx_v_src, __pyx_v_dst, __pyx_v_cdata, __pyx_v_forward); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 557; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }

  /* "h5py/_conv.pyx":558
 *     if command == H5T_CONV_INIT:
 *         enum_int_converter_init(src, dst, cdata, forward)
 *     elif command == H5T_CONV_FREE:             # <<<<<<<<<<<<<<
 *         enum_int_converter_free(cdata)
 *     elif command == H5T_CONV_CONV:
 */
  __pyx_t_2 = (__pyx_v_command == H5T_CONV_FREE);
  if (__pyx_t_2) {

    /* "h5py/_conv.pyx":559
 *         enum_int_converter_init(src, dst, cdata, forward)
 *     elif command == H5T_CONV_FREE:
 *         enum_int_converter_free(cdata)             # <<<<<<<<<<<<<<
 *     elif command == H5T_CONV_CONV:
 *         return enum_int_converter_conv(src, dst, cdata, nl, buf_stride,
 */
    __pyx_f_4h5py_5_conv_enum_int_converter_free(__pyx_v_cdata);
    goto __pyx_L3;
  }

  /* "h5py/_conv.pyx":560
 *     elif command == H5T_CONV_FREE:
 *         enum_int_converter_free(cdata)
 *     elif command == H5T_CONV_CONV:             # <<<<<<<<<<<<<<
 *         return enum_int_converter_conv(src, dst, cdata, nl, buf_stride,
 *                                        bkg_stride, buf_i, bkg_i, dxpl, forward)
 */
  __pyx_t_2 = (__pyx_v_command == H5T_CONV_CONV);
  if (__pyx_t_2) {

    /* "h5py/_conv.pyx":562
 *     elif command == H5T_CONV_CONV:
 *         return enum_int_converter_conv(src, dst, cdata, nl, buf_stride,
 *                                        bkg_stride, buf_i, bkg_i, dxpl, forward)             # <<<<<<<<<<<<<<
 *     else:
 *         return -2
 */
    __pyx_t_3 = __pyx_f_4h5py_5_conv_enum_int_converter_conv(__pyx_v_src, __pyx_v_dst, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, __pyx_v_forward); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 561; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_3;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_conv.pyx":564
 *                                        bkg_stride, buf_i, bkg_i, dxpl, forward)
 *     else:
 *         return -2             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_r = -2;
    goto __pyx_L0;
  }
  __pyx_L3:;

  /* "h5py/_conv.pyx":566
 *         return -2
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.enum_int_converter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":569
 * 
 * 
 * cdef herr_t enum2int(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_enum2int(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enum2int", 0);

  /* "h5py/_conv.pyx":573
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return enum_int_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, 1)             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t int2enum(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_enum_int_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, 1); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 572; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.enum2int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":575
 *              buf_i, bkg_i, dxpl, 1)
 * 
 * cdef herr_t int2enum(hid_t src_id, hid_t dst_id, H5T_cdata_t *cdata,             # <<<<<<<<<<<<<<
 *                     size_t nl, size_t buf_stride, size_t bkg_stride, void *buf_i,
 *                     void *bkg_i, hid_t dxpl) except -1:
 */

static herr_t __pyx_f_4h5py_5_conv_int2enum(hid_t __pyx_v_src_id, hid_t __pyx_v_dst_id, H5T_cdata_t *__pyx_v_cdata, size_t __pyx_v_nl, size_t __pyx_v_buf_stride, size_t __pyx_v_bkg_stride, void *__pyx_v_buf_i, void *__pyx_v_bkg_i, hid_t __pyx_v_dxpl) {
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("int2enum", 0);

  /* "h5py/_conv.pyx":579
 *                     void *bkg_i, hid_t dxpl) except -1:
 *     return enum_int_converter(src_id, dst_id, cdata, nl, buf_stride, bkg_stride,
 *              buf_i, bkg_i, dxpl, 0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_5_conv_enum_int_converter(__pyx_v_src_id, __pyx_v_dst_id, __pyx_v_cdata, __pyx_v_nl, __pyx_v_buf_stride, __pyx_v_bkg_stride, __pyx_v_buf_i, __pyx_v_bkg_i, __pyx_v_dxpl, 0); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 578; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.int2enum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":582
 * 
 * 
 * cpdef int register_converters() except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef hid_t vlstring
 */

static PyObject *__pyx_pw_4h5py_5_conv_3register_converters(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_4h5py_5_conv_register_converters(CYTHON_UNUSED int __pyx_skip_dispatch) {
  hid_t __pyx_v_vlstring;
  hid_t __pyx_v_pyobj;
  hid_t __pyx_v_enum;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  herr_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("register_converters", 0);

  /* "h5py/_conv.pyx":588
 *     cdef hid_t enum
 * 
 *     vlstring = H5Tcopy(H5T_C_S1)             # <<<<<<<<<<<<<<
 *     H5Tset_size(vlstring, H5T_VARIABLE)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tcopy(H5T_C_S1); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_vlstring = __pyx_t_1;

  /* "h5py/_conv.pyx":589
 * 
 *     vlstring = H5Tcopy(H5T_C_S1)
 *     H5Tset_size(vlstring, H5T_VARIABLE)             # <<<<<<<<<<<<<<
 * 
 *     enum = H5Tenum_create(H5T_STD_I32LE)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tset_size(__pyx_v_vlstring, H5T_VARIABLE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 589; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":591
 *     H5Tset_size(vlstring, H5T_VARIABLE)
 * 
 *     enum = H5Tenum_create(H5T_STD_I32LE)             # <<<<<<<<<<<<<<
 * 
 *     pyobj = get_python_obj()
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tenum_create(H5T_STD_I32LE); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_enum = __pyx_t_1;

  /* "h5py/_conv.pyx":593
 *     enum = H5Tenum_create(H5T_STD_I32LE)
 * 
 *     pyobj = get_python_obj()             # <<<<<<<<<<<<<<
 * 
 *     H5Tregister(H5T_PERS_HARD, "vlen2str", vlstring, pyobj, vlen2str)
 */
  __pyx_v_pyobj = __pyx_f_4h5py_5_conv_get_python_obj(0);

  /* "h5py/_conv.pyx":595
 *     pyobj = get_python_obj()
 * 
 *     H5Tregister(H5T_PERS_HARD, "vlen2str", vlstring, pyobj, vlen2str)             # <<<<<<<<<<<<<<
 *     H5Tregister(H5T_PERS_HARD, "str2vlen", pyobj, vlstring, str2vlen)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_HARD, __pyx_k__vlen2str, __pyx_v_vlstring, __pyx_v_pyobj, __pyx_f_4h5py_5_conv_vlen2str); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":596
 * 
 *     H5Tregister(H5T_PERS_HARD, "vlen2str", vlstring, pyobj, vlen2str)
 *     H5Tregister(H5T_PERS_HARD, "str2vlen", pyobj, vlstring, str2vlen)             # <<<<<<<<<<<<<<
 * 
 *     H5Tregister(H5T_PERS_SOFT, "vlen2fixed", vlstring, H5T_C_S1, vlen2fixed)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_HARD, __pyx_k__str2vlen, __pyx_v_pyobj, __pyx_v_vlstring, __pyx_f_4h5py_5_conv_str2vlen); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":598
 *     H5Tregister(H5T_PERS_HARD, "str2vlen", pyobj, vlstring, str2vlen)
 * 
 *     H5Tregister(H5T_PERS_SOFT, "vlen2fixed", vlstring, H5T_C_S1, vlen2fixed)             # <<<<<<<<<<<<<<
 *     H5Tregister(H5T_PERS_SOFT, "fixed2vlen", H5T_C_S1, vlstring, fixed2vlen)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_SOFT, __pyx_k__vlen2fixed, __pyx_v_vlstring, H5T_C_S1, __pyx_f_4h5py_5_conv_vlen2fixed); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 598; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":599
 * 
 *     H5Tregister(H5T_PERS_SOFT, "vlen2fixed", vlstring, H5T_C_S1, vlen2fixed)
 *     H5Tregister(H5T_PERS_SOFT, "fixed2vlen", H5T_C_S1, vlstring, fixed2vlen)             # <<<<<<<<<<<<<<
 * 
 *     H5Tregister(H5T_PERS_HARD, "objref2pyref", H5T_STD_REF_OBJ, pyobj, objref2pyref)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_SOFT, __pyx_k__fixed2vlen, H5T_C_S1, __pyx_v_vlstring, __pyx_f_4h5py_5_conv_fixed2vlen); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":601
 *     H5Tregister(H5T_PERS_SOFT, "fixed2vlen", H5T_C_S1, vlstring, fixed2vlen)
 * 
 *     H5Tregister(H5T_PERS_HARD, "objref2pyref", H5T_STD_REF_OBJ, pyobj, objref2pyref)             # <<<<<<<<<<<<<<
 *     H5Tregister(H5T_PERS_HARD, "pyref2objref", pyobj, H5T_STD_REF_OBJ, pyref2objref)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_HARD, __pyx_k__objref2pyref, H5T_STD_REF_OBJ, __pyx_v_pyobj, __pyx_f_4h5py_5_conv_objref2pyref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":602
 * 
 *     H5Tregister(H5T_PERS_HARD, "objref2pyref", H5T_STD_REF_OBJ, pyobj, objref2pyref)
 *     H5Tregister(H5T_PERS_HARD, "pyref2objref", pyobj, H5T_STD_REF_OBJ, pyref2objref)             # <<<<<<<<<<<<<<
 * 
 *     H5Tregister(H5T_PERS_HARD, "regref2pyref", H5T_STD_REF_DSETREG, pyobj, regref2pyref)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_HARD, __pyx_k__pyref2objref, __pyx_v_pyobj, H5T_STD_REF_OBJ, __pyx_f_4h5py_5_conv_pyref2objref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 602; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":604
 *     H5Tregister(H5T_PERS_HARD, "pyref2objref", pyobj, H5T_STD_REF_OBJ, pyref2objref)
 * 
 *     H5Tregister(H5T_PERS_HARD, "regref2pyref", H5T_STD_REF_DSETREG, pyobj, regref2pyref)             # <<<<<<<<<<<<<<
 *     H5Tregister(H5T_PERS_HARD, "pyref2regref", pyobj, H5T_STD_REF_DSETREG, pyref2regref)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_HARD, __pyx_k__regref2pyref, H5T_STD_REF_DSETREG, __pyx_v_pyobj, __pyx_f_4h5py_5_conv_regref2pyref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 604; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":605
 * 
 *     H5Tregister(H5T_PERS_HARD, "regref2pyref", H5T_STD_REF_DSETREG, pyobj, regref2pyref)
 *     H5Tregister(H5T_PERS_HARD, "pyref2regref", pyobj, H5T_STD_REF_DSETREG, pyref2regref)             # <<<<<<<<<<<<<<
 * 
 *     H5Tregister(H5T_PERS_SOFT, "enum2int", enum, H5T_STD_I32LE, enum2int)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_HARD, __pyx_k__pyref2regref, __pyx_v_pyobj, H5T_STD_REF_DSETREG, __pyx_f_4h5py_5_conv_pyref2regref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 605; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":607
 *     H5Tregister(H5T_PERS_HARD, "pyref2regref", pyobj, H5T_STD_REF_DSETREG, pyref2regref)
 * 
 *     H5Tregister(H5T_PERS_SOFT, "enum2int", enum, H5T_STD_I32LE, enum2int)             # <<<<<<<<<<<<<<
 *     H5Tregister(H5T_PERS_SOFT, "int2enum", H5T_STD_I32LE, enum, int2enum)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_SOFT, __pyx_k__enum2int, __pyx_v_enum, H5T_STD_I32LE, __pyx_f_4h5py_5_conv_enum2int); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 607; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":608
 * 
 *     H5Tregister(H5T_PERS_SOFT, "enum2int", enum, H5T_STD_I32LE, enum2int)
 *     H5Tregister(H5T_PERS_SOFT, "int2enum", H5T_STD_I32LE, enum, int2enum)             # <<<<<<<<<<<<<<
 * 
 *     H5Tclose(vlstring)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tregister(H5T_PERS_SOFT, __pyx_k__int2enum, H5T_STD_I32LE, __pyx_v_enum, __pyx_f_4h5py_5_conv_int2enum); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 608; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":610
 *     H5Tregister(H5T_PERS_SOFT, "int2enum", H5T_STD_I32LE, enum, int2enum)
 * 
 *     H5Tclose(vlstring)             # <<<<<<<<<<<<<<
 *     H5Tclose(enum)
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_vlstring); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 610; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":611
 * 
 *     H5Tclose(vlstring)
 *     H5Tclose(enum)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_enum); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":613
 *     H5Tclose(enum)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cpdef int unregister_converters() except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.register_converters", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_5_conv_3register_converters(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_5_conv_3register_converters(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_converters (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_5_conv_2register_converters(__pyx_self);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":582
 * 
 * 
 * cpdef int register_converters() except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef hid_t vlstring
 */

static PyObject *__pyx_pf_4h5py_5_conv_2register_converters(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("register_converters", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_5_conv_register_converters(0); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 582; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py._conv.register_converters", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":615
 *     return 0
 * 
 * cpdef int unregister_converters() except -1:             # <<<<<<<<<<<<<<
 * 
 *     H5Tunregister(H5T_PERS_HARD, "vlen2str", -1, -1, vlen2str)
 */

static PyObject *__pyx_pw_4h5py_5_conv_5unregister_converters(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_4h5py_5_conv_unregister_converters(CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unregister_converters", 0);

  /* "h5py/_conv.pyx":617
 * cpdef int unregister_converters() except -1:
 * 
 *     H5Tunregister(H5T_PERS_HARD, "vlen2str", -1, -1, vlen2str)             # <<<<<<<<<<<<<<
 *     H5Tunregister(H5T_PERS_HARD, "str2vlen", -1, -1, str2vlen)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_HARD, __pyx_k__vlen2str, -1, -1, __pyx_f_4h5py_5_conv_vlen2str); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 617; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":618
 * 
 *     H5Tunregister(H5T_PERS_HARD, "vlen2str", -1, -1, vlen2str)
 *     H5Tunregister(H5T_PERS_HARD, "str2vlen", -1, -1, str2vlen)             # <<<<<<<<<<<<<<
 * 
 *     H5Tunregister(H5T_PERS_SOFT, "vlen2fixed", -1, -1, vlen2fixed)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_HARD, __pyx_k__str2vlen, -1, -1, __pyx_f_4h5py_5_conv_str2vlen); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 618; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":620
 *     H5Tunregister(H5T_PERS_HARD, "str2vlen", -1, -1, str2vlen)
 * 
 *     H5Tunregister(H5T_PERS_SOFT, "vlen2fixed", -1, -1, vlen2fixed)             # <<<<<<<<<<<<<<
 *     H5Tunregister(H5T_PERS_SOFT, "fixed2vlen", -1, -1, fixed2vlen)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_SOFT, __pyx_k__vlen2fixed, -1, -1, __pyx_f_4h5py_5_conv_vlen2fixed); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 620; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":621
 * 
 *     H5Tunregister(H5T_PERS_SOFT, "vlen2fixed", -1, -1, vlen2fixed)
 *     H5Tunregister(H5T_PERS_SOFT, "fixed2vlen", -1, -1, fixed2vlen)             # <<<<<<<<<<<<<<
 * 
 *     H5Tunregister(H5T_PERS_HARD, "objref2pyref", -1, -1, objref2pyref)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_SOFT, __pyx_k__fixed2vlen, -1, -1, __pyx_f_4h5py_5_conv_fixed2vlen); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 621; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":623
 *     H5Tunregister(H5T_PERS_SOFT, "fixed2vlen", -1, -1, fixed2vlen)
 * 
 *     H5Tunregister(H5T_PERS_HARD, "objref2pyref", -1, -1, objref2pyref)             # <<<<<<<<<<<<<<
 *     H5Tunregister(H5T_PERS_HARD, "pyref2objref", -1, -1, pyref2objref)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_HARD, __pyx_k__objref2pyref, -1, -1, __pyx_f_4h5py_5_conv_objref2pyref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":624
 * 
 *     H5Tunregister(H5T_PERS_HARD, "objref2pyref", -1, -1, objref2pyref)
 *     H5Tunregister(H5T_PERS_HARD, "pyref2objref", -1, -1, pyref2objref)             # <<<<<<<<<<<<<<
 * 
 *     H5Tunregister(H5T_PERS_HARD, "regref2pyref", -1, -1, regref2pyref)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_HARD, __pyx_k__pyref2objref, -1, -1, __pyx_f_4h5py_5_conv_pyref2objref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 624; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":626
 *     H5Tunregister(H5T_PERS_HARD, "pyref2objref", -1, -1, pyref2objref)
 * 
 *     H5Tunregister(H5T_PERS_HARD, "regref2pyref", -1, -1, regref2pyref)             # <<<<<<<<<<<<<<
 *     H5Tunregister(H5T_PERS_HARD, "pyref2regref", -1, -1, pyref2regref)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_HARD, __pyx_k__regref2pyref, -1, -1, __pyx_f_4h5py_5_conv_regref2pyref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":627
 * 
 *     H5Tunregister(H5T_PERS_HARD, "regref2pyref", -1, -1, regref2pyref)
 *     H5Tunregister(H5T_PERS_HARD, "pyref2regref", -1, -1, pyref2regref)             # <<<<<<<<<<<<<<
 * 
 *     H5Tunregister(H5T_PERS_SOFT, "enum2int", -1, -1, enum2int)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_HARD, __pyx_k__pyref2regref, -1, -1, __pyx_f_4h5py_5_conv_pyref2regref); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 627; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":629
 *     H5Tunregister(H5T_PERS_HARD, "pyref2regref", -1, -1, pyref2regref)
 * 
 *     H5Tunregister(H5T_PERS_SOFT, "enum2int", -1, -1, enum2int)             # <<<<<<<<<<<<<<
 *     H5Tunregister(H5T_PERS_SOFT, "int2enum", -1, -1, int2enum)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_SOFT, __pyx_k__enum2int, -1, -1, __pyx_f_4h5py_5_conv_enum2int); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 629; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":630
 * 
 *     H5Tunregister(H5T_PERS_SOFT, "enum2int", -1, -1, enum2int)
 *     H5Tunregister(H5T_PERS_SOFT, "int2enum", -1, -1, int2enum)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tunregister(H5T_PERS_SOFT, __pyx_k__int2enum, -1, -1, __pyx_f_4h5py_5_conv_int2enum); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 630; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/_conv.pyx":632
 *     H5Tunregister(H5T_PERS_SOFT, "int2enum", -1, -1, int2enum)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._conv.unregister_converters", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_5_conv_5unregister_converters(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_5_conv_5unregister_converters(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unregister_converters (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_5_conv_4unregister_converters(__pyx_self);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_conv.pyx":615
 *     return 0
 * 
 * cpdef int unregister_converters() except -1:             # <<<<<<<<<<<<<<
 * 
 *     H5Tunregister(H5T_PERS_HARD, "vlen2str", -1, -1, vlen2str)
 */

static PyObject *__pyx_pf_4h5py_5_conv_4unregister_converters(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unregister_converters", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_5_conv_unregister_converters(0); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 615; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py._conv.unregister_converters", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {__Pyx_NAMESTR("get_python_obj"), (PyCFunction)__pyx_pw_4h5py_5_conv_1get_python_obj, METH_NOARGS, __Pyx_DOCSTR(0)},
  {__Pyx_NAMESTR("register_converters"), (PyCFunction)__pyx_pw_4h5py_5_conv_3register_converters, METH_NOARGS, __Pyx_DOCSTR(0)},
  {__Pyx_NAMESTR("unregister_converters"), (PyCFunction)__pyx_pw_4h5py_5_conv_5unregister_converters, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    __Pyx_NAMESTR("_conv"),
    __Pyx_DOCSTR(__pyx_k_13), /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_11, __pyx_k_11, sizeof(__pyx_k_11), 0, 0, 1, 0},
  {&__pyx_kp_s_3, __pyx_k_3, sizeof(__pyx_k_3), 0, 0, 1, 0},
  {&__pyx_kp_s_5, __pyx_k_5, sizeof(__pyx_k_5), 0, 0, 1, 0},
  {&__pyx_kp_s_7, __pyx_k_7, sizeof(__pyx_k_7), 0, 0, 1, 0},
  {&__pyx_kp_s_9, __pyx_k_9, sizeof(__pyx_k_9), 0, 0, 1, 0},
  {&__pyx_n_s__MemoryError, __pyx_k__MemoryError, sizeof(__pyx_k__MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s__TypeError, __pyx_k__TypeError, sizeof(__pyx_k__TypeError), 0, 0, 1, 1},
  {&__pyx_n_s__ValueError, __pyx_k__ValueError, sizeof(__pyx_k__ValueError), 0, 0, 1, 1},
  {&__pyx_n_s____main__, __pyx_k____main__, sizeof(__pyx_k____main__), 0, 0, 1, 1},
  {&__pyx_n_s____test__, __pyx_k____test__, sizeof(__pyx_k____test__), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetName(__pyx_b, __pyx_n_s__ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_TypeError = __Pyx_GetName(__pyx_b, __pyx_n_s__TypeError); if (!__pyx_builtin_TypeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_MemoryError = __Pyx_GetName(__pyx_b, __pyx_n_s__MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "h5py/_conv.pyx":220
 *                         #temp_encoded = PyString_AsDecodedObject(temp_object, "utf8", NULL)
 *                     except:
 *                         raise ValueError("Byte string is not valid utf-8 and can't be stored in a utf-8 dataset")             # <<<<<<<<<<<<<<
 *                 temp_string = PyBytes_AsString(temp_object)
 *                 temp_string_len = PyBytes_Size(temp_object)
 */
  __pyx_k_tuple_4 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_3)); if (unlikely(!__pyx_k_tuple_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_4);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_4));

  /* "h5py/_conv.pyx":246
 *                     temp_string_len = PyBytes_Size(temp_encoded)
 *                 else:
 *                     raise TypeError("Unrecognized dataset encoding")             # <<<<<<<<<<<<<<
 * 
 *         if strlen(temp_string) != temp_string_len:
 */
  __pyx_k_tuple_6 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_5)); if (unlikely(!__pyx_k_tuple_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_6);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_6));

  /* "h5py/_conv.pyx":249
 * 
 *         if strlen(temp_string) != temp_string_len:
 *             raise ValueError("VLEN strings do not support embedded NULLs")             # <<<<<<<<<<<<<<
 * 
 *         buf_cstring[0] = <char*>malloc(temp_string_len+1)
 */
  __pyx_k_tuple_8 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_7)); if (unlikely(!__pyx_k_tuple_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 249; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_8);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_8));

  /* "h5py/_conv.pyx":364
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, Reference):
 *             raise TypeError("Can't convert incompatible object to HDF5 object reference")             # <<<<<<<<<<<<<<
 *         ref = <Reference>(buf_obj[0])
 *         buf_ref[0] = ref.ref.obj_ref
 */
  __pyx_k_tuple_10 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_9)); if (unlikely(!__pyx_k_tuple_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 364; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_10);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_10));

  /* "h5py/_conv.pyx":405
 *         obj = <object>(buf_obj[0])
 *         if not isinstance(obj, RegionReference):
 *             raise TypeError("Can't convert incompatible object to HDF5 region reference")             # <<<<<<<<<<<<<<
 *         ref = <RegionReference>(buf_obj[0])
 *         memcpy(buf_ref, ref.ref.reg_ref, sizeof(hdset_reg_ref_t))
 */
  __pyx_k_tuple_12 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_11)); if (unlikely(!__pyx_k_tuple_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_12);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_12));
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_conv(void); /*proto*/
PyMODINIT_FUNC init_conv(void)
#else
PyMODINIT_FUNC PyInit__conv(void); /*proto*/
PyMODINIT_FUNC PyInit__conv(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__conv(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4(__Pyx_NAMESTR("_conv"), __pyx_methods, __Pyx_DOCSTR(__pyx_k_13), 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "h5py._conv")) {
      if (unlikely(PyDict_SetItemString(modules, "h5py._conv", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  __pyx_b = PyImport_AddModule(__Pyx_NAMESTR(__Pyx_BUILTIN_MODULE_NAME)); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_module_is_main_h5py___conv) {
    if (__Pyx_SetAttrString(__pyx_m, "__name__", __pyx_n_s____main__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  __pyx_ptype_4h5py_3h5r_Reference = __Pyx_ImportType("h5py.h5r", "Reference", sizeof(struct __pyx_obj_4h5py_3h5r_Reference), 1); if (unlikely(!__pyx_ptype_4h5py_3h5r_Reference)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5r_RegionReference = __Pyx_ImportType("h5py.h5r", "RegionReference", sizeof(struct __pyx_obj_4h5py_3h5r_RegionReference), 1); if (unlikely(!__pyx_ptype_4h5py_3h5r_RegionReference)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("h5py.defs"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tcreate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tcreate, "hid_t (enum H5T_class_t, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tcopy", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tcopy, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tequal", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tequal, "htri_t (hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tlock", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tlock, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_size", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_size, "size_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_super", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_super, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tclose", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tclose, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tconvert", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tconvert, "herr_t (hid_t, hid_t, size_t, void *, void *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tset_size", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tset_size, "herr_t (hid_t, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_cset", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_cset, "H5T_cset_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tis_variable_str", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tis_variable_str, "htri_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tenum_create", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tenum_create, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tset_tag", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tset_tag, "herr_t (hid_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tregister", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tregister, "herr_t (H5T_pers_t, char *, hid_t, hid_t, H5T_conv_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tunregister", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tunregister, "herr_t (H5T_pers_t, char *, hid_t, hid_t, H5T_conv_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/

  /* "h5py/_conv.pyx":50
 * 
 * # Create Python object equivalents
 * cdef hid_t H5PY_OBJ = 0             # <<<<<<<<<<<<<<
 * 
 * cpdef hid_t get_python_obj():
 */
  __pyx_v_4h5py_5_conv_H5PY_OBJ = 0;

  /* "h5py/_conv.pyx":1
 * # cython: profile=False             # <<<<<<<<<<<<<<
 * 
 * # This file is part of h5py, a Python interface to the HDF5 library.
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_2));
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s____test__, ((PyObject *)__pyx_t_2)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    __Pyx_AddTraceback("init h5py._conv", __pyx_clineno, __pyx_lineno, __pyx_filename);
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init h5py._conv");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif /* CYTHON_REFNANNY */

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name) {
    PyObject *result;
    result = PyObject_GetAttr(dict, name);
    if (!result) {
        if (dict != __pyx_b) {
            PyErr_Clear();
            result = PyObject_GetAttr(__pyx_b, name);
        }
        if (!result) {
            PyErr_SetObject(PyExc_NameError, name);
        }
    }
    return result;
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
        goto bad;
    #endif
    Py_INCREF(local_type);
    Py_INCREF(local_value);
    Py_INCREF(local_tb);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    /* Make sure tstate is in a consistent state when we XDECREF
       these objects (DECREF may run arbitrary code). */
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    #if PY_VERSION_HEX < 0x02050000
    if (PyClass_Check(type)) {
    #else
    if (PyType_Check(type)) {
    #endif
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        #if PY_VERSION_HEX < 0x02050000
            if (PyInstance_Check(type)) {
                type = (PyObject*) ((PyInstanceObject*)type)->in_class;
                Py_INCREF(type);
            }
            else {
                type = 0;
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception must be an old-style class or instance");
                goto raise_error;
            }
        #else
            type = (PyObject*) Py_TYPE(type);
            Py_INCREF(type);
            if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception class must be a subclass of BaseException");
                goto raise_error;
            }
        #endif
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else /* Python 3+ */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *args;
        if (!value)
            args = PyTuple_New(0);
        else if (PyTuple_Check(value)) {
            Py_INCREF(value);
            args = value;
        }
        else
            args = PyTuple_Pack(1, value);
        if (!args)
            goto bad;
        owned_instance = PyEval_CallObject(type, args);
        Py_DECREF(args);
        if (!owned_instance)
            goto bad;
        value = owned_instance;
        if (!PyExceptionInstance_Check(value)) {
            PyErr_Format(PyExc_TypeError,
                         "calling %R should have returned an instance of "
                         "BaseException, not %R",
                         type, Py_TYPE(value));
            goto bad;
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause && cause != Py_None) {
        PyObject *fixed_cause;
        if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        }
        else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        }
        else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t val) {
    const hid_t neg_one = (hid_t)-1, const_zero = (hid_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hid_t) == sizeof(char))  ||
        (sizeof(hid_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hid_t) == sizeof(int)) ||
               (sizeof(hid_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hid_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_htri_t(htri_t val) {
    const htri_t neg_one = (htri_t)-1, const_zero = (htri_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(htri_t) == sizeof(char))  ||
        (sizeof(htri_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(htri_t) == sizeof(int)) ||
               (sizeof(htri_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(htri_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(htri_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject* x) {
    const unsigned char neg_one = (unsigned char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned char" :
                    "value too large to convert to unsigned char");
            }
            return (unsigned char)-1;
        }
        return (unsigned char)val;
    }
    return (unsigned char)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject* x) {
    const unsigned short neg_one = (unsigned short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned short" :
                    "value too large to convert to unsigned short");
            }
            return (unsigned short)-1;
        }
        return (unsigned short)val;
    }
    return (unsigned short)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject* x) {
    const unsigned int neg_one = (unsigned int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned int" :
                    "value too large to convert to unsigned int");
            }
            return (unsigned int)-1;
        }
        return (unsigned int)val;
    }
    return (unsigned int)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject* x) {
    const char neg_one = (char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to char" :
                    "value too large to convert to char");
            }
            return (char)-1;
        }
        return (char)val;
    }
    return (char)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject* x) {
    const short neg_one = (short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to short" :
                    "value too large to convert to short");
            }
            return (short)-1;
        }
        return (short)val;
    }
    return (short)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject* x) {
    const signed char neg_one = (signed char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed char" :
                    "value too large to convert to signed char");
            }
            return (signed char)-1;
        }
        return (signed char)val;
    }
    return (signed char)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject* x) {
    const signed short neg_one = (signed short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed short" :
                    "value too large to convert to signed short");
            }
            return (signed short)-1;
        }
        return (signed short)val;
    }
    return (signed short)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject* x) {
    const signed int neg_one = (signed int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed int" :
                    "value too large to convert to signed int");
            }
            return (signed int)-1;
        }
        return (signed int)val;
    }
    return (signed int)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject* x) {
    const unsigned long neg_one = (unsigned long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned long");
            return (unsigned long)-1;
        }
        return (unsigned long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned long");
                return (unsigned long)-1;
            }
            return (unsigned long)PyLong_AsUnsignedLong(x);
        } else {
            return (unsigned long)PyLong_AsLong(x);
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned long)-1;
        val = __Pyx_PyInt_AsUnsignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject* x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned PY_LONG_LONG");
            return (unsigned PY_LONG_LONG)-1;
        }
        return (unsigned PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned PY_LONG_LONG");
                return (unsigned PY_LONG_LONG)-1;
            }
            return (unsigned PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (unsigned PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsUnsignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject* x) {
    const long neg_one = (long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to long");
            return (long)-1;
        }
        return (long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to long");
                return (long)-1;
            }
            return (long)PyLong_AsUnsignedLong(x);
        } else {
            return (long)PyLong_AsLong(x);
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long)-1;
        val = __Pyx_PyInt_AsLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject* x) {
    const PY_LONG_LONG neg_one = (PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to PY_LONG_LONG");
            return (PY_LONG_LONG)-1;
        }
        return (PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to PY_LONG_LONG");
                return (PY_LONG_LONG)-1;
            }
            return (PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject* x) {
    const signed long neg_one = (signed long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed long");
            return (signed long)-1;
        }
        return (signed long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed long");
                return (signed long)-1;
            }
            return (signed long)PyLong_AsUnsignedLong(x);
        } else {
            return (signed long)PyLong_AsLong(x);
        }
    } else {
        signed long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed long)-1;
        val = __Pyx_PyInt_AsSignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject* x) {
    const signed PY_LONG_LONG neg_one = (signed PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed PY_LONG_LONG");
            return (signed PY_LONG_LONG)-1;
        }
        return (signed PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed PY_LONG_LONG");
                return (signed PY_LONG_LONG)-1;
            }
            return (signed PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (signed PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        signed PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsSignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        #if PY_VERSION_HEX < 0x02050000
        return PyErr_Warn(NULL, message);
        #else
        return PyErr_WarnEx(NULL, message, 1);
        #endif
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%s.%s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        #if PY_VERSION_HEX < 0x02050000
        if (PyErr_Warn(NULL, warning) < 0) goto bad;
        #else
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
        #endif
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%s.%s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%s does not export expected C function %s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3 && PY_MINOR_VERSION==0)
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,            /*int argcount,*/
        0,            /*int kwonlyargcount,*/
        0,            /*int nlocals,*/
        0,            /*int stacksize,*/
        0,            /*int flags,*/
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,      /*int firstlineno,*/
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_globals = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_globals = PyModule_GetDict(__pyx_m);
    if (!py_globals) goto bad;
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        py_globals,          /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else  /* Python 3+ has unicode identifiers */
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_VERSION_HEX < 0x03000000
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_VERSION_HEX < 0x03000000
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%s__ returned non-%s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject* x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
#if PY_VERSION_HEX < 0x02050000
   if (ival <= LONG_MAX)
       return PyInt_FromLong((long)ival);
   else {
       unsigned char *bytes = (unsigned char *) &ival;
       int one = 1; int little = (int)*(unsigned char*)&one;
       return _PyLong_FromByteArray(bytes, sizeof(size_t), little, 0);
   }
#else
   return PyInt_FromSize_t(ival);
#endif
}
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject* x) {
   unsigned PY_LONG_LONG val = __Pyx_PyInt_AsUnsignedLongLong(x);
   if (unlikely(val == (unsigned PY_LONG_LONG)-1 && PyErr_Occurred())) {
       return (size_t)-1;
   } else if (unlikely(val != (unsigned PY_LONG_LONG)(size_t)val)) {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to size_t");
       return (size_t)-1;
   }
   return (size_t)val;
}


#endif /* Py_PYTHON_H */
