/* Generated by Cython 0.18 on Mon Dec  9 16:00:33 2013 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02040000
    #error Cython requires Python 2.4+.
#else
#include <stddef.h> /* For offsetof */
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if PY_VERSION_HEX < 0x02050000
  typedef int Py_ssize_t;
  #define PY_SSIZE_T_MAX INT_MAX
  #define PY_SSIZE_T_MIN INT_MIN
  #define PY_FORMAT_SIZE_T ""
  #define CYTHON_FORMAT_SSIZE_T ""
  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
  #define PyInt_AsSsize_t(o)   __Pyx_PyInt_AsInt(o)
  #define PyNumber_Index(o)    ((PyNumber_Check(o) && !PyFloat_Check(o)) ? PyNumber_Int(o) : \
                                (PyErr_Format(PyExc_TypeError, \
                                              "expected index value, got %.200s", Py_TYPE(o)->tp_name), \
                                 (PyObject*)0))
  #define __Pyx_PyIndex_Check(o) (PyNumber_Check(o) && !PyFloat_Check(o) && \
                                  !PyComplex_Check(o))
  #define PyIndex_Check __Pyx_PyIndex_Check
  #define PyErr_WarnEx(category, message, stacklevel) PyErr_Warn(category, message)
  #define __PYX_BUILD_PY_SSIZE_T "i"
#else
  #define __PYX_BUILD_PY_SSIZE_T "n"
  #define CYTHON_FORMAT_SSIZE_T "z"
  #define __Pyx_PyIndex_Check PyIndex_Check
#endif
#if PY_VERSION_HEX < 0x02060000
  #define Py_REFCNT(ob) (((PyObject*)(ob))->ob_refcnt)
  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
  #define Py_SIZE(ob)   (((PyVarObject*)(ob))->ob_size)
  #define PyVarObject_HEAD_INIT(type, size) \
          PyObject_HEAD_INIT(type) size,
  #define PyType_Modified(t)
  typedef struct {
     void *buf;
     PyObject *obj;
     Py_ssize_t len;
     Py_ssize_t itemsize;
     int readonly;
     int ndim;
     char *format;
     Py_ssize_t *shape;
     Py_ssize_t *strides;
     Py_ssize_t *suboffsets;
     void *internal;
  } Py_buffer;
  #define PyBUF_SIMPLE 0
  #define PyBUF_WRITABLE 0x0001
  #define PyBUF_FORMAT 0x0004
  #define PyBUF_ND 0x0008
  #define PyBUF_STRIDES (0x0010 | PyBUF_ND)
  #define PyBUF_C_CONTIGUOUS (0x0020 | PyBUF_STRIDES)
  #define PyBUF_F_CONTIGUOUS (0x0040 | PyBUF_STRIDES)
  #define PyBUF_ANY_CONTIGUOUS (0x0080 | PyBUF_STRIDES)
  #define PyBUF_INDIRECT (0x0100 | PyBUF_STRIDES)
  #define PyBUF_RECORDS (PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_WRITABLE)
  #define PyBUF_FULL (PyBUF_INDIRECT | PyBUF_FORMAT | PyBUF_WRITABLE)
  typedef int (*getbufferproc)(PyObject *, Py_buffer *, int);
  typedef void (*releasebufferproc)(PyObject *, Py_buffer *);
#endif
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
  #define PyUnicode_FromString(s) PyUnicode_Decode(s, strlen(s), "UTF-8", "strict")
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#if (PY_VERSION_HEX < 0x02060000) || (PY_MAJOR_VERSION >= 3)
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((k=k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PyBytesObject                PyStringObject
  #define PyBytes_Type                 PyString_Type
  #define PyBytes_Check                PyString_Check
  #define PyBytes_CheckExact           PyString_CheckExact
  #define PyBytes_FromString           PyString_FromString
  #define PyBytes_FromStringAndSize    PyString_FromStringAndSize
  #define PyBytes_FromFormat           PyString_FromFormat
  #define PyBytes_DecodeEscape         PyString_DecodeEscape
  #define PyBytes_AsString             PyString_AsString
  #define PyBytes_AsStringAndSize      PyString_AsStringAndSize
  #define PyBytes_Size                 PyString_Size
  #define PyBytes_AS_STRING            PyString_AS_STRING
  #define PyBytes_GET_SIZE             PyString_GET_SIZE
  #define PyBytes_Repr                 PyString_Repr
  #define PyBytes_Concat               PyString_Concat
  #define PyBytes_ConcatAndDel         PyString_ConcatAndDel
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PySet_Check(obj)             PyObject_TypeCheck(obj, &PySet_Type)
  #define PyFrozenSet_Check(obj)       PyObject_TypeCheck(obj, &PyFrozenSet_Type)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_VERSION_HEX < 0x03020000
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if (PY_MAJOR_VERSION < 3) || (PY_VERSION_HEX >= 0x03010300)
  #define __Pyx_PySequence_GetSlice(obj, a, b) PySequence_GetSlice(obj, a, b)
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) PySequence_SetSlice(obj, a, b, value)
  #define __Pyx_PySequence_DelSlice(obj, a, b) PySequence_DelSlice(obj, a, b)
#else
  #define __Pyx_PySequence_GetSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), (PyObject*)0) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_GetSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object is unsliceable", (obj)->ob_type->tp_name), (PyObject*)0)))
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_SetSlice(obj, a, b, value)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice assignment", (obj)->ob_type->tp_name), -1)))
  #define __Pyx_PySequence_DelSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_DelSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice deletion", (obj)->ob_type->tp_name), -1)))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),((char *)(n)))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),((char *)(n)),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),((char *)(n)))
#else
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),(n))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),(n),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),(n))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_NAMESTR(n) ((char *)(n))
  #define __Pyx_DOCSTR(n)  ((char *)(n))
#else
  #define __Pyx_NAMESTR(n) (n)
  #define __Pyx_DOCSTR(n)  (n)
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__h5py__h5o
#define __PYX_HAVE_API__h5py__h5o
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "unistd.h"
#include "stdint.h"
#include "api_compat.h"
#include "lzf_filter.h"
#include "hdf5.h"
#include "H5Cpublic.h"
#include "hdf5_hl.h"
#include "numpy/arrayobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const long n; const char* encoding; const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry; /*proto*/

#define __Pyx_PyBytes_FromUString(s) PyBytes_FromString((char*)s)
#define __Pyx_PyBytes_AsUString(s)   ((unsigned char*) PyBytes_AsString(s))
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject*);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))


#ifdef __GNUC__
  /* Test for GCC > 2.95 */
  #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95))
    #define likely(x)   __builtin_expect(!!(x), 1)
    #define unlikely(x) __builtin_expect(!!(x), 0)
  #else /* __GNUC__ > 2 ... */
    #define likely(x)   (x)
    #define unlikely(x) (x)
  #endif /* __GNUC__ > 2 ... */
#else /* __GNUC__ */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
    
static PyObject *__pyx_m;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "h5o.pyx",
  "_objects.pxd",
  "h5g.pxd",
  "h5p.pxd",
  "numpy.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_4h5py_8_objects_ObjectID;
struct __pyx_obj_4h5py_3h5p_PropID;
struct __pyx_obj_4h5py_3h5p_PropInstanceID;
struct __pyx_obj_4h5py_3h5p_PropFAID;
struct __pyx_obj_4h5py_3h5o__ObjectVisitor;
struct __pyx_obj_4h5py_3h5o__ObjInfoBase;
struct __pyx_obj_4h5py_3h5o__OHdr;
struct __pyx_obj_4h5py_3h5p_PropCreateID;
struct __pyx_obj_4h5py_3h5p_PropFCID;
struct __pyx_obj_4h5py_3h5p_PropClassID;
struct __pyx_obj_4h5py_3h5o__ObjInfo;
struct __pyx_obj_4h5py_3h5p_PropOCID;
struct __pyx_obj_4h5py_3h5o_ObjInfo;
struct __pyx_obj_4h5py_3h5p_PropLCID;
struct __pyx_obj_4h5py_3h5g_GroupID;
struct __pyx_obj_4h5py_3h5p_PropGCID;
struct __pyx_obj_4h5py_3h5o__OHdrMesg;
struct __pyx_obj_4h5py_3h5p_PropDCID;
struct __pyx_obj_4h5py_3h5p_PropLAID;
struct __pyx_obj_4h5py_3h5o__OHdrSpace;
struct __pyx_obj_4h5py_3h5p_PropCopyID;
struct __pyx_obj_4h5py_3h5p_PropDXID;
struct __pyx_opt_args_4h5py_5utils_check_numpy_read;
struct __pyx_opt_args_4h5py_5utils_check_numpy_write;

/* "utils.pxd":17
 * cdef void efree(void* ptr)
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1
 * 
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_read {
  int __pyx_n;
  hid_t space_id;
};

/* "utils.pxd":18
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * 
 * cdef int convert_tuple(object tuple, hsize_t *dims, hsize_t rank) except -1
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_write {
  int __pyx_n;
  hid_t space_id;
};

/* "_objects.pxd":12
 * from defs cimport *
 * 
 * cdef class ObjectID:             # <<<<<<<<<<<<<<
 * 
 *     cdef object __weakref__
 */
struct __pyx_obj_4h5py_8_objects_ObjectID {
  PyObject_HEAD
  PyObject *__weakref__;
  hid_t id;
  int locked;
  PyObject *_hash;
};


/* "h5p.pxd":16
 * # --- Base classes ---
 * 
 * cdef class PropID(ObjectID):             # <<<<<<<<<<<<<<
 *     """ Base class for all property lists """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
};


/* "h5p.pxd":26
 *     pass
 * 
 * cdef class PropInstanceID(PropID):             # <<<<<<<<<<<<<<
 *     """ Represents an instance of a property list class (i.e. an actual list
 *         which can be passed on to other API functions).
 */
struct __pyx_obj_4h5py_3h5p_PropInstanceID {
  struct __pyx_obj_4h5py_3h5p_PropID __pyx_base;
};


/* "h5p.pxd":59
 * # --- Object access ---
 * 
 * cdef class PropFAID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ File access property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropFAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5py/h5o.pyx":245
 * # === Visit routines ==========================================================
 * 
 * cdef class _ObjectVisitor:             # <<<<<<<<<<<<<<
 * 
 *     cdef object func
 */
struct __pyx_obj_4h5py_3h5o__ObjectVisitor {
  PyObject_HEAD
  PyObject *func;
  PyObject *retval;
  struct __pyx_obj_4h5py_3h5o_ObjInfo *objinfo;
};


/* "h5py/h5o.pyx":36
 * # === Giant H5O_info_t structure ==============================================
 * 
 * cdef class _ObjInfoBase:             # <<<<<<<<<<<<<<
 * 
 *     cdef H5O_info_t *istr
 */
struct __pyx_obj_4h5py_3h5o__ObjInfoBase {
  PyObject_HEAD
  H5O_info_t *istr;
};


/* "h5py/h5o.pyx":70
 *         return hash((self.total, self.meta, self.mesg, self.free))
 * 
 * cdef class _OHdr(_ObjInfoBase):             # <<<<<<<<<<<<<<
 * 
 *     cdef public _OHdrSpace space
 */
struct __pyx_obj_4h5py_3h5o__OHdr {
  struct __pyx_obj_4h5py_3h5o__ObjInfoBase __pyx_base;
  struct __pyx_obj_4h5py_3h5o__OHdrSpace *space;
  struct __pyx_obj_4h5py_3h5o__OHdrMesg *mesg;
};


/* "h5p.pxd":32
 *     pass
 * 
 * cdef class PropCreateID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Base class for all object creation lists.
 * 
 */
struct __pyx_obj_4h5py_3h5p_PropCreateID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5p.pxd":52
 *     pass
 * 
 * cdef class PropFCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ File creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropFCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5p.pxd":20
 *     pass
 * 
 * cdef class PropClassID(PropID):             # <<<<<<<<<<<<<<
 *     """ Represents an HDF5 property list class.  These can be either (locked)
 *         library-defined classes or user-created classes.
 */
struct __pyx_obj_4h5py_3h5p_PropClassID {
  struct __pyx_obj_4h5py_3h5p_PropID __pyx_base;
};


/* "h5py/h5o.pyx":89
 *         return hash((self.version, self.nmesgs, self.space, self.mesg))
 * 
 * cdef class _ObjInfo(_ObjInfoBase):             # <<<<<<<<<<<<<<
 * 
 *     property fileno:
 */
struct __pyx_obj_4h5py_3h5o__ObjInfo {
  struct __pyx_obj_4h5py_3h5o__ObjInfoBase __pyx_base;
};


/* "h5p.pxd":44
 * # --- Object creation ---
 * 
 * cdef class PropOCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ Object creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropOCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5py/h5o.pyx":107
 *         return hash((self.fileno, self.addr, self.type, self.rc))
 * 
 * cdef class ObjInfo(_ObjInfo):             # <<<<<<<<<<<<<<
 * 
 *     """
 */
struct __pyx_obj_4h5py_3h5o_ObjInfo {
  struct __pyx_obj_4h5py_3h5o__ObjInfo __pyx_base;
  H5O_info_t infostruct;
  struct __pyx_obj_4h5py_3h5o__OHdr *hdr;
};


/* "h5p.pxd":70
 * # --- New in 1.8 ---
 * 
 * cdef class PropLCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ Link creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropLCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5g.pxd":14
 * from _objects cimport ObjectID
 * 
 * cdef class GroupID(ObjectID):             # <<<<<<<<<<<<<<
 * 
 *     cdef readonly object links
 */
struct __pyx_obj_4h5py_3h5g_GroupID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
  PyObject *links;
};


/* "h5p.pxd":78
 *     cdef char* _buf
 * 
 * cdef class PropGCID(PropOCID):             # <<<<<<<<<<<<<<
 *     """ Group creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropGCID {
  struct __pyx_obj_4h5py_3h5p_PropOCID __pyx_base;
};


/* "h5py/h5o.pyx":40
 *     cdef H5O_info_t *istr
 * 
 * cdef class _OHdrMesg(_ObjInfoBase):             # <<<<<<<<<<<<<<
 * 
 *     property present:
 */
struct __pyx_obj_4h5py_3h5o__OHdrMesg {
  struct __pyx_obj_4h5py_3h5o__ObjInfoBase __pyx_base;
};


/* "h5p.pxd":48
 *     pass
 * 
 * cdef class PropDCID(PropOCID):             # <<<<<<<<<<<<<<
 *     """ Dataset creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropDCID {
  struct __pyx_obj_4h5py_3h5p_PropOCID __pyx_base;
};


/* "h5p.pxd":74
 *     pass
 * 
 * cdef class PropLAID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Link access property list """
 *     cdef char* _buf
 */
struct __pyx_obj_4h5py_3h5p_PropLAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
  char *_buf;
};


/* "h5py/h5o.pyx":52
 *         return hash((self.present, self.shared))
 * 
 * cdef class _OHdrSpace(_ObjInfoBase):             # <<<<<<<<<<<<<<
 * 
 *     property total:
 */
struct __pyx_obj_4h5py_3h5o__OHdrSpace {
  struct __pyx_obj_4h5py_3h5o__ObjInfoBase __pyx_base;
};


/* "h5p.pxd":39
 *     pass
 * 
 * cdef class PropCopyID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Property list for copying objects (as in h5o.copy) """
 * 
 */
struct __pyx_obj_4h5py_3h5p_PropCopyID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5p.pxd":63
 *     pass
 * 
 * cdef class PropDXID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Dataset transfer property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropDXID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};

#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname); /*proto*/
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif /* CYTHON_REFNANNY */
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name); /*proto*/

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found); /*proto*/

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed); /*proto*/

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type); /*proto*/

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name); /*proto*/

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name); /*proto*/

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact); /*proto*/

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb); /*proto*/
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause); /*proto*/

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_haddr_t(haddr_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t);

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject *);

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject *);

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject *);

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject *);

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject *);

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject *);

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject *);

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject *);

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject *);

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject *);

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject *);

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name); /*proto*/

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);  /*proto*/

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig); /*proto*/

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename); /*proto*/

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t); /*proto*/


/* Module declarations from 'h5py.api_types_ext' */

/* Module declarations from 'h5py.api_types_hdf5' */

/* Module declarations from 'h5py.defs' */
static hid_t (*__pyx_f_4h5py_4defs_H5Oopen)(hid_t, char *, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Oget_info)(hid_t, H5O_info_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Oget_info_by_name)(hid_t, char *, H5O_info_t *, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Oget_info_by_idx)(hid_t, char *, H5_index_t, H5_iter_order_t, hsize_t, H5O_info_t *, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Olink)(hid_t, hid_t, char *, hid_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Ocopy)(hid_t, char *, hid_t, char *, hid_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Oset_comment_by_name)(hid_t, char *, char *, hid_t); /*proto*/
static Py_ssize_t (*__pyx_f_4h5py_4defs_H5Oget_comment_by_name)(hid_t, char *, char *, size_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Ovisit_by_name)(hid_t, char *, H5_index_t, H5_iter_order_t, H5O_iterate_t, void *, hid_t); /*proto*/

/* Module declarations from 'h5py._objects' */
static PyTypeObject *__pyx_ptype_4h5py_8_objects_ObjectID = 0;
static hid_t (*__pyx_f_4h5py_8_objects_pdefault)(struct __pyx_obj_4h5py_8_objects_ObjectID *); /*proto*/

/* Module declarations from 'h5py.h5g' */
static PyTypeObject *__pyx_ptype_4h5py_3h5g_GroupID = 0;

/* Module declarations from 'h5py.h5i' */
static struct __pyx_obj_4h5py_8_objects_ObjectID *(*__pyx_f_4h5py_3h5i_wrap_identifier)(hid_t, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'h5py.h5p' */
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropClassID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropInstanceID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropCreateID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropCopyID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropOCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropFCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropFAID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDXID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropLCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropLAID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropGCID = 0;

/* Module declarations from 'numpy' */

/* Module declarations from 'h5py.numpy' */
static PyTypeObject *__pyx_ptype_4h5py_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_4h5py_5numpy_ndarray = 0;

/* Module declarations from 'h5py.utils' */
static void *(*__pyx_f_4h5py_5utils_emalloc)(size_t); /*proto*/
static void (*__pyx_f_4h5py_5utils_efree)(void *); /*proto*/

/* Module declarations from 'h5py.h5o' */
static PyTypeObject *__pyx_ptype_4h5py_3h5o__ObjInfoBase = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5o__OHdrMesg = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5o__OHdrSpace = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5o__OHdr = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5o__ObjInfo = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5o_ObjInfo = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5o__ObjectVisitor = 0;
static herr_t __pyx_f_4h5py_3h5o_cb_obj_iterate(hid_t, char *, H5O_info_t *, void *); /*proto*/
static herr_t __pyx_f_4h5py_3h5o_cb_obj_simple(hid_t, char *, H5O_info_t *, void *); /*proto*/
#define __Pyx_MODULE_NAME "h5py.h5o"
int __pyx_module_is_main_h5py__h5o = 0;

/* Implementation of 'h5py.h5o' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_pf_4h5py_3h5o_9_OHdrMesg_7present___get__(struct __pyx_obj_4h5py_3h5o__OHdrMesg *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_9_OHdrMesg_6shared___get__(struct __pyx_obj_4h5py_3h5o__OHdrMesg *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_9_OHdrMesg__hash(struct __pyx_obj_4h5py_3h5o__OHdrMesg *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_5total___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_4meta___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_4mesg___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_4free___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace__hash(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_7version___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_6nmesgs___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5o_5_OHdr___init__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_2_hash(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_5space___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5o_5_OHdr_5space_2__set__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_4h5py_3h5o_5_OHdr_5space_4__del__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_4mesg___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5o_5_OHdr_4mesg_2__set__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_4h5py_3h5o_5_OHdr_4mesg_4__del__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_6fileno___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_4addr___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_4type___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_2rc___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo__hash(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5o_7ObjInfo___init__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_7ObjInfo_2__copy__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_7ObjInfo_3hdr___get__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr_2__set__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr_4__del__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_get_info(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_name, int __pyx_v_index, char *__pyx_v_obj_name, int __pyx_v_index_type, int __pyx_v_order, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_2open(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_4link(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_obj, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_loc, char *__pyx_v_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_6copy(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_src_loc, char *__pyx_v_src_name, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_dst_loc, char *__pyx_v_dst_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_copypl, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_8set_comment(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_comment, char *__pyx_v_obj_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_10get_comment(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, CYTHON_UNUSED char *__pyx_v_comment, char *__pyx_v_obj_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl); /* proto */
static int __pyx_pf_4h5py_3h5o_14_ObjectVisitor___init__(struct __pyx_obj_4h5py_3h5o__ObjectVisitor *__pyx_v_self, PyObject *__pyx_v_func); /* proto */
static PyObject *__pyx_pf_4h5py_3h5o_12visit(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, PyObject *__pyx_v_func, int __pyx_v_idx_type, int __pyx_v_order, char *__pyx_v_obj_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl, int __pyx_v_info); /* proto */
static char __pyx_k_1[] = ".";
static char __pyx_k_4[] = "At most one of name or index may be specified";
static char __pyx_k_8[] = "\n    Module for HDF5 \"H5O\" functions.\n";
static char __pyx_k_9[] = "COPY_SHALLOW_HIERARCHY_FLAG";
static char __pyx_k_10[] = "COPY_EXPAND_SOFT_LINK_FLAG";
static char __pyx_k_11[] = "COPY_EXPAND_EXT_LINK_FLAG";
static char __pyx_k_12[] = "COPY_EXPAND_REFERENCE_FLAG";
static char __pyx_k_13[] = "COPY_WITHOUT_ATTR_FLAG";
static char __pyx_k_14[] = "COPY_PRESERVE_NULL_FLAG";
static char __pyx_k_17[] = "/home/computer/h5py/h5py/h5o.pyx";
static char __pyx_k_18[] = "h5py.h5o";
static char __pyx_k__rc[] = "rc";
static char __pyx_k__buf[] = "buf";
static char __pyx_k__loc[] = "loc";
static char __pyx_k__obj[] = "obj";
static char __pyx_k__addr[] = "addr";
static char __pyx_k__copy[] = "copy";
static char __pyx_k__free[] = "free";
static char __pyx_k__func[] = "func";
static char __pyx_k__info[] = "info";
static char __pyx_k__lapl[] = "lapl";
static char __pyx_k__lcpl[] = "lcpl";
static char __pyx_k__link[] = "link";
static char __pyx_k__mesg[] = "mesg";
static char __pyx_k__meta[] = "meta";
static char __pyx_k__name[] = "name";
static char __pyx_k__open[] = "open";
static char __pyx_k__size[] = "size";
static char __pyx_k__type[] = "type";
static char __pyx_k__cfunc[] = "cfunc";
static char __pyx_k__index[] = "index";
static char __pyx_k__order[] = "order";
static char __pyx_k__total[] = "total";
static char __pyx_k__visit[] = "visit";
static char __pyx_k__copypl[] = "copypl";
static char __pyx_k__fileno[] = "fileno";
static char __pyx_k__nmesgs[] = "nmesgs";
static char __pyx_k__shared[] = "shared";
static char __pyx_k__comment[] = "comment";
static char __pyx_k__dst_loc[] = "dst_loc";
static char __pyx_k__present[] = "present";
static char __pyx_k__pstring[] = "pstring";
static char __pyx_k__src_loc[] = "src_loc";
static char __pyx_k__version[] = "version";
static char __pyx_k____main__[] = "__main__";
static char __pyx_k____test__[] = "__test__";
static char __pyx_k__dst_name[] = "dst_name";
static char __pyx_k__get_info[] = "get_info";
static char __pyx_k__idx_type[] = "idx_type";
static char __pyx_k__obj_name[] = "obj_name";
static char __pyx_k__src_name[] = "src_name";
static char __pyx_k__TypeError[] = "TypeError";
static char __pyx_k__TYPE_GROUP[] = "TYPE_GROUP";
static char __pyx_k__index_type[] = "index_type";
static char __pyx_k__get_comment[] = "get_comment";
static char __pyx_k__set_comment[] = "set_comment";
static char __pyx_k__TYPE_DATASET[] = "TYPE_DATASET";
static char __pyx_k__TYPE_NAMED_DATATYPE[] = "TYPE_NAMED_DATATYPE";
static PyObject *__pyx_n_s_10;
static PyObject *__pyx_n_s_11;
static PyObject *__pyx_n_s_12;
static PyObject *__pyx_n_s_13;
static PyObject *__pyx_n_s_14;
static PyObject *__pyx_kp_s_17;
static PyObject *__pyx_n_s_18;
static PyObject *__pyx_kp_s_4;
static PyObject *__pyx_n_s_9;
static PyObject *__pyx_n_s__TYPE_DATASET;
static PyObject *__pyx_n_s__TYPE_GROUP;
static PyObject *__pyx_n_s__TYPE_NAMED_DATATYPE;
static PyObject *__pyx_n_s__TypeError;
static PyObject *__pyx_n_s____main__;
static PyObject *__pyx_n_s____test__;
static PyObject *__pyx_n_s__addr;
static PyObject *__pyx_n_s__buf;
static PyObject *__pyx_n_s__cfunc;
static PyObject *__pyx_n_s__comment;
static PyObject *__pyx_n_s__copy;
static PyObject *__pyx_n_s__copypl;
static PyObject *__pyx_n_s__dst_loc;
static PyObject *__pyx_n_s__dst_name;
static PyObject *__pyx_n_s__fileno;
static PyObject *__pyx_n_s__free;
static PyObject *__pyx_n_s__func;
static PyObject *__pyx_n_s__get_comment;
static PyObject *__pyx_n_s__get_info;
static PyObject *__pyx_n_s__idx_type;
static PyObject *__pyx_n_s__index;
static PyObject *__pyx_n_s__index_type;
static PyObject *__pyx_n_s__info;
static PyObject *__pyx_n_s__lapl;
static PyObject *__pyx_n_s__lcpl;
static PyObject *__pyx_n_s__link;
static PyObject *__pyx_n_s__loc;
static PyObject *__pyx_n_s__mesg;
static PyObject *__pyx_n_s__meta;
static PyObject *__pyx_n_s__name;
static PyObject *__pyx_n_s__nmesgs;
static PyObject *__pyx_n_s__obj;
static PyObject *__pyx_n_s__obj_name;
static PyObject *__pyx_n_s__open;
static PyObject *__pyx_n_s__order;
static PyObject *__pyx_n_s__present;
static PyObject *__pyx_n_s__pstring;
static PyObject *__pyx_n_s__rc;
static PyObject *__pyx_n_s__set_comment;
static PyObject *__pyx_n_s__shared;
static PyObject *__pyx_n_s__size;
static PyObject *__pyx_n_s__src_loc;
static PyObject *__pyx_n_s__src_name;
static PyObject *__pyx_n_s__total;
static PyObject *__pyx_n_s__type;
static PyObject *__pyx_n_s__version;
static PyObject *__pyx_n_s__visit;
static int __pyx_k_2;
static int __pyx_k_3;
static int __pyx_k_6;
static int __pyx_k_7;
static PyObject *__pyx_k_tuple_5;
static PyObject *__pyx_k_tuple_15;
static PyObject *__pyx_k_tuple_19;
static PyObject *__pyx_k_tuple_21;
static PyObject *__pyx_k_tuple_23;
static PyObject *__pyx_k_tuple_25;
static PyObject *__pyx_k_tuple_27;
static PyObject *__pyx_k_tuple_29;
static PyObject *__pyx_k_codeobj_16;
static PyObject *__pyx_k_codeobj_20;
static PyObject *__pyx_k_codeobj_22;
static PyObject *__pyx_k_codeobj_24;
static PyObject *__pyx_k_codeobj_26;
static PyObject *__pyx_k_codeobj_28;
static PyObject *__pyx_k_codeobj_30;

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_9_OHdrMesg_7present_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_9_OHdrMesg_7present_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_9_OHdrMesg_7present___get__(((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":43
 * 
 *     property present:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.mesg.present
 *     property shared:
 */

static PyObject *__pyx_pf_4h5py_3h5o_9_OHdrMesg_7present___get__(struct __pyx_obj_4h5py_3h5o__OHdrMesg *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":44
 *     property present:
 *         def __get__(self):
 *             return self.istr[0].hdr.mesg.present             # <<<<<<<<<<<<<<
 *     property shared:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->__pyx_base.istr[0]).hdr.mesg.present); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdrMesg.present.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_9_OHdrMesg_6shared_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_9_OHdrMesg_6shared_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_9_OHdrMesg_6shared___get__(((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":46
 *             return self.istr[0].hdr.mesg.present
 *     property shared:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.mesg.shared
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_9_OHdrMesg_6shared___get__(struct __pyx_obj_4h5py_3h5o__OHdrMesg *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":47
 *     property shared:
 *         def __get__(self):
 *             return self.istr[0].hdr.mesg.shared             # <<<<<<<<<<<<<<
 * 
 *     def _hash(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->__pyx_base.istr[0]).hdr.mesg.shared); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdrMesg.shared.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_9_OHdrMesg_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_9_OHdrMesg_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_hash (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_9_OHdrMesg__hash(((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":49
 *             return self.istr[0].hdr.mesg.shared
 * 
 *     def _hash(self):             # <<<<<<<<<<<<<<
 *         return hash((self.present, self.shared))
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_9_OHdrMesg__hash(struct __pyx_obj_4h5py_3h5o__OHdrMesg *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_hash_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_hash", 0);

  /* "h5py/h5o.pyx":50
 * 
 *     def _hash(self):
 *         return hash((self.present, self.shared))             # <<<<<<<<<<<<<<
 * 
 * cdef class _OHdrSpace(_ObjInfoBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__present); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__shared); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_4 = PyObject_Hash(((PyObject *)__pyx_t_3)); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_FromHash_t(__pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5o._OHdrMesg._hash", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_5total_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_5total_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_10_OHdrSpace_5total___get__(((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":55
 * 
 *     property total:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.space.total
 *     property meta:
 */

static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_5total___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":56
 *     property total:
 *         def __get__(self):
 *             return self.istr[0].hdr.space.total             # <<<<<<<<<<<<<<
 *     property meta:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_hsize_t((__pyx_v_self->__pyx_base.istr[0]).hdr.space.total); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdrSpace.total.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_4meta_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_4meta_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_10_OHdrSpace_4meta___get__(((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":58
 *             return self.istr[0].hdr.space.total
 *     property meta:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.space.meta
 *     property mesg:
 */

static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_4meta___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":59
 *     property meta:
 *         def __get__(self):
 *             return self.istr[0].hdr.space.meta             # <<<<<<<<<<<<<<
 *     property mesg:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_hsize_t((__pyx_v_self->__pyx_base.istr[0]).hdr.space.meta); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdrSpace.meta.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_4mesg_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_4mesg_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_10_OHdrSpace_4mesg___get__(((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":61
 *             return self.istr[0].hdr.space.meta
 *     property mesg:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.space.mesg
 *     property free:
 */

static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_4mesg___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":62
 *     property mesg:
 *         def __get__(self):
 *             return self.istr[0].hdr.space.mesg             # <<<<<<<<<<<<<<
 *     property free:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_hsize_t((__pyx_v_self->__pyx_base.istr[0]).hdr.space.mesg); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdrSpace.mesg.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_4free_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_4free_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_10_OHdrSpace_4free___get__(((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":64
 *             return self.istr[0].hdr.space.mesg
 *     property free:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.space.free
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace_4free___get__(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":65
 *     property free:
 *         def __get__(self):
 *             return self.istr[0].hdr.space.free             # <<<<<<<<<<<<<<
 * 
 *     def _hash(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_hsize_t((__pyx_v_self->__pyx_base.istr[0]).hdr.space.free); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdrSpace.free.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_10_OHdrSpace_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_hash (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_10_OHdrSpace__hash(((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":67
 *             return self.istr[0].hdr.space.free
 * 
 *     def _hash(self):             # <<<<<<<<<<<<<<
 *         return hash((self.total, self.meta, self.mesg, self.free))
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_10_OHdrSpace__hash(struct __pyx_obj_4h5py_3h5o__OHdrSpace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_hash_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_hash", 0);

  /* "h5py/h5o.pyx":68
 * 
 *     def _hash(self):
 *         return hash((self.total, self.meta, self.mesg, self.free))             # <<<<<<<<<<<<<<
 * 
 * cdef class _OHdr(_ObjInfoBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__total); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__meta); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__mesg); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__free); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = PyObject_Hash(((PyObject *)__pyx_t_5)); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_FromHash_t(__pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("h5py.h5o._OHdrSpace._hash", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_7version_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_7version_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_7version___get__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":76
 * 
 *     property version:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.version
 *     property nmesgs:
 */

static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_7version___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":77
 *     property version:
 *         def __get__(self):
 *             return self.istr[0].hdr.version             # <<<<<<<<<<<<<<
 *     property nmesgs:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->__pyx_base.istr[0]).hdr.version); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdr.version.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_6nmesgs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_6nmesgs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_6nmesgs___get__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":79
 *             return self.istr[0].hdr.version
 *     property nmesgs:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].hdr.nmesgs
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_6nmesgs___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":80
 *     property nmesgs:
 *         def __get__(self):
 *             return self.istr[0].hdr.nmesgs             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->__pyx_base.istr[0]).hdr.nmesgs); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdr.nmesgs.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_5_OHdr_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5o_5_OHdr_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr___init__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":82
 *             return self.istr[0].hdr.nmesgs
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.space = _OHdrSpace()
 *         self.mesg = _OHdrMesg()
 */

static int __pyx_pf_4h5py_3h5o_5_OHdr___init__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "h5py/h5o.pyx":83
 * 
 *     def __init__(self):
 *         self.space = _OHdrSpace()             # <<<<<<<<<<<<<<
 *         self.mesg = _OHdrMesg()
 * 
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o__OHdrSpace)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 83; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->space);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->space));
  __pyx_v_self->space = ((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5o.pyx":84
 *     def __init__(self):
 *         self.space = _OHdrSpace()
 *         self.mesg = _OHdrMesg()             # <<<<<<<<<<<<<<
 * 
 *     def _hash(self):
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o__OHdrMesg)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->mesg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->mesg));
  __pyx_v_self->mesg = ((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)__pyx_t_1);
  __pyx_t_1 = 0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._OHdr.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_3_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_3_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_hash (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_2_hash(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":86
 *         self.mesg = _OHdrMesg()
 * 
 *     def _hash(self):             # <<<<<<<<<<<<<<
 *         return hash((self.version, self.nmesgs, self.space, self.mesg))
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_2_hash(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_hash_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_hash", 0);

  /* "h5py/h5o.pyx":87
 * 
 *     def _hash(self):
 *         return hash((self.version, self.nmesgs, self.space, self.mesg))             # <<<<<<<<<<<<<<
 * 
 * cdef class _ObjInfo(_ObjInfoBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__version); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__nmesgs); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->space));
  PyTuple_SET_ITEM(__pyx_t_3, 2, ((PyObject *)__pyx_v_self->space));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->space));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mesg));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->mesg));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->mesg));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_4 = PyObject_Hash(((PyObject *)__pyx_t_3)); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_FromHash_t(__pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5o._OHdr._hash", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_5space_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_5space_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_5space___get__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":72
 * cdef class _OHdr(_ObjInfoBase):
 * 
 *     cdef public _OHdrSpace space             # <<<<<<<<<<<<<<
 *     cdef public _OHdrMesg mesg
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_5space___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->space));
  __pyx_r = ((PyObject *)__pyx_v_self->space);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_5_OHdr_5space_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4h5py_3h5o_5_OHdr_5space_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_5space_2__set__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self), ((PyObject *)__pyx_v_value));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4h5py_3h5o_5_OHdr_5space_2__set__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_4h5py_3h5o__OHdrSpace))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 72; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->space);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->space));
  __pyx_v_self->space = ((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)__pyx_v_value);

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5o._OHdr.space.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_5_OHdr_5space_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_4h5py_3h5o_5_OHdr_5space_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_5space_4__del__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4h5py_3h5o_5_OHdr_5space_4__del__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->space);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->space));
  __pyx_v_self->space = ((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)Py_None);

  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_4mesg_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_5_OHdr_4mesg_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_4mesg___get__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":73
 * 
 *     cdef public _OHdrSpace space
 *     cdef public _OHdrMesg mesg             # <<<<<<<<<<<<<<
 * 
 *     property version:
 */

static PyObject *__pyx_pf_4h5py_3h5o_5_OHdr_4mesg___get__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->mesg));
  __pyx_r = ((PyObject *)__pyx_v_self->mesg);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_4mesg_2__set__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self), ((PyObject *)__pyx_v_value));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4h5py_3h5o_5_OHdr_4mesg_2__set__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_4h5py_3h5o__OHdrMesg))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->mesg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->mesg));
  __pyx_v_self->mesg = ((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)__pyx_v_value);

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5o._OHdr.mesg.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_5_OHdr_4mesg_4__del__(((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4h5py_3h5o_5_OHdr_4mesg_4__del__(struct __pyx_obj_4h5py_3h5o__OHdr *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->mesg);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->mesg));
  __pyx_v_self->mesg = ((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)Py_None);

  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_6fileno_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_6fileno_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_8_ObjInfo_6fileno___get__(((struct __pyx_obj_4h5py_3h5o__ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":92
 * 
 *     property fileno:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].fileno
 *     property addr:
 */

static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_6fileno___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":93
 *     property fileno:
 *         def __get__(self):
 *             return self.istr[0].fileno             # <<<<<<<<<<<<<<
 *     property addr:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->__pyx_base.istr[0]).fileno); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._ObjInfo.fileno.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_4addr_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_4addr_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_8_ObjInfo_4addr___get__(((struct __pyx_obj_4h5py_3h5o__ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":95
 *             return self.istr[0].fileno
 *     property addr:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].addr
 *     property type:
 */

static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_4addr___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":96
 *     property addr:
 *         def __get__(self):
 *             return self.istr[0].addr             # <<<<<<<<<<<<<<
 *     property type:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_haddr_t((__pyx_v_self->__pyx_base.istr[0]).addr); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._ObjInfo.addr.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_8_ObjInfo_4type___get__(((struct __pyx_obj_4h5py_3h5o__ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":98
 *             return self.istr[0].addr
 *     property type:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return <int>self.istr[0].type
 *     property rc:
 */

static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_4type___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":99
 *     property type:
 *         def __get__(self):
 *             return <int>self.istr[0].type             # <<<<<<<<<<<<<<
 *     property rc:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromLong(((int)(__pyx_v_self->__pyx_base.istr[0]).type)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._ObjInfo.type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_2rc_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_2rc_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_8_ObjInfo_2rc___get__(((struct __pyx_obj_4h5py_3h5o__ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":101
 *             return <int>self.istr[0].type
 *     property rc:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.istr[0].rc
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo_2rc___get__(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5o.pyx":102
 *     property rc:
 *         def __get__(self):
 *             return self.istr[0].rc             # <<<<<<<<<<<<<<
 * 
 *     def _hash(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->__pyx_base.istr[0]).rc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 102; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._ObjInfo.rc.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_8_ObjInfo_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_hash (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_8_ObjInfo__hash(((struct __pyx_obj_4h5py_3h5o__ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":104
 *             return self.istr[0].rc
 * 
 *     def _hash(self):             # <<<<<<<<<<<<<<
 *         return hash((self.fileno, self.addr, self.type, self.rc))
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5o_8_ObjInfo__hash(struct __pyx_obj_4h5py_3h5o__ObjInfo *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_hash_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_hash", 0);

  /* "h5py/h5o.pyx":105
 * 
 *     def _hash(self):
 *         return hash((self.fileno, self.addr, self.type, self.rc))             # <<<<<<<<<<<<<<
 * 
 * cdef class ObjInfo(_ObjInfo):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__fileno); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__addr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__type); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__rc); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = PyObject_Hash(((PyObject *)__pyx_t_5)); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_FromHash_t(__pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("h5py.h5o._ObjInfo._hash", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_7ObjInfo_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5o_7ObjInfo_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_4h5py_3h5o_7ObjInfo___init__(((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":116
 *     cdef public _OHdr hdr
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.hdr = _OHdr()
 * 
 */

static int __pyx_pf_4h5py_3h5o_7ObjInfo___init__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "h5py/h5o.pyx":117
 * 
 *     def __init__(self):
 *         self.hdr = _OHdr()             # <<<<<<<<<<<<<<
 * 
 *         self.istr = &self.infostruct
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o__OHdr)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->hdr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->hdr));
  __pyx_v_self->hdr = ((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5o.pyx":119
 *         self.hdr = _OHdr()
 * 
 *         self.istr = &self.infostruct             # <<<<<<<<<<<<<<
 *         self.hdr.istr = &self.infostruct
 *         self.hdr.space.istr = &self.infostruct
 */
  __pyx_v_self->__pyx_base.__pyx_base.istr = (&__pyx_v_self->infostruct);

  /* "h5py/h5o.pyx":120
 * 
 *         self.istr = &self.infostruct
 *         self.hdr.istr = &self.infostruct             # <<<<<<<<<<<<<<
 *         self.hdr.space.istr = &self.infostruct
 *         self.hdr.mesg.istr = &self.infostruct
 */
  __pyx_v_self->hdr->__pyx_base.istr = (&__pyx_v_self->infostruct);

  /* "h5py/h5o.pyx":121
 *         self.istr = &self.infostruct
 *         self.hdr.istr = &self.infostruct
 *         self.hdr.space.istr = &self.infostruct             # <<<<<<<<<<<<<<
 *         self.hdr.mesg.istr = &self.infostruct
 * 
 */
  __pyx_v_self->hdr->space->__pyx_base.istr = (&__pyx_v_self->infostruct);

  /* "h5py/h5o.pyx":122
 *         self.hdr.istr = &self.infostruct
 *         self.hdr.space.istr = &self.infostruct
 *         self.hdr.mesg.istr = &self.infostruct             # <<<<<<<<<<<<<<
 * 
 *     def __copy__(self):
 */
  __pyx_v_self->hdr->mesg->__pyx_base.istr = (&__pyx_v_self->infostruct);

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o.ObjInfo.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_7ObjInfo_3__copy__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_7ObjInfo_3__copy__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__copy__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_7ObjInfo_2__copy__(((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":124
 *         self.hdr.mesg.istr = &self.infostruct
 * 
 *     def __copy__(self):             # <<<<<<<<<<<<<<
 *         cdef ObjInfo newcopy
 *         newcopy = ObjInfo()
 */

static PyObject *__pyx_pf_4h5py_3h5o_7ObjInfo_2__copy__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self) {
  struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_newcopy = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  H5O_info_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__copy__", 0);

  /* "h5py/h5o.pyx":126
 *     def __copy__(self):
 *         cdef ObjInfo newcopy
 *         newcopy = ObjInfo()             # <<<<<<<<<<<<<<
 *         newcopy.infostruct = self.infostruct
 *         return newcopy
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o_ObjInfo)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 126; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_newcopy = ((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5o.pyx":127
 *         cdef ObjInfo newcopy
 *         newcopy = ObjInfo()
 *         newcopy.infostruct = self.infostruct             # <<<<<<<<<<<<<<
 *         return newcopy
 * 
 */
  __pyx_t_2 = __pyx_v_self->infostruct;
  __pyx_v_newcopy->infostruct = __pyx_t_2;

  /* "h5py/h5o.pyx":128
 *         newcopy = ObjInfo()
 *         newcopy.infostruct = self.infostruct
 *         return newcopy             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_newcopy));
  __pyx_r = ((PyObject *)__pyx_v_newcopy);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o.ObjInfo.__copy__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_newcopy);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr___get__(((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":114
 * 
 *     cdef H5O_info_t infostruct
 *     cdef public _OHdr hdr             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self):
 */

static PyObject *__pyx_pf_4h5py_3h5o_7ObjInfo_3hdr___get__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->hdr));
  __pyx_r = ((PyObject *)__pyx_v_self->hdr);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr_2__set__(((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_v_self), ((PyObject *)__pyx_v_value));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr_2__set__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_4h5py_3h5o__OHdr))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->hdr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->hdr));
  __pyx_v_self->hdr = ((struct __pyx_obj_4h5py_3h5o__OHdr *)__pyx_v_value);

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5o.ObjInfo.hdr.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr_4__del__(((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4h5py_3h5o_7ObjInfo_3hdr_4__del__(struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->hdr);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->hdr));
  __pyx_v_self->hdr = ((struct __pyx_obj_4h5py_3h5o__OHdr *)Py_None);

  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_1get_info(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_get_info[] = "(ObjectID loc, STRING name=, INT index=, **kwds) => ObjInfo\n\n    Get information describing an object in an HDF5 file.  Provide the object\n    itself, or the containing group and exactly one of \"name\" or \"index\".\n\n    STRING obj_name (\".\")\n        When \"index\" is specified, look in this subgroup instead.\n        Otherwise ignored.\n\n    PropID lapl (None)\n        Link access property list\n\n    INT index_type (h5.INDEX_NAME)\n\n    INT order (h5.ITER_NATIVE)\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_1get_info = {__Pyx_NAMESTR("get_info"), (PyCFunction)__pyx_pw_4h5py_3h5o_1get_info, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_get_info)};
static PyObject *__pyx_pw_4h5py_3h5o_1get_info(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  char *__pyx_v_name;
  int __pyx_v_index;
  char *__pyx_v_obj_name;
  int __pyx_v_index_type;
  int __pyx_v_order;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_info (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__name,&__pyx_n_s__index,&__pyx_n_s__obj_name,&__pyx_n_s__index_type,&__pyx_n_s__order,&__pyx_n_s__lapl,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};

    /* "h5py/h5o.pyx":133
 * def get_info(ObjectID loc not None, char* name=NULL, int index=-1, *,
 *         char* obj_name='.', int index_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *         PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name=, INT index=, **kwds) => ObjInfo
 * 
 */
    values[6] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__index);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (kw_args > 0 && likely(kw_args <= 4)) {
        Py_ssize_t index;
        for (index = 3; index < 7 && kw_args > 0; index++) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_info") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    if (values[1]) {
      __pyx_v_name = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {

      /* "h5py/h5o.pyx":131
 * 
 * 
 * def get_info(ObjectID loc not None, char* name=NULL, int index=-1, *,             # <<<<<<<<<<<<<<
 *         char* obj_name='.', int index_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *         PropID lapl=None):
 */
      __pyx_v_name = ((char *)NULL);
    }
    if (values[2]) {
      __pyx_v_index = __Pyx_PyInt_AsInt(values[2]); if (unlikely((__pyx_v_index == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_index = ((int)-1);
    }
    if (values[3]) {
      __pyx_v_obj_name = PyBytes_AsString(values[3]); if (unlikely((!__pyx_v_obj_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_obj_name = ((char *)__pyx_k_1);
    }
    if (values[4]) {
      __pyx_v_index_type = __Pyx_PyInt_AsInt(values[4]); if (unlikely((__pyx_v_index_type == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_index_type = __pyx_k_2;
    }
    if (values[5]) {
      __pyx_v_order = __Pyx_PyInt_AsInt(values[5]); if (unlikely((__pyx_v_order == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_order = __pyx_k_3;
    }
    __pyx_v_lapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[6]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_info", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.get_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_get_info(__pyx_self, __pyx_v_loc, __pyx_v_name, __pyx_v_index, __pyx_v_obj_name, __pyx_v_index_type, __pyx_v_order, __pyx_v_lapl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5o_get_info(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_name, int __pyx_v_index, char *__pyx_v_obj_name, int __pyx_v_index_type, int __pyx_v_order, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl) {
  struct __pyx_obj_4h5py_3h5o_ObjInfo *__pyx_v_info = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  herr_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_info", 0);

  /* "h5py/h5o.pyx":151
 *     """
 *     cdef ObjInfo info
 *     info = ObjInfo()             # <<<<<<<<<<<<<<
 * 
 *     if name != NULL and index >= 0:
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o_ObjInfo)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 151; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_info = ((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5o.pyx":153
 *     info = ObjInfo()
 * 
 *     if name != NULL and index >= 0:             # <<<<<<<<<<<<<<
 *         raise TypeError("At most one of name or index may be specified")
 *     elif name != NULL and index < 0:
 */
  __pyx_t_2 = (__pyx_v_name != NULL);
  if (__pyx_t_2) {
    __pyx_t_3 = (__pyx_v_index >= 0);
    __pyx_t_4 = __pyx_t_3;
  } else {
    __pyx_t_4 = __pyx_t_2;
  }
  if (__pyx_t_4) {

    /* "h5py/h5o.pyx":154
 * 
 *     if name != NULL and index >= 0:
 *         raise TypeError("At most one of name or index may be specified")             # <<<<<<<<<<<<<<
 *     elif name != NULL and index < 0:
 *         H5Oget_info_by_name(loc.id, name, &info.infostruct, pdefault(lapl))
 */
    __pyx_t_1 = PyObject_Call(__pyx_builtin_TypeError, ((PyObject *)__pyx_k_tuple_5), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }

  /* "h5py/h5o.pyx":155
 *     if name != NULL and index >= 0:
 *         raise TypeError("At most one of name or index may be specified")
 *     elif name != NULL and index < 0:             # <<<<<<<<<<<<<<
 *         H5Oget_info_by_name(loc.id, name, &info.infostruct, pdefault(lapl))
 *     elif name == NULL and index >= 0:
 */
  __pyx_t_4 = (__pyx_v_name != NULL);
  if (__pyx_t_4) {
    __pyx_t_2 = (__pyx_v_index < 0);
    __pyx_t_3 = __pyx_t_2;
  } else {
    __pyx_t_3 = __pyx_t_4;
  }
  if (__pyx_t_3) {

    /* "h5py/h5o.pyx":156
 *         raise TypeError("At most one of name or index may be specified")
 *     elif name != NULL and index < 0:
 *         H5Oget_info_by_name(loc.id, name, &info.infostruct, pdefault(lapl))             # <<<<<<<<<<<<<<
 *     elif name == NULL and index >= 0:
 *         H5Oget_info_by_idx(loc.id, obj_name, <H5_index_t>index_type,
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Oget_info_by_name(__pyx_v_loc->id, __pyx_v_name, (&__pyx_v_info->infostruct), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }

  /* "h5py/h5o.pyx":157
 *     elif name != NULL and index < 0:
 *         H5Oget_info_by_name(loc.id, name, &info.infostruct, pdefault(lapl))
 *     elif name == NULL and index >= 0:             # <<<<<<<<<<<<<<
 *         H5Oget_info_by_idx(loc.id, obj_name, <H5_index_t>index_type,
 *             <H5_iter_order_t>order, index, &info.infostruct, pdefault(lapl))
 */
  __pyx_t_3 = (__pyx_v_name == NULL);
  if (__pyx_t_3) {
    __pyx_t_4 = (__pyx_v_index >= 0);
    __pyx_t_2 = __pyx_t_4;
  } else {
    __pyx_t_2 = __pyx_t_3;
  }
  if (__pyx_t_2) {

    /* "h5py/h5o.pyx":159
 *     elif name == NULL and index >= 0:
 *         H5Oget_info_by_idx(loc.id, obj_name, <H5_index_t>index_type,
 *             <H5_iter_order_t>order, index, &info.infostruct, pdefault(lapl))             # <<<<<<<<<<<<<<
 *     else:
 *         H5Oget_info(loc.id, &info.infostruct)
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Oget_info_by_idx(__pyx_v_loc->id, __pyx_v_obj_name, ((H5_index_t)__pyx_v_index_type), ((H5_iter_order_t)__pyx_v_order), __pyx_v_index, (&__pyx_v_info->infostruct), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/h5o.pyx":161
 *             <H5_iter_order_t>order, index, &info.infostruct, pdefault(lapl))
 *     else:
 *         H5Oget_info(loc.id, &info.infostruct)             # <<<<<<<<<<<<<<
 * 
 *     return info
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Oget_info(__pyx_v_loc->id, (&__pyx_v_info->infostruct)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L3:;

  /* "h5py/h5o.pyx":163
 *         H5Oget_info(loc.id, &info.infostruct)
 * 
 *     return info             # <<<<<<<<<<<<<<
 * 
 * # === General object operations ===============================================
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_info));
  __pyx_r = ((PyObject *)__pyx_v_info);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o.get_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_info);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_3open(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_2open[] = "(ObjectID loc, STRING name, PropID lapl=None) => ObjectID\n\n    Open a group, dataset, or named datatype attached to an existing group.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_3open = {__Pyx_NAMESTR("open"), (PyCFunction)__pyx_pw_4h5py_3h5o_3open, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_2open)};
static PyObject *__pyx_pw_4h5py_3h5o_3open(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  char *__pyx_v_name;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("open (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__name,&__pyx_n_s__lapl,0};
    PyObject* values[3] = {0,0,0};

    /* "h5py/h5o.pyx":168
 * 
 * 
 * def open(ObjectID loc not None, char* name, PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name, PropID lapl=None) => ObjectID
 * 
 */
    values[2] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("open", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__lapl);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "open") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_name = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_lapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("open", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_2open(__pyx_self, __pyx_v_loc, __pyx_v_name, __pyx_v_lapl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5o_2open(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);

  /* "h5py/h5o.pyx":173
 *     Open a group, dataset, or named datatype attached to an existing group.
 *     """
 *     return wrap_identifier(H5Oopen(loc.id, name, pdefault(lapl)))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Oopen(__pyx_v_loc->id, __pyx_v_name, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((PyObject *)__pyx_f_4h5py_3h5i_wrap_identifier(__pyx_t_1, 0)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5o.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_5link(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_4link[] = "(ObjectID obj, GroupID loc, STRING name, PropID lcpl=None,\n    PropID lapl=None)\n\n    Create a new hard link to an object.  Useful for objects created with\n    h5g.create_anon() or h5d.create_anon().\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_5link = {__Pyx_NAMESTR("link"), (PyCFunction)__pyx_pw_4h5py_3h5o_5link, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_4link)};
static PyObject *__pyx_pw_4h5py_3h5o_5link(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_obj = 0;
  struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_loc = 0;
  char *__pyx_v_name;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl = 0;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("link (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__obj,&__pyx_n_s__loc,&__pyx_n_s__name,&__pyx_n_s__lcpl,&__pyx_n_s__lapl,0};
    PyObject* values[5] = {0,0,0,0,0};

    /* "h5py/h5o.pyx":177
 * 
 * def link(ObjectID obj not None, GroupID loc not None, char* name,
 *     PropID lcpl=None, PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID obj, GroupID loc, STRING name, PropID lcpl=None,
 *     PropID lapl=None)
 */
    values[3] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    values[4] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__obj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("link", 0, 3, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("link", 0, 3, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__lcpl);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__lapl);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "link") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_obj = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_loc = ((struct __pyx_obj_4h5py_3h5g_GroupID *)values[1]);
    __pyx_v_name = PyBytes_AsString(values[2]); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_lcpl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[3]);
    __pyx_v_lapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[4]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("link", 0, 3, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_obj), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "obj", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_3h5g_GroupID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lcpl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lcpl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_4link(__pyx_self, __pyx_v_obj, __pyx_v_loc, __pyx_v_name, __pyx_v_lcpl, __pyx_v_lapl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":176
 * 
 * 
 * def link(ObjectID obj not None, GroupID loc not None, char* name,             # <<<<<<<<<<<<<<
 *     PropID lcpl=None, PropID lapl=None):
 *     """(ObjectID obj, GroupID loc, STRING name, PropID lcpl=None,
 */

static PyObject *__pyx_pf_4h5py_3h5o_4link(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_obj, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_loc, char *__pyx_v_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);

  /* "h5py/h5o.pyx":184
 *     h5g.create_anon() or h5d.create_anon().
 *     """
 *     H5Olink(obj.id, loc.id, name, pdefault(lcpl), pdefault(lapl))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Olink(__pyx_v_obj->id, __pyx_v_loc->__pyx_base.id, __pyx_v_name, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lcpl)), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 184; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5o.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_7copy(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_6copy[] = "(ObjectID src_loc, STRING src_name, GroupID dst_loc, STRING dst_name,\n    PropID copypl=None, PropID lcpl=None)\n\n    Copy a group, dataset or named datatype from one location to another.  The\n    source and destination need not be in the same file.\n\n    The default behavior is a recursive copy of the object and all objects\n    below it.  This behavior is modified via the \"copypl\" property list.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_7copy = {__Pyx_NAMESTR("copy"), (PyCFunction)__pyx_pw_4h5py_3h5o_7copy, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_6copy)};
static PyObject *__pyx_pw_4h5py_3h5o_7copy(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_src_loc = 0;
  char *__pyx_v_src_name;
  struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_dst_loc = 0;
  char *__pyx_v_dst_name;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_copypl = 0;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__src_loc,&__pyx_n_s__src_name,&__pyx_n_s__dst_loc,&__pyx_n_s__dst_name,&__pyx_n_s__copypl,&__pyx_n_s__lcpl,0};
    PyObject* values[6] = {0,0,0,0,0,0};

    /* "h5py/h5o.pyx":188
 * 
 * def copy(ObjectID src_loc not None, char* src_name, GroupID dst_loc not None,
 *     char* dst_name, PropID copypl=None, PropID lcpl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID src_loc, STRING src_name, GroupID dst_loc, STRING dst_name,
 *     PropID copypl=None, PropID lcpl=None)
 */
    values[4] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    values[5] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__src_loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__src_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("copy", 0, 4, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__dst_loc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("copy", 0, 4, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__dst_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("copy", 0, 4, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__copypl);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__lcpl);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "copy") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_src_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_src_name = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_src_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_dst_loc = ((struct __pyx_obj_4h5py_3h5g_GroupID *)values[2]);
    __pyx_v_dst_name = PyBytes_AsString(values[3]); if (unlikely((!__pyx_v_dst_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_copypl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[4]);
    __pyx_v_lcpl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[5]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("copy", 0, 4, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_src_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "src_loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dst_loc), __pyx_ptype_4h5py_3h5g_GroupID, 0, "dst_loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_copypl), __pyx_ptype_4h5py_3h5p_PropID, 1, "copypl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lcpl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lcpl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_6copy(__pyx_self, __pyx_v_src_loc, __pyx_v_src_name, __pyx_v_dst_loc, __pyx_v_dst_name, __pyx_v_copypl, __pyx_v_lcpl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":187
 * 
 * 
 * def copy(ObjectID src_loc not None, char* src_name, GroupID dst_loc not None,             # <<<<<<<<<<<<<<
 *     char* dst_name, PropID copypl=None, PropID lcpl=None):
 *     """(ObjectID src_loc, STRING src_name, GroupID dst_loc, STRING dst_name,
 */

static PyObject *__pyx_pf_4h5py_3h5o_6copy(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_src_loc, char *__pyx_v_src_name, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_dst_loc, char *__pyx_v_dst_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_copypl, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "h5py/h5o.pyx":199
 *     """
 *     H5Ocopy(src_loc.id, src_name, dst_loc.id, dst_name, pdefault(copypl),
 *         pdefault(lcpl))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Ocopy(__pyx_v_src_loc->id, __pyx_v_src_name, __pyx_v_dst_loc->__pyx_base.id, __pyx_v_dst_name, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_copypl)), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lcpl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5o.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_9set_comment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_8set_comment[] = "(ObjectID loc, STRING comment, **kwds)\n\n    Set the comment for any-file resident object.  Keywords:\n\n    STRING obj_name (\".\")\n        Set comment on this group member instead\n\n    PropID lapl (None)\n        Link access property list\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_9set_comment = {__Pyx_NAMESTR("set_comment"), (PyCFunction)__pyx_pw_4h5py_3h5o_9set_comment, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_8set_comment)};
static PyObject *__pyx_pw_4h5py_3h5o_9set_comment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  char *__pyx_v_comment;
  char *__pyx_v_obj_name;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_comment (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__comment,&__pyx_n_s__obj_name,&__pyx_n_s__lapl,0};
    PyObject* values[4] = {0,0,0,0};

    /* "h5py/h5o.pyx":203
 * 
 * def set_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",
 *     PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING comment, **kwds)
 * 
 */
    values[3] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__comment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_comment", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 2; index < 4 && kw_args > 0; index++) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_comment") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_comment = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_comment) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_obj_name = PyBytes_AsString(values[2]); if (unlikely((!__pyx_v_obj_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_obj_name = ((char *)__pyx_k_1);
    }
    __pyx_v_lapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_comment", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.set_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_8set_comment(__pyx_self, __pyx_v_loc, __pyx_v_comment, __pyx_v_obj_name, __pyx_v_lapl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":202
 * 
 * 
 * def set_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",             # <<<<<<<<<<<<<<
 *     PropID lapl=None):
 *     """(ObjectID loc, STRING comment, **kwds)
 */

static PyObject *__pyx_pf_4h5py_3h5o_8set_comment(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_comment, char *__pyx_v_obj_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_comment", 0);

  /* "h5py/h5o.pyx":214
 *         Link access property list
 *     """
 *     H5Oset_comment_by_name(loc.id, obj_name, comment, pdefault(lapl))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Oset_comment_by_name(__pyx_v_loc->id, __pyx_v_obj_name, __pyx_v_comment, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5o.set_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_11get_comment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_10get_comment[] = "(ObjectID loc, STRING comment, **kwds)\n\n    Get the comment for any-file resident object.  Keywords:\n\n    STRING obj_name (\".\")\n        Set comment on this group member instead\n\n    PropID lapl (None)\n        Link access property list\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_11get_comment = {__Pyx_NAMESTR("get_comment"), (PyCFunction)__pyx_pw_4h5py_3h5o_11get_comment, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_10get_comment)};
static PyObject *__pyx_pw_4h5py_3h5o_11get_comment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  CYTHON_UNUSED char *__pyx_v_comment;
  char *__pyx_v_obj_name;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_comment (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__comment,&__pyx_n_s__obj_name,&__pyx_n_s__lapl,0};
    PyObject* values[4] = {0,0,0,0};

    /* "h5py/h5o.pyx":219
 * 
 * def get_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",
 *     PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING comment, **kwds)
 * 
 */
    values[3] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__comment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_comment", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 2; index < 4 && kw_args > 0; index++) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_comment") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_comment = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_comment) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_obj_name = PyBytes_AsString(values[2]); if (unlikely((!__pyx_v_obj_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_obj_name = ((char *)__pyx_k_1);
    }
    __pyx_v_lapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_comment", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.get_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_10get_comment(__pyx_self, __pyx_v_loc, __pyx_v_comment, __pyx_v_obj_name, __pyx_v_lapl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":218
 * 
 * 
 * def get_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",             # <<<<<<<<<<<<<<
 *     PropID lapl=None):
 *     """(ObjectID loc, STRING comment, **kwds)
 */

static PyObject *__pyx_pf_4h5py_3h5o_10get_comment(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, CYTHON_UNUSED char *__pyx_v_comment, char *__pyx_v_obj_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl) {
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buf;
  char *__pyx_v_pstring;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_comment", 0);

  /* "h5py/h5o.pyx":233
 *     cdef char* buf
 * 
 *     size = H5Oget_comment_by_name(loc.id, obj_name, NULL, 0, pdefault(lapl))             # <<<<<<<<<<<<<<
 *     buf = <char*>emalloc(size+1)
 *     try:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Oget_comment_by_name(__pyx_v_loc->id, __pyx_v_obj_name, NULL, 0, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_size = __pyx_t_1;

  /* "h5py/h5o.pyx":234
 * 
 *     size = H5Oget_comment_by_name(loc.id, obj_name, NULL, 0, pdefault(lapl))
 *     buf = <char*>emalloc(size+1)             # <<<<<<<<<<<<<<
 *     try:
 *         H5Oget_comment_by_name(loc.id, obj_name, buf, size+1, pdefault(lapl))
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc((__pyx_v_size + 1)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 234; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = ((char *)__pyx_t_2);

  /* "h5py/h5o.pyx":235
 *     size = H5Oget_comment_by_name(loc.id, obj_name, NULL, 0, pdefault(lapl))
 *     buf = <char*>emalloc(size+1)
 *     try:             # <<<<<<<<<<<<<<
 *         H5Oget_comment_by_name(loc.id, obj_name, buf, size+1, pdefault(lapl))
 *         pstring = buf
 */
  /*try:*/ {

    /* "h5py/h5o.pyx":236
 *     buf = <char*>emalloc(size+1)
 *     try:
 *         H5Oget_comment_by_name(loc.id, obj_name, buf, size+1, pdefault(lapl))             # <<<<<<<<<<<<<<
 *         pstring = buf
 *     finally:
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Oget_comment_by_name(__pyx_v_loc->id, __pyx_v_obj_name, __pyx_v_buf, (__pyx_v_size + 1), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5o.pyx":237
 *     try:
 *         H5Oget_comment_by_name(loc.id, obj_name, buf, size+1, pdefault(lapl))
 *         pstring = buf             # <<<<<<<<<<<<<<
 *     finally:
 *         efree(buf)
 */
    __pyx_v_pstring = __pyx_v_buf;
  }

  /* "h5py/h5o.pyx":239
 *         pstring = buf
 *     finally:
 *         efree(buf)             # <<<<<<<<<<<<<<
 * 
 *     return pstring
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_buf);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  /* "h5py/h5o.pyx":241
 *         efree(buf)
 * 
 *     return pstring             # <<<<<<<<<<<<<<
 * 
 * # === Visit routines ==========================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyBytes_FromString(__pyx_v_pstring); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_3));
  __pyx_r = ((PyObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5o.get_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5o_14_ObjectVisitor_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5o_14_ObjectVisitor_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_func = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__func,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__func)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_func = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o._ObjectVisitor.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5o_14_ObjectVisitor___init__(((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)__pyx_v_self), __pyx_v_func);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":251
 *     cdef ObjInfo objinfo
 * 
 *     def __init__(self, func):             # <<<<<<<<<<<<<<
 *         self.func = func
 *         self.retval = None
 */

static int __pyx_pf_4h5py_3h5o_14_ObjectVisitor___init__(struct __pyx_obj_4h5py_3h5o__ObjectVisitor *__pyx_v_self, PyObject *__pyx_v_func) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "h5py/h5o.pyx":252
 * 
 *     def __init__(self, func):
 *         self.func = func             # <<<<<<<<<<<<<<
 *         self.retval = None
 *         self.objinfo = ObjInfo()
 */
  __Pyx_INCREF(__pyx_v_func);
  __Pyx_GIVEREF(__pyx_v_func);
  __Pyx_GOTREF(__pyx_v_self->func);
  __Pyx_DECREF(__pyx_v_self->func);
  __pyx_v_self->func = __pyx_v_func;

  /* "h5py/h5o.pyx":253
 *     def __init__(self, func):
 *         self.func = func
 *         self.retval = None             # <<<<<<<<<<<<<<
 *         self.objinfo = ObjInfo()
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->retval);
  __Pyx_DECREF(__pyx_v_self->retval);
  __pyx_v_self->retval = Py_None;

  /* "h5py/h5o.pyx":254
 *         self.func = func
 *         self.retval = None
 *         self.objinfo = ObjInfo()             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t cb_obj_iterate(hid_t obj, char* name, H5O_info_t *info, void* data) except 2:
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o_ObjInfo)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->objinfo);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->objinfo));
  __pyx_v_self->objinfo = ((struct __pyx_obj_4h5py_3h5o_ObjInfo *)__pyx_t_1);
  __pyx_t_1 = 0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5o._ObjectVisitor.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":256
 *         self.objinfo = ObjInfo()
 * 
 * cdef herr_t cb_obj_iterate(hid_t obj, char* name, H5O_info_t *info, void* data) except 2:             # <<<<<<<<<<<<<<
 * 
 *     cdef _ObjectVisitor visit
 */

static herr_t __pyx_f_4h5py_3h5o_cb_obj_iterate(CYTHON_UNUSED hid_t __pyx_v_obj, char *__pyx_v_name, H5O_info_t *__pyx_v_info, void *__pyx_v_data) {
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *__pyx_v_visit = 0;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb_obj_iterate", 0);

  /* "h5py/h5o.pyx":261
 * 
 *     # HDF5 doesn't respect callback return for ".", so skip it
 *     if strcmp(name, ".") == 0:             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_1 = (strcmp(__pyx_v_name, __pyx_k_1) == 0);
  if (__pyx_t_1) {

    /* "h5py/h5o.pyx":262
 *     # HDF5 doesn't respect callback return for ".", so skip it
 *     if strcmp(name, ".") == 0:
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     visit = <_ObjectVisitor>data
 */
    __pyx_r = 0;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5o.pyx":264
 *         return 0
 * 
 *     visit = <_ObjectVisitor>data             # <<<<<<<<<<<<<<
 *     visit.objinfo.infostruct = info[0]
 *     visit.retval = visit.func(name, visit.objinfo)
 */
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)__pyx_v_data)));
  __pyx_v_visit = ((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)__pyx_v_data);

  /* "h5py/h5o.pyx":265
 * 
 *     visit = <_ObjectVisitor>data
 *     visit.objinfo.infostruct = info[0]             # <<<<<<<<<<<<<<
 *     visit.retval = visit.func(name, visit.objinfo)
 * 
 */
  __pyx_v_visit->objinfo->infostruct = (__pyx_v_info[0]);

  /* "h5py/h5o.pyx":266
 *     visit = <_ObjectVisitor>data
 *     visit.objinfo.infostruct = info[0]
 *     visit.retval = visit.func(name, visit.objinfo)             # <<<<<<<<<<<<<<
 * 
 *     if visit.retval is not None:
 */
  __pyx_t_2 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_2));
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_t_2));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_2));
  __Pyx_INCREF(((PyObject *)__pyx_v_visit->objinfo));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_visit->objinfo));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_visit->objinfo));
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_v_visit->func, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_visit->retval);
  __Pyx_DECREF(__pyx_v_visit->retval);
  __pyx_v_visit->retval = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "h5py/h5o.pyx":268
 *     visit.retval = visit.func(name, visit.objinfo)
 * 
 *     if visit.retval is not None:             # <<<<<<<<<<<<<<
 *         return 1
 *     return 0
 */
  __pyx_t_1 = (__pyx_v_visit->retval != Py_None);
  if (__pyx_t_1) {

    /* "h5py/h5o.pyx":269
 * 
 *     if visit.retval is not None:
 *         return 1             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_r = 1;
    goto __pyx_L0;
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "h5py/h5o.pyx":270
 *     if visit.retval is not None:
 *         return 1
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t cb_obj_simple(hid_t obj, char* name, H5O_info_t *info, void* data) except 2:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5o.cb_obj_iterate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 2;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_visit);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":272
 *     return 0
 * 
 * cdef herr_t cb_obj_simple(hid_t obj, char* name, H5O_info_t *info, void* data) except 2:             # <<<<<<<<<<<<<<
 * 
 *     cdef _ObjectVisitor visit
 */

static herr_t __pyx_f_4h5py_3h5o_cb_obj_simple(CYTHON_UNUSED hid_t __pyx_v_obj, char *__pyx_v_name, CYTHON_UNUSED H5O_info_t *__pyx_v_info, void *__pyx_v_data) {
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *__pyx_v_visit = 0;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb_obj_simple", 0);

  /* "h5py/h5o.pyx":277
 * 
 *     # Not all versions of HDF5 respect callback value for ".", so skip it
 *     if strcmp(name, ".") == 0:             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_1 = (strcmp(__pyx_v_name, __pyx_k_1) == 0);
  if (__pyx_t_1) {

    /* "h5py/h5o.pyx":278
 *     # Not all versions of HDF5 respect callback value for ".", so skip it
 *     if strcmp(name, ".") == 0:
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     visit = <_ObjectVisitor>data
 */
    __pyx_r = 0;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5o.pyx":280
 *         return 0
 * 
 *     visit = <_ObjectVisitor>data             # <<<<<<<<<<<<<<
 *     visit.retval = visit.func(name)
 * 
 */
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)__pyx_v_data)));
  __pyx_v_visit = ((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)__pyx_v_data);

  /* "h5py/h5o.pyx":281
 * 
 *     visit = <_ObjectVisitor>data
 *     visit.retval = visit.func(name)             # <<<<<<<<<<<<<<
 * 
 *     if visit.retval is not None:
 */
  __pyx_t_2 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_2));
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_t_2));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_2));
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_v_visit->func, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 281; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_visit->retval);
  __Pyx_DECREF(__pyx_v_visit->retval);
  __pyx_v_visit->retval = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "h5py/h5o.pyx":283
 *     visit.retval = visit.func(name)
 * 
 *     if visit.retval is not None:             # <<<<<<<<<<<<<<
 *         return 1
 *     return 0
 */
  __pyx_t_1 = (__pyx_v_visit->retval != Py_None);
  if (__pyx_t_1) {

    /* "h5py/h5o.pyx":284
 * 
 *     if visit.retval is not None:
 *         return 1             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_r = 1;
    goto __pyx_L0;
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "h5py/h5o.pyx":285
 *     if visit.retval is not None:
 *         return 1
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5o.cb_obj_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 2;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_visit);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5o_13visit(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5o_12visit[] = "(ObjectID loc, CALLABLE func, **kwds) => <Return value from func>\n\n    Iterate a function or callable object over all objects below the\n    specified one.  Your callable should conform to the signature::\n\n        func(STRING name) => Result\n\n    or if the keyword argument \"info\" is True::\n\n        func(STRING name, ObjInfo info) => Result\n\n    Returning None continues iteration; returning anything else aborts\n    iteration and returns that value.  Keywords:\n\n    BOOL info (False)\n        Callback is func(STRING, Objinfo)\n\n    STRING obj_name (\".\")\n        Visit a subgroup of \"loc\" instead\n\n    PropLAID lapl (None)\n        Control how \"obj_name\" is interpreted\n\n    INT idx_type (h5.INDEX_NAME)\n        What indexing strategy to use\n\n    INT order (h5.ITER_NATIVE)\n        Order in which iteration occurs\n\n    Compatibility note:  No callback is executed for the starting path (\".\"),\n    as some versions of HDF5 don't correctly handle a return value for this\n    case.  This differs from the behavior of the native H5Ovisit, which\n    provides a literal \".\" as the first value.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5o_13visit = {__Pyx_NAMESTR("visit"), (PyCFunction)__pyx_pw_4h5py_3h5o_13visit, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5o_12visit)};
static PyObject *__pyx_pw_4h5py_3h5o_13visit(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  PyObject *__pyx_v_func = 0;
  int __pyx_v_idx_type;
  int __pyx_v_order;
  char *__pyx_v_obj_name;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl = 0;
  int __pyx_v_info;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("visit (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__func,&__pyx_n_s__idx_type,&__pyx_n_s__order,&__pyx_n_s__obj_name,&__pyx_n_s__lapl,&__pyx_n_s__info,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};

    /* "h5py/h5o.pyx":290
 * def visit(ObjectID loc not None, object func, *,
 *           int idx_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *           char* obj_name=".", PropID lapl=None, bint info=0):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, CALLABLE func, **kwds) => <Return value from func>
 * 
 */
    values[5] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__func)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("visit", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (kw_args > 0 && likely(kw_args <= 5)) {
        Py_ssize_t index;
        for (index = 2; index < 7 && kw_args > 0; index++) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "visit") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_func = values[1];
    if (values[2]) {
      __pyx_v_idx_type = __Pyx_PyInt_AsInt(values[2]); if (unlikely((__pyx_v_idx_type == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_idx_type = __pyx_k_6;
    }
    if (values[3]) {
      __pyx_v_order = __Pyx_PyInt_AsInt(values[3]); if (unlikely((__pyx_v_order == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_order = __pyx_k_7;
    }
    if (values[4]) {
      __pyx_v_obj_name = PyBytes_AsString(values[4]); if (unlikely((!__pyx_v_obj_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_obj_name = ((char *)__pyx_k_1);
    }
    __pyx_v_lapl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[5]);
    if (values[6]) {
      __pyx_v_info = __Pyx_PyObject_IsTrue(values[6]); if (unlikely((__pyx_v_info == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_info = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("visit", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5o.visit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lapl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lapl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5o_12visit(__pyx_self, __pyx_v_loc, __pyx_v_func, __pyx_v_idx_type, __pyx_v_order, __pyx_v_obj_name, __pyx_v_lapl, __pyx_v_info);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5o.pyx":288
 * 
 * 
 * def visit(ObjectID loc not None, object func, *,             # <<<<<<<<<<<<<<
 *           int idx_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *           char* obj_name=".", PropID lapl=None, bint info=0):
 */

static PyObject *__pyx_pf_4h5py_3h5o_12visit(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, PyObject *__pyx_v_func, int __pyx_v_idx_type, int __pyx_v_order, char *__pyx_v_obj_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lapl, int __pyx_v_info) {
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *__pyx_v_visit = 0;
  H5O_iterate_t __pyx_v_cfunc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  herr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("visit", 0);

  /* "h5py/h5o.pyx":325
 *     provides a literal "." as the first value.
 *     """
 *     cdef _ObjectVisitor visit = _ObjectVisitor(func)             # <<<<<<<<<<<<<<
 *     cdef H5O_iterate_t cfunc
 * 
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_func);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_func);
  __Pyx_GIVEREF(__pyx_v_func);
  __pyx_t_2 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5o__ObjectVisitor)), ((PyObject *)__pyx_t_1), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 325; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(((PyObject *)__pyx_t_1)); __pyx_t_1 = 0;
  __pyx_v_visit = ((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "h5py/h5o.pyx":328
 *     cdef H5O_iterate_t cfunc
 * 
 *     if info:             # <<<<<<<<<<<<<<
 *         cfunc = cb_obj_iterate
 *     else:
 */
  if (__pyx_v_info) {

    /* "h5py/h5o.pyx":329
 * 
 *     if info:
 *         cfunc = cb_obj_iterate             # <<<<<<<<<<<<<<
 *     else:
 *         cfunc = cb_obj_simple
 */
    __pyx_v_cfunc = __pyx_f_4h5py_3h5o_cb_obj_iterate;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/h5o.pyx":331
 *         cfunc = cb_obj_iterate
 *     else:
 *         cfunc = cb_obj_simple             # <<<<<<<<<<<<<<
 * 
 *     H5Ovisit_by_name(loc.id, obj_name, <H5_index_t>idx_type,
 */
    __pyx_v_cfunc = __pyx_f_4h5py_3h5o_cb_obj_simple;
  }
  __pyx_L3:;

  /* "h5py/h5o.pyx":334
 * 
 *     H5Ovisit_by_name(loc.id, obj_name, <H5_index_t>idx_type,
 *         <H5_iter_order_t>order, cfunc, <void*>visit, pdefault(lapl))             # <<<<<<<<<<<<<<
 * 
 *     return visit.retval
 */
  __pyx_t_3 = __pyx_f_4h5py_4defs_H5Ovisit_by_name(__pyx_v_loc->id, __pyx_v_obj_name, ((H5_index_t)__pyx_v_idx_type), ((H5_iter_order_t)__pyx_v_order), __pyx_v_cfunc, ((void *)__pyx_v_visit), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lapl))); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":336
 *         <H5_iter_order_t>order, cfunc, <void*>visit, pdefault(lapl))
 * 
 *     return visit.retval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_visit->retval);
  __pyx_r = __pyx_v_visit->retval;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5o.visit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_visit);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_4h5py_3h5o__ObjInfoBase(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o = (*t->tp_alloc)(t, 0);
  if (!o) return 0;
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase(PyObject *o) {
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5o__ObjInfoBase[] = {
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__ObjInfoBase = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__ObjInfoBase = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__ObjInfoBase = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__ObjInfoBase = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o__ObjInfoBase = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o._ObjInfoBase"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o__ObjInfoBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__ObjInfoBase, /*tp_as_number*/
  &__pyx_tp_as_sequence__ObjInfoBase, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__ObjInfoBase, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__ObjInfoBase, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o__ObjInfoBase, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o__ObjInfoBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5o__OHdrMesg(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5o__ObjInfoBase(t, a, k);
  if (!o) return 0;
  return o;
}

static PyObject *__pyx_getprop_4h5py_3h5o_9_OHdrMesg_present(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_9_OHdrMesg_7present_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_9_OHdrMesg_shared(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_9_OHdrMesg_6shared_1__get__(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5o__OHdrMesg[] = {
  {__Pyx_NAMESTR("_hash"), (PyCFunction)__pyx_pw_4h5py_3h5o_9_OHdrMesg_1_hash, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5o__OHdrMesg[] = {
  {(char *)"present", __pyx_getprop_4h5py_3h5o_9_OHdrMesg_present, 0, 0, 0},
  {(char *)"shared", __pyx_getprop_4h5py_3h5o_9_OHdrMesg_shared, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__OHdrMesg = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__OHdrMesg = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__OHdrMesg = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__OHdrMesg = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o__OHdrMesg = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o._OHdrMesg"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o__OHdrMesg), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__OHdrMesg, /*tp_as_number*/
  &__pyx_tp_as_sequence__OHdrMesg, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__OHdrMesg, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__OHdrMesg, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o__OHdrMesg, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5o__OHdrMesg, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o__OHdrMesg, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5o__OHdrSpace(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5o__ObjInfoBase(t, a, k);
  if (!o) return 0;
  return o;
}

static PyObject *__pyx_getprop_4h5py_3h5o_10_OHdrSpace_total(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_10_OHdrSpace_5total_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_10_OHdrSpace_meta(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_10_OHdrSpace_4meta_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_10_OHdrSpace_mesg(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_10_OHdrSpace_4mesg_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_10_OHdrSpace_free(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_10_OHdrSpace_4free_1__get__(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5o__OHdrSpace[] = {
  {__Pyx_NAMESTR("_hash"), (PyCFunction)__pyx_pw_4h5py_3h5o_10_OHdrSpace_1_hash, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5o__OHdrSpace[] = {
  {(char *)"total", __pyx_getprop_4h5py_3h5o_10_OHdrSpace_total, 0, 0, 0},
  {(char *)"meta", __pyx_getprop_4h5py_3h5o_10_OHdrSpace_meta, 0, 0, 0},
  {(char *)"mesg", __pyx_getprop_4h5py_3h5o_10_OHdrSpace_mesg, 0, 0, 0},
  {(char *)"free", __pyx_getprop_4h5py_3h5o_10_OHdrSpace_free, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__OHdrSpace = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__OHdrSpace = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__OHdrSpace = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__OHdrSpace = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o__OHdrSpace = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o._OHdrSpace"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o__OHdrSpace), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__OHdrSpace, /*tp_as_number*/
  &__pyx_tp_as_sequence__OHdrSpace, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__OHdrSpace, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__OHdrSpace, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o__OHdrSpace, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5o__OHdrSpace, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o__OHdrSpace, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5o__OHdr(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4h5py_3h5o__OHdr *p;
  PyObject *o = __pyx_tp_new_4h5py_3h5o__ObjInfoBase(t, a, k);
  if (!o) return 0;
  p = ((struct __pyx_obj_4h5py_3h5o__OHdr *)o);
  p->space = ((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)Py_None); Py_INCREF(Py_None);
  p->mesg = ((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5o__OHdr(PyObject *o) {
  struct __pyx_obj_4h5py_3h5o__OHdr *p = (struct __pyx_obj_4h5py_3h5o__OHdr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->space);
  Py_CLEAR(p->mesg);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase(o);
}

static int __pyx_tp_traverse_4h5py_3h5o__OHdr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4h5py_3h5o__OHdr *p = (struct __pyx_obj_4h5py_3h5o__OHdr *)o;
  e = ((likely(__pyx_ptype_4h5py_3h5o__ObjInfoBase)) ? ((__pyx_ptype_4h5py_3h5o__ObjInfoBase->tp_traverse) ? __pyx_ptype_4h5py_3h5o__ObjInfoBase->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4h5py_3h5o__OHdr)); if (e) return e;
  if (p->space) {
    e = (*v)(((PyObject*)p->space), a); if (e) return e;
  }
  if (p->mesg) {
    e = (*v)(((PyObject*)p->mesg), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5o__OHdr(PyObject *o) {
  struct __pyx_obj_4h5py_3h5o__OHdr *p = (struct __pyx_obj_4h5py_3h5o__OHdr *)o;
  PyObject* tmp;
  if (likely(__pyx_ptype_4h5py_3h5o__ObjInfoBase)) { if (__pyx_ptype_4h5py_3h5o__ObjInfoBase->tp_clear) __pyx_ptype_4h5py_3h5o__ObjInfoBase->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4h5py_3h5o__OHdr);
  tmp = ((PyObject*)p->space);
  p->space = ((struct __pyx_obj_4h5py_3h5o__OHdrSpace *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->mesg);
  p->mesg = ((struct __pyx_obj_4h5py_3h5o__OHdrMesg *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4h5py_3h5o_5_OHdr_version(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_5_OHdr_7version_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_5_OHdr_nmesgs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_5_OHdr_6nmesgs_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_5_OHdr_space(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_5_OHdr_5space_1__get__(o);
}

static int __pyx_setprop_4h5py_3h5o_5_OHdr_space(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4h5py_3h5o_5_OHdr_5space_3__set__(o, v);
  }
  else {
    return __pyx_pw_4h5py_3h5o_5_OHdr_5space_5__del__(o);
  }
}

static PyObject *__pyx_getprop_4h5py_3h5o_5_OHdr_mesg(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_1__get__(o);
}

static int __pyx_setprop_4h5py_3h5o_5_OHdr_mesg(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_3__set__(o, v);
  }
  else {
    return __pyx_pw_4h5py_3h5o_5_OHdr_4mesg_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_4h5py_3h5o__OHdr[] = {
  {__Pyx_NAMESTR("_hash"), (PyCFunction)__pyx_pw_4h5py_3h5o_5_OHdr_3_hash, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5o__OHdr[] = {
  {(char *)"version", __pyx_getprop_4h5py_3h5o_5_OHdr_version, 0, 0, 0},
  {(char *)"nmesgs", __pyx_getprop_4h5py_3h5o_5_OHdr_nmesgs, 0, 0, 0},
  {(char *)"space", __pyx_getprop_4h5py_3h5o_5_OHdr_space, __pyx_setprop_4h5py_3h5o_5_OHdr_space, 0, 0},
  {(char *)"mesg", __pyx_getprop_4h5py_3h5o_5_OHdr_mesg, __pyx_setprop_4h5py_3h5o_5_OHdr_mesg, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__OHdr = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__OHdr = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__OHdr = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__OHdr = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o__OHdr = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o._OHdr"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o__OHdr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o__OHdr, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__OHdr, /*tp_as_number*/
  &__pyx_tp_as_sequence__OHdr, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__OHdr, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__OHdr, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5o__OHdr, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5o__OHdr, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o__OHdr, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5o__OHdr, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_4h5py_3h5o_5_OHdr_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o__OHdr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5o__ObjInfo(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_tp_new_4h5py_3h5o__ObjInfoBase(t, a, k);
  if (!o) return 0;
  return o;
}

static PyObject *__pyx_getprop_4h5py_3h5o_8_ObjInfo_fileno(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_8_ObjInfo_6fileno_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_8_ObjInfo_addr(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_8_ObjInfo_4addr_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_8_ObjInfo_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_8_ObjInfo_4type_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5o_8_ObjInfo_rc(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_8_ObjInfo_2rc_1__get__(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5o__ObjInfo[] = {
  {__Pyx_NAMESTR("_hash"), (PyCFunction)__pyx_pw_4h5py_3h5o_8_ObjInfo_1_hash, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5o__ObjInfo[] = {
  {(char *)"fileno", __pyx_getprop_4h5py_3h5o_8_ObjInfo_fileno, 0, 0, 0},
  {(char *)"addr", __pyx_getprop_4h5py_3h5o_8_ObjInfo_addr, 0, 0, 0},
  {(char *)"type", __pyx_getprop_4h5py_3h5o_8_ObjInfo_type, 0, 0, 0},
  {(char *)"rc", __pyx_getprop_4h5py_3h5o_8_ObjInfo_rc, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__ObjInfo = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__ObjInfo = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__ObjInfo = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__ObjInfo = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o__ObjInfo = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o._ObjInfo"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o__ObjInfo), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__ObjInfo, /*tp_as_number*/
  &__pyx_tp_as_sequence__ObjInfo, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__ObjInfo, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__ObjInfo, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o__ObjInfo, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5o__ObjInfo, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o__ObjInfo, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5o_ObjInfo(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4h5py_3h5o_ObjInfo *p;
  PyObject *o = __pyx_tp_new_4h5py_3h5o__ObjInfoBase(t, a, k);
  if (!o) return 0;
  p = ((struct __pyx_obj_4h5py_3h5o_ObjInfo *)o);
  p->hdr = ((struct __pyx_obj_4h5py_3h5o__OHdr *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5o_ObjInfo(PyObject *o) {
  struct __pyx_obj_4h5py_3h5o_ObjInfo *p = (struct __pyx_obj_4h5py_3h5o_ObjInfo *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->hdr);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4h5py_3h5o__ObjInfoBase(o);
}

static int __pyx_tp_traverse_4h5py_3h5o_ObjInfo(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4h5py_3h5o_ObjInfo *p = (struct __pyx_obj_4h5py_3h5o_ObjInfo *)o;
  e = ((likely(__pyx_ptype_4h5py_3h5o__ObjInfo)) ? ((__pyx_ptype_4h5py_3h5o__ObjInfo->tp_traverse) ? __pyx_ptype_4h5py_3h5o__ObjInfo->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4h5py_3h5o_ObjInfo)); if (e) return e;
  if (p->hdr) {
    e = (*v)(((PyObject*)p->hdr), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5o_ObjInfo(PyObject *o) {
  struct __pyx_obj_4h5py_3h5o_ObjInfo *p = (struct __pyx_obj_4h5py_3h5o_ObjInfo *)o;
  PyObject* tmp;
  if (likely(__pyx_ptype_4h5py_3h5o__ObjInfo)) { if (__pyx_ptype_4h5py_3h5o__ObjInfo->tp_clear) __pyx_ptype_4h5py_3h5o__ObjInfo->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4h5py_3h5o_ObjInfo);
  tmp = ((PyObject*)p->hdr);
  p->hdr = ((struct __pyx_obj_4h5py_3h5o__OHdr *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4h5py_3h5o_7ObjInfo_hdr(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_1__get__(o);
}

static int __pyx_setprop_4h5py_3h5o_7ObjInfo_hdr(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_3__set__(o, v);
  }
  else {
    return __pyx_pw_4h5py_3h5o_7ObjInfo_3hdr_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_4h5py_3h5o_ObjInfo[] = {
  {__Pyx_NAMESTR("__copy__"), (PyCFunction)__pyx_pw_4h5py_3h5o_7ObjInfo_3__copy__, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5o_ObjInfo[] = {
  {(char *)"hdr", __pyx_getprop_4h5py_3h5o_7ObjInfo_hdr, __pyx_setprop_4h5py_3h5o_7ObjInfo_hdr, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_ObjInfo = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_ObjInfo = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_ObjInfo = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_ObjInfo = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o_ObjInfo = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o.ObjInfo"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o_ObjInfo), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o_ObjInfo, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_ObjInfo, /*tp_as_number*/
  &__pyx_tp_as_sequence_ObjInfo, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_ObjInfo, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_ObjInfo, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Represents the H5O_info_t structure\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5o_ObjInfo, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5o_ObjInfo, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o_ObjInfo, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5o_ObjInfo, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_4h5py_3h5o_7ObjInfo_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o_ObjInfo, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5o__ObjectVisitor(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *p;
  PyObject *o = (*t->tp_alloc)(t, 0);
  if (!o) return 0;
  p = ((struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)o);
  p->func = Py_None; Py_INCREF(Py_None);
  p->retval = Py_None; Py_INCREF(Py_None);
  p->objinfo = ((struct __pyx_obj_4h5py_3h5o_ObjInfo *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5o__ObjectVisitor(PyObject *o) {
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *p = (struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->func);
  Py_CLEAR(p->retval);
  Py_CLEAR(p->objinfo);
  PyObject_GC_Track(o);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4h5py_3h5o__ObjectVisitor(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *p = (struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)o;
  if (p->func) {
    e = (*v)(p->func, a); if (e) return e;
  }
  if (p->retval) {
    e = (*v)(p->retval, a); if (e) return e;
  }
  if (p->objinfo) {
    e = (*v)(((PyObject*)p->objinfo), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5o__ObjectVisitor(PyObject *o) {
  struct __pyx_obj_4h5py_3h5o__ObjectVisitor *p = (struct __pyx_obj_4h5py_3h5o__ObjectVisitor *)o;
  PyObject* tmp;
  tmp = ((PyObject*)p->func);
  p->func = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->retval);
  p->retval = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->objinfo);
  p->objinfo = ((struct __pyx_obj_4h5py_3h5o_ObjInfo *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4h5py_3h5o__ObjectVisitor[] = {
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__ObjectVisitor = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__ObjectVisitor = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__ObjectVisitor = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__ObjectVisitor = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5o__ObjectVisitor = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5o._ObjectVisitor"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5o__ObjectVisitor), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5o__ObjectVisitor, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__ObjectVisitor, /*tp_as_number*/
  &__pyx_tp_as_sequence__ObjectVisitor, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__ObjectVisitor, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__ObjectVisitor, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5o__ObjectVisitor, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5o__ObjectVisitor, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5o__ObjectVisitor, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_4h5py_3h5o_14_ObjectVisitor_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5o__ObjectVisitor, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    __Pyx_NAMESTR("h5o"),
    __Pyx_DOCSTR(__pyx_k_8), /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_10, __pyx_k_10, sizeof(__pyx_k_10), 0, 0, 1, 1},
  {&__pyx_n_s_11, __pyx_k_11, sizeof(__pyx_k_11), 0, 0, 1, 1},
  {&__pyx_n_s_12, __pyx_k_12, sizeof(__pyx_k_12), 0, 0, 1, 1},
  {&__pyx_n_s_13, __pyx_k_13, sizeof(__pyx_k_13), 0, 0, 1, 1},
  {&__pyx_n_s_14, __pyx_k_14, sizeof(__pyx_k_14), 0, 0, 1, 1},
  {&__pyx_kp_s_17, __pyx_k_17, sizeof(__pyx_k_17), 0, 0, 1, 0},
  {&__pyx_n_s_18, __pyx_k_18, sizeof(__pyx_k_18), 0, 0, 1, 1},
  {&__pyx_kp_s_4, __pyx_k_4, sizeof(__pyx_k_4), 0, 0, 1, 0},
  {&__pyx_n_s_9, __pyx_k_9, sizeof(__pyx_k_9), 0, 0, 1, 1},
  {&__pyx_n_s__TYPE_DATASET, __pyx_k__TYPE_DATASET, sizeof(__pyx_k__TYPE_DATASET), 0, 0, 1, 1},
  {&__pyx_n_s__TYPE_GROUP, __pyx_k__TYPE_GROUP, sizeof(__pyx_k__TYPE_GROUP), 0, 0, 1, 1},
  {&__pyx_n_s__TYPE_NAMED_DATATYPE, __pyx_k__TYPE_NAMED_DATATYPE, sizeof(__pyx_k__TYPE_NAMED_DATATYPE), 0, 0, 1, 1},
  {&__pyx_n_s__TypeError, __pyx_k__TypeError, sizeof(__pyx_k__TypeError), 0, 0, 1, 1},
  {&__pyx_n_s____main__, __pyx_k____main__, sizeof(__pyx_k____main__), 0, 0, 1, 1},
  {&__pyx_n_s____test__, __pyx_k____test__, sizeof(__pyx_k____test__), 0, 0, 1, 1},
  {&__pyx_n_s__addr, __pyx_k__addr, sizeof(__pyx_k__addr), 0, 0, 1, 1},
  {&__pyx_n_s__buf, __pyx_k__buf, sizeof(__pyx_k__buf), 0, 0, 1, 1},
  {&__pyx_n_s__cfunc, __pyx_k__cfunc, sizeof(__pyx_k__cfunc), 0, 0, 1, 1},
  {&__pyx_n_s__comment, __pyx_k__comment, sizeof(__pyx_k__comment), 0, 0, 1, 1},
  {&__pyx_n_s__copy, __pyx_k__copy, sizeof(__pyx_k__copy), 0, 0, 1, 1},
  {&__pyx_n_s__copypl, __pyx_k__copypl, sizeof(__pyx_k__copypl), 0, 0, 1, 1},
  {&__pyx_n_s__dst_loc, __pyx_k__dst_loc, sizeof(__pyx_k__dst_loc), 0, 0, 1, 1},
  {&__pyx_n_s__dst_name, __pyx_k__dst_name, sizeof(__pyx_k__dst_name), 0, 0, 1, 1},
  {&__pyx_n_s__fileno, __pyx_k__fileno, sizeof(__pyx_k__fileno), 0, 0, 1, 1},
  {&__pyx_n_s__free, __pyx_k__free, sizeof(__pyx_k__free), 0, 0, 1, 1},
  {&__pyx_n_s__func, __pyx_k__func, sizeof(__pyx_k__func), 0, 0, 1, 1},
  {&__pyx_n_s__get_comment, __pyx_k__get_comment, sizeof(__pyx_k__get_comment), 0, 0, 1, 1},
  {&__pyx_n_s__get_info, __pyx_k__get_info, sizeof(__pyx_k__get_info), 0, 0, 1, 1},
  {&__pyx_n_s__idx_type, __pyx_k__idx_type, sizeof(__pyx_k__idx_type), 0, 0, 1, 1},
  {&__pyx_n_s__index, __pyx_k__index, sizeof(__pyx_k__index), 0, 0, 1, 1},
  {&__pyx_n_s__index_type, __pyx_k__index_type, sizeof(__pyx_k__index_type), 0, 0, 1, 1},
  {&__pyx_n_s__info, __pyx_k__info, sizeof(__pyx_k__info), 0, 0, 1, 1},
  {&__pyx_n_s__lapl, __pyx_k__lapl, sizeof(__pyx_k__lapl), 0, 0, 1, 1},
  {&__pyx_n_s__lcpl, __pyx_k__lcpl, sizeof(__pyx_k__lcpl), 0, 0, 1, 1},
  {&__pyx_n_s__link, __pyx_k__link, sizeof(__pyx_k__link), 0, 0, 1, 1},
  {&__pyx_n_s__loc, __pyx_k__loc, sizeof(__pyx_k__loc), 0, 0, 1, 1},
  {&__pyx_n_s__mesg, __pyx_k__mesg, sizeof(__pyx_k__mesg), 0, 0, 1, 1},
  {&__pyx_n_s__meta, __pyx_k__meta, sizeof(__pyx_k__meta), 0, 0, 1, 1},
  {&__pyx_n_s__name, __pyx_k__name, sizeof(__pyx_k__name), 0, 0, 1, 1},
  {&__pyx_n_s__nmesgs, __pyx_k__nmesgs, sizeof(__pyx_k__nmesgs), 0, 0, 1, 1},
  {&__pyx_n_s__obj, __pyx_k__obj, sizeof(__pyx_k__obj), 0, 0, 1, 1},
  {&__pyx_n_s__obj_name, __pyx_k__obj_name, sizeof(__pyx_k__obj_name), 0, 0, 1, 1},
  {&__pyx_n_s__open, __pyx_k__open, sizeof(__pyx_k__open), 0, 0, 1, 1},
  {&__pyx_n_s__order, __pyx_k__order, sizeof(__pyx_k__order), 0, 0, 1, 1},
  {&__pyx_n_s__present, __pyx_k__present, sizeof(__pyx_k__present), 0, 0, 1, 1},
  {&__pyx_n_s__pstring, __pyx_k__pstring, sizeof(__pyx_k__pstring), 0, 0, 1, 1},
  {&__pyx_n_s__rc, __pyx_k__rc, sizeof(__pyx_k__rc), 0, 0, 1, 1},
  {&__pyx_n_s__set_comment, __pyx_k__set_comment, sizeof(__pyx_k__set_comment), 0, 0, 1, 1},
  {&__pyx_n_s__shared, __pyx_k__shared, sizeof(__pyx_k__shared), 0, 0, 1, 1},
  {&__pyx_n_s__size, __pyx_k__size, sizeof(__pyx_k__size), 0, 0, 1, 1},
  {&__pyx_n_s__src_loc, __pyx_k__src_loc, sizeof(__pyx_k__src_loc), 0, 0, 1, 1},
  {&__pyx_n_s__src_name, __pyx_k__src_name, sizeof(__pyx_k__src_name), 0, 0, 1, 1},
  {&__pyx_n_s__total, __pyx_k__total, sizeof(__pyx_k__total), 0, 0, 1, 1},
  {&__pyx_n_s__type, __pyx_k__type, sizeof(__pyx_k__type), 0, 0, 1, 1},
  {&__pyx_n_s__version, __pyx_k__version, sizeof(__pyx_k__version), 0, 0, 1, 1},
  {&__pyx_n_s__visit, __pyx_k__visit, sizeof(__pyx_k__visit), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetName(__pyx_b, __pyx_n_s__TypeError); if (!__pyx_builtin_TypeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "h5py/h5o.pyx":154
 * 
 *     if name != NULL and index >= 0:
 *         raise TypeError("At most one of name or index may be specified")             # <<<<<<<<<<<<<<
 *     elif name != NULL and index < 0:
 *         H5Oget_info_by_name(loc.id, name, &info.infostruct, pdefault(lapl))
 */
  __pyx_k_tuple_5 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_4)); if (unlikely(!__pyx_k_tuple_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_5);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_5));

  /* "h5py/h5o.pyx":131
 * 
 * 
 * def get_info(ObjectID loc not None, char* name=NULL, int index=-1, *,             # <<<<<<<<<<<<<<
 *         char* obj_name='.', int index_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *         PropID lapl=None):
 */
  __pyx_k_tuple_15 = PyTuple_Pack(8, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__name), ((PyObject *)__pyx_n_s__index), ((PyObject *)__pyx_n_s__obj_name), ((PyObject *)__pyx_n_s__index_type), ((PyObject *)__pyx_n_s__order), ((PyObject *)__pyx_n_s__lapl), ((PyObject *)__pyx_n_s__info)); if (unlikely(!__pyx_k_tuple_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_15);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_15));
  __pyx_k_codeobj_16 = (PyObject*)__Pyx_PyCode_New(7, 4, 8, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__get_info, 131, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":168
 * 
 * 
 * def open(ObjectID loc not None, char* name, PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name, PropID lapl=None) => ObjectID
 * 
 */
  __pyx_k_tuple_19 = PyTuple_Pack(3, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__name), ((PyObject *)__pyx_n_s__lapl)); if (unlikely(!__pyx_k_tuple_19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_19);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_19));
  __pyx_k_codeobj_20 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__open, 168, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_20)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":176
 * 
 * 
 * def link(ObjectID obj not None, GroupID loc not None, char* name,             # <<<<<<<<<<<<<<
 *     PropID lcpl=None, PropID lapl=None):
 *     """(ObjectID obj, GroupID loc, STRING name, PropID lcpl=None,
 */
  __pyx_k_tuple_21 = PyTuple_Pack(5, ((PyObject *)__pyx_n_s__obj), ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__name), ((PyObject *)__pyx_n_s__lcpl), ((PyObject *)__pyx_n_s__lapl)); if (unlikely(!__pyx_k_tuple_21)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_21);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_21));
  __pyx_k_codeobj_22 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__link, 176, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":187
 * 
 * 
 * def copy(ObjectID src_loc not None, char* src_name, GroupID dst_loc not None,             # <<<<<<<<<<<<<<
 *     char* dst_name, PropID copypl=None, PropID lcpl=None):
 *     """(ObjectID src_loc, STRING src_name, GroupID dst_loc, STRING dst_name,
 */
  __pyx_k_tuple_23 = PyTuple_Pack(6, ((PyObject *)__pyx_n_s__src_loc), ((PyObject *)__pyx_n_s__src_name), ((PyObject *)__pyx_n_s__dst_loc), ((PyObject *)__pyx_n_s__dst_name), ((PyObject *)__pyx_n_s__copypl), ((PyObject *)__pyx_n_s__lcpl)); if (unlikely(!__pyx_k_tuple_23)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_23);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_23));
  __pyx_k_codeobj_24 = (PyObject*)__Pyx_PyCode_New(6, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__copy, 187, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":202
 * 
 * 
 * def set_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",             # <<<<<<<<<<<<<<
 *     PropID lapl=None):
 *     """(ObjectID loc, STRING comment, **kwds)
 */
  __pyx_k_tuple_25 = PyTuple_Pack(4, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__comment), ((PyObject *)__pyx_n_s__obj_name), ((PyObject *)__pyx_n_s__lapl)); if (unlikely(!__pyx_k_tuple_25)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_25);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_25));
  __pyx_k_codeobj_26 = (PyObject*)__Pyx_PyCode_New(4, 2, 4, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__set_comment, 202, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_26)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":218
 * 
 * 
 * def get_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",             # <<<<<<<<<<<<<<
 *     PropID lapl=None):
 *     """(ObjectID loc, STRING comment, **kwds)
 */
  __pyx_k_tuple_27 = PyTuple_Pack(7, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__comment), ((PyObject *)__pyx_n_s__obj_name), ((PyObject *)__pyx_n_s__lapl), ((PyObject *)__pyx_n_s__size), ((PyObject *)__pyx_n_s__buf), ((PyObject *)__pyx_n_s__pstring)); if (unlikely(!__pyx_k_tuple_27)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_27);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_27));
  __pyx_k_codeobj_28 = (PyObject*)__Pyx_PyCode_New(4, 2, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__get_comment, 218, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5o.pyx":288
 * 
 * 
 * def visit(ObjectID loc not None, object func, *,             # <<<<<<<<<<<<<<
 *           int idx_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *           char* obj_name=".", PropID lapl=None, bint info=0):
 */
  __pyx_k_tuple_29 = PyTuple_Pack(9, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__func), ((PyObject *)__pyx_n_s__idx_type), ((PyObject *)__pyx_n_s__order), ((PyObject *)__pyx_n_s__obj_name), ((PyObject *)__pyx_n_s__lapl), ((PyObject *)__pyx_n_s__info), ((PyObject *)__pyx_n_s__visit), ((PyObject *)__pyx_n_s__cfunc)); if (unlikely(!__pyx_k_tuple_29)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_29);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_29));
  __pyx_k_codeobj_30 = (PyObject*)__Pyx_PyCode_New(7, 5, 9, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_17, __pyx_n_s__visit, 288, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC inith5o(void); /*proto*/
PyMODINIT_FUNC inith5o(void)
#else
PyMODINIT_FUNC PyInit_h5o(void); /*proto*/
PyMODINIT_FUNC PyInit_h5o(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_h5o(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4(__Pyx_NAMESTR("h5o"), __pyx_methods, __Pyx_DOCSTR(__pyx_k_8), 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "h5py.h5o")) {
      if (unlikely(PyDict_SetItemString(modules, "h5py.h5o", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  __pyx_b = PyImport_AddModule(__Pyx_NAMESTR(__Pyx_BUILTIN_MODULE_NAME)); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_module_is_main_h5py__h5o) {
    if (__Pyx_SetAttrString(__pyx_m, "__name__", __pyx_n_s____main__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_4h5py_3h5o__ObjInfoBase) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_ObjInfoBase", (PyObject *)&__pyx_type_4h5py_3h5o__ObjInfoBase) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o__ObjInfoBase = &__pyx_type_4h5py_3h5o__ObjInfoBase;
  __pyx_type_4h5py_3h5o__OHdrMesg.tp_base = __pyx_ptype_4h5py_3h5o__ObjInfoBase;
  if (PyType_Ready(&__pyx_type_4h5py_3h5o__OHdrMesg) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_OHdrMesg", (PyObject *)&__pyx_type_4h5py_3h5o__OHdrMesg) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o__OHdrMesg = &__pyx_type_4h5py_3h5o__OHdrMesg;
  __pyx_type_4h5py_3h5o__OHdrSpace.tp_base = __pyx_ptype_4h5py_3h5o__ObjInfoBase;
  if (PyType_Ready(&__pyx_type_4h5py_3h5o__OHdrSpace) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_OHdrSpace", (PyObject *)&__pyx_type_4h5py_3h5o__OHdrSpace) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o__OHdrSpace = &__pyx_type_4h5py_3h5o__OHdrSpace;
  __pyx_type_4h5py_3h5o__OHdr.tp_base = __pyx_ptype_4h5py_3h5o__ObjInfoBase;
  if (PyType_Ready(&__pyx_type_4h5py_3h5o__OHdr) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_OHdr", (PyObject *)&__pyx_type_4h5py_3h5o__OHdr) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o__OHdr = &__pyx_type_4h5py_3h5o__OHdr;
  __pyx_type_4h5py_3h5o__ObjInfo.tp_base = __pyx_ptype_4h5py_3h5o__ObjInfoBase;
  if (PyType_Ready(&__pyx_type_4h5py_3h5o__ObjInfo) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_ObjInfo", (PyObject *)&__pyx_type_4h5py_3h5o__ObjInfo) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o__ObjInfo = &__pyx_type_4h5py_3h5o__ObjInfo;
  __pyx_type_4h5py_3h5o_ObjInfo.tp_base = __pyx_ptype_4h5py_3h5o__ObjInfo;
  if (PyType_Ready(&__pyx_type_4h5py_3h5o_ObjInfo) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "ObjInfo", (PyObject *)&__pyx_type_4h5py_3h5o_ObjInfo) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o_ObjInfo = &__pyx_type_4h5py_3h5o_ObjInfo;
  if (PyType_Ready(&__pyx_type_4h5py_3h5o__ObjectVisitor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_ObjectVisitor", (PyObject *)&__pyx_type_4h5py_3h5o__ObjectVisitor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5o__ObjectVisitor = &__pyx_type_4h5py_3h5o__ObjectVisitor;
  /*--- Type import code ---*/
  __pyx_ptype_4h5py_8_objects_ObjectID = __Pyx_ImportType("h5py._objects", "ObjectID", sizeof(struct __pyx_obj_4h5py_8_objects_ObjectID), 1); if (unlikely(!__pyx_ptype_4h5py_8_objects_ObjectID)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5g_GroupID = __Pyx_ImportType("h5py.h5g", "GroupID", sizeof(struct __pyx_obj_4h5py_3h5g_GroupID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5g_GroupID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropID = __Pyx_ImportType("h5py.h5p", "PropID", sizeof(struct __pyx_obj_4h5py_3h5p_PropID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropClassID = __Pyx_ImportType("h5py.h5p", "PropClassID", sizeof(struct __pyx_obj_4h5py_3h5p_PropClassID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropClassID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropInstanceID = __Pyx_ImportType("h5py.h5p", "PropInstanceID", sizeof(struct __pyx_obj_4h5py_3h5p_PropInstanceID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropInstanceID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropCreateID = __Pyx_ImportType("h5py.h5p", "PropCreateID", sizeof(struct __pyx_obj_4h5py_3h5p_PropCreateID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropCreateID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropCopyID = __Pyx_ImportType("h5py.h5p", "PropCopyID", sizeof(struct __pyx_obj_4h5py_3h5p_PropCopyID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropCopyID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropOCID = __Pyx_ImportType("h5py.h5p", "PropOCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropOCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropOCID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropDCID = __Pyx_ImportType("h5py.h5p", "PropDCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropDCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropDCID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropFCID = __Pyx_ImportType("h5py.h5p", "PropFCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropFCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropFCID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropFAID = __Pyx_ImportType("h5py.h5p", "PropFAID", sizeof(struct __pyx_obj_4h5py_3h5p_PropFAID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropFAID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropDXID = __Pyx_ImportType("h5py.h5p", "PropDXID", sizeof(struct __pyx_obj_4h5py_3h5p_PropDXID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropDXID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropLCID = __Pyx_ImportType("h5py.h5p", "PropLCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropLCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropLCID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropLAID = __Pyx_ImportType("h5py.h5p", "PropLAID", sizeof(struct __pyx_obj_4h5py_3h5p_PropLAID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropLAID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 74; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropGCID = __Pyx_ImportType("h5py.h5p", "PropGCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropGCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropGCID)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_dtype)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_ndarray)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("h5py.defs"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Oopen", (void (**)(void))&__pyx_f_4h5py_4defs_H5Oopen, "hid_t (hid_t, char *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Oget_info", (void (**)(void))&__pyx_f_4h5py_4defs_H5Oget_info, "herr_t (hid_t, H5O_info_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Oget_info_by_name", (void (**)(void))&__pyx_f_4h5py_4defs_H5Oget_info_by_name, "herr_t (hid_t, char *, H5O_info_t *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Oget_info_by_idx", (void (**)(void))&__pyx_f_4h5py_4defs_H5Oget_info_by_idx, "herr_t (hid_t, char *, H5_index_t, H5_iter_order_t, hsize_t, H5O_info_t *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Olink", (void (**)(void))&__pyx_f_4h5py_4defs_H5Olink, "herr_t (hid_t, hid_t, char *, hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Ocopy", (void (**)(void))&__pyx_f_4h5py_4defs_H5Ocopy, "herr_t (hid_t, char *, hid_t, char *, hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Oset_comment_by_name", (void (**)(void))&__pyx_f_4h5py_4defs_H5Oset_comment_by_name, "herr_t (hid_t, char *, char *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Oget_comment_by_name", (void (**)(void))&__pyx_f_4h5py_4defs_H5Oget_comment_by_name, "Py_ssize_t (hid_t, char *, char *, size_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Ovisit_by_name", (void (**)(void))&__pyx_f_4h5py_4defs_H5Ovisit_by_name, "herr_t (hid_t, char *, H5_index_t, H5_iter_order_t, H5O_iterate_t, void *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_ImportModule("h5py._objects"); if (!__pyx_t_2) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "pdefault", (void (**)(void))&__pyx_f_4h5py_8_objects_pdefault, "hid_t (struct __pyx_obj_4h5py_8_objects_ObjectID *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_ImportModule("h5py.h5i"); if (!__pyx_t_3) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_3, "wrap_identifier", (void (**)(void))&__pyx_f_4h5py_3h5i_wrap_identifier, "struct __pyx_obj_4h5py_8_objects_ObjectID *(hid_t, int __pyx_skip_dispatch)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_ImportModule("h5py.utils"); if (!__pyx_t_4) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_4, "emalloc", (void (**)(void))&__pyx_f_4h5py_5utils_emalloc, "void *(size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_4, "efree", (void (**)(void))&__pyx_f_4h5py_5utils_efree, "void (void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  /*--- Execution code ---*/

  /* "h5py/h5o.pyx":23
 * # === Public constants ========================================================
 * 
 * TYPE_GROUP = H5O_TYPE_GROUP             # <<<<<<<<<<<<<<
 * TYPE_DATASET = H5O_TYPE_DATASET
 * TYPE_NAMED_DATATYPE = H5O_TYPE_NAMED_DATATYPE
 */
  __pyx_t_5 = PyInt_FromLong(H5O_TYPE_GROUP); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__TYPE_GROUP, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":24
 * 
 * TYPE_GROUP = H5O_TYPE_GROUP
 * TYPE_DATASET = H5O_TYPE_DATASET             # <<<<<<<<<<<<<<
 * TYPE_NAMED_DATATYPE = H5O_TYPE_NAMED_DATATYPE
 * 
 */
  __pyx_t_5 = PyInt_FromLong(H5O_TYPE_DATASET); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__TYPE_DATASET, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":25
 * TYPE_GROUP = H5O_TYPE_GROUP
 * TYPE_DATASET = H5O_TYPE_DATASET
 * TYPE_NAMED_DATATYPE = H5O_TYPE_NAMED_DATATYPE             # <<<<<<<<<<<<<<
 * 
 * COPY_SHALLOW_HIERARCHY_FLAG = H5O_COPY_SHALLOW_HIERARCHY_FLAG
 */
  __pyx_t_5 = PyInt_FromLong(H5O_TYPE_NAMED_DATATYPE); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__TYPE_NAMED_DATATYPE, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":27
 * TYPE_NAMED_DATATYPE = H5O_TYPE_NAMED_DATATYPE
 * 
 * COPY_SHALLOW_HIERARCHY_FLAG = H5O_COPY_SHALLOW_HIERARCHY_FLAG             # <<<<<<<<<<<<<<
 * COPY_EXPAND_SOFT_LINK_FLAG  = H5O_COPY_EXPAND_SOFT_LINK_FLAG
 * COPY_EXPAND_EXT_LINK_FLAG   = H5O_COPY_EXPAND_EXT_LINK_FLAG
 */
  __pyx_t_5 = PyLong_FromUnsignedLong(H5O_COPY_SHALLOW_HIERARCHY_FLAG); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_9, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":28
 * 
 * COPY_SHALLOW_HIERARCHY_FLAG = H5O_COPY_SHALLOW_HIERARCHY_FLAG
 * COPY_EXPAND_SOFT_LINK_FLAG  = H5O_COPY_EXPAND_SOFT_LINK_FLAG             # <<<<<<<<<<<<<<
 * COPY_EXPAND_EXT_LINK_FLAG   = H5O_COPY_EXPAND_EXT_LINK_FLAG
 * COPY_EXPAND_REFERENCE_FLAG  = H5O_COPY_EXPAND_REFERENCE_FLAG
 */
  __pyx_t_5 = PyLong_FromUnsignedLong(H5O_COPY_EXPAND_SOFT_LINK_FLAG); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_10, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":29
 * COPY_SHALLOW_HIERARCHY_FLAG = H5O_COPY_SHALLOW_HIERARCHY_FLAG
 * COPY_EXPAND_SOFT_LINK_FLAG  = H5O_COPY_EXPAND_SOFT_LINK_FLAG
 * COPY_EXPAND_EXT_LINK_FLAG   = H5O_COPY_EXPAND_EXT_LINK_FLAG             # <<<<<<<<<<<<<<
 * COPY_EXPAND_REFERENCE_FLAG  = H5O_COPY_EXPAND_REFERENCE_FLAG
 * COPY_WITHOUT_ATTR_FLAG      = H5O_COPY_WITHOUT_ATTR_FLAG
 */
  __pyx_t_5 = PyLong_FromUnsignedLong(H5O_COPY_EXPAND_EXT_LINK_FLAG); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_11, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":30
 * COPY_EXPAND_SOFT_LINK_FLAG  = H5O_COPY_EXPAND_SOFT_LINK_FLAG
 * COPY_EXPAND_EXT_LINK_FLAG   = H5O_COPY_EXPAND_EXT_LINK_FLAG
 * COPY_EXPAND_REFERENCE_FLAG  = H5O_COPY_EXPAND_REFERENCE_FLAG             # <<<<<<<<<<<<<<
 * COPY_WITHOUT_ATTR_FLAG      = H5O_COPY_WITHOUT_ATTR_FLAG
 * COPY_PRESERVE_NULL_FLAG     = H5O_COPY_PRESERVE_NULL_FLAG
 */
  __pyx_t_5 = PyLong_FromUnsignedLong(H5O_COPY_EXPAND_REFERENCE_FLAG); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_12, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":31
 * COPY_EXPAND_EXT_LINK_FLAG   = H5O_COPY_EXPAND_EXT_LINK_FLAG
 * COPY_EXPAND_REFERENCE_FLAG  = H5O_COPY_EXPAND_REFERENCE_FLAG
 * COPY_WITHOUT_ATTR_FLAG      = H5O_COPY_WITHOUT_ATTR_FLAG             # <<<<<<<<<<<<<<
 * COPY_PRESERVE_NULL_FLAG     = H5O_COPY_PRESERVE_NULL_FLAG
 * 
 */
  __pyx_t_5 = PyLong_FromUnsignedLong(H5O_COPY_WITHOUT_ATTR_FLAG); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_13, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":32
 * COPY_EXPAND_REFERENCE_FLAG  = H5O_COPY_EXPAND_REFERENCE_FLAG
 * COPY_WITHOUT_ATTR_FLAG      = H5O_COPY_WITHOUT_ATTR_FLAG
 * COPY_PRESERVE_NULL_FLAG     = H5O_COPY_PRESERVE_NULL_FLAG             # <<<<<<<<<<<<<<
 * 
 * # === Giant H5O_info_t structure ==============================================
 */
  __pyx_t_5 = PyLong_FromUnsignedLong(H5O_COPY_PRESERVE_NULL_FLAG); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_14, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":132
 * 
 * def get_info(ObjectID loc not None, char* name=NULL, int index=-1, *,
 *         char* obj_name='.', int index_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,             # <<<<<<<<<<<<<<
 *         PropID lapl=None):
 *     """(ObjectID loc, STRING name=, INT index=, **kwds) => ObjInfo
 */
  __pyx_k_2 = H5_INDEX_NAME;
  __pyx_k_3 = H5_ITER_NATIVE;

  /* "h5py/h5o.pyx":131
 * 
 * 
 * def get_info(ObjectID loc not None, char* name=NULL, int index=-1, *,             # <<<<<<<<<<<<<<
 *         char* obj_name='.', int index_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *         PropID lapl=None):
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_1get_info, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__get_info, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":168
 * 
 * 
 * def open(ObjectID loc not None, char* name, PropID lapl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name, PropID lapl=None) => ObjectID
 * 
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_3open, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__open, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":176
 * 
 * 
 * def link(ObjectID obj not None, GroupID loc not None, char* name,             # <<<<<<<<<<<<<<
 *     PropID lcpl=None, PropID lapl=None):
 *     """(ObjectID obj, GroupID loc, STRING name, PropID lcpl=None,
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_5link, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__link, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":187
 * 
 * 
 * def copy(ObjectID src_loc not None, char* src_name, GroupID dst_loc not None,             # <<<<<<<<<<<<<<
 *     char* dst_name, PropID copypl=None, PropID lcpl=None):
 *     """(ObjectID src_loc, STRING src_name, GroupID dst_loc, STRING dst_name,
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_7copy, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__copy, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":202
 * 
 * 
 * def set_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",             # <<<<<<<<<<<<<<
 *     PropID lapl=None):
 *     """(ObjectID loc, STRING comment, **kwds)
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_9set_comment, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__set_comment, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":218
 * 
 * 
 * def get_comment(ObjectID loc not None, char* comment, *, char* obj_name=".",             # <<<<<<<<<<<<<<
 *     PropID lapl=None):
 *     """(ObjectID loc, STRING comment, **kwds)
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_11get_comment, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__get_comment, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":289
 * 
 * def visit(ObjectID loc not None, object func, *,
 *           int idx_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,             # <<<<<<<<<<<<<<
 *           char* obj_name=".", PropID lapl=None, bint info=0):
 *     """(ObjectID loc, CALLABLE func, **kwds) => <Return value from func>
 */
  __pyx_k_6 = H5_INDEX_NAME;
  __pyx_k_7 = H5_ITER_NATIVE;

  /* "h5py/h5o.pyx":288
 * 
 * 
 * def visit(ObjectID loc not None, object func, *,             # <<<<<<<<<<<<<<
 *           int idx_type=H5_INDEX_NAME, int order=H5_ITER_NATIVE,
 *           char* obj_name=".", PropID lapl=None, bint info=0):
 */
  __pyx_t_5 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5o_13visit, NULL, __pyx_n_s_18); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__visit, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "h5py/h5o.pyx":1
 * # This file is part of h5py, a Python interface to the HDF5 library.             # <<<<<<<<<<<<<<
 * #
 * # http://www.h5py.org
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_5));
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s____test__, ((PyObject *)__pyx_t_5)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    __Pyx_AddTraceback("init h5py.h5o", __pyx_clineno, __pyx_lineno, __pyx_filename);
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init h5py.h5o");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif /* CYTHON_REFNANNY */

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name) {
    PyObject *result;
    result = PyObject_GetAttr(dict, name);
    if (!result) {
        if (dict != __pyx_b) {
            PyErr_Clear();
            result = PyObject_GetAttr(__pyx_b, name);
        }
        if (!result) {
            PyErr_SetObject(PyExc_NameError, name);
        }
    }
    return result;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%s() takes %s %" CYTHON_FORMAT_SSIZE_T "d positional argument%s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CPYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%s() got an unexpected keyword argument '%s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%s() got an unexpected keyword argument '%s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (!type) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (Py_TYPE(obj) == type) return 1;
    }
    else {
        if (PyObject_TypeCheck(obj, type)) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%s' has incorrect type (expected %s, got %s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    #if PY_VERSION_HEX < 0x02050000
    if (PyClass_Check(type)) {
    #else
    if (PyType_Check(type)) {
    #endif
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        #if PY_VERSION_HEX < 0x02050000
            if (PyInstance_Check(type)) {
                type = (PyObject*) ((PyInstanceObject*)type)->in_class;
                Py_INCREF(type);
            }
            else {
                type = 0;
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception must be an old-style class or instance");
                goto raise_error;
            }
        #else
            type = (PyObject*) Py_TYPE(type);
            Py_INCREF(type);
            if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception class must be a subclass of BaseException");
                goto raise_error;
            }
        #endif
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else /* Python 3+ */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *args;
        if (!value)
            args = PyTuple_New(0);
        else if (PyTuple_Check(value)) {
            Py_INCREF(value);
            args = value;
        }
        else
            args = PyTuple_Pack(1, value);
        if (!args)
            goto bad;
        owned_instance = PyEval_CallObject(type, args);
        Py_DECREF(args);
        if (!owned_instance)
            goto bad;
        value = owned_instance;
        if (!PyExceptionInstance_Check(value)) {
            PyErr_Format(PyExc_TypeError,
                         "calling %R should have returned an instance of "
                         "BaseException, not %R",
                         type, Py_TYPE(value));
            goto bad;
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause && cause != Py_None) {
        PyObject *fixed_cause;
        if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        }
        else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        }
        else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t val) {
    const hsize_t neg_one = (hsize_t)-1, const_zero = (hsize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hsize_t) == sizeof(char))  ||
        (sizeof(hsize_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hsize_t) == sizeof(int)) ||
               (sizeof(hsize_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hsize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hsize_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_haddr_t(haddr_t val) {
    const haddr_t neg_one = (haddr_t)-1, const_zero = (haddr_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(haddr_t) == sizeof(char))  ||
        (sizeof(haddr_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(haddr_t) == sizeof(int)) ||
               (sizeof(haddr_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(haddr_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(haddr_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t val) {
    const hid_t neg_one = (hid_t)-1, const_zero = (hid_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hid_t) == sizeof(char))  ||
        (sizeof(hid_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hid_t) == sizeof(int)) ||
               (sizeof(hid_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hid_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject* x) {
    const unsigned char neg_one = (unsigned char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned char" :
                    "value too large to convert to unsigned char");
            }
            return (unsigned char)-1;
        }
        return (unsigned char)val;
    }
    return (unsigned char)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject* x) {
    const unsigned short neg_one = (unsigned short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned short" :
                    "value too large to convert to unsigned short");
            }
            return (unsigned short)-1;
        }
        return (unsigned short)val;
    }
    return (unsigned short)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject* x) {
    const unsigned int neg_one = (unsigned int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned int" :
                    "value too large to convert to unsigned int");
            }
            return (unsigned int)-1;
        }
        return (unsigned int)val;
    }
    return (unsigned int)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject* x) {
    const char neg_one = (char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to char" :
                    "value too large to convert to char");
            }
            return (char)-1;
        }
        return (char)val;
    }
    return (char)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject* x) {
    const short neg_one = (short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to short" :
                    "value too large to convert to short");
            }
            return (short)-1;
        }
        return (short)val;
    }
    return (short)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject* x) {
    const signed char neg_one = (signed char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed char" :
                    "value too large to convert to signed char");
            }
            return (signed char)-1;
        }
        return (signed char)val;
    }
    return (signed char)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject* x) {
    const signed short neg_one = (signed short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed short" :
                    "value too large to convert to signed short");
            }
            return (signed short)-1;
        }
        return (signed short)val;
    }
    return (signed short)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject* x) {
    const signed int neg_one = (signed int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed int" :
                    "value too large to convert to signed int");
            }
            return (signed int)-1;
        }
        return (signed int)val;
    }
    return (signed int)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject* x) {
    const unsigned long neg_one = (unsigned long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned long");
            return (unsigned long)-1;
        }
        return (unsigned long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned long");
                return (unsigned long)-1;
            }
            return (unsigned long)PyLong_AsUnsignedLong(x);
        } else {
            return (unsigned long)PyLong_AsLong(x);
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned long)-1;
        val = __Pyx_PyInt_AsUnsignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject* x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned PY_LONG_LONG");
            return (unsigned PY_LONG_LONG)-1;
        }
        return (unsigned PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned PY_LONG_LONG");
                return (unsigned PY_LONG_LONG)-1;
            }
            return (unsigned PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (unsigned PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsUnsignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject* x) {
    const long neg_one = (long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to long");
            return (long)-1;
        }
        return (long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to long");
                return (long)-1;
            }
            return (long)PyLong_AsUnsignedLong(x);
        } else {
            return (long)PyLong_AsLong(x);
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long)-1;
        val = __Pyx_PyInt_AsLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject* x) {
    const PY_LONG_LONG neg_one = (PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to PY_LONG_LONG");
            return (PY_LONG_LONG)-1;
        }
        return (PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to PY_LONG_LONG");
                return (PY_LONG_LONG)-1;
            }
            return (PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject* x) {
    const signed long neg_one = (signed long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed long");
            return (signed long)-1;
        }
        return (signed long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed long");
                return (signed long)-1;
            }
            return (signed long)PyLong_AsUnsignedLong(x);
        } else {
            return (signed long)PyLong_AsLong(x);
        }
    } else {
        signed long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed long)-1;
        val = __Pyx_PyInt_AsSignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject* x) {
    const signed PY_LONG_LONG neg_one = (signed PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed PY_LONG_LONG");
            return (signed PY_LONG_LONG)-1;
        }
        return (signed PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed PY_LONG_LONG");
                return (signed PY_LONG_LONG)-1;
            }
            return (signed PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (signed PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        signed PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsSignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        #if PY_VERSION_HEX < 0x02050000
        return PyErr_Warn(NULL, message);
        #else
        return PyErr_WarnEx(NULL, message, 1);
        #endif
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%s.%s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        #if PY_VERSION_HEX < 0x02050000
        if (PyErr_Warn(NULL, warning) < 0) goto bad;
        #else
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
        #endif
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%s.%s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%s does not export expected C function %s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3 && PY_MINOR_VERSION==0)
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,            /*int argcount,*/
        0,            /*int kwonlyargcount,*/
        0,            /*int nlocals,*/
        0,            /*int stacksize,*/
        0,            /*int flags,*/
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,      /*int firstlineno,*/
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_globals = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_globals = PyModule_GetDict(__pyx_m);
    if (!py_globals) goto bad;
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        py_globals,          /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else  /* Python 3+ has unicode identifiers */
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_VERSION_HEX < 0x03000000
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_VERSION_HEX < 0x03000000
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%s__ returned non-%s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject* x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
#if PY_VERSION_HEX < 0x02050000
   if (ival <= LONG_MAX)
       return PyInt_FromLong((long)ival);
   else {
       unsigned char *bytes = (unsigned char *) &ival;
       int one = 1; int little = (int)*(unsigned char*)&one;
       return _PyLong_FromByteArray(bytes, sizeof(size_t), little, 0);
   }
#else
   return PyInt_FromSize_t(ival);
#endif
}
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject* x) {
   unsigned PY_LONG_LONG val = __Pyx_PyInt_AsUnsignedLongLong(x);
   if (unlikely(val == (unsigned PY_LONG_LONG)-1 && PyErr_Occurred())) {
       return (size_t)-1;
   } else if (unlikely(val != (unsigned PY_LONG_LONG)(size_t)val)) {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to size_t");
       return (size_t)-1;
   }
   return (size_t)val;
}


#endif /* Py_PYTHON_H */
