/* Generated by Cython 0.18 on Mon Dec  9 16:00:33 2013 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02040000
    #error Cython requires Python 2.4+.
#else
#include <stddef.h> /* For offsetof */
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if PY_VERSION_HEX < 0x02050000
  typedef int Py_ssize_t;
  #define PY_SSIZE_T_MAX INT_MAX
  #define PY_SSIZE_T_MIN INT_MIN
  #define PY_FORMAT_SIZE_T ""
  #define CYTHON_FORMAT_SSIZE_T ""
  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
  #define PyInt_AsSsize_t(o)   __Pyx_PyInt_AsInt(o)
  #define PyNumber_Index(o)    ((PyNumber_Check(o) && !PyFloat_Check(o)) ? PyNumber_Int(o) : \
                                (PyErr_Format(PyExc_TypeError, \
                                              "expected index value, got %.200s", Py_TYPE(o)->tp_name), \
                                 (PyObject*)0))
  #define __Pyx_PyIndex_Check(o) (PyNumber_Check(o) && !PyFloat_Check(o) && \
                                  !PyComplex_Check(o))
  #define PyIndex_Check __Pyx_PyIndex_Check
  #define PyErr_WarnEx(category, message, stacklevel) PyErr_Warn(category, message)
  #define __PYX_BUILD_PY_SSIZE_T "i"
#else
  #define __PYX_BUILD_PY_SSIZE_T "n"
  #define CYTHON_FORMAT_SSIZE_T "z"
  #define __Pyx_PyIndex_Check PyIndex_Check
#endif
#if PY_VERSION_HEX < 0x02060000
  #define Py_REFCNT(ob) (((PyObject*)(ob))->ob_refcnt)
  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
  #define Py_SIZE(ob)   (((PyVarObject*)(ob))->ob_size)
  #define PyVarObject_HEAD_INIT(type, size) \
          PyObject_HEAD_INIT(type) size,
  #define PyType_Modified(t)
  typedef struct {
     void *buf;
     PyObject *obj;
     Py_ssize_t len;
     Py_ssize_t itemsize;
     int readonly;
     int ndim;
     char *format;
     Py_ssize_t *shape;
     Py_ssize_t *strides;
     Py_ssize_t *suboffsets;
     void *internal;
  } Py_buffer;
  #define PyBUF_SIMPLE 0
  #define PyBUF_WRITABLE 0x0001
  #define PyBUF_FORMAT 0x0004
  #define PyBUF_ND 0x0008
  #define PyBUF_STRIDES (0x0010 | PyBUF_ND)
  #define PyBUF_C_CONTIGUOUS (0x0020 | PyBUF_STRIDES)
  #define PyBUF_F_CONTIGUOUS (0x0040 | PyBUF_STRIDES)
  #define PyBUF_ANY_CONTIGUOUS (0x0080 | PyBUF_STRIDES)
  #define PyBUF_INDIRECT (0x0100 | PyBUF_STRIDES)
  #define PyBUF_RECORDS (PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_WRITABLE)
  #define PyBUF_FULL (PyBUF_INDIRECT | PyBUF_FORMAT | PyBUF_WRITABLE)
  typedef int (*getbufferproc)(PyObject *, Py_buffer *, int);
  typedef void (*releasebufferproc)(PyObject *, Py_buffer *);
#endif
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
  #define PyUnicode_FromString(s) PyUnicode_Decode(s, strlen(s), "UTF-8", "strict")
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#if (PY_VERSION_HEX < 0x02060000) || (PY_MAJOR_VERSION >= 3)
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((k=k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PyBytesObject                PyStringObject
  #define PyBytes_Type                 PyString_Type
  #define PyBytes_Check                PyString_Check
  #define PyBytes_CheckExact           PyString_CheckExact
  #define PyBytes_FromString           PyString_FromString
  #define PyBytes_FromStringAndSize    PyString_FromStringAndSize
  #define PyBytes_FromFormat           PyString_FromFormat
  #define PyBytes_DecodeEscape         PyString_DecodeEscape
  #define PyBytes_AsString             PyString_AsString
  #define PyBytes_AsStringAndSize      PyString_AsStringAndSize
  #define PyBytes_Size                 PyString_Size
  #define PyBytes_AS_STRING            PyString_AS_STRING
  #define PyBytes_GET_SIZE             PyString_GET_SIZE
  #define PyBytes_Repr                 PyString_Repr
  #define PyBytes_Concat               PyString_Concat
  #define PyBytes_ConcatAndDel         PyString_ConcatAndDel
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PySet_Check(obj)             PyObject_TypeCheck(obj, &PySet_Type)
  #define PyFrozenSet_Check(obj)       PyObject_TypeCheck(obj, &PyFrozenSet_Type)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_VERSION_HEX < 0x03020000
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if (PY_MAJOR_VERSION < 3) || (PY_VERSION_HEX >= 0x03010300)
  #define __Pyx_PySequence_GetSlice(obj, a, b) PySequence_GetSlice(obj, a, b)
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) PySequence_SetSlice(obj, a, b, value)
  #define __Pyx_PySequence_DelSlice(obj, a, b) PySequence_DelSlice(obj, a, b)
#else
  #define __Pyx_PySequence_GetSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), (PyObject*)0) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_GetSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object is unsliceable", (obj)->ob_type->tp_name), (PyObject*)0)))
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_SetSlice(obj, a, b, value)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice assignment", (obj)->ob_type->tp_name), -1)))
  #define __Pyx_PySequence_DelSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_DelSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice deletion", (obj)->ob_type->tp_name), -1)))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),((char *)(n)))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),((char *)(n)),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),((char *)(n)))
#else
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),(n))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),(n),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),(n))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_NAMESTR(n) ((char *)(n))
  #define __Pyx_DOCSTR(n)  ((char *)(n))
#else
  #define __Pyx_NAMESTR(n) (n)
  #define __Pyx_DOCSTR(n)  (n)
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__h5py__h5g
#define __PYX_HAVE_API__h5py__h5g
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "unistd.h"
#include "stdint.h"
#include "api_compat.h"
#include "lzf_filter.h"
#include "hdf5.h"
#include "H5Cpublic.h"
#include "hdf5_hl.h"
#include "numpy/arrayobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const long n; const char* encoding; const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry; /*proto*/

#define __Pyx_PyBytes_FromUString(s) PyBytes_FromString((char*)s)
#define __Pyx_PyBytes_AsUString(s)   ((unsigned char*) PyBytes_AsString(s))
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject*);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))


#ifdef __GNUC__
  /* Test for GCC > 2.95 */
  #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95))
    #define likely(x)   __builtin_expect(!!(x), 1)
    #define unlikely(x) __builtin_expect(!!(x), 0)
  #else /* __GNUC__ > 2 ... */
    #define likely(x)   (x)
    #define unlikely(x) (x)
  #endif /* __GNUC__ > 2 ... */
#else /* __GNUC__ */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
    
static PyObject *__pyx_m;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "h5g.pyx",
  "numpy.pxd",
  "h5p.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_4h5py_3h5g_GroupIter;
struct __pyx_obj_4h5py_8_objects_ObjectID;
struct __pyx_obj_4h5py_3h5p_PropID;
struct __pyx_obj_4h5py_3h5p_PropInstanceID;
struct __pyx_obj_4h5py_3h5p_PropCreateID;
struct __pyx_obj_4h5py_3h5p_PropOCID;
struct __pyx_obj_4h5py_3h5g_GroupID;
struct __pyx_obj_4h5py_3h5p_PropFAID;
struct __pyx_obj_4h5py_3h5p_PropGCID;
struct __pyx_obj_4h5py_3h5p_PropDXID;
struct __pyx_obj_4h5py_3h5p_PropDCID;
struct __pyx_obj_4h5py_3h5p_PropLAID;
struct __pyx_obj_4h5py_3h5g__GroupVisitor;
struct __pyx_obj_4h5py_3h5p_PropFCID;
struct __pyx_obj_4h5py_3h5p_PropClassID;
struct __pyx_obj_4h5py_3h5p_PropCopyID;
struct __pyx_obj_4h5py_3h5p_PropLCID;
struct __pyx_obj_4h5py_3h5g_GroupStat;
struct __pyx_opt_args_4h5py_5utils_check_numpy_read;
struct __pyx_opt_args_4h5py_5utils_check_numpy_write;

/* "utils.pxd":17
 * cdef void efree(void* ptr)
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1
 * 
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_read {
  int __pyx_n;
  hid_t space_id;
};

/* "utils.pxd":18
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * 
 * cdef int convert_tuple(object tuple, hsize_t *dims, hsize_t rank) except -1
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_write {
  int __pyx_n;
  hid_t space_id;
};
struct __pyx_t_4h5py_7_errors_err_cookie;
typedef struct __pyx_t_4h5py_7_errors_err_cookie __pyx_t_4h5py_7_errors_err_cookie;

/* "_errors.pxd":230
 * # --- Functions for managing the HDF5 error callback mechanism ---
 * 
 * ctypedef struct err_cookie:             # <<<<<<<<<<<<<<
 *     # Defines the error handler state (callback and callback data)
 *     H5E_auto_t func
 */
struct __pyx_t_4h5py_7_errors_err_cookie {
  H5E_auto_t func;
  void *data;
};

/* "h5py/h5g.pyx":79
 * 
 * 
 * cdef class GroupIter:             # <<<<<<<<<<<<<<
 * 
 *     """
 */
struct __pyx_obj_4h5py_3h5g_GroupIter {
  PyObject_HEAD
  unsigned long idx;
  unsigned long nobjs;
  struct __pyx_obj_4h5py_3h5g_GroupID *grp;
  PyObject *names;
};


/* "_objects.pxd":12
 * from defs cimport *
 * 
 * cdef class ObjectID:             # <<<<<<<<<<<<<<
 * 
 *     cdef object __weakref__
 */
struct __pyx_obj_4h5py_8_objects_ObjectID {
  PyObject_HEAD
  PyObject *__weakref__;
  hid_t id;
  int locked;
  PyObject *_hash;
};


/* "h5p.pxd":16
 * # --- Base classes ---
 * 
 * cdef class PropID(ObjectID):             # <<<<<<<<<<<<<<
 *     """ Base class for all property lists """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
};


/* "h5p.pxd":26
 *     pass
 * 
 * cdef class PropInstanceID(PropID):             # <<<<<<<<<<<<<<
 *     """ Represents an instance of a property list class (i.e. an actual list
 *         which can be passed on to other API functions).
 */
struct __pyx_obj_4h5py_3h5p_PropInstanceID {
  struct __pyx_obj_4h5py_3h5p_PropID __pyx_base;
};


/* "h5p.pxd":32
 *     pass
 * 
 * cdef class PropCreateID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Base class for all object creation lists.
 * 
 */
struct __pyx_obj_4h5py_3h5p_PropCreateID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5p.pxd":44
 * # --- Object creation ---
 * 
 * cdef class PropOCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ Object creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropOCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5py/h5g.pxd":14
 * from _objects cimport ObjectID
 * 
 * cdef class GroupID(ObjectID):             # <<<<<<<<<<<<<<
 * 
 *     cdef readonly object links
 */
struct __pyx_obj_4h5py_3h5g_GroupID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
  PyObject *links;
};


/* "h5p.pxd":59
 * # --- Object access ---
 * 
 * cdef class PropFAID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ File access property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropFAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5p.pxd":78
 *     cdef char* _buf
 * 
 * cdef class PropGCID(PropOCID):             # <<<<<<<<<<<<<<
 *     """ Group creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropGCID {
  struct __pyx_obj_4h5py_3h5p_PropOCID __pyx_base;
};


/* "h5p.pxd":63
 *     pass
 * 
 * cdef class PropDXID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Dataset transfer property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropDXID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5p.pxd":48
 *     pass
 * 
 * cdef class PropDCID(PropOCID):             # <<<<<<<<<<<<<<
 *     """ Dataset creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropDCID {
  struct __pyx_obj_4h5py_3h5p_PropOCID __pyx_base;
};


/* "h5p.pxd":74
 *     pass
 * 
 * cdef class PropLAID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Link access property list """
 *     cdef char* _buf
 */
struct __pyx_obj_4h5py_3h5p_PropLAID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
  char *_buf;
};


/* "h5py/h5g.pyx":152
 * 
 * 
 * cdef class _GroupVisitor:             # <<<<<<<<<<<<<<
 * 
 *     cdef object func
 */
struct __pyx_obj_4h5py_3h5g__GroupVisitor {
  PyObject_HEAD
  PyObject *func;
  PyObject *retval;
};


/* "h5p.pxd":52
 *     pass
 * 
 * cdef class PropFCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ File creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropFCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5p.pxd":20
 *     pass
 * 
 * cdef class PropClassID(PropID):             # <<<<<<<<<<<<<<
 *     """ Represents an HDF5 property list class.  These can be either (locked)
 *         library-defined classes or user-created classes.
 */
struct __pyx_obj_4h5py_3h5p_PropClassID {
  struct __pyx_obj_4h5py_3h5p_PropID __pyx_base;
};


/* "h5p.pxd":39
 *     pass
 * 
 * cdef class PropCopyID(PropInstanceID):             # <<<<<<<<<<<<<<
 *     """ Property list for copying objects (as in h5o.copy) """
 * 
 */
struct __pyx_obj_4h5py_3h5p_PropCopyID {
  struct __pyx_obj_4h5py_3h5p_PropInstanceID __pyx_base;
};


/* "h5p.pxd":70
 * # --- New in 1.8 ---
 * 
 * cdef class PropLCID(PropCreateID):             # <<<<<<<<<<<<<<
 *     """ Link creation property list """
 *     pass
 */
struct __pyx_obj_4h5py_3h5p_PropLCID {
  struct __pyx_obj_4h5py_3h5p_PropCreateID __pyx_base;
};


/* "h5py/h5g.pyx":37
 * LINK_SOFT  = H5G_LINK_SOFT
 * 
 * cdef class GroupStat:             # <<<<<<<<<<<<<<
 *     """Represents the H5G_stat_t structure containing group member info.
 * 
 */
struct __pyx_obj_4h5py_3h5g_GroupStat {
  PyObject_HEAD
  H5G_stat_t infostruct;
};

#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname); /*proto*/
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif /* CYTHON_REFNANNY */
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name); /*proto*/

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name); /*proto*/

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name); /*proto*/

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found); /*proto*/

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact); /*proto*/

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb); /*proto*/
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause); /*proto*/

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
#define __Pyx_GetItemInt_List(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_GetItemInt_List_Fast(o, i) : \
                                                    __Pyx_GetItemInt_Generic(o, to_py_func(i)))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    else if ((-PyList_GET_SIZE(o) <= i) & (i < 0)) {
        PyObject *r = PyList_GET_ITEM(o, PyList_GET_SIZE(o) + i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
#define __Pyx_GetItemInt_Tuple(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_GetItemInt_Tuple_Fast(o, i) : \
                                                    __Pyx_GetItemInt_Generic(o, to_py_func(i)))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    else if ((-PyTuple_GET_SIZE(o) <= i) & (i < 0)) {
        PyObject *r = PyTuple_GET_ITEM(o, PyTuple_GET_SIZE(o) + i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
#define __Pyx_GetItemInt(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_GetItemInt_Fast(o, i) : \
                                                    __Pyx_GetItemInt_Generic(o, to_py_func(i)))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (PyList_CheckExact(o)) {
        Py_ssize_t n = (likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if (likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = (likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if (likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {  /* inlined PySequence_GetItem() */
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (unlikely(l < 0)) return NULL;
                i += l;
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

#define __Pyx_DelItemInt(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_DelItemInt_Fast(o, i) : \
                                                    __Pyx_DelItem_Generic(o, to_py_func(i)))
static CYTHON_INLINE int __Pyx_DelItem_Generic(PyObject *o, PyObject *j) {
    int r;
    if (!j) return -1;
    r = PyObject_DelItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_PYPY
    if (PySequence_Check(o)) {
        return PySequence_DelItem(o, i);
    }
#else
    PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
    if (likely(m && m->sq_ass_item)) {
        if (unlikely(i < 0) && likely(m->sq_length)) {
            Py_ssize_t l = m->sq_length(o);
            if (unlikely(l < 0)) return -1;
            i += l;
        }
        return m->sq_ass_item(o, i, (PyObject *)NULL);
    }
#endif
    return __Pyx_DelItem_Generic(o, PyInt_FromSsize_t(i));
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb); /*proto*/
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb); /*proto*/

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level); /*proto*/

static CYTHON_INLINE void __Pyx_RaiseImportError(PyObject *name);

static CYTHON_INLINE hid_t __Pyx_PyInt_from_py_hid_t(PyObject *);

static CYTHON_INLINE hsize_t __Pyx_PyInt_from_py_hsize_t(PyObject *);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_time_t(time_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t);

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject *);

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject *);

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject *);

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject *);

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject *);

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject *);

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject *);

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject *);

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject *);

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject *);

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject *);

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name); /*proto*/

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);  /*proto*/

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig); /*proto*/

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename); /*proto*/

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t); /*proto*/


/* Module declarations from 'h5py.api_types_ext' */

/* Module declarations from 'h5py.api_types_hdf5' */

/* Module declarations from 'h5py.defs' */
static hid_t (*__pyx_f_4h5py_4defs_H5Gopen)(hid_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gclose)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Glink2)(hid_t, char *, H5G_link_t, hid_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gunlink)(hid_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gmove2)(hid_t, char *, hid_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gget_num_objs)(hid_t, hsize_t *); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Gget_objname_by_idx)(hid_t, hsize_t, char *, size_t); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Gget_objtype_by_idx)(hid_t, hsize_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Giterate)(hid_t, char *, int *, H5G_iterate_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gget_objinfo)(hid_t, char *, int, H5G_stat_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gget_linkval)(hid_t, char *, size_t, char *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Gset_comment)(hid_t, char *, char *); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Gget_comment)(hid_t, char *, size_t, char *); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Gcreate_anon)(hid_t, hid_t, hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Gcreate2)(hid_t, char *, hid_t, hid_t, hid_t); /*proto*/

/* Module declarations from 'h5py._objects' */
static PyTypeObject *__pyx_ptype_4h5py_8_objects_ObjectID = 0;
static hid_t (*__pyx_f_4h5py_8_objects_pdefault)(struct __pyx_obj_4h5py_8_objects_ObjectID *); /*proto*/

/* Module declarations from 'numpy' */

/* Module declarations from 'h5py.numpy' */
static PyTypeObject *__pyx_ptype_4h5py_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_4h5py_5numpy_ndarray = 0;

/* Module declarations from 'h5py.utils' */
static void *(*__pyx_f_4h5py_5utils_emalloc)(size_t); /*proto*/
static void (*__pyx_f_4h5py_5utils_efree)(void *); /*proto*/

/* Module declarations from 'h5py.h5p' */
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropClassID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropInstanceID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropCreateID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropCopyID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropOCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropFCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropFAID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropDXID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropLCID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropLAID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5p_PropGCID = 0;

/* Module declarations from 'h5py._hdf5' */

/* Module declarations from 'h5py._errors' */
static __pyx_t_4h5py_7_errors_err_cookie (*__pyx_f_4h5py_7_errors_set_error_handler)(__pyx_t_4h5py_7_errors_err_cookie); /*proto*/

/* Module declarations from 'h5py.h5g' */
static PyTypeObject *__pyx_ptype_4h5py_3h5g_GroupID = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5g_GroupStat = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5g_GroupIter = 0;
static PyTypeObject *__pyx_ptype_4h5py_3h5g__GroupVisitor = 0;
static herr_t __pyx_f_4h5py_3h5g_cb_group_iter(hid_t, char *, void *); /*proto*/
#define __Pyx_MODULE_NAME "h5py.h5g"
int __pyx_module_is_main_h5py__h5g = 0;

/* Implementation of 'h5py.h5g' */
static PyObject *__pyx_builtin_StopIteration;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_6fileno___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_5objno___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_5nlink___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_4type___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_5mtime___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_7linklen___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat__hash(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self); /* proto */
static int __pyx_pf_4h5py_3h5g_9GroupIter___init__(struct __pyx_obj_4h5py_3h5g_GroupIter *__pyx_v_self, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_grp); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupIter_2__iter__(struct __pyx_obj_4h5py_3h5g_GroupIter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_9GroupIter_4__next__(struct __pyx_obj_4h5py_3h5g_GroupIter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_open(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_2create(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, PyObject *__pyx_v_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_gcpl); /* proto */
static int __pyx_pf_4h5py_3h5g_13_GroupVisitor___init__(struct __pyx_obj_4h5py_3h5g__GroupVisitor *__pyx_v_self, PyObject *__pyx_v_func); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_4iterate(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_loc, PyObject *__pyx_v_func, int __pyx_v_startidx, char *__pyx_v_obj_name); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_6get_objinfo(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_obj, PyObject *__pyx_v_name, int __pyx_v_follow_link); /* proto */
static int __pyx_pf_4h5py_3h5g_7GroupID___init__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, hid_t __pyx_v_id_); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_2_close(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_4link(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_current_name, char *__pyx_v_new_name, int __pyx_v_link_type, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_remote); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_6unlink(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_8move(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_current_name, char *__pyx_v_new_name, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_remote); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_10get_num_objs(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_12get_objname_by_idx(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, hsize_t __pyx_v_idx); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_14get_objtype_by_idx(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, hsize_t __pyx_v_idx); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_16get_linkval(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_18set_comment(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name, char *__pyx_v_comment); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_20get_comment(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name); /* proto */
static int __pyx_pf_4h5py_3h5g_7GroupID_22__contains__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_24__iter__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_4h5py_3h5g_7GroupID_26__len__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_5links___get__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self); /* proto */
static char __pyx_k_1[] = ".";
static char __pyx_k_2[] = "Starting index must be non-negative";
static char __pyx_k_6[] = "\"%s\" is not a symbolic link.";
static char __pyx_k_7[] = "\n    Low-level HDF5 \"H5G\" group interface.\n";
static char __pyx_k_10[] = "/home/computer/h5py/h5py/h5g.pyx";
static char __pyx_k_11[] = "h5py.h5g";
static char __pyx_k__i[] = "i";
static char __pyx_k__gid[] = "gid";
static char __pyx_k__grp[] = "grp";
static char __pyx_k__h5l[] = "h5l";
static char __pyx_k__id_[] = "id_";
static char __pyx_k__loc[] = "loc";
static char __pyx_k__obj[] = "obj";
static char __pyx_k__vis[] = "vis";
static char __pyx_k__LINK[] = "LINK";
static char __pyx_k__TYPE[] = "TYPE";
static char __pyx_k__func[] = "func";
static char __pyx_k__gcpl[] = "gcpl";
static char __pyx_k__h5py[] = "h5py";
static char __pyx_k__lcpl[] = "lcpl";
static char __pyx_k__lock[] = "lock";
static char __pyx_k__name[] = "name";
static char __pyx_k__open[] = "open";
static char __pyx_k__type[] = "type";
static char __pyx_k__GROUP[] = "GROUP";
static char __pyx_k___name[] = "_name";
static char __pyx_k__cname[] = "cname";
static char __pyx_k__mtime[] = "mtime";
static char __pyx_k__nlink[] = "nlink";
static char __pyx_k__objno[] = "objno";
static char __pyx_k__valid[] = "valid";
static char __pyx_k__append[] = "append";
static char __pyx_k__create[] = "create";
static char __pyx_k__fileno[] = "fileno";
static char __pyx_k__remote[] = "remote";
static char __pyx_k__DATASET[] = "DATASET";
static char __pyx_k__UNKNOWN[] = "UNKNOWN";
static char __pyx_k__comment[] = "comment";
static char __pyx_k__iterate[] = "iterate";
static char __pyx_k__linklen[] = "linklen";
static char __pyx_k__statobj[] = "statobj";
static char __pyx_k____exit__[] = "__exit__";
static char __pyx_k____main__[] = "__main__";
static char __pyx_k____test__[] = "__test__";
static char __pyx_k___objects[] = "_objects";
static char __pyx_k__new_name[] = "new_name";
static char __pyx_k__obj_name[] = "obj_name";
static char __pyx_k__registry[] = "registry";
static char __pyx_k__startidx[] = "startidx";
static char __pyx_k__LINK_HARD[] = "LINK_HARD";
static char __pyx_k__LINK_SOFT[] = "LINK_SOFT";
static char __pyx_k__LinkProxy[] = "LinkProxy";
static char __pyx_k____enter__[] = "__enter__";
static char __pyx_k__link_type[] = "link_type";
static char __pyx_k__LINK_ERROR[] = "LINK_ERROR";
static char __pyx_k__ValueError[] = "ValueError";
static char __pyx_k__follow_link[] = "follow_link";
static char __pyx_k__get_objinfo[] = "get_objinfo";
static char __pyx_k__current_name[] = "current_name";
static char __pyx_k__get_num_objs[] = "get_num_objs";
static char __pyx_k__StopIteration[] = "StopIteration";
static PyObject *__pyx_kp_b_1;
static PyObject *__pyx_kp_s_10;
static PyObject *__pyx_n_s_11;
static PyObject *__pyx_kp_s_2;
static PyObject *__pyx_kp_s_6;
static PyObject *__pyx_n_s__DATASET;
static PyObject *__pyx_n_s__GROUP;
static PyObject *__pyx_n_s__LINK;
static PyObject *__pyx_n_s__LINK_ERROR;
static PyObject *__pyx_n_s__LINK_HARD;
static PyObject *__pyx_n_s__LINK_SOFT;
static PyObject *__pyx_n_s__LinkProxy;
static PyObject *__pyx_n_s__StopIteration;
static PyObject *__pyx_n_s__TYPE;
static PyObject *__pyx_n_s__UNKNOWN;
static PyObject *__pyx_n_s__ValueError;
static PyObject *__pyx_n_s____enter__;
static PyObject *__pyx_n_s____exit__;
static PyObject *__pyx_n_s____main__;
static PyObject *__pyx_n_s____test__;
static PyObject *__pyx_n_s___name;
static PyObject *__pyx_n_s___objects;
static PyObject *__pyx_n_s__append;
static PyObject *__pyx_n_s__cname;
static PyObject *__pyx_n_s__comment;
static PyObject *__pyx_n_s__create;
static PyObject *__pyx_n_s__current_name;
static PyObject *__pyx_n_s__fileno;
static PyObject *__pyx_n_s__follow_link;
static PyObject *__pyx_n_s__func;
static PyObject *__pyx_n_s__gcpl;
static PyObject *__pyx_n_s__get_num_objs;
static PyObject *__pyx_n_s__get_objinfo;
static PyObject *__pyx_n_s__gid;
static PyObject *__pyx_n_s__grp;
static PyObject *__pyx_n_s__h5l;
static PyObject *__pyx_n_s__h5py;
static PyObject *__pyx_n_s__i;
static PyObject *__pyx_n_s__id_;
static PyObject *__pyx_n_s__iterate;
static PyObject *__pyx_n_s__lcpl;
static PyObject *__pyx_n_s__link_type;
static PyObject *__pyx_n_s__linklen;
static PyObject *__pyx_n_s__loc;
static PyObject *__pyx_n_s__lock;
static PyObject *__pyx_n_s__mtime;
static PyObject *__pyx_n_s__name;
static PyObject *__pyx_n_s__new_name;
static PyObject *__pyx_n_s__nlink;
static PyObject *__pyx_n_s__obj;
static PyObject *__pyx_n_s__obj_name;
static PyObject *__pyx_n_s__objno;
static PyObject *__pyx_n_s__open;
static PyObject *__pyx_n_s__registry;
static PyObject *__pyx_n_s__remote;
static PyObject *__pyx_n_s__startidx;
static PyObject *__pyx_n_s__statobj;
static PyObject *__pyx_n_s__type;
static PyObject *__pyx_n_s__valid;
static PyObject *__pyx_n_s__vis;
static PyObject *__pyx_int_1;
static int __pyx_k_5;
static PyObject *__pyx_k_tuple_3;
static PyObject *__pyx_k_tuple_4;
static PyObject *__pyx_k_tuple_8;
static PyObject *__pyx_k_tuple_12;
static PyObject *__pyx_k_tuple_14;
static PyObject *__pyx_k_tuple_16;
static PyObject *__pyx_k_codeobj_9;
static PyObject *__pyx_k_codeobj_13;
static PyObject *__pyx_k_codeobj_15;
static PyObject *__pyx_k_codeobj_17;

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_6fileno_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_6fileno_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat_6fileno___get__(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":57
 * 
 *     property fileno:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return (self.infostruct.fileno[0], self.infostruct.fileno[1])
 *     property objno:
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_6fileno___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5g.pyx":58
 *     property fileno:
 *         def __get__(self):
 *             return (self.infostruct.fileno[0], self.infostruct.fileno[1])             # <<<<<<<<<<<<<<
 *     property objno:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->infostruct.fileno[0])); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromUnsignedLong((__pyx_v_self->infostruct.fileno[1])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = ((PyObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5g.GroupStat.fileno.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_5objno_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_5objno_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat_5objno___get__(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":60
 *             return (self.infostruct.fileno[0], self.infostruct.fileno[1])
 *     property objno:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return (self.infostruct.objno[0], self.infostruct.objno[1])
 *     property nlink:
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_5objno___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5g.pyx":61
 *     property objno:
 *         def __get__(self):
 *             return (self.infostruct.objno[0], self.infostruct.objno[1])             # <<<<<<<<<<<<<<
 *     property nlink:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong((__pyx_v_self->infostruct.objno[0])); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromUnsignedLong((__pyx_v_self->infostruct.objno[1])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 61; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = ((PyObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5g.GroupStat.objno.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_5nlink_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_5nlink_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat_5nlink___get__(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":63
 *             return (self.infostruct.objno[0], self.infostruct.objno[1])
 *     property nlink:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.infostruct.nlink
 *     property type:
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_5nlink___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5g.pyx":64
 *     property nlink:
 *         def __get__(self):
 *             return self.infostruct.nlink             # <<<<<<<<<<<<<<
 *     property type:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromUnsignedLong(__pyx_v_self->infostruct.nlink); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5g.GroupStat.nlink.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat_4type___get__(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":66
 *             return self.infostruct.nlink
 *     property type:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.infostruct.type
 *     property mtime:
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_4type___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5g.pyx":67
 *     property type:
 *         def __get__(self):
 *             return self.infostruct.type             # <<<<<<<<<<<<<<
 *     property mtime:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromLong(__pyx_v_self->infostruct.type); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 67; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5g.GroupStat.type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_5mtime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_5mtime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat_5mtime___get__(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":69
 *             return self.infostruct.type
 *     property mtime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.infostruct.mtime
 *     property linklen:
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_5mtime___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5g.pyx":70
 *     property mtime:
 *         def __get__(self):
 *             return self.infostruct.mtime             # <<<<<<<<<<<<<<
 *     property linklen:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_to_py_time_t(__pyx_v_self->infostruct.mtime); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5g.GroupStat.mtime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_7linklen_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_7linklen_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat_7linklen___get__(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":72
 *             return self.infostruct.mtime
 *     property linklen:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.infostruct.linklen
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat_7linklen___get__(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5g.pyx":73
 *     property linklen:
 *         def __get__(self):
 *             return self.infostruct.linklen             # <<<<<<<<<<<<<<
 * 
 *     def _hash(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->infostruct.linklen); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5g.GroupStat.linklen.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupStat_1_hash(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_hash (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupStat__hash(((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":75
 *             return self.infostruct.linklen
 * 
 *     def _hash(self):             # <<<<<<<<<<<<<<
 *         return hash((self.fileno, self.objno, self.nlink, self.type, self.mtime, self.linklen))
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupStat__hash(struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_hash_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_hash", 0);

  /* "h5py/h5g.pyx":76
 * 
 *     def _hash(self):
 *         return hash((self.fileno, self.objno, self.nlink, self.type, self.mtime, self.linklen))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__fileno); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__objno); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__nlink); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__type); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__mtime); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__linklen); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 5, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_6);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_8 = PyObject_Hash(((PyObject *)__pyx_t_7)); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_7)); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyInt_FromHash_t(__pyx_t_8); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 76; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("h5py.h5g.GroupStat._hash", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5g_9GroupIter_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5g_9GroupIter_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_grp = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__grp,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__grp)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 92; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_grp = ((struct __pyx_obj_4h5py_3h5g_GroupID *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 92; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupIter.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_grp), __pyx_ptype_4h5py_3h5g_GroupID, 0, "grp", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 92; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupIter___init__(((struct __pyx_obj_4h5py_3h5g_GroupIter *)__pyx_v_self), __pyx_v_grp);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":92
 * 
 * 
 *     def __init__(self, GroupID grp not None):             # <<<<<<<<<<<<<<
 * 
 *         self.idx = 0
 */

static int __pyx_pf_4h5py_3h5g_9GroupIter___init__(struct __pyx_obj_4h5py_3h5g_GroupIter *__pyx_v_self, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_grp) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  unsigned long __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "h5py/h5g.pyx":94
 *     def __init__(self, GroupID grp not None):
 * 
 *         self.idx = 0             # <<<<<<<<<<<<<<
 *         self.grp = grp
 *         self.nobjs = grp.get_num_objs()
 */
  __pyx_v_self->idx = 0;

  /* "h5py/h5g.pyx":95
 * 
 *         self.idx = 0
 *         self.grp = grp             # <<<<<<<<<<<<<<
 *         self.nobjs = grp.get_num_objs()
 *         self.names = []
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_grp));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_grp));
  __Pyx_GOTREF(__pyx_v_self->grp);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->grp));
  __pyx_v_self->grp = __pyx_v_grp;

  /* "h5py/h5g.pyx":96
 *         self.idx = 0
 *         self.grp = grp
 *         self.nobjs = grp.get_num_objs()             # <<<<<<<<<<<<<<
 *         self.names = []
 * 
 */
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_grp), __pyx_n_s__get_num_objs); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_AsUnsignedLong(__pyx_t_2); if (unlikely((__pyx_t_3 == (unsigned long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->nobjs = __pyx_t_3;

  /* "h5py/h5g.pyx":97
 *         self.grp = grp
 *         self.nobjs = grp.get_num_objs()
 *         self.names = []             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(((PyObject *)__pyx_t_2));
  __Pyx_GOTREF(__pyx_v_self->names);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->names));
  __pyx_v_self->names = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5g.GroupIter.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupIter_3__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupIter_3__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupIter_2__iter__(((struct __pyx_obj_4h5py_3h5g_GroupIter *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":100
 * 
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 * 
 *         return self
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupIter_2__iter__(struct __pyx_obj_4h5py_3h5g_GroupIter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "h5py/h5g.pyx":102
 *     def __iter__(self):
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_9GroupIter_5__next__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_9GroupIter_5__next__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__next__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_9GroupIter_4__next__(((struct __pyx_obj_4h5py_3h5g_GroupIter *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":105
 * 
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 * 
 *         if self.idx == self.nobjs:
 */

static PyObject *__pyx_pf_4h5py_3h5g_9GroupIter_4__next__(struct __pyx_obj_4h5py_3h5g_GroupIter *__pyx_v_self) {
  PyObject *__pyx_v_retval = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__", 0);

  /* "h5py/h5g.pyx":107
 *     def __next__(self):
 * 
 *         if self.idx == self.nobjs:             # <<<<<<<<<<<<<<
 *             self.grp = None
 *             self.names = None
 */
  __pyx_t_1 = (__pyx_v_self->idx == __pyx_v_self->nobjs);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":108
 * 
 *         if self.idx == self.nobjs:
 *             self.grp = None             # <<<<<<<<<<<<<<
 *             self.names = None
 *             raise StopIteration
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->grp);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->grp));
    __pyx_v_self->grp = ((struct __pyx_obj_4h5py_3h5g_GroupID *)Py_None);

    /* "h5py/h5g.pyx":109
 *         if self.idx == self.nobjs:
 *             self.grp = None
 *             self.names = None             # <<<<<<<<<<<<<<
 *             raise StopIteration
 * 
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->names);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->names));
    __pyx_v_self->names = ((PyObject*)Py_None);

    /* "h5py/h5g.pyx":110
 *             self.grp = None
 *             self.names = None
 *             raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *         if self.idx == 0:
 */
    __Pyx_Raise(__pyx_builtin_StopIteration, 0, 0, 0);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":112
 *             raise StopIteration
 * 
 *         if self.idx == 0:             # <<<<<<<<<<<<<<
 *             self.grp.links.iterate(self.names.append)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->idx == 0);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":113
 * 
 *         if self.idx == 0:
 *             self.grp.links.iterate(self.names.append)             # <<<<<<<<<<<<<<
 * 
 *         retval = self.names[self.idx]
 */
    __pyx_t_2 = PyObject_GetAttr(__pyx_v_self->grp->links, __pyx_n_s__iterate); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_GetAttr(((PyObject *)__pyx_v_self->names), __pyx_n_s__append); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_t_4), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "h5py/h5g.pyx":115
 *             self.grp.links.iterate(self.names.append)
 * 
 *         retval = self.names[self.idx]             # <<<<<<<<<<<<<<
 *         self.idx += 1
 * 
 */
  if (unlikely(((PyObject *)__pyx_v_self->names) == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = __Pyx_GetItemInt_List(((PyObject *)__pyx_v_self->names), __pyx_v_self->idx, sizeof(unsigned long)+1, PyLong_FromUnsignedLong); if (!__pyx_t_3) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_retval = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "h5py/h5g.pyx":116
 * 
 *         retval = self.names[self.idx]
 *         self.idx += 1             # <<<<<<<<<<<<<<
 * 
 *         return retval
 */
  __pyx_v_self->idx = (__pyx_v_self->idx + 1);

  /* "h5py/h5g.pyx":118
 *         self.idx += 1
 * 
 *         return retval             # <<<<<<<<<<<<<<
 * 
 * # === Basic group management ==================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_retval);
  __pyx_r = __pyx_v_retval;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5g.GroupIter.__next__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_retval);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_1open(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_open[] = "(ObjectID loc, STRING name) => GroupID\n\n    Open an existing HDF5 group, attached to some other group.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5g_1open = {__Pyx_NAMESTR("open"), (PyCFunction)__pyx_pw_4h5py_3h5g_1open, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_open)};
static PyObject *__pyx_pw_4h5py_3h5g_1open(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  char *__pyx_v_name;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("open (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("open", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "open") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_name = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("open", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_open(__pyx_self, __pyx_v_loc, __pyx_v_name);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":123
 * 
 * 
 * def open(ObjectID loc not None, char* name):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name) => GroupID
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_open(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, char *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);

  /* "h5py/h5g.pyx":128
 *     Open an existing HDF5 group, attached to some other group.
 *     """
 *     return GroupID.open(H5Gopen(loc.id, name))             # <<<<<<<<<<<<<<
 * 
 * def create(ObjectID loc not None, object name, PropID lcpl=None,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5g_GroupID)), __pyx_n_s__open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Gopen(__pyx_v_loc->id, __pyx_v_name); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyInt_to_py_hid_t(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_4), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5g.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_3create(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_2create[] = "(ObjectID loc, STRING name or None, PropLCID lcpl=None,\n        PropGCID gcpl=None)\n    => GroupID\n\n    Create a new group, under a given parent group.  If name is None,\n    an anonymous group will be created in the file.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5g_3create = {__Pyx_NAMESTR("create"), (PyCFunction)__pyx_pw_4h5py_3h5g_3create, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_2create)};
static PyObject *__pyx_pw_4h5py_3h5g_3create(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc = 0;
  PyObject *__pyx_v_name = 0;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl = 0;
  struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_gcpl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__name,&__pyx_n_s__lcpl,&__pyx_n_s__gcpl,0};
    PyObject* values[4] = {0,0,0,0};

    /* "h5py/h5g.pyx":130
 *     return GroupID.open(H5Gopen(loc.id, name))
 * 
 * def create(ObjectID loc not None, object name, PropID lcpl=None,             # <<<<<<<<<<<<<<
 *            PropID gcpl=None):
 *     """(ObjectID loc, STRING name or None, PropLCID lcpl=None,
 */
    values[2] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);

    /* "h5py/h5g.pyx":131
 * 
 * def create(ObjectID loc not None, object name, PropID lcpl=None,
 *            PropID gcpl=None):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name or None, PropLCID lcpl=None,
 *         PropGCID gcpl=None)
 */
    values[3] = (PyObject *)((struct __pyx_obj_4h5py_3h5p_PropID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create", 0, 2, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__lcpl);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__gcpl);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_name = values[1];
    __pyx_v_lcpl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[2]);
    __pyx_v_gcpl = ((struct __pyx_obj_4h5py_3h5p_PropID *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lcpl), __pyx_ptype_4h5py_3h5p_PropID, 1, "lcpl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_gcpl), __pyx_ptype_4h5py_3h5p_PropID, 1, "gcpl", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_2create(__pyx_self, __pyx_v_loc, __pyx_v_name, __pyx_v_lcpl, __pyx_v_gcpl);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":130
 *     return GroupID.open(H5Gopen(loc.id, name))
 * 
 * def create(ObjectID loc not None, object name, PropID lcpl=None,             # <<<<<<<<<<<<<<
 *            PropID gcpl=None):
 *     """(ObjectID loc, STRING name or None, PropLCID lcpl=None,
 */

static PyObject *__pyx_pf_4h5py_3h5g_2create(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_loc, PyObject *__pyx_v_name, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_lcpl, struct __pyx_obj_4h5py_3h5p_PropID *__pyx_v_gcpl) {
  hid_t __pyx_v_gid;
  char *__pyx_v_cname;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  char *__pyx_t_2;
  hid_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create", 0);

  /* "h5py/h5g.pyx":140
 *     """
 *     cdef hid_t gid
 *     cdef char* cname = NULL             # <<<<<<<<<<<<<<
 *     if name is not None:
 *         cname = name
 */
  __pyx_v_cname = NULL;

  /* "h5py/h5g.pyx":141
 *     cdef hid_t gid
 *     cdef char* cname = NULL
 *     if name is not None:             # <<<<<<<<<<<<<<
 *         cname = name
 * 
 */
  __pyx_t_1 = (__pyx_v_name != Py_None);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":142
 *     cdef char* cname = NULL
 *     if name is not None:
 *         cname = name             # <<<<<<<<<<<<<<
 * 
 *     if cname != NULL:
 */
    __pyx_t_2 = PyBytes_AsString(__pyx_v_name); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_cname = __pyx_t_2;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":144
 *         cname = name
 * 
 *     if cname != NULL:             # <<<<<<<<<<<<<<
 *         gid = H5Gcreate2(loc.id, cname, pdefault(lcpl), pdefault(gcpl), H5P_DEFAULT)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_cname != NULL);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":145
 * 
 *     if cname != NULL:
 *         gid = H5Gcreate2(loc.id, cname, pdefault(lcpl), pdefault(gcpl), H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *     else:
 *         gid = H5Gcreate_anon(loc.id, pdefault(gcpl), H5P_DEFAULT)
 */
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Gcreate2(__pyx_v_loc->id, __pyx_v_cname, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_lcpl)), __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_gcpl)), H5P_DEFAULT); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_gid = __pyx_t_3;
    goto __pyx_L4;
  }
  /*else*/ {

    /* "h5py/h5g.pyx":147
 *         gid = H5Gcreate2(loc.id, cname, pdefault(lcpl), pdefault(gcpl), H5P_DEFAULT)
 *     else:
 *         gid = H5Gcreate_anon(loc.id, pdefault(gcpl), H5P_DEFAULT)             # <<<<<<<<<<<<<<
 * 
 *     return GroupID.open(gid)
 */
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Gcreate_anon(__pyx_v_loc->id, __pyx_f_4h5py_8_objects_pdefault(((struct __pyx_obj_4h5py_8_objects_ObjectID *)__pyx_v_gcpl)), H5P_DEFAULT); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_gid = __pyx_t_3;
  }
  __pyx_L4:;

  /* "h5py/h5g.pyx":149
 *         gid = H5Gcreate_anon(loc.id, pdefault(gcpl), H5P_DEFAULT)
 * 
 *     return GroupID.open(gid)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5g_GroupID)), __pyx_n_s__open); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_to_py_hid_t(__pyx_v_gid); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = PyObject_Call(__pyx_t_4, ((PyObject *)__pyx_t_6), NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_6)); __pyx_t_6 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("h5py.h5g.create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5g_13_GroupVisitor_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5g_13_GroupVisitor_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_func = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__func,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__func)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_func = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 157; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g._GroupVisitor.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_13_GroupVisitor___init__(((struct __pyx_obj_4h5py_3h5g__GroupVisitor *)__pyx_v_self), __pyx_v_func);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":157
 *     cdef object retval
 * 
 *     def __init__(self, func):             # <<<<<<<<<<<<<<
 *         self.func = func
 *         self.retval = None
 */

static int __pyx_pf_4h5py_3h5g_13_GroupVisitor___init__(struct __pyx_obj_4h5py_3h5g__GroupVisitor *__pyx_v_self, PyObject *__pyx_v_func) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "h5py/h5g.pyx":158
 * 
 *     def __init__(self, func):
 *         self.func = func             # <<<<<<<<<<<<<<
 *         self.retval = None
 * 
 */
  __Pyx_INCREF(__pyx_v_func);
  __Pyx_GIVEREF(__pyx_v_func);
  __Pyx_GOTREF(__pyx_v_self->func);
  __Pyx_DECREF(__pyx_v_self->func);
  __pyx_v_self->func = __pyx_v_func;

  /* "h5py/h5g.pyx":159
 *     def __init__(self, func):
 *         self.func = func
 *         self.retval = None             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t cb_group_iter(hid_t gid, char *name, void* vis_in) except 2:
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->retval);
  __Pyx_DECREF(__pyx_v_self->retval);
  __pyx_v_self->retval = Py_None;

  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":161
 *         self.retval = None
 * 
 * cdef herr_t cb_group_iter(hid_t gid, char *name, void* vis_in) except 2:             # <<<<<<<<<<<<<<
 * 
 *     cdef _GroupVisitor vis = <_GroupVisitor>vis_in
 */

static herr_t __pyx_f_4h5py_3h5g_cb_group_iter(CYTHON_UNUSED hid_t __pyx_v_gid, char *__pyx_v_name, void *__pyx_v_vis_in) {
  struct __pyx_obj_4h5py_3h5g__GroupVisitor *__pyx_v_vis = 0;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb_group_iter", 0);

  /* "h5py/h5g.pyx":163
 * cdef herr_t cb_group_iter(hid_t gid, char *name, void* vis_in) except 2:
 * 
 *     cdef _GroupVisitor vis = <_GroupVisitor>vis_in             # <<<<<<<<<<<<<<
 * 
 *     vis.retval = vis.func(name)
 */
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_4h5py_3h5g__GroupVisitor *)__pyx_v_vis_in)));
  __pyx_v_vis = ((struct __pyx_obj_4h5py_3h5g__GroupVisitor *)__pyx_v_vis_in);

  /* "h5py/h5g.pyx":165
 *     cdef _GroupVisitor vis = <_GroupVisitor>vis_in
 * 
 *     vis.retval = vis.func(name)             # <<<<<<<<<<<<<<
 * 
 *     if vis.retval is not None:
 */
  __pyx_t_1 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_t_1));
  __Pyx_GIVEREF(((PyObject *)__pyx_t_1));
  __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_Call(__pyx_v_vis->func, ((PyObject *)__pyx_t_2), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_vis->retval);
  __Pyx_DECREF(__pyx_v_vis->retval);
  __pyx_v_vis->retval = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "h5py/h5g.pyx":167
 *     vis.retval = vis.func(name)
 * 
 *     if vis.retval is not None:             # <<<<<<<<<<<<<<
 *         return 1
 *     return 0
 */
  __pyx_t_3 = (__pyx_v_vis->retval != Py_None);
  if (__pyx_t_3) {

    /* "h5py/h5g.pyx":168
 * 
 *     if vis.retval is not None:
 *         return 1             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_r = 1;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":169
 *     if vis.retval is not None:
 *         return 1
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5g.cb_group_iter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 2;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_vis);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_5iterate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_4iterate[] = " (GroupID loc, CALLABLE func, UINT startidx=0, **kwds)\n    => Return value from func\n\n    Iterate a callable (function, method or callable object) over the\n    members of a group.  Your callable should have the signature::\n\n        func(STRING name) => Result\n\n    Returning None continues iteration; returning anything else aborts\n    iteration and returns that value. Keywords:\n\n    STRING obj_name (\".\")\n        Iterate over this subgroup instead\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5g_5iterate = {__Pyx_NAMESTR("iterate"), (PyCFunction)__pyx_pw_4h5py_3h5g_5iterate, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_4iterate)};
static PyObject *__pyx_pw_4h5py_3h5g_5iterate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_loc = 0;
  PyObject *__pyx_v_func = 0;
  int __pyx_v_startidx;
  char *__pyx_v_obj_name;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__loc,&__pyx_n_s__func,&__pyx_n_s__startidx,&__pyx_n_s__obj_name,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__loc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__func)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("iterate", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__startidx);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (kw_args == 1) {
        const Py_ssize_t index = 3;
        PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
        if (value) { values[index] = value; kw_args--; }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "iterate") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_loc = ((struct __pyx_obj_4h5py_3h5g_GroupID *)values[0]);
    __pyx_v_func = values[1];
    if (values[2]) {
      __pyx_v_startidx = __Pyx_PyInt_AsInt(values[2]); if (unlikely((__pyx_v_startidx == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_startidx = ((int)0);
    }
    if (values[3]) {
      __pyx_v_obj_name = PyBytes_AsString(values[3]); if (unlikely((!__pyx_v_obj_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_obj_name = ((char *)__pyx_k_1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterate", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.iterate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_loc), __pyx_ptype_4h5py_3h5g_GroupID, 0, "loc", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_4iterate(__pyx_self, __pyx_v_loc, __pyx_v_func, __pyx_v_startidx, __pyx_v_obj_name);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":172
 * 
 * 
 * def iterate(GroupID loc not None, object func, int startidx=0, *,             # <<<<<<<<<<<<<<
 *             char* obj_name='.'):
 *     """ (GroupID loc, CALLABLE func, UINT startidx=0, **kwds)
 */

static PyObject *__pyx_pf_4h5py_3h5g_4iterate(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_loc, PyObject *__pyx_v_func, int __pyx_v_startidx, char *__pyx_v_obj_name) {
  int __pyx_v_i;
  struct __pyx_obj_4h5py_3h5g__GroupVisitor *__pyx_v_vis = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  herr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("iterate", 0);

  /* "h5py/h5g.pyx":188
 *         Iterate over this subgroup instead
 *     """
 *     if startidx < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("Starting index must be non-negative")
 * 
 */
  __pyx_t_1 = (__pyx_v_startidx < 0);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":189
 *     """
 *     if startidx < 0:
 *         raise ValueError("Starting index must be non-negative")             # <<<<<<<<<<<<<<
 * 
 *     cdef int i = startidx
 */
    __pyx_t_2 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_k_tuple_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":191
 *         raise ValueError("Starting index must be non-negative")
 * 
 *     cdef int i = startidx             # <<<<<<<<<<<<<<
 *     cdef _GroupVisitor vis = _GroupVisitor(func)
 * 
 */
  __pyx_v_i = __pyx_v_startidx;

  /* "h5py/h5g.pyx":192
 * 
 *     cdef int i = startidx
 *     cdef _GroupVisitor vis = _GroupVisitor(func)             # <<<<<<<<<<<<<<
 * 
 *     H5Giterate(loc.id, obj_name, &i, <H5G_iterate_t>cb_group_iter, <void*>vis)
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_func);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_func);
  __Pyx_GIVEREF(__pyx_v_func);
  __pyx_t_3 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5g__GroupVisitor)), ((PyObject *)__pyx_t_2), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  __pyx_v_vis = ((struct __pyx_obj_4h5py_3h5g__GroupVisitor *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "h5py/h5g.pyx":194
 *     cdef _GroupVisitor vis = _GroupVisitor(func)
 * 
 *     H5Giterate(loc.id, obj_name, &i, <H5G_iterate_t>cb_group_iter, <void*>vis)             # <<<<<<<<<<<<<<
 * 
 *     return vis.retval
 */
  __pyx_t_4 = __pyx_f_4h5py_4defs_H5Giterate(__pyx_v_loc->__pyx_base.id, __pyx_v_obj_name, (&__pyx_v_i), ((H5G_iterate_t)__pyx_f_4h5py_3h5g_cb_group_iter), ((void *)__pyx_v_vis)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 194; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":196
 *     H5Giterate(loc.id, obj_name, &i, <H5G_iterate_t>cb_group_iter, <void*>vis)
 * 
 *     return vis.retval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_vis->retval);
  __pyx_r = __pyx_v_vis->retval;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5g.iterate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_vis);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7get_objinfo(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_6get_objinfo[] = "(ObjectID obj, STRING name='.', BOOL follow_link=True) => GroupStat object\n\n    Obtain information about a named object.  If \"name\" is provided,\n    \"obj\" is taken to be a GroupID object containing the target.\n    The return value is a GroupStat object; see that class's docstring\n    for a description of its attributes.\n\n    If follow_link is True (default) and the object is a symbolic link,\n    the information returned describes its target.  Otherwise the\n    information describes the link itself.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5g_7get_objinfo = {__Pyx_NAMESTR("get_objinfo"), (PyCFunction)__pyx_pw_4h5py_3h5g_7get_objinfo, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_6get_objinfo)};
static PyObject *__pyx_pw_4h5py_3h5g_7get_objinfo(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_obj = 0;
  PyObject *__pyx_v_name = 0;
  int __pyx_v_follow_link;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_objinfo (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__obj,&__pyx_n_s__name,&__pyx_n_s__follow_link,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)__pyx_kp_b_1);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__obj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__follow_link);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_objinfo") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_obj = ((struct __pyx_obj_4h5py_8_objects_ObjectID *)values[0]);
    __pyx_v_name = values[1];
    if (values[2]) {
      __pyx_v_follow_link = __Pyx_PyInt_AsInt(values[2]); if (unlikely((__pyx_v_follow_link == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_follow_link = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_objinfo", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.get_objinfo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_obj), __pyx_ptype_4h5py_8_objects_ObjectID, 0, "obj", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_6get_objinfo(__pyx_self, __pyx_v_obj, __pyx_v_name, __pyx_v_follow_link);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":199
 * 
 * 
 * def get_objinfo(ObjectID obj not None, object name=b'.', int follow_link=1):             # <<<<<<<<<<<<<<
 *     """(ObjectID obj, STRING name='.', BOOL follow_link=True) => GroupStat object
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_6get_objinfo(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4h5py_8_objects_ObjectID *__pyx_v_obj, PyObject *__pyx_v_name, int __pyx_v_follow_link) {
  struct __pyx_obj_4h5py_3h5g_GroupStat *__pyx_v_statobj = 0;
  char *__pyx_v__name;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  herr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_objinfo", 0);

  /* "h5py/h5g.pyx":212
 *     """
 *     cdef GroupStat statobj
 *     statobj = GroupStat()             # <<<<<<<<<<<<<<
 *     cdef char* _name
 *     _name = name
 */
  __pyx_t_1 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5g_GroupStat)), ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_statobj = ((struct __pyx_obj_4h5py_3h5g_GroupStat *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5g.pyx":214
 *     statobj = GroupStat()
 *     cdef char* _name
 *     _name = name             # <<<<<<<<<<<<<<
 * 
 *     H5Gget_objinfo(obj.id, _name, follow_link, &statobj.infostruct)
 */
  __pyx_t_2 = PyBytes_AsString(__pyx_v_name); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v__name = __pyx_t_2;

  /* "h5py/h5g.pyx":216
 *     _name = name
 * 
 *     H5Gget_objinfo(obj.id, _name, follow_link, &statobj.infostruct)             # <<<<<<<<<<<<<<
 * 
 *     return statobj
 */
  __pyx_t_3 = __pyx_f_4h5py_4defs_H5Gget_objinfo(__pyx_v_obj->id, __pyx_v__name, __pyx_v_follow_link, (&__pyx_v_statobj->infostruct)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 216; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":218
 *     H5Gget_objinfo(obj.id, _name, follow_link, &statobj.infostruct)
 * 
 *     return statobj             # <<<<<<<<<<<<<<
 * 
 * # === Group member management =================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_statobj));
  __pyx_r = ((PyObject *)__pyx_v_statobj);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("h5py.h5g.get_objinfo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_statobj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5g_7GroupID_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4h5py_3h5g_7GroupID_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  hid_t __pyx_v_id_;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__id_,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__id_)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_id_ = __Pyx_PyInt_from_py_hid_t(values[0]); if (unlikely((__pyx_v_id_ == (hid_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID___init__(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), __pyx_v_id_);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":246
 *     """
 * 
 *     def __init__(self, hid_t id_):             # <<<<<<<<<<<<<<
 *         import h5l
 *         self.links = h5l.LinkProxy(id_)
 */

static int __pyx_pf_4h5py_3h5g_7GroupID___init__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, hid_t __pyx_v_id_) {
  PyObject *__pyx_v_h5l = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "h5py/h5g.pyx":247
 * 
 *     def __init__(self, hid_t id_):
 *         import h5l             # <<<<<<<<<<<<<<
 *         self.links = h5l.LinkProxy(id_)
 * 
 */
  __pyx_t_1 = __Pyx_Import(((PyObject *)__pyx_n_s__h5l), 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_h5l = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "h5py/h5g.pyx":248
 *     def __init__(self, hid_t id_):
 *         import h5l
 *         self.links = h5l.LinkProxy(id_)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyObject_GetAttr(__pyx_v_h5l, __pyx_n_s__LinkProxy); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_to_py_hid_t(__pyx_v_id_); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->links);
  __Pyx_DECREF(__pyx_v_self->links);
  __pyx_v_self->links = __pyx_t_2;
  __pyx_t_2 = 0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5g.GroupID.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_h5l);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_3_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_2_close[] = "()\n\n        Terminate access through this identifier.  You shouldn't have to\n        call this manually; group identifiers are automatically released\n        when their Python wrappers are freed.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_3_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_close (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_2_close(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":251
 * 
 * 
 *     def _close(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_2_close(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  herr_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_close", 0);

  /* "h5py/h5g.pyx":258
 *         when their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Gclose(self.id)
 *             if not self.valid:
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_GetName(__pyx_m, __pyx_n_s___objects); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s__registry); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_GetAttr(__pyx_t_2, __pyx_n_s__lock); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s____exit__); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s____enter__); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "h5py/h5g.pyx":259
 *         """
 *         with _objects.registry.lock:
 *             H5Gclose(self.id)             # <<<<<<<<<<<<<<
 *             if not self.valid:
 *                 del _objects.registry[self.id]
 */
          __pyx_t_8 = __pyx_f_4h5py_4defs_H5Gclose(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L7_error;}

          /* "h5py/h5g.pyx":260
 *         with _objects.registry.lock:
 *             H5Gclose(self.id)
 *             if not self.valid:             # <<<<<<<<<<<<<<
 *                 del _objects.registry[self.id]
 * 
 */
          __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__valid); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_10 = (!__pyx_t_9);
          if (__pyx_t_10) {

            /* "h5py/h5g.pyx":261
 *             H5Gclose(self.id)
 *             if not self.valid:
 *                 del _objects.registry[self.id]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_1 = __Pyx_GetName(__pyx_m, __pyx_n_s___objects); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_4 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s__registry); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (__Pyx_DelItemInt(__pyx_t_4, __pyx_v_self->__pyx_base.id, sizeof(hid_t), __Pyx_PyInt_to_py_hid_t) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            goto __pyx_L15;
          }
          __pyx_L15:;
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L14_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "h5py/h5g.pyx":258
 *         when their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Gclose(self.id)
 *             if not self.valid:
 */
        /*except:*/ {
          __Pyx_AddTraceback("h5py.h5g.GroupID._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_2);
          __Pyx_GIVEREF(__pyx_t_2);
          __pyx_t_12 = PyObject_Call(__pyx_t_3, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __pyx_t_9 = (!__pyx_t_10);
          if (__pyx_t_9) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_ErrRestore(__pyx_t_4, __pyx_t_1, __pyx_t_2);
            __pyx_t_4 = 0; __pyx_t_1 = 0; __pyx_t_2 = 0; 
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
            goto __pyx_L18;
          }
          __pyx_L18:;
          __Pyx_DECREF(((PyObject *)__pyx_t_11)); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L14_try_end:;
      }
    }
    /*finally:*/ {
      if (__pyx_t_3) {
        __pyx_t_7 = PyObject_Call(__pyx_t_3, __pyx_k_tuple_4, NULL);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    goto __pyx_L19;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L1_error;
    __pyx_L19:;
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("h5py.h5g.GroupID._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_4link[] = "(STRING current_name, STRING new_name, INT link_type=LINK_HARD,\n        GroupID remote=None)\n\n        Create a new hard or soft link.  current_name identifies\n        the link target (object the link will point to).  The new link is\n        identified by new_name and (optionally) another group \"remote\".\n\n        Link types are:\n\n        LINK_HARD\n            Hard link to existing object (default)\n\n        LINK_SOFT\n            Symbolic link; link target need not exist.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_5link(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  char *__pyx_v_current_name;
  char *__pyx_v_new_name;
  int __pyx_v_link_type;
  struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_remote = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("link (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__current_name,&__pyx_n_s__new_name,&__pyx_n_s__link_type,&__pyx_n_s__remote,0};
    PyObject* values[4] = {0,0,0,0};

    /* "h5py/h5g.pyx":265
 * 
 *     def link(self, char* current_name, char* new_name,
 *              int link_type=H5G_LINK_HARD, GroupID remote=None):             # <<<<<<<<<<<<<<
 *         """(STRING current_name, STRING new_name, INT link_type=LINK_HARD,
 *         GroupID remote=None)
 */
    values[3] = (PyObject *)((struct __pyx_obj_4h5py_3h5g_GroupID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__current_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__new_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("link", 0, 2, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__link_type);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__remote);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "link") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_current_name = PyBytes_AsString(values[0]); if (unlikely((!__pyx_v_current_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_new_name = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_new_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_link_type = __Pyx_PyInt_AsInt(values[2]); if (unlikely((__pyx_v_link_type == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_link_type = __pyx_k_5;
    }
    __pyx_v_remote = ((struct __pyx_obj_4h5py_3h5g_GroupID *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("link", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_remote), __pyx_ptype_4h5py_3h5g_GroupID, 1, "remote", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 265; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_4link(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), __pyx_v_current_name, __pyx_v_new_name, __pyx_v_link_type, __pyx_v_remote);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":264
 * 
 * 
 *     def link(self, char* current_name, char* new_name,             # <<<<<<<<<<<<<<
 *              int link_type=H5G_LINK_HARD, GroupID remote=None):
 *         """(STRING current_name, STRING new_name, INT link_type=LINK_HARD,
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_4link(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_current_name, char *__pyx_v_new_name, int __pyx_v_link_type, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_remote) {
  hid_t __pyx_v_remote_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hid_t __pyx_t_2;
  herr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("link", 0);

  /* "h5py/h5g.pyx":282
 *         """
 *         cdef hid_t remote_id
 *         if remote is None:             # <<<<<<<<<<<<<<
 *             remote_id = self.id
 *         else:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_remote) == Py_None);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":283
 *         cdef hid_t remote_id
 *         if remote is None:
 *             remote_id = self.id             # <<<<<<<<<<<<<<
 *         else:
 *             remote_id = remote.id
 */
    __pyx_t_2 = __pyx_v_self->__pyx_base.id;
    __pyx_v_remote_id = __pyx_t_2;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/h5g.pyx":285
 *             remote_id = self.id
 *         else:
 *             remote_id = remote.id             # <<<<<<<<<<<<<<
 * 
 *         H5Glink2(self.id, current_name, <H5G_link_t>link_type, remote_id, new_name)
 */
    __pyx_t_2 = __pyx_v_remote->__pyx_base.id;
    __pyx_v_remote_id = __pyx_t_2;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":287
 *             remote_id = remote.id
 * 
 *         H5Glink2(self.id, current_name, <H5G_link_t>link_type, remote_id, new_name)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_4h5py_4defs_H5Glink2(__pyx_v_self->__pyx_base.id, __pyx_v_current_name, ((H5G_link_t)__pyx_v_link_type), __pyx_v_remote_id, __pyx_v_new_name); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.link", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_7unlink(PyObject *__pyx_v_self, PyObject *__pyx_arg_name); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_6unlink[] = "(STRING name)\n\n        Remove a link to an object from this group.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_7unlink(PyObject *__pyx_v_self, PyObject *__pyx_arg_name) {
  char *__pyx_v_name;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unlink (wrapper)", 0);
  assert(__pyx_arg_name); {
    __pyx_v_name = PyBytes_AsString(__pyx_arg_name); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 290; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.unlink", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_6unlink(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), ((char *)__pyx_v_name));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":290
 * 
 * 
 *     def unlink(self, char* name):             # <<<<<<<<<<<<<<
 *         """(STRING name)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_6unlink(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unlink", 0);

  /* "h5py/h5g.pyx":295
 *         Remove a link to an object from this group.
 *         """
 *         H5Gunlink(self.id, name)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gunlink(__pyx_v_self->__pyx_base.id, __pyx_v_name); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.unlink", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_9move(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_8move[] = "(STRING current_name, STRING new_name, GroupID remote=None)\n\n        Relink an object.  current_name identifies the object.\n        new_name and (optionally) another group \"remote\" determine\n        where it should be moved.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_9move(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  char *__pyx_v_current_name;
  char *__pyx_v_new_name;
  struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_remote = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("move (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__current_name,&__pyx_n_s__new_name,&__pyx_n_s__remote,0};
    PyObject* values[3] = {0,0,0};

    /* "h5py/h5g.pyx":298
 * 
 * 
 *     def move(self, char* current_name, char* new_name, GroupID remote=None):             # <<<<<<<<<<<<<<
 *         """(STRING current_name, STRING new_name, GroupID remote=None)
 * 
 */
    values[2] = (PyObject *)((struct __pyx_obj_4h5py_3h5g_GroupID *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__current_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__new_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("move", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__remote);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "move") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_current_name = PyBytes_AsString(values[0]); if (unlikely((!__pyx_v_current_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_new_name = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_new_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_remote = ((struct __pyx_obj_4h5py_3h5g_GroupID *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("move", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.move", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_remote), __pyx_ptype_4h5py_3h5g_GroupID, 1, "remote", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_8move(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), __pyx_v_current_name, __pyx_v_new_name, __pyx_v_remote);
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_8move(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_current_name, char *__pyx_v_new_name, struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_remote) {
  hid_t __pyx_v_remote_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hid_t __pyx_t_2;
  herr_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("move", 0);

  /* "h5py/h5g.pyx":306
 *         """
 *         cdef hid_t remote_id
 *         if remote is None:             # <<<<<<<<<<<<<<
 *             remote_id = self.id
 *         else:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_remote) == Py_None);
  if (__pyx_t_1) {

    /* "h5py/h5g.pyx":307
 *         cdef hid_t remote_id
 *         if remote is None:
 *             remote_id = self.id             # <<<<<<<<<<<<<<
 *         else:
 *             remote_id = remote.id
 */
    __pyx_t_2 = __pyx_v_self->__pyx_base.id;
    __pyx_v_remote_id = __pyx_t_2;
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/h5g.pyx":309
 *             remote_id = self.id
 *         else:
 *             remote_id = remote.id             # <<<<<<<<<<<<<<
 * 
 *         H5Gmove2(self.id, current_name, remote_id, new_name)
 */
    __pyx_t_2 = __pyx_v_remote->__pyx_base.id;
    __pyx_v_remote_id = __pyx_t_2;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":311
 *             remote_id = remote.id
 * 
 *         H5Gmove2(self.id, current_name, remote_id, new_name)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_4h5py_4defs_H5Gmove2(__pyx_v_self->__pyx_base.id, __pyx_v_current_name, __pyx_v_remote_id, __pyx_v_new_name); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.move", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_11get_num_objs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_10get_num_objs[] = "() => INT number_of_objects\n\n        Get the number of objects directly attached to a given group.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_11get_num_objs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_num_objs (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_10get_num_objs(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":314
 * 
 * 
 *     def get_num_objs(self):             # <<<<<<<<<<<<<<
 *         """() => INT number_of_objects
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_10get_num_objs(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self) {
  hsize_t __pyx_v_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_num_objs", 0);

  /* "h5py/h5g.pyx":320
 *         """
 *         cdef hsize_t size
 *         H5Gget_num_objs(self.id, &size)             # <<<<<<<<<<<<<<
 *         return size
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_num_objs(__pyx_v_self->__pyx_base.id, (&__pyx_v_size)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":321
 *         cdef hsize_t size
 *         H5Gget_num_objs(self.id, &size)
 *         return size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_to_py_hsize_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 321; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_num_objs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_13get_objname_by_idx(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_12get_objname_by_idx[] = "(INT idx) => STRING\n\n        Get the name of a group member given its zero-based index.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_13get_objname_by_idx(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx) {
  hsize_t __pyx_v_idx;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_objname_by_idx (wrapper)", 0);
  assert(__pyx_arg_idx); {
    __pyx_v_idx = __Pyx_PyInt_from_py_hsize_t(__pyx_arg_idx); if (unlikely((__pyx_v_idx == (hsize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 324; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_objname_by_idx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_12get_objname_by_idx(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), ((hsize_t)__pyx_v_idx));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":324
 * 
 * 
 *     def get_objname_by_idx(self, hsize_t idx):             # <<<<<<<<<<<<<<
 *         """(INT idx) => STRING
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_12get_objname_by_idx(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, hsize_t __pyx_v_idx) {
  int __pyx_v_size;
  char *__pyx_v_buf;
  char *__pyx_v_pystring;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_objname_by_idx", 0);

  /* "h5py/h5g.pyx":331
 *         cdef int size
 *         cdef char* buf
 *         buf = NULL             # <<<<<<<<<<<<<<
 * 
 *         size = H5Gget_objname_by_idx(self.id, idx, NULL, 0)
 */
  __pyx_v_buf = NULL;

  /* "h5py/h5g.pyx":333
 *         buf = NULL
 * 
 *         size = H5Gget_objname_by_idx(self.id, idx, NULL, 0)             # <<<<<<<<<<<<<<
 * 
 *         buf = <char*>emalloc(sizeof(char)*(size+1))
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_objname_by_idx(__pyx_v_self->__pyx_base.id, __pyx_v_idx, NULL, 0); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_size = __pyx_t_1;

  /* "h5py/h5g.pyx":335
 *         size = H5Gget_objname_by_idx(self.id, idx, NULL, 0)
 * 
 *         buf = <char*>emalloc(sizeof(char)*(size+1))             # <<<<<<<<<<<<<<
 *         try:
 *             H5Gget_objname_by_idx(self.id, idx, buf, size+1)
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(char)) * (__pyx_v_size + 1))); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 335; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = ((char *)__pyx_t_2);

  /* "h5py/h5g.pyx":336
 * 
 *         buf = <char*>emalloc(sizeof(char)*(size+1))
 *         try:             # <<<<<<<<<<<<<<
 *             H5Gget_objname_by_idx(self.id, idx, buf, size+1)
 *             pystring = buf
 */
  /*try:*/ {

    /* "h5py/h5g.pyx":337
 *         buf = <char*>emalloc(sizeof(char)*(size+1))
 *         try:
 *             H5Gget_objname_by_idx(self.id, idx, buf, size+1)             # <<<<<<<<<<<<<<
 *             pystring = buf
 *             return pystring
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_objname_by_idx(__pyx_v_self->__pyx_base.id, __pyx_v_idx, __pyx_v_buf, (__pyx_v_size + 1)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 337; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5g.pyx":338
 *         try:
 *             H5Gget_objname_by_idx(self.id, idx, buf, size+1)
 *             pystring = buf             # <<<<<<<<<<<<<<
 *             return pystring
 *         finally:
 */
    __pyx_v_pystring = __pyx_v_buf;

    /* "h5py/h5g.pyx":339
 *             H5Gget_objname_by_idx(self.id, idx, buf, size+1)
 *             pystring = buf
 *             return pystring             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(buf)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyBytes_FromString(__pyx_v_pystring); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 339; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_3));
    __pyx_r = ((PyObject *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L3;
  }

  /* "h5py/h5g.pyx":341
 *             return pystring
 *         finally:
 *             efree(buf)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_buf);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_objname_by_idx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_15get_objtype_by_idx(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_14get_objtype_by_idx[] = "(INT idx) => INT object_type_code\n\n        Get the type of an object attached to a group, given its zero-based\n        index.  Possible return values are:\n\n        - LINK\n        - GROUP\n        - DATASET\n        - TYPE\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_15get_objtype_by_idx(PyObject *__pyx_v_self, PyObject *__pyx_arg_idx) {
  hsize_t __pyx_v_idx;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_objtype_by_idx (wrapper)", 0);
  assert(__pyx_arg_idx); {
    __pyx_v_idx = __Pyx_PyInt_from_py_hsize_t(__pyx_arg_idx); if (unlikely((__pyx_v_idx == (hsize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 344; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_objtype_by_idx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_14get_objtype_by_idx(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), ((hsize_t)__pyx_v_idx));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":344
 * 
 * 
 *     def get_objtype_by_idx(self, hsize_t idx):             # <<<<<<<<<<<<<<
 *         """(INT idx) => INT object_type_code
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_14get_objtype_by_idx(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, hsize_t __pyx_v_idx) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_objtype_by_idx", 0);

  /* "h5py/h5g.pyx":355
 *         - TYPE
 *         """
 *         return <int>H5Gget_objtype_by_idx(self.id, idx)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_objtype_by_idx(__pyx_v_self->__pyx_base.id, __pyx_v_idx); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 355; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_objtype_by_idx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_17get_linkval(PyObject *__pyx_v_self, PyObject *__pyx_arg_name); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_16get_linkval[] = "(STRING name) => STRING link_value\n\n        Retrieve the value (target name) of a symbolic link.\n        Limited to 2048 characters on Windows.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_17get_linkval(PyObject *__pyx_v_self, PyObject *__pyx_arg_name) {
  char *__pyx_v_name;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_linkval (wrapper)", 0);
  assert(__pyx_arg_name); {
    __pyx_v_name = PyBytes_AsString(__pyx_arg_name); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 358; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_linkval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_16get_linkval(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), ((char *)__pyx_v_name));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":358
 * 
 * 
 *     def get_linkval(self, char* name):             # <<<<<<<<<<<<<<
 *         """(STRING name) => STRING link_value
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_16get_linkval(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name) {
  char *__pyx_v_value;
  H5G_stat_t __pyx_v_statbuf;
  PyObject *__pyx_v_linklen = NULL;
  char *__pyx_v_pyvalue;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  void *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_linkval", 0);

  /* "h5py/h5g.pyx":366
 *         cdef char* value
 *         cdef H5G_stat_t statbuf
 *         value = NULL             # <<<<<<<<<<<<<<
 * 
 *         H5Gget_objinfo(self.id, name, 0, &statbuf)
 */
  __pyx_v_value = NULL;

  /* "h5py/h5g.pyx":368
 *         value = NULL
 * 
 *         H5Gget_objinfo(self.id, name, 0, &statbuf)             # <<<<<<<<<<<<<<
 * 
 *         if statbuf.type != H5G_LINK:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_objinfo(__pyx_v_self->__pyx_base.id, __pyx_v_name, 0, (&__pyx_v_statbuf)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":370
 *         H5Gget_objinfo(self.id, name, 0, &statbuf)
 * 
 *         if statbuf.type != H5G_LINK:             # <<<<<<<<<<<<<<
 *             raise ValueError('"%s" is not a symbolic link.' % name)
 * 
 */
  __pyx_t_2 = (__pyx_v_statbuf.type != H5G_LINK);
  if (__pyx_t_2) {

    /* "h5py/h5g.pyx":371
 * 
 *         if statbuf.type != H5G_LINK:
 *             raise ValueError('"%s" is not a symbolic link.' % name)             # <<<<<<<<<<<<<<
 * 
 *         IF UNAME_SYSNAME == "Windows":
 */
    __pyx_t_3 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 371; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_3));
    __pyx_t_4 = PyNumber_Remainder(((PyObject *)__pyx_kp_s_6), ((PyObject *)__pyx_t_3)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 371; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_4));
    __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 371; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_t_4));
    __Pyx_GIVEREF(((PyObject *)__pyx_t_4));
    __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 371; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 371; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5g.pyx":376
 *             linklen = 2049  # Windows statbuf.linklen seems broken
 *         ELSE:
 *             linklen = statbuf.linklen+1             # <<<<<<<<<<<<<<
 *         value = <char*>emalloc(sizeof(char)*linklen)
 *         try:
 */
  __pyx_t_4 = __Pyx_PyInt_FromSize_t((__pyx_v_statbuf.linklen + 1)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 376; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_linklen = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "h5py/h5g.pyx":377
 *         ELSE:
 *             linklen = statbuf.linklen+1
 *         value = <char*>emalloc(sizeof(char)*linklen)             # <<<<<<<<<<<<<<
 *         try:
 *             H5Gget_linkval(self.id, name, linklen, value)
 */
  __pyx_t_4 = __Pyx_PyInt_FromSize_t((sizeof(char))); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_4, __pyx_v_linklen); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyInt_AsSize_t(__pyx_t_3); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __pyx_f_4h5py_5utils_emalloc(__pyx_t_5); if (unlikely(__pyx_t_6 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_value = ((char *)__pyx_t_6);

  /* "h5py/h5g.pyx":378
 *             linklen = statbuf.linklen+1
 *         value = <char*>emalloc(sizeof(char)*linklen)
 *         try:             # <<<<<<<<<<<<<<
 *             H5Gget_linkval(self.id, name, linklen, value)
 *             value[linklen-1] = c'\0'  # in case HDF5 doesn't null terminate on Windows
 */
  /*try:*/ {

    /* "h5py/h5g.pyx":379
 *         value = <char*>emalloc(sizeof(char)*linklen)
 *         try:
 *             H5Gget_linkval(self.id, name, linklen, value)             # <<<<<<<<<<<<<<
 *             value[linklen-1] = c'\0'  # in case HDF5 doesn't null terminate on Windows
 *             pyvalue = value
 */
    __pyx_t_5 = __Pyx_PyInt_AsSize_t(__pyx_v_linklen); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_linkval(__pyx_v_self->__pyx_base.id, __pyx_v_name, __pyx_t_5, __pyx_v_value); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L5;}

    /* "h5py/h5g.pyx":380
 *         try:
 *             H5Gget_linkval(self.id, name, linklen, value)
 *             value[linklen-1] = c'\0'  # in case HDF5 doesn't null terminate on Windows             # <<<<<<<<<<<<<<
 *             pyvalue = value
 *             return pyvalue
 */
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_linklen, __pyx_int_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 380; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 380; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    (__pyx_v_value[__pyx_t_7]) = '\x00';

    /* "h5py/h5g.pyx":381
 *             H5Gget_linkval(self.id, name, linklen, value)
 *             value[linklen-1] = c'\0'  # in case HDF5 doesn't null terminate on Windows
 *             pyvalue = value             # <<<<<<<<<<<<<<
 *             return pyvalue
 *         finally:
 */
    __pyx_v_pyvalue = __pyx_v_value;

    /* "h5py/h5g.pyx":382
 *             value[linklen-1] = c'\0'  # in case HDF5 doesn't null terminate on Windows
 *             pyvalue = value
 *             return pyvalue             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(value)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyBytes_FromString(__pyx_v_pyvalue); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 382; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_3));
    __pyx_r = ((PyObject *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L4;
  }

  /* "h5py/h5g.pyx":384
 *             return pyvalue
 *         finally:
 *             efree(value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L6;
    __pyx_L4: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L6;
    __pyx_L5: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L6;
    }
    __pyx_L6:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_value);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_linkval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_linklen);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_19set_comment(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_18set_comment[] = "(STRING name, STRING comment)\n\n        Set the comment on a group member.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_19set_comment(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  char *__pyx_v_name;
  char *__pyx_v_comment;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_comment (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__name,&__pyx_n_s__comment,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__comment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_comment", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 387; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_comment") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 387; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_name = PyBytes_AsString(values[0]); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 387; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_comment = PyBytes_AsString(values[1]); if (unlikely((!__pyx_v_comment) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 387; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_comment", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 387; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.set_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_18set_comment(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), __pyx_v_name, __pyx_v_comment);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":387
 * 
 * 
 *     def set_comment(self, char* name, char* comment):             # <<<<<<<<<<<<<<
 *         """(STRING name, STRING comment)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_18set_comment(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name, char *__pyx_v_comment) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_comment", 0);

  /* "h5py/h5g.pyx":392
 *         Set the comment on a group member.
 *         """
 *         H5Gset_comment(self.id, name, comment)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gset_comment(__pyx_v_self->__pyx_base.id, __pyx_v_name, __pyx_v_comment); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 392; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.set_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_21get_comment(PyObject *__pyx_v_self, PyObject *__pyx_arg_name); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_20get_comment[] = "(STRING name) => STRING comment\n\n        Retrieve the comment for a group member.\n        ";
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_21get_comment(PyObject *__pyx_v_self, PyObject *__pyx_arg_name) {
  char *__pyx_v_name;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_comment (wrapper)", 0);
  assert(__pyx_arg_name); {
    __pyx_v_name = PyBytes_AsString(__pyx_arg_name); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 395; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_20get_comment(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), ((char *)__pyx_v_name));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":395
 * 
 * 
 *     def get_comment(self, char* name):             # <<<<<<<<<<<<<<
 *         """(STRING name) => STRING comment
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_20get_comment(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name) {
  int __pyx_v_cmnt_len;
  char *__pyx_v_cmnt;
  char *__pyx_v_py_cmnt;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_comment", 0);

  /* "h5py/h5g.pyx":402
 *         cdef int cmnt_len
 *         cdef char* cmnt
 *         cmnt = NULL             # <<<<<<<<<<<<<<
 * 
 *         cmnt_len = H5Gget_comment(self.id, name, 0, NULL)
 */
  __pyx_v_cmnt = NULL;

  /* "h5py/h5g.pyx":404
 *         cmnt = NULL
 * 
 *         cmnt_len = H5Gget_comment(self.id, name, 0, NULL)             # <<<<<<<<<<<<<<
 *         assert cmnt_len >= 0
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_comment(__pyx_v_self->__pyx_base.id, __pyx_v_name, 0, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 404; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_cmnt_len = __pyx_t_1;

  /* "h5py/h5g.pyx":405
 * 
 *         cmnt_len = H5Gget_comment(self.id, name, 0, NULL)
 *         assert cmnt_len >= 0             # <<<<<<<<<<<<<<
 * 
 *         cmnt = <char*>emalloc(sizeof(char)*(cmnt_len+1))
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!(__pyx_v_cmnt_len >= 0))) {
    PyErr_SetNone(PyExc_AssertionError);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #endif

  /* "h5py/h5g.pyx":407
 *         assert cmnt_len >= 0
 * 
 *         cmnt = <char*>emalloc(sizeof(char)*(cmnt_len+1))             # <<<<<<<<<<<<<<
 *         try:
 *             H5Gget_comment(self.id, name, cmnt_len+1, cmnt)
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(char)) * (__pyx_v_cmnt_len + 1))); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 407; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_cmnt = ((char *)__pyx_t_2);

  /* "h5py/h5g.pyx":408
 * 
 *         cmnt = <char*>emalloc(sizeof(char)*(cmnt_len+1))
 *         try:             # <<<<<<<<<<<<<<
 *             H5Gget_comment(self.id, name, cmnt_len+1, cmnt)
 *             py_cmnt = cmnt
 */
  /*try:*/ {

    /* "h5py/h5g.pyx":409
 *         cmnt = <char*>emalloc(sizeof(char)*(cmnt_len+1))
 *         try:
 *             H5Gget_comment(self.id, name, cmnt_len+1, cmnt)             # <<<<<<<<<<<<<<
 *             py_cmnt = cmnt
 *             return py_cmnt
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_comment(__pyx_v_self->__pyx_base.id, __pyx_v_name, (__pyx_v_cmnt_len + 1), __pyx_v_cmnt); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 409; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5g.pyx":410
 *         try:
 *             H5Gget_comment(self.id, name, cmnt_len+1, cmnt)
 *             py_cmnt = cmnt             # <<<<<<<<<<<<<<
 *             return py_cmnt
 *         finally:
 */
    __pyx_v_py_cmnt = __pyx_v_cmnt;

    /* "h5py/h5g.pyx":411
 *             H5Gget_comment(self.id, name, cmnt_len+1, cmnt)
 *             py_cmnt = cmnt
 *             return py_cmnt             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(cmnt)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyBytes_FromString(__pyx_v_py_cmnt); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 411; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_3));
    __pyx_r = ((PyObject *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L3;
  }

  /* "h5py/h5g.pyx":413
 *             return py_cmnt
 *         finally:
 *             efree(cmnt)             # <<<<<<<<<<<<<<
 * 
 *     # === Special methods =====================================================
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_cmnt);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5g.GroupID.get_comment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_4h5py_3h5g_7GroupID_23__contains__(PyObject *__pyx_v_self, PyObject *__pyx_arg_name); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_22__contains__[] = "(STRING name)\n\n        Determine if a group member of the given name is present\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_4h5py_3h5g_7GroupID_22__contains__;
#endif
static int __pyx_pw_4h5py_3h5g_7GroupID_23__contains__(PyObject *__pyx_v_self, PyObject *__pyx_arg_name) {
  char *__pyx_v_name;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  assert(__pyx_arg_name); {
    __pyx_v_name = PyBytes_AsString(__pyx_arg_name); if (unlikely((!__pyx_v_name) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_22__contains__(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self), ((char *)__pyx_v_name));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":418
 * 
 * 
 *     def __contains__(self, char* name):             # <<<<<<<<<<<<<<
 *         """(STRING name)
 * 
 */

static int __pyx_pf_4h5py_3h5g_7GroupID_22__contains__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self, char *__pyx_v_name) {
  __pyx_t_4h5py_7_errors_err_cookie __pyx_v_old_handler;
  __pyx_t_4h5py_7_errors_err_cookie __pyx_v_new_handler;
  herr_t __pyx_v_retval;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "h5py/h5g.pyx":427
 *         cdef herr_t retval
 * 
 *         new_handler.func = NULL             # <<<<<<<<<<<<<<
 *         new_handler.data = NULL
 * 
 */
  __pyx_v_new_handler.func = NULL;

  /* "h5py/h5g.pyx":428
 * 
 *         new_handler.func = NULL
 *         new_handler.data = NULL             # <<<<<<<<<<<<<<
 * 
 *         old_handler = set_error_handler(new_handler)
 */
  __pyx_v_new_handler.data = NULL;

  /* "h5py/h5g.pyx":430
 *         new_handler.data = NULL
 * 
 *         old_handler = set_error_handler(new_handler)             # <<<<<<<<<<<<<<
 *         retval = _hdf5.H5Gget_objinfo(self.id, name, 0, NULL)
 *         set_error_handler(old_handler)
 */
  __pyx_v_old_handler = __pyx_f_4h5py_7_errors_set_error_handler(__pyx_v_new_handler);

  /* "h5py/h5g.pyx":431
 * 
 *         old_handler = set_error_handler(new_handler)
 *         retval = _hdf5.H5Gget_objinfo(self.id, name, 0, NULL)             # <<<<<<<<<<<<<<
 *         set_error_handler(old_handler)
 * 
 */
  __pyx_t_1 = H5Gget_objinfo(__pyx_v_self->__pyx_base.id, __pyx_v_name, 0, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 431; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_retval = __pyx_t_1;

  /* "h5py/h5g.pyx":432
 *         old_handler = set_error_handler(new_handler)
 *         retval = _hdf5.H5Gget_objinfo(self.id, name, 0, NULL)
 *         set_error_handler(old_handler)             # <<<<<<<<<<<<<<
 * 
 *         return bool(retval >= 0)
 */
  __pyx_f_4h5py_7_errors_set_error_handler(__pyx_v_old_handler);

  /* "h5py/h5g.pyx":434
 *         set_error_handler(old_handler)
 * 
 *         return bool(retval >= 0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = (__pyx_v_retval >= 0);
  __pyx_r = (!(!__pyx_t_2));
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_25__iter__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_24__iter__[] = " Return an iterator over the names of group members. ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_4h5py_3h5g_7GroupID_24__iter__;
#endif
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_25__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_24__iter__(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":437
 * 
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         """ Return an iterator over the names of group members. """
 *         return GroupIter(self)
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_24__iter__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "h5py/h5g.pyx":439
 *     def __iter__(self):
 *         """ Return an iterator over the names of group members. """
 *         return GroupIter(self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 439; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __pyx_t_2 = PyObject_Call(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5g_GroupIter)), ((PyObject *)__pyx_t_1), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 439; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(((PyObject *)__pyx_t_1)); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5g.GroupID.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static Py_ssize_t __pyx_pw_4h5py_3h5g_7GroupID_27__len__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_4h5py_3h5g_7GroupID_26__len__[] = " Number of group members ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_4h5py_3h5g_7GroupID_26__len__;
#endif
static Py_ssize_t __pyx_pw_4h5py_3h5g_7GroupID_27__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_26__len__(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pyx":442
 * 
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         """ Number of group members """
 *         cdef hsize_t size
 */

static Py_ssize_t __pyx_pf_4h5py_3h5g_7GroupID_26__len__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self) {
  hsize_t __pyx_v_size;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "h5py/h5g.pyx":445
 *         """ Number of group members """
 *         cdef hsize_t size
 *         H5Gget_num_objs(self.id, &size)             # <<<<<<<<<<<<<<
 *         return size
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Gget_num_objs(__pyx_v_self->__pyx_base.id, (&__pyx_v_size)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":446
 *         cdef hsize_t size
 *         H5Gget_num_objs(self.id, &size)
 *         return size             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_size;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5g.GroupID.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_5links_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5g_7GroupID_5links_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5g_7GroupID_5links___get__(((struct __pyx_obj_4h5py_3h5g_GroupID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5g.pxd":16
 * cdef class GroupID(ObjectID):
 * 
 *     cdef readonly object links             # <<<<<<<<<<<<<<
 * 
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5g_7GroupID_5links___get__(struct __pyx_obj_4h5py_3h5g_GroupID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->links);
  __pyx_r = __pyx_v_self->links;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_4h5py_3h5g_GroupID(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4h5py_3h5g_GroupID *p;
  PyObject *o = __pyx_ptype_4h5py_8_objects_ObjectID->tp_new(t, a, k);
  if (!o) return 0;
  p = ((struct __pyx_obj_4h5py_3h5g_GroupID *)o);
  p->links = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5g_GroupID(PyObject *o) {
  struct __pyx_obj_4h5py_3h5g_GroupID *p = (struct __pyx_obj_4h5py_3h5g_GroupID *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->links);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_4h5py_8_objects_ObjectID)) __pyx_ptype_4h5py_8_objects_ObjectID->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_4h5py_3h5g_GroupID);
}

static int __pyx_tp_traverse_4h5py_3h5g_GroupID(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4h5py_3h5g_GroupID *p = (struct __pyx_obj_4h5py_3h5g_GroupID *)o;
  e = ((likely(__pyx_ptype_4h5py_8_objects_ObjectID)) ? ((__pyx_ptype_4h5py_8_objects_ObjectID->tp_traverse) ? __pyx_ptype_4h5py_8_objects_ObjectID->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4h5py_3h5g_GroupID)); if (e) return e;
  if (p->links) {
    e = (*v)(p->links, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5g_GroupID(PyObject *o) {
  struct __pyx_obj_4h5py_3h5g_GroupID *p = (struct __pyx_obj_4h5py_3h5g_GroupID *)o;
  PyObject* tmp;
  if (likely(__pyx_ptype_4h5py_8_objects_ObjectID)) { if (__pyx_ptype_4h5py_8_objects_ObjectID->tp_clear) __pyx_ptype_4h5py_8_objects_ObjectID->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4h5py_3h5g_GroupID);
  tmp = ((PyObject*)p->links);
  p->links = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4h5py_3h5g_7GroupID_links(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_7GroupID_5links_1__get__(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5g_GroupID[] = {
  {__Pyx_NAMESTR("_close"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_3_close, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_2_close)},
  {__Pyx_NAMESTR("link"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_5link, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_4link)},
  {__Pyx_NAMESTR("unlink"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_7unlink, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_6unlink)},
  {__Pyx_NAMESTR("move"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_9move, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_8move)},
  {__Pyx_NAMESTR("get_num_objs"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_11get_num_objs, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_10get_num_objs)},
  {__Pyx_NAMESTR("get_objname_by_idx"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_13get_objname_by_idx, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_12get_objname_by_idx)},
  {__Pyx_NAMESTR("get_objtype_by_idx"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_15get_objtype_by_idx, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_14get_objtype_by_idx)},
  {__Pyx_NAMESTR("get_linkval"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_17get_linkval, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_16get_linkval)},
  {__Pyx_NAMESTR("set_comment"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_19set_comment, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_18set_comment)},
  {__Pyx_NAMESTR("get_comment"), (PyCFunction)__pyx_pw_4h5py_3h5g_7GroupID_21get_comment, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5g_7GroupID_20get_comment)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5g_GroupID[] = {
  {(char *)"links", __pyx_getprop_4h5py_3h5g_7GroupID_links, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_GroupID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_GroupID = {
  __pyx_pw_4h5py_3h5g_7GroupID_27__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_4h5py_3h5g_7GroupID_23__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_GroupID = {
  __pyx_pw_4h5py_3h5g_7GroupID_27__len__, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_GroupID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5g_GroupID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5g.GroupID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5g_GroupID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5g_GroupID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_GroupID, /*tp_as_number*/
  &__pyx_tp_as_sequence_GroupID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_GroupID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_GroupID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Represents an HDF5 group identifier\n\n        Python extensions:\n\n        __contains__\n            Test for group member (\"if name in grpid\")\n\n        __iter__\n            Get an iterator over member names\n\n        __len__\n            Number of members in this group; len(grpid) = N\n\n        If HDF5 1.8.X is used, the attribute \"links\" contains a proxy object\n        providing access to the H5L family of routines.  See the docs\n        for h5py.h5l.LinkProxy for more information.\n\n        * Hashable: Yes, unless anonymous\n        * Equality: True HDF5 identity unless anonymous\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5g_GroupID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5g_GroupID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_4h5py_3h5g_7GroupID_25__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5g_GroupID, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5g_GroupID, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_4h5py_3h5g_7GroupID_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5g_GroupID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5g_GroupStat(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o = (*t->tp_alloc)(t, 0);
  if (!o) return 0;
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5g_GroupStat(PyObject *o) {
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_4h5py_3h5g_9GroupStat_fileno(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_9GroupStat_6fileno_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5g_9GroupStat_objno(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_9GroupStat_5objno_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5g_9GroupStat_nlink(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_9GroupStat_5nlink_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5g_9GroupStat_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_9GroupStat_4type_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5g_9GroupStat_mtime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_9GroupStat_5mtime_1__get__(o);
}

static PyObject *__pyx_getprop_4h5py_3h5g_9GroupStat_linklen(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5g_9GroupStat_7linklen_1__get__(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5g_GroupStat[] = {
  {__Pyx_NAMESTR("_hash"), (PyCFunction)__pyx_pw_4h5py_3h5g_9GroupStat_1_hash, METH_NOARGS, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5g_GroupStat[] = {
  {(char *)"fileno", __pyx_getprop_4h5py_3h5g_9GroupStat_fileno, 0, 0, 0},
  {(char *)"objno", __pyx_getprop_4h5py_3h5g_9GroupStat_objno, 0, 0, 0},
  {(char *)"nlink", __pyx_getprop_4h5py_3h5g_9GroupStat_nlink, 0, 0, 0},
  {(char *)"type", __pyx_getprop_4h5py_3h5g_9GroupStat_type, 0, 0, 0},
  {(char *)"mtime", __pyx_getprop_4h5py_3h5g_9GroupStat_mtime, 0, 0, 0},
  {(char *)"linklen", __pyx_getprop_4h5py_3h5g_9GroupStat_linklen, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_GroupStat = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_GroupStat = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_GroupStat = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_GroupStat = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5g_GroupStat = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5g.GroupStat"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5g_GroupStat), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5g_GroupStat, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_GroupStat, /*tp_as_number*/
  &__pyx_tp_as_sequence_GroupStat, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_GroupStat, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_GroupStat, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  __Pyx_DOCSTR("Represents the H5G_stat_t structure containing group member info.\n\n    Fields (read-only):\n\n    * fileno:   2-tuple uniquely identifying the current file\n    * objno:    2-tuple uniquely identifying this object\n    * nlink:    Number of hard links to this object\n    * mtime:    Modification time of this object\n    * linklen:  Length of the symbolic link name, or 0 if not a link.\n\n    \"Uniquely identifying\" means unique among currently open files,\n    not universally unique.\n\n    * Hashable: Yes\n    * Equality: Yes\n    "), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5g_GroupStat, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5g_GroupStat, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5g_GroupStat, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5g_GroupIter(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4h5py_3h5g_GroupIter *p;
  PyObject *o = (*t->tp_alloc)(t, 0);
  if (!o) return 0;
  p = ((struct __pyx_obj_4h5py_3h5g_GroupIter *)o);
  p->grp = ((struct __pyx_obj_4h5py_3h5g_GroupID *)Py_None); Py_INCREF(Py_None);
  p->names = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5g_GroupIter(PyObject *o) {
  struct __pyx_obj_4h5py_3h5g_GroupIter *p = (struct __pyx_obj_4h5py_3h5g_GroupIter *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->grp);
  Py_CLEAR(p->names);
  PyObject_GC_Track(o);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4h5py_3h5g_GroupIter(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4h5py_3h5g_GroupIter *p = (struct __pyx_obj_4h5py_3h5g_GroupIter *)o;
  if (p->grp) {
    e = (*v)(((PyObject*)p->grp), a); if (e) return e;
  }
  if (p->names) {
    e = (*v)(p->names, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5g_GroupIter(PyObject *o) {
  struct __pyx_obj_4h5py_3h5g_GroupIter *p = (struct __pyx_obj_4h5py_3h5g_GroupIter *)o;
  PyObject* tmp;
  tmp = ((PyObject*)p->grp);
  p->grp = ((struct __pyx_obj_4h5py_3h5g_GroupID *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->names);
  p->names = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4h5py_3h5g_GroupIter[] = {
  {__Pyx_NAMESTR("__next__"), (PyCFunction)__pyx_pw_4h5py_3h5g_9GroupIter_5__next__, METH_NOARGS|METH_COEXIST, __Pyx_DOCSTR(0)},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_GroupIter = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_GroupIter = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_GroupIter = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_GroupIter = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5g_GroupIter = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5g.GroupIter"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5g_GroupIter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5g_GroupIter, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_GroupIter, /*tp_as_number*/
  &__pyx_tp_as_sequence_GroupIter, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_GroupIter, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_GroupIter, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Iterator over the names of group members.  After this iterator is\n        exhausted, it releases its reference to the group ID.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5g_GroupIter, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5g_GroupIter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_4h5py_3h5g_9GroupIter_3__iter__, /*tp_iter*/
  __pyx_pw_4h5py_3h5g_9GroupIter_5__next__, /*tp_iternext*/
  __pyx_methods_4h5py_3h5g_GroupIter, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_4h5py_3h5g_9GroupIter_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5g_GroupIter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyObject *__pyx_tp_new_4h5py_3h5g__GroupVisitor(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4h5py_3h5g__GroupVisitor *p;
  PyObject *o = (*t->tp_alloc)(t, 0);
  if (!o) return 0;
  p = ((struct __pyx_obj_4h5py_3h5g__GroupVisitor *)o);
  p->func = Py_None; Py_INCREF(Py_None);
  p->retval = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5g__GroupVisitor(PyObject *o) {
  struct __pyx_obj_4h5py_3h5g__GroupVisitor *p = (struct __pyx_obj_4h5py_3h5g__GroupVisitor *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->func);
  Py_CLEAR(p->retval);
  PyObject_GC_Track(o);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4h5py_3h5g__GroupVisitor(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4h5py_3h5g__GroupVisitor *p = (struct __pyx_obj_4h5py_3h5g__GroupVisitor *)o;
  if (p->func) {
    e = (*v)(p->func, a); if (e) return e;
  }
  if (p->retval) {
    e = (*v)(p->retval, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5g__GroupVisitor(PyObject *o) {
  struct __pyx_obj_4h5py_3h5g__GroupVisitor *p = (struct __pyx_obj_4h5py_3h5g__GroupVisitor *)o;
  PyObject* tmp;
  tmp = ((PyObject*)p->func);
  p->func = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->retval);
  p->retval = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4h5py_3h5g__GroupVisitor[] = {
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number__GroupVisitor = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence__GroupVisitor = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__GroupVisitor = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer__GroupVisitor = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5g__GroupVisitor = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5g._GroupVisitor"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5g__GroupVisitor), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5g__GroupVisitor, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number__GroupVisitor, /*tp_as_number*/
  &__pyx_tp_as_sequence__GroupVisitor, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__GroupVisitor, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer__GroupVisitor, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5g__GroupVisitor, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5g__GroupVisitor, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5g__GroupVisitor, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_4h5py_3h5g_13_GroupVisitor_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5g__GroupVisitor, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    __Pyx_NAMESTR("h5g"),
    __Pyx_DOCSTR(__pyx_k_7), /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_b_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 0, 0},
  {&__pyx_kp_s_10, __pyx_k_10, sizeof(__pyx_k_10), 0, 0, 1, 0},
  {&__pyx_n_s_11, __pyx_k_11, sizeof(__pyx_k_11), 0, 0, 1, 1},
  {&__pyx_kp_s_2, __pyx_k_2, sizeof(__pyx_k_2), 0, 0, 1, 0},
  {&__pyx_kp_s_6, __pyx_k_6, sizeof(__pyx_k_6), 0, 0, 1, 0},
  {&__pyx_n_s__DATASET, __pyx_k__DATASET, sizeof(__pyx_k__DATASET), 0, 0, 1, 1},
  {&__pyx_n_s__GROUP, __pyx_k__GROUP, sizeof(__pyx_k__GROUP), 0, 0, 1, 1},
  {&__pyx_n_s__LINK, __pyx_k__LINK, sizeof(__pyx_k__LINK), 0, 0, 1, 1},
  {&__pyx_n_s__LINK_ERROR, __pyx_k__LINK_ERROR, sizeof(__pyx_k__LINK_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s__LINK_HARD, __pyx_k__LINK_HARD, sizeof(__pyx_k__LINK_HARD), 0, 0, 1, 1},
  {&__pyx_n_s__LINK_SOFT, __pyx_k__LINK_SOFT, sizeof(__pyx_k__LINK_SOFT), 0, 0, 1, 1},
  {&__pyx_n_s__LinkProxy, __pyx_k__LinkProxy, sizeof(__pyx_k__LinkProxy), 0, 0, 1, 1},
  {&__pyx_n_s__StopIteration, __pyx_k__StopIteration, sizeof(__pyx_k__StopIteration), 0, 0, 1, 1},
  {&__pyx_n_s__TYPE, __pyx_k__TYPE, sizeof(__pyx_k__TYPE), 0, 0, 1, 1},
  {&__pyx_n_s__UNKNOWN, __pyx_k__UNKNOWN, sizeof(__pyx_k__UNKNOWN), 0, 0, 1, 1},
  {&__pyx_n_s__ValueError, __pyx_k__ValueError, sizeof(__pyx_k__ValueError), 0, 0, 1, 1},
  {&__pyx_n_s____enter__, __pyx_k____enter__, sizeof(__pyx_k____enter__), 0, 0, 1, 1},
  {&__pyx_n_s____exit__, __pyx_k____exit__, sizeof(__pyx_k____exit__), 0, 0, 1, 1},
  {&__pyx_n_s____main__, __pyx_k____main__, sizeof(__pyx_k____main__), 0, 0, 1, 1},
  {&__pyx_n_s____test__, __pyx_k____test__, sizeof(__pyx_k____test__), 0, 0, 1, 1},
  {&__pyx_n_s___name, __pyx_k___name, sizeof(__pyx_k___name), 0, 0, 1, 1},
  {&__pyx_n_s___objects, __pyx_k___objects, sizeof(__pyx_k___objects), 0, 0, 1, 1},
  {&__pyx_n_s__append, __pyx_k__append, sizeof(__pyx_k__append), 0, 0, 1, 1},
  {&__pyx_n_s__cname, __pyx_k__cname, sizeof(__pyx_k__cname), 0, 0, 1, 1},
  {&__pyx_n_s__comment, __pyx_k__comment, sizeof(__pyx_k__comment), 0, 0, 1, 1},
  {&__pyx_n_s__create, __pyx_k__create, sizeof(__pyx_k__create), 0, 0, 1, 1},
  {&__pyx_n_s__current_name, __pyx_k__current_name, sizeof(__pyx_k__current_name), 0, 0, 1, 1},
  {&__pyx_n_s__fileno, __pyx_k__fileno, sizeof(__pyx_k__fileno), 0, 0, 1, 1},
  {&__pyx_n_s__follow_link, __pyx_k__follow_link, sizeof(__pyx_k__follow_link), 0, 0, 1, 1},
  {&__pyx_n_s__func, __pyx_k__func, sizeof(__pyx_k__func), 0, 0, 1, 1},
  {&__pyx_n_s__gcpl, __pyx_k__gcpl, sizeof(__pyx_k__gcpl), 0, 0, 1, 1},
  {&__pyx_n_s__get_num_objs, __pyx_k__get_num_objs, sizeof(__pyx_k__get_num_objs), 0, 0, 1, 1},
  {&__pyx_n_s__get_objinfo, __pyx_k__get_objinfo, sizeof(__pyx_k__get_objinfo), 0, 0, 1, 1},
  {&__pyx_n_s__gid, __pyx_k__gid, sizeof(__pyx_k__gid), 0, 0, 1, 1},
  {&__pyx_n_s__grp, __pyx_k__grp, sizeof(__pyx_k__grp), 0, 0, 1, 1},
  {&__pyx_n_s__h5l, __pyx_k__h5l, sizeof(__pyx_k__h5l), 0, 0, 1, 1},
  {&__pyx_n_s__h5py, __pyx_k__h5py, sizeof(__pyx_k__h5py), 0, 0, 1, 1},
  {&__pyx_n_s__i, __pyx_k__i, sizeof(__pyx_k__i), 0, 0, 1, 1},
  {&__pyx_n_s__id_, __pyx_k__id_, sizeof(__pyx_k__id_), 0, 0, 1, 1},
  {&__pyx_n_s__iterate, __pyx_k__iterate, sizeof(__pyx_k__iterate), 0, 0, 1, 1},
  {&__pyx_n_s__lcpl, __pyx_k__lcpl, sizeof(__pyx_k__lcpl), 0, 0, 1, 1},
  {&__pyx_n_s__link_type, __pyx_k__link_type, sizeof(__pyx_k__link_type), 0, 0, 1, 1},
  {&__pyx_n_s__linklen, __pyx_k__linklen, sizeof(__pyx_k__linklen), 0, 0, 1, 1},
  {&__pyx_n_s__loc, __pyx_k__loc, sizeof(__pyx_k__loc), 0, 0, 1, 1},
  {&__pyx_n_s__lock, __pyx_k__lock, sizeof(__pyx_k__lock), 0, 0, 1, 1},
  {&__pyx_n_s__mtime, __pyx_k__mtime, sizeof(__pyx_k__mtime), 0, 0, 1, 1},
  {&__pyx_n_s__name, __pyx_k__name, sizeof(__pyx_k__name), 0, 0, 1, 1},
  {&__pyx_n_s__new_name, __pyx_k__new_name, sizeof(__pyx_k__new_name), 0, 0, 1, 1},
  {&__pyx_n_s__nlink, __pyx_k__nlink, sizeof(__pyx_k__nlink), 0, 0, 1, 1},
  {&__pyx_n_s__obj, __pyx_k__obj, sizeof(__pyx_k__obj), 0, 0, 1, 1},
  {&__pyx_n_s__obj_name, __pyx_k__obj_name, sizeof(__pyx_k__obj_name), 0, 0, 1, 1},
  {&__pyx_n_s__objno, __pyx_k__objno, sizeof(__pyx_k__objno), 0, 0, 1, 1},
  {&__pyx_n_s__open, __pyx_k__open, sizeof(__pyx_k__open), 0, 0, 1, 1},
  {&__pyx_n_s__registry, __pyx_k__registry, sizeof(__pyx_k__registry), 0, 0, 1, 1},
  {&__pyx_n_s__remote, __pyx_k__remote, sizeof(__pyx_k__remote), 0, 0, 1, 1},
  {&__pyx_n_s__startidx, __pyx_k__startidx, sizeof(__pyx_k__startidx), 0, 0, 1, 1},
  {&__pyx_n_s__statobj, __pyx_k__statobj, sizeof(__pyx_k__statobj), 0, 0, 1, 1},
  {&__pyx_n_s__type, __pyx_k__type, sizeof(__pyx_k__type), 0, 0, 1, 1},
  {&__pyx_n_s__valid, __pyx_k__valid, sizeof(__pyx_k__valid), 0, 0, 1, 1},
  {&__pyx_n_s__vis, __pyx_k__vis, sizeof(__pyx_k__vis), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_StopIteration = __Pyx_GetName(__pyx_b, __pyx_n_s__StopIteration); if (!__pyx_builtin_StopIteration) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetName(__pyx_b, __pyx_n_s__ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "h5py/h5g.pyx":189
 *     """
 *     if startidx < 0:
 *         raise ValueError("Starting index must be non-negative")             # <<<<<<<<<<<<<<
 * 
 *     cdef int i = startidx
 */
  __pyx_k_tuple_3 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_2)); if (unlikely(!__pyx_k_tuple_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_3);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_3));

  /* "h5py/h5g.pyx":258
 *         when their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Gclose(self.id)
 *             if not self.valid:
 */
  __pyx_k_tuple_4 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_k_tuple_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_4);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_4));

  /* "h5py/h5g.pyx":123
 * 
 * 
 * def open(ObjectID loc not None, char* name):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name) => GroupID
 * 
 */
  __pyx_k_tuple_8 = PyTuple_Pack(2, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__name)); if (unlikely(!__pyx_k_tuple_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_8);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_8));
  __pyx_k_codeobj_9 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_10, __pyx_n_s__open, 123, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":130
 *     return GroupID.open(H5Gopen(loc.id, name))
 * 
 * def create(ObjectID loc not None, object name, PropID lcpl=None,             # <<<<<<<<<<<<<<
 *            PropID gcpl=None):
 *     """(ObjectID loc, STRING name or None, PropLCID lcpl=None,
 */
  __pyx_k_tuple_12 = PyTuple_Pack(6, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__name), ((PyObject *)__pyx_n_s__lcpl), ((PyObject *)__pyx_n_s__gcpl), ((PyObject *)__pyx_n_s__gid), ((PyObject *)__pyx_n_s__cname)); if (unlikely(!__pyx_k_tuple_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_12);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_12));
  __pyx_k_codeobj_13 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_10, __pyx_n_s__create, 130, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":172
 * 
 * 
 * def iterate(GroupID loc not None, object func, int startidx=0, *,             # <<<<<<<<<<<<<<
 *             char* obj_name='.'):
 *     """ (GroupID loc, CALLABLE func, UINT startidx=0, **kwds)
 */
  __pyx_k_tuple_14 = PyTuple_Pack(6, ((PyObject *)__pyx_n_s__loc), ((PyObject *)__pyx_n_s__func), ((PyObject *)__pyx_n_s__startidx), ((PyObject *)__pyx_n_s__obj_name), ((PyObject *)__pyx_n_s__i), ((PyObject *)__pyx_n_s__vis)); if (unlikely(!__pyx_k_tuple_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_14);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_14));
  __pyx_k_codeobj_15 = (PyObject*)__Pyx_PyCode_New(4, 1, 6, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_14, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_10, __pyx_n_s__iterate, 172, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5g.pyx":199
 * 
 * 
 * def get_objinfo(ObjectID obj not None, object name=b'.', int follow_link=1):             # <<<<<<<<<<<<<<
 *     """(ObjectID obj, STRING name='.', BOOL follow_link=True) => GroupStat object
 * 
 */
  __pyx_k_tuple_16 = PyTuple_Pack(5, ((PyObject *)__pyx_n_s__obj), ((PyObject *)__pyx_n_s__name), ((PyObject *)__pyx_n_s__follow_link), ((PyObject *)__pyx_n_s__statobj), ((PyObject *)__pyx_n_s___name)); if (unlikely(!__pyx_k_tuple_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_16);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_16));
  __pyx_k_codeobj_17 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_10, __pyx_n_s__get_objinfo, 199, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC inith5g(void); /*proto*/
PyMODINIT_FUNC inith5g(void)
#else
PyMODINIT_FUNC PyInit_h5g(void); /*proto*/
PyMODINIT_FUNC PyInit_h5g(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_h5g(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4(__Pyx_NAMESTR("h5g"), __pyx_methods, __Pyx_DOCSTR(__pyx_k_7), 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "h5py.h5g")) {
      if (unlikely(PyDict_SetItemString(modules, "h5py.h5g", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  __pyx_b = PyImport_AddModule(__Pyx_NAMESTR(__Pyx_BUILTIN_MODULE_NAME)); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_module_is_main_h5py__h5g) {
    if (__Pyx_SetAttrString(__pyx_m, "__name__", __pyx_n_s____main__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_ptype_4h5py_8_objects_ObjectID = __Pyx_ImportType("h5py._objects", "ObjectID", sizeof(struct __pyx_obj_4h5py_8_objects_ObjectID), 1); if (unlikely(!__pyx_ptype_4h5py_8_objects_ObjectID)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_4h5py_3h5g_GroupID.tp_base = __pyx_ptype_4h5py_8_objects_ObjectID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5g_GroupID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = __Pyx_GetAttrString((PyObject *)&__pyx_type_4h5py_3h5g_GroupID, "__contains__"); if (unlikely(!wrapper)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_4h5py_3h5g_7GroupID_22__contains__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4h5py_3h5g_7GroupID_22__contains__.doc = __pyx_doc_4h5py_3h5g_7GroupID_22__contains__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4h5py_3h5g_7GroupID_22__contains__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = __Pyx_GetAttrString((PyObject *)&__pyx_type_4h5py_3h5g_GroupID, "__iter__"); if (unlikely(!wrapper)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_4h5py_3h5g_7GroupID_24__iter__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4h5py_3h5g_7GroupID_24__iter__.doc = __pyx_doc_4h5py_3h5g_7GroupID_24__iter__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4h5py_3h5g_7GroupID_24__iter__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = __Pyx_GetAttrString((PyObject *)&__pyx_type_4h5py_3h5g_GroupID, "__len__"); if (unlikely(!wrapper)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_4h5py_3h5g_7GroupID_26__len__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_4h5py_3h5g_7GroupID_26__len__.doc = __pyx_doc_4h5py_3h5g_7GroupID_26__len__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_4h5py_3h5g_7GroupID_26__len__;
    }
  }
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "GroupID", (PyObject *)&__pyx_type_4h5py_3h5g_GroupID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5g_GroupID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5g_GroupID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5g_GroupID, __pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5g_GroupID = &__pyx_type_4h5py_3h5g_GroupID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5g_GroupStat) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "GroupStat", (PyObject *)&__pyx_type_4h5py_3h5g_GroupStat) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5g_GroupStat = &__pyx_type_4h5py_3h5g_GroupStat;
  if (PyType_Ready(&__pyx_type_4h5py_3h5g_GroupIter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "GroupIter", (PyObject *)&__pyx_type_4h5py_3h5g_GroupIter) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5g_GroupIter = &__pyx_type_4h5py_3h5g_GroupIter;
  if (PyType_Ready(&__pyx_type_4h5py_3h5g__GroupVisitor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "_GroupVisitor", (PyObject *)&__pyx_type_4h5py_3h5g__GroupVisitor) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5g__GroupVisitor = &__pyx_type_4h5py_3h5g__GroupVisitor;
  /*--- Type import code ---*/
  __pyx_ptype_4h5py_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_dtype)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_ndarray)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropID = __Pyx_ImportType("h5py.h5p", "PropID", sizeof(struct __pyx_obj_4h5py_3h5p_PropID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropClassID = __Pyx_ImportType("h5py.h5p", "PropClassID", sizeof(struct __pyx_obj_4h5py_3h5p_PropClassID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropClassID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropInstanceID = __Pyx_ImportType("h5py.h5p", "PropInstanceID", sizeof(struct __pyx_obj_4h5py_3h5p_PropInstanceID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropInstanceID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropCreateID = __Pyx_ImportType("h5py.h5p", "PropCreateID", sizeof(struct __pyx_obj_4h5py_3h5p_PropCreateID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropCreateID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropCopyID = __Pyx_ImportType("h5py.h5p", "PropCopyID", sizeof(struct __pyx_obj_4h5py_3h5p_PropCopyID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropCopyID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropOCID = __Pyx_ImportType("h5py.h5p", "PropOCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropOCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropOCID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropDCID = __Pyx_ImportType("h5py.h5p", "PropDCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropDCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropDCID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropFCID = __Pyx_ImportType("h5py.h5p", "PropFCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropFCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropFCID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropFAID = __Pyx_ImportType("h5py.h5p", "PropFAID", sizeof(struct __pyx_obj_4h5py_3h5p_PropFAID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropFAID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropDXID = __Pyx_ImportType("h5py.h5p", "PropDXID", sizeof(struct __pyx_obj_4h5py_3h5p_PropDXID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropDXID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropLCID = __Pyx_ImportType("h5py.h5p", "PropLCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropLCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropLCID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropLAID = __Pyx_ImportType("h5py.h5p", "PropLAID", sizeof(struct __pyx_obj_4h5py_3h5p_PropLAID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropLAID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 74; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_3h5p_PropGCID = __Pyx_ImportType("h5py.h5p", "PropGCID", sizeof(struct __pyx_obj_4h5py_3h5p_PropGCID), 1); if (unlikely(!__pyx_ptype_4h5py_3h5p_PropGCID)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 78; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("h5py.defs"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gopen", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gopen, "hid_t (hid_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gclose", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gclose, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Glink2", (void (**)(void))&__pyx_f_4h5py_4defs_H5Glink2, "herr_t (hid_t, char *, H5G_link_t, hid_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gunlink", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gunlink, "herr_t (hid_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gmove2", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gmove2, "herr_t (hid_t, char *, hid_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gget_num_objs", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gget_num_objs, "herr_t (hid_t, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gget_objname_by_idx", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gget_objname_by_idx, "int (hid_t, hsize_t, char *, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gget_objtype_by_idx", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gget_objtype_by_idx, "int (hid_t, hsize_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Giterate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Giterate, "herr_t (hid_t, char *, int *, H5G_iterate_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gget_objinfo", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gget_objinfo, "herr_t (hid_t, char *, int, H5G_stat_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gget_linkval", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gget_linkval, "herr_t (hid_t, char *, size_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gset_comment", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gset_comment, "herr_t (hid_t, char *, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gget_comment", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gget_comment, "int (hid_t, char *, size_t, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gcreate_anon", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gcreate_anon, "hid_t (hid_t, hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Gcreate2", (void (**)(void))&__pyx_f_4h5py_4defs_H5Gcreate2, "hid_t (hid_t, char *, hid_t, hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_ImportModule("h5py._objects"); if (!__pyx_t_2) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "pdefault", (void (**)(void))&__pyx_f_4h5py_8_objects_pdefault, "hid_t (struct __pyx_obj_4h5py_8_objects_ObjectID *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_ImportModule("h5py.utils"); if (!__pyx_t_3) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_3, "emalloc", (void (**)(void))&__pyx_f_4h5py_5utils_emalloc, "void *(size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_3, "efree", (void (**)(void))&__pyx_f_4h5py_5utils_efree, "void (void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_ImportModule("h5py._errors"); if (!__pyx_t_4) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_4, "set_error_handler", (void (**)(void))&__pyx_f_4h5py_7_errors_set_error_handler, "__pyx_t_4h5py_7_errors_err_cookie (__pyx_t_4h5py_7_errors_err_cookie)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  /*--- Execution code ---*/

  /* "h5py/h5g.pyx":21
 * from _errors cimport set_error_handler, err_cookie
 * 
 * from h5py import _objects             # <<<<<<<<<<<<<<
 * 
 * # === Public constants and data structures ====================================
 */
  __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_n_s___objects));
  PyList_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_n_s___objects));
  __Pyx_GIVEREF(((PyObject *)__pyx_n_s___objects));
  __pyx_t_6 = __Pyx_Import(((PyObject *)__pyx_n_s__h5py), ((PyObject *)__pyx_t_5), -1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
  __pyx_t_5 = PyObject_GetAttr(__pyx_t_6, __pyx_n_s___objects);
  if (__pyx_t_5 == NULL) {
    if (PyErr_ExceptionMatches(PyExc_AttributeError)) __Pyx_RaiseImportError(__pyx_n_s___objects);
    if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s___objects, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":26
 * 
 * # Enumerated object types for groups "H5G_obj_t"
 * UNKNOWN  = H5G_UNKNOWN             # <<<<<<<<<<<<<<
 * LINK     = H5G_LINK
 * GROUP    = H5G_GROUP
 */
  __pyx_t_6 = PyInt_FromLong(H5G_UNKNOWN); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__UNKNOWN, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":27
 * # Enumerated object types for groups "H5G_obj_t"
 * UNKNOWN  = H5G_UNKNOWN
 * LINK     = H5G_LINK             # <<<<<<<<<<<<<<
 * GROUP    = H5G_GROUP
 * DATASET  = H5G_DATASET
 */
  __pyx_t_6 = PyInt_FromLong(H5G_LINK); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__LINK, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 27; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":28
 * UNKNOWN  = H5G_UNKNOWN
 * LINK     = H5G_LINK
 * GROUP    = H5G_GROUP             # <<<<<<<<<<<<<<
 * DATASET  = H5G_DATASET
 * TYPE = H5G_TYPE
 */
  __pyx_t_6 = PyInt_FromLong(H5G_GROUP); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__GROUP, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":29
 * LINK     = H5G_LINK
 * GROUP    = H5G_GROUP
 * DATASET  = H5G_DATASET             # <<<<<<<<<<<<<<
 * TYPE = H5G_TYPE
 * 
 */
  __pyx_t_6 = PyInt_FromLong(H5G_DATASET); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__DATASET, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 29; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":30
 * GROUP    = H5G_GROUP
 * DATASET  = H5G_DATASET
 * TYPE = H5G_TYPE             # <<<<<<<<<<<<<<
 * 
 * # Enumerated link types "H5G_link_t"
 */
  __pyx_t_6 = PyInt_FromLong(H5G_TYPE); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__TYPE, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":33
 * 
 * # Enumerated link types "H5G_link_t"
 * LINK_ERROR = H5G_LINK_ERROR             # <<<<<<<<<<<<<<
 * LINK_HARD  = H5G_LINK_HARD
 * LINK_SOFT  = H5G_LINK_SOFT
 */
  __pyx_t_6 = PyInt_FromLong(H5G_LINK_ERROR); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__LINK_ERROR, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":34
 * # Enumerated link types "H5G_link_t"
 * LINK_ERROR = H5G_LINK_ERROR
 * LINK_HARD  = H5G_LINK_HARD             # <<<<<<<<<<<<<<
 * LINK_SOFT  = H5G_LINK_SOFT
 * 
 */
  __pyx_t_6 = PyInt_FromLong(H5G_LINK_HARD); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__LINK_HARD, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":35
 * LINK_ERROR = H5G_LINK_ERROR
 * LINK_HARD  = H5G_LINK_HARD
 * LINK_SOFT  = H5G_LINK_SOFT             # <<<<<<<<<<<<<<
 * 
 * cdef class GroupStat:
 */
  __pyx_t_6 = PyInt_FromLong(H5G_LINK_SOFT); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__LINK_SOFT, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":123
 * 
 * 
 * def open(ObjectID loc not None, char* name):             # <<<<<<<<<<<<<<
 *     """(ObjectID loc, STRING name) => GroupID
 * 
 */
  __pyx_t_6 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5g_1open, NULL, __pyx_n_s_11); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__open, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":130
 *     return GroupID.open(H5Gopen(loc.id, name))
 * 
 * def create(ObjectID loc not None, object name, PropID lcpl=None,             # <<<<<<<<<<<<<<
 *            PropID gcpl=None):
 *     """(ObjectID loc, STRING name or None, PropLCID lcpl=None,
 */
  __pyx_t_6 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5g_3create, NULL, __pyx_n_s_11); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__create, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":172
 * 
 * 
 * def iterate(GroupID loc not None, object func, int startidx=0, *,             # <<<<<<<<<<<<<<
 *             char* obj_name='.'):
 *     """ (GroupID loc, CALLABLE func, UINT startidx=0, **kwds)
 */
  __pyx_t_6 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5g_5iterate, NULL, __pyx_n_s_11); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__iterate, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":199
 * 
 * 
 * def get_objinfo(ObjectID obj not None, object name=b'.', int follow_link=1):             # <<<<<<<<<<<<<<
 *     """(ObjectID obj, STRING name='.', BOOL follow_link=True) => GroupStat object
 * 
 */
  __pyx_t_6 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5g_7get_objinfo, NULL, __pyx_n_s_11); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__get_objinfo, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "h5py/h5g.pyx":265
 * 
 *     def link(self, char* current_name, char* new_name,
 *              int link_type=H5G_LINK_HARD, GroupID remote=None):             # <<<<<<<<<<<<<<
 *         """(STRING current_name, STRING new_name, INT link_type=LINK_HARD,
 *         GroupID remote=None)
 */
  __pyx_k_5 = H5G_LINK_HARD;

  /* "h5py/h5g.pyx":1
 * # This file is part of h5py, a Python interface to the HDF5 library.             # <<<<<<<<<<<<<<
 * #
 * # http://www.h5py.org
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_6));
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s____test__, ((PyObject *)__pyx_t_6)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_6)); __pyx_t_6 = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  if (__pyx_m) {
    __Pyx_AddTraceback("init h5py.h5g", __pyx_clineno, __pyx_lineno, __pyx_filename);
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init h5py.h5g");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif /* CYTHON_REFNANNY */

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name) {
    PyObject *result;
    result = PyObject_GetAttr(dict, name);
    if (!result) {
        if (dict != __pyx_b) {
            PyErr_Clear();
            result = PyObject_GetAttr(__pyx_b, name);
        }
        if (!result) {
            PyErr_SetObject(PyExc_NameError, name);
        }
    }
    return result;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%s() got an unexpected keyword argument '%s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%s() takes %s %" CYTHON_FORMAT_SSIZE_T "d positional argument%s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (!type) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (Py_TYPE(obj) == type) return 1;
    }
    else {
        if (PyObject_TypeCheck(obj, type)) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%s' has incorrect type (expected %s, got %s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    #if PY_VERSION_HEX < 0x02050000
    if (PyClass_Check(type)) {
    #else
    if (PyType_Check(type)) {
    #endif
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        #if PY_VERSION_HEX < 0x02050000
            if (PyInstance_Check(type)) {
                type = (PyObject*) ((PyInstanceObject*)type)->in_class;
                Py_INCREF(type);
            }
            else {
                type = 0;
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception must be an old-style class or instance");
                goto raise_error;
            }
        #else
            type = (PyObject*) Py_TYPE(type);
            Py_INCREF(type);
            if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception class must be a subclass of BaseException");
                goto raise_error;
            }
        #endif
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else /* Python 3+ */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *args;
        if (!value)
            args = PyTuple_New(0);
        else if (PyTuple_Check(value)) {
            Py_INCREF(value);
            args = value;
        }
        else
            args = PyTuple_Pack(1, value);
        if (!args)
            goto bad;
        owned_instance = PyEval_CallObject(type, args);
        Py_DECREF(args);
        if (!owned_instance)
            goto bad;
        value = owned_instance;
        if (!PyExceptionInstance_Check(value)) {
            PyErr_Format(PyExc_TypeError,
                         "calling %R should have returned an instance of "
                         "BaseException, not %R",
                         type, Py_TYPE(value));
            goto bad;
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause && cause != Py_None) {
        PyObject *fixed_cause;
        if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        }
        else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        }
        else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
        goto bad;
    #endif
    Py_INCREF(local_type);
    Py_INCREF(local_value);
    Py_INCREF(local_tb);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    /* Make sure tstate is in a consistent state when we XDECREF
       these objects (DECREF may run arbitrary code). */
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import = 0;
    py_import = __Pyx_GetAttrString(__pyx_b, "__import__");
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    #if PY_VERSION_HEX >= 0x02050000
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0; /* try absolute import on failure */
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
    #else
    if (level>0) {
        PyErr_SetString(PyExc_RuntimeError, "Relative import is not supported for Python <=2.4.");
        goto bad;
    }
    module = PyObject_CallFunctionObjArgs(py_import,
        name, global_dict, empty_dict, list, NULL);
    #endif
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static CYTHON_INLINE void __Pyx_RaiseImportError(PyObject *name) {
#if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_ImportError, "cannot import name %.230s",
                 PyString_AsString(name));
#else
    PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
#endif
}

static CYTHON_INLINE hid_t __Pyx_PyInt_from_py_hid_t(PyObject* x) {
    const hid_t neg_one = (hid_t)-1, const_zero = (hid_t)0;
    const int is_unsigned = const_zero < neg_one;
    if (sizeof(hid_t) == sizeof(char)) {
        if (is_unsigned)
            return (hid_t)__Pyx_PyInt_AsUnsignedChar(x);
        else
            return (hid_t)__Pyx_PyInt_AsSignedChar(x);
    } else if (sizeof(hid_t) == sizeof(short)) {
        if (is_unsigned)
            return (hid_t)__Pyx_PyInt_AsUnsignedShort(x);
        else
            return (hid_t)__Pyx_PyInt_AsSignedShort(x);
    } else if (sizeof(hid_t) == sizeof(int)) {
        if (is_unsigned)
            return (hid_t)__Pyx_PyInt_AsUnsignedInt(x);
        else
            return (hid_t)__Pyx_PyInt_AsSignedInt(x);
    } else if (sizeof(hid_t) == sizeof(long)) {
        if (is_unsigned)
            return (hid_t)__Pyx_PyInt_AsUnsignedLong(x);
        else
            return (hid_t)__Pyx_PyInt_AsSignedLong(x);
    } else if (sizeof(hid_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return (hid_t)__Pyx_PyInt_AsUnsignedLongLong(x);
        else
            return (hid_t)__Pyx_PyInt_AsSignedLongLong(x);
    }  else {
        #if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
        #else
        hid_t val;
        PyObject *v = __Pyx_PyNumber_Int(x);
        #if PY_VERSION_HEX < 0x03000000
        if (likely(v) && !PyLong_Check(v)) {
            PyObject *tmp = v;
            v = PyNumber_Long(tmp);
            Py_DECREF(tmp);
        }
        #endif
        if (likely(v)) {
            int one = 1; int is_little = (int)*(unsigned char *)&one;
            unsigned char *bytes = (unsigned char *)&val;
            int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                          bytes, sizeof(val),
                                          is_little, !is_unsigned);
            Py_DECREF(v);
            if (likely(!ret))
                return val;
        }
        #endif
        return (hid_t)-1;
    }
}

static CYTHON_INLINE hsize_t __Pyx_PyInt_from_py_hsize_t(PyObject* x) {
    const hsize_t neg_one = (hsize_t)-1, const_zero = (hsize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if (sizeof(hsize_t) == sizeof(char)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedChar(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedChar(x);
    } else if (sizeof(hsize_t) == sizeof(short)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedShort(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedShort(x);
    } else if (sizeof(hsize_t) == sizeof(int)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedInt(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedInt(x);
    } else if (sizeof(hsize_t) == sizeof(long)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedLong(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedLong(x);
    } else if (sizeof(hsize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return (hsize_t)__Pyx_PyInt_AsUnsignedLongLong(x);
        else
            return (hsize_t)__Pyx_PyInt_AsSignedLongLong(x);
    }  else {
        #if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
        PyErr_SetString(PyExc_RuntimeError,
                        "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
        #else
        hsize_t val;
        PyObject *v = __Pyx_PyNumber_Int(x);
        #if PY_VERSION_HEX < 0x03000000
        if (likely(v) && !PyLong_Check(v)) {
            PyObject *tmp = v;
            v = PyNumber_Long(tmp);
            Py_DECREF(tmp);
        }
        #endif
        if (likely(v)) {
            int one = 1; int is_little = (int)*(unsigned char *)&one;
            unsigned char *bytes = (unsigned char *)&val;
            int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                          bytes, sizeof(val),
                                          is_little, !is_unsigned);
            Py_DECREF(v);
            if (likely(!ret))
                return val;
        }
        #endif
        return (hsize_t)-1;
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_time_t(time_t val) {
    const time_t neg_one = (time_t)-1, const_zero = (time_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(time_t) == sizeof(char))  ||
        (sizeof(time_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(time_t) == sizeof(int)) ||
               (sizeof(time_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(time_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(time_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t val) {
    const hid_t neg_one = (hid_t)-1, const_zero = (hid_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hid_t) == sizeof(char))  ||
        (sizeof(hid_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hid_t) == sizeof(int)) ||
               (sizeof(hid_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hid_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t val) {
    const hsize_t neg_one = (hsize_t)-1, const_zero = (hsize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hsize_t) == sizeof(char))  ||
        (sizeof(hsize_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hsize_t) == sizeof(int)) ||
               (sizeof(hsize_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hsize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hsize_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject* x) {
    const unsigned char neg_one = (unsigned char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned char" :
                    "value too large to convert to unsigned char");
            }
            return (unsigned char)-1;
        }
        return (unsigned char)val;
    }
    return (unsigned char)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject* x) {
    const unsigned short neg_one = (unsigned short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned short" :
                    "value too large to convert to unsigned short");
            }
            return (unsigned short)-1;
        }
        return (unsigned short)val;
    }
    return (unsigned short)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject* x) {
    const unsigned int neg_one = (unsigned int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned int" :
                    "value too large to convert to unsigned int");
            }
            return (unsigned int)-1;
        }
        return (unsigned int)val;
    }
    return (unsigned int)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject* x) {
    const char neg_one = (char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to char" :
                    "value too large to convert to char");
            }
            return (char)-1;
        }
        return (char)val;
    }
    return (char)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject* x) {
    const short neg_one = (short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to short" :
                    "value too large to convert to short");
            }
            return (short)-1;
        }
        return (short)val;
    }
    return (short)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject* x) {
    const signed char neg_one = (signed char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed char" :
                    "value too large to convert to signed char");
            }
            return (signed char)-1;
        }
        return (signed char)val;
    }
    return (signed char)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject* x) {
    const signed short neg_one = (signed short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed short" :
                    "value too large to convert to signed short");
            }
            return (signed short)-1;
        }
        return (signed short)val;
    }
    return (signed short)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject* x) {
    const signed int neg_one = (signed int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed int" :
                    "value too large to convert to signed int");
            }
            return (signed int)-1;
        }
        return (signed int)val;
    }
    return (signed int)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject* x) {
    const unsigned long neg_one = (unsigned long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned long");
            return (unsigned long)-1;
        }
        return (unsigned long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned long");
                return (unsigned long)-1;
            }
            return (unsigned long)PyLong_AsUnsignedLong(x);
        } else {
            return (unsigned long)PyLong_AsLong(x);
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned long)-1;
        val = __Pyx_PyInt_AsUnsignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject* x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned PY_LONG_LONG");
            return (unsigned PY_LONG_LONG)-1;
        }
        return (unsigned PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned PY_LONG_LONG");
                return (unsigned PY_LONG_LONG)-1;
            }
            return (unsigned PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (unsigned PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsUnsignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject* x) {
    const long neg_one = (long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to long");
            return (long)-1;
        }
        return (long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to long");
                return (long)-1;
            }
            return (long)PyLong_AsUnsignedLong(x);
        } else {
            return (long)PyLong_AsLong(x);
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long)-1;
        val = __Pyx_PyInt_AsLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject* x) {
    const PY_LONG_LONG neg_one = (PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to PY_LONG_LONG");
            return (PY_LONG_LONG)-1;
        }
        return (PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to PY_LONG_LONG");
                return (PY_LONG_LONG)-1;
            }
            return (PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject* x) {
    const signed long neg_one = (signed long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed long");
            return (signed long)-1;
        }
        return (signed long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed long");
                return (signed long)-1;
            }
            return (signed long)PyLong_AsUnsignedLong(x);
        } else {
            return (signed long)PyLong_AsLong(x);
        }
    } else {
        signed long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed long)-1;
        val = __Pyx_PyInt_AsSignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject* x) {
    const signed PY_LONG_LONG neg_one = (signed PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed PY_LONG_LONG");
            return (signed PY_LONG_LONG)-1;
        }
        return (signed PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed PY_LONG_LONG");
                return (signed PY_LONG_LONG)-1;
            }
            return (signed PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (signed PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        signed PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsSignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        #if PY_VERSION_HEX < 0x02050000
        return PyErr_Warn(NULL, message);
        #else
        return PyErr_WarnEx(NULL, message, 1);
        #endif
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%s.%s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        #if PY_VERSION_HEX < 0x02050000
        if (PyErr_Warn(NULL, warning) < 0) goto bad;
        #else
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
        #endif
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%s.%s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%s does not export expected C function %s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3 && PY_MINOR_VERSION==0)
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,            /*int argcount,*/
        0,            /*int kwonlyargcount,*/
        0,            /*int nlocals,*/
        0,            /*int stacksize,*/
        0,            /*int flags,*/
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,      /*int firstlineno,*/
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_globals = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_globals = PyModule_GetDict(__pyx_m);
    if (!py_globals) goto bad;
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        py_globals,          /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else  /* Python 3+ has unicode identifiers */
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_VERSION_HEX < 0x03000000
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_VERSION_HEX < 0x03000000
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%s__ returned non-%s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject* x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
#if PY_VERSION_HEX < 0x02050000
   if (ival <= LONG_MAX)
       return PyInt_FromLong((long)ival);
   else {
       unsigned char *bytes = (unsigned char *) &ival;
       int one = 1; int little = (int)*(unsigned char*)&one;
       return _PyLong_FromByteArray(bytes, sizeof(size_t), little, 0);
   }
#else
   return PyInt_FromSize_t(ival);
#endif
}
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject* x) {
   unsigned PY_LONG_LONG val = __Pyx_PyInt_AsUnsignedLongLong(x);
   if (unlikely(val == (unsigned PY_LONG_LONG)-1 && PyErr_Occurred())) {
       return (size_t)-1;
   } else if (unlikely(val != (unsigned PY_LONG_LONG)(size_t)val)) {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to size_t");
       return (size_t)-1;
   }
   return (size_t)val;
}


#endif /* Py_PYTHON_H */
