/* Generated by Cython 0.18 on Mon Dec  9 16:00:33 2013 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02040000
    #error Cython requires Python 2.4+.
#else
#include <stddef.h> /* For offsetof */
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if PY_VERSION_HEX < 0x02050000
  typedef int Py_ssize_t;
  #define PY_SSIZE_T_MAX INT_MAX
  #define PY_SSIZE_T_MIN INT_MIN
  #define PY_FORMAT_SIZE_T ""
  #define CYTHON_FORMAT_SSIZE_T ""
  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
  #define PyInt_AsSsize_t(o)   __Pyx_PyInt_AsInt(o)
  #define PyNumber_Index(o)    ((PyNumber_Check(o) && !PyFloat_Check(o)) ? PyNumber_Int(o) : \
                                (PyErr_Format(PyExc_TypeError, \
                                              "expected index value, got %.200s", Py_TYPE(o)->tp_name), \
                                 (PyObject*)0))
  #define __Pyx_PyIndex_Check(o) (PyNumber_Check(o) && !PyFloat_Check(o) && \
                                  !PyComplex_Check(o))
  #define PyIndex_Check __Pyx_PyIndex_Check
  #define PyErr_WarnEx(category, message, stacklevel) PyErr_Warn(category, message)
  #define __PYX_BUILD_PY_SSIZE_T "i"
#else
  #define __PYX_BUILD_PY_SSIZE_T "n"
  #define CYTHON_FORMAT_SSIZE_T "z"
  #define __Pyx_PyIndex_Check PyIndex_Check
#endif
#if PY_VERSION_HEX < 0x02060000
  #define Py_REFCNT(ob) (((PyObject*)(ob))->ob_refcnt)
  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
  #define Py_SIZE(ob)   (((PyVarObject*)(ob))->ob_size)
  #define PyVarObject_HEAD_INIT(type, size) \
          PyObject_HEAD_INIT(type) size,
  #define PyType_Modified(t)
  typedef struct {
     void *buf;
     PyObject *obj;
     Py_ssize_t len;
     Py_ssize_t itemsize;
     int readonly;
     int ndim;
     char *format;
     Py_ssize_t *shape;
     Py_ssize_t *strides;
     Py_ssize_t *suboffsets;
     void *internal;
  } Py_buffer;
  #define PyBUF_SIMPLE 0
  #define PyBUF_WRITABLE 0x0001
  #define PyBUF_FORMAT 0x0004
  #define PyBUF_ND 0x0008
  #define PyBUF_STRIDES (0x0010 | PyBUF_ND)
  #define PyBUF_C_CONTIGUOUS (0x0020 | PyBUF_STRIDES)
  #define PyBUF_F_CONTIGUOUS (0x0040 | PyBUF_STRIDES)
  #define PyBUF_ANY_CONTIGUOUS (0x0080 | PyBUF_STRIDES)
  #define PyBUF_INDIRECT (0x0100 | PyBUF_STRIDES)
  #define PyBUF_RECORDS (PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_WRITABLE)
  #define PyBUF_FULL (PyBUF_INDIRECT | PyBUF_FORMAT | PyBUF_WRITABLE)
  typedef int (*getbufferproc)(PyObject *, Py_buffer *, int);
  typedef void (*releasebufferproc)(PyObject *, Py_buffer *);
#endif
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
  #define PyUnicode_FromString(s) PyUnicode_Decode(s, strlen(s), "UTF-8", "strict")
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#if (PY_VERSION_HEX < 0x02060000) || (PY_MAJOR_VERSION >= 3)
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((k=k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PyBytesObject                PyStringObject
  #define PyBytes_Type                 PyString_Type
  #define PyBytes_Check                PyString_Check
  #define PyBytes_CheckExact           PyString_CheckExact
  #define PyBytes_FromString           PyString_FromString
  #define PyBytes_FromStringAndSize    PyString_FromStringAndSize
  #define PyBytes_FromFormat           PyString_FromFormat
  #define PyBytes_DecodeEscape         PyString_DecodeEscape
  #define PyBytes_AsString             PyString_AsString
  #define PyBytes_AsStringAndSize      PyString_AsStringAndSize
  #define PyBytes_Size                 PyString_Size
  #define PyBytes_AS_STRING            PyString_AS_STRING
  #define PyBytes_GET_SIZE             PyString_GET_SIZE
  #define PyBytes_Repr                 PyString_Repr
  #define PyBytes_Concat               PyString_Concat
  #define PyBytes_ConcatAndDel         PyString_ConcatAndDel
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PySet_Check(obj)             PyObject_TypeCheck(obj, &PySet_Type)
  #define PyFrozenSet_Check(obj)       PyObject_TypeCheck(obj, &PyFrozenSet_Type)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_VERSION_HEX < 0x03020000
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if (PY_MAJOR_VERSION < 3) || (PY_VERSION_HEX >= 0x03010300)
  #define __Pyx_PySequence_GetSlice(obj, a, b) PySequence_GetSlice(obj, a, b)
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) PySequence_SetSlice(obj, a, b, value)
  #define __Pyx_PySequence_DelSlice(obj, a, b) PySequence_DelSlice(obj, a, b)
#else
  #define __Pyx_PySequence_GetSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), (PyObject*)0) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_GetSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object is unsliceable", (obj)->ob_type->tp_name), (PyObject*)0)))
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_SetSlice(obj, a, b, value)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice assignment", (obj)->ob_type->tp_name), -1)))
  #define __Pyx_PySequence_DelSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_DelSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice deletion", (obj)->ob_type->tp_name), -1)))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),((char *)(n)))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),((char *)(n)),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),((char *)(n)))
#else
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),(n))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),(n),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),(n))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_NAMESTR(n) ((char *)(n))
  #define __Pyx_DOCSTR(n)  ((char *)(n))
#else
  #define __Pyx_NAMESTR(n) (n)
  #define __Pyx_DOCSTR(n)  (n)
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__h5py__h5s
#define __PYX_HAVE_API__h5py__h5s
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "unistd.h"
#include "stdint.h"
#include "api_compat.h"
#include "lzf_filter.h"
#include "hdf5.h"
#include "H5Cpublic.h"
#include "hdf5_hl.h"
#include "numpy/arrayobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const long n; const char* encoding; const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry; /*proto*/

#define __Pyx_PyBytes_FromUString(s) PyBytes_FromString((char*)s)
#define __Pyx_PyBytes_AsUString(s)   ((unsigned char*) PyBytes_AsString(s))
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject*);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))


#ifdef __GNUC__
  /* Test for GCC > 2.95 */
  #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95))
    #define likely(x)   __builtin_expect(!!(x), 1)
    #define unlikely(x) __builtin_expect(!!(x), 0)
  #else /* __GNUC__ > 2 ... */
    #define likely(x)   (x)
    #define unlikely(x) (x)
  #endif /* __GNUC__ > 2 ... */
#else /* __GNUC__ */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
    
static PyObject *__pyx_m;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "h5s.pyx",
  "numpy.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_4h5py_8_objects_ObjectID;
struct __pyx_obj_4h5py_3h5s_SpaceID;
struct __pyx_opt_args_4h5py_5utils_check_numpy_read;
struct __pyx_opt_args_4h5py_5utils_check_numpy_write;

/* "utils.pxd":17
 * cdef void efree(void* ptr)
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1
 * 
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_read {
  int __pyx_n;
  hid_t space_id;
};

/* "utils.pxd":18
 * 
 * cpdef int check_numpy_read(ndarray arr, hid_t space_id=*) except -1
 * cpdef int check_numpy_write(ndarray arr, hid_t space_id=*) except -1             # <<<<<<<<<<<<<<
 * 
 * cdef int convert_tuple(object tuple, hsize_t *dims, hsize_t rank) except -1
 */
struct __pyx_opt_args_4h5py_5utils_check_numpy_write {
  int __pyx_n;
  hid_t space_id;
};

/* "_objects.pxd":12
 * from defs cimport *
 * 
 * cdef class ObjectID:             # <<<<<<<<<<<<<<
 * 
 *     cdef object __weakref__
 */
struct __pyx_obj_4h5py_8_objects_ObjectID {
  PyObject_HEAD
  PyObject *__weakref__;
  hid_t id;
  int locked;
  PyObject *_hash;
};


/* "h5py/h5s.pxd":14
 * from _objects cimport class ObjectID
 * 
 * cdef class SpaceID(ObjectID):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_4h5py_3h5s_SpaceID {
  struct __pyx_obj_4h5py_8_objects_ObjectID __pyx_base;
};

#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname); /*proto*/
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif /* CYTHON_REFNANNY */
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name); /*proto*/

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type); /*proto*/

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name); /*proto*/

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name); /*proto*/

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found); /*proto*/

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb); /*proto*/
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

#define __Pyx_DelItemInt(o, i, size, to_py_func) (((size) <= sizeof(Py_ssize_t)) ? \
                                                    __Pyx_DelItemInt_Fast(o, i) : \
                                                    __Pyx_DelItem_Generic(o, to_py_func(i)))
static CYTHON_INLINE int __Pyx_DelItem_Generic(PyObject *o, PyObject *j) {
    int r;
    if (!j) return -1;
    r = PyObject_DelItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i) {
#if CYTHON_COMPILING_IN_PYPY
    if (PySequence_Check(o)) {
        return PySequence_DelItem(o, i);
    }
#else
    PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
    if (likely(m && m->sq_ass_item)) {
        if (unlikely(i < 0) && likely(m->sq_length)) {
            Py_ssize_t l = m->sq_length(o);
            if (unlikely(l < 0)) return -1;
            i += l;
        }
        return m->sq_ass_item(o, i, (PyObject *)NULL);
    }
#endif
    return __Pyx_DelItem_Generic(o, PyInt_FromSsize_t(i));
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause); /*proto*/

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact); /*proto*/

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb); /*proto*/
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb); /*proto*/

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level); /*proto*/

static CYTHON_INLINE void __Pyx_RaiseImportError(PyObject *name);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t);

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hssize_t(hssize_t);

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject *);

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject *);

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject *);

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject *);

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject *);

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject *);

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject *);

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject *);

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject *);

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject *);

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject *);

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name); /*proto*/

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);  /*proto*/

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig); /*proto*/

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename); /*proto*/

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t); /*proto*/


/* Module declarations from 'h5py.api_types_ext' */

/* Module declarations from 'h5py.api_types_hdf5' */

/* Module declarations from 'h5py.defs' */
static hid_t (*__pyx_f_4h5py_4defs_H5Screate)(H5S_class_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Scopy)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sclose)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Screate_simple)(int, hsize_t *, hsize_t *); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Sis_simple)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Soffset_simple)(hid_t, hssize_t *); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims)(hid_t); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Sget_simple_extent_dims)(hid_t, hsize_t *, hsize_t *); /*proto*/
static hssize_t (*__pyx_f_4h5py_4defs_H5Sget_simple_extent_npoints)(hid_t); /*proto*/
static H5S_class_t (*__pyx_f_4h5py_4defs_H5Sget_simple_extent_type)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sextent_copy)(hid_t, hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sset_extent_simple)(hid_t, int, hsize_t *, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sset_extent_none)(hid_t); /*proto*/
static H5S_sel_type (*__pyx_f_4h5py_4defs_H5Sget_select_type)(hid_t); /*proto*/
static hssize_t (*__pyx_f_4h5py_4defs_H5Sget_select_npoints)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sget_select_bounds)(hid_t, hsize_t *, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sselect_all)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sselect_none)(hid_t); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Sselect_valid)(hid_t); /*proto*/
static hssize_t (*__pyx_f_4h5py_4defs_H5Sget_select_elem_npoints)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sget_select_elem_pointlist)(hid_t, hsize_t, hsize_t, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sselect_elements)(hid_t, H5S_seloper_t, size_t, hsize_t **); /*proto*/
static hssize_t (*__pyx_f_4h5py_4defs_H5Sget_select_hyper_nblocks)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sget_select_hyper_blocklist)(hid_t, hsize_t, hsize_t, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sselect_hyperslab)(hid_t, H5S_seloper_t, hsize_t *, hsize_t *, hsize_t *, hsize_t *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sencode)(hid_t, void *, size_t *); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Sdecode)(void *); /*proto*/

/* Module declarations from 'h5py._objects' */
static PyTypeObject *__pyx_ptype_4h5py_8_objects_ObjectID = 0;

/* Module declarations from 'numpy' */

/* Module declarations from 'h5py.numpy' */
static PyTypeObject *__pyx_ptype_4h5py_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_4h5py_5numpy_ndarray = 0;

/* Module declarations from 'h5py.utils' */
static void *(*__pyx_f_4h5py_5utils_emalloc)(size_t); /*proto*/
static void (*__pyx_f_4h5py_5utils_efree)(void *); /*proto*/
static int (*__pyx_f_4h5py_5utils_convert_tuple)(PyObject *, hsize_t *, hsize_t); /*proto*/
static PyObject *(*__pyx_f_4h5py_5utils_convert_dims)(hsize_t *, hsize_t); /*proto*/
static int (*__pyx_f_4h5py_5utils_require_tuple)(PyObject *, int, int, char *); /*proto*/
static PyObject *(*__pyx_f_4h5py_5utils_create_numpy_hsize)(int, hsize_t *); /*proto*/
static PyObject *(*__pyx_f_4h5py_5utils_create_hsize_array)(PyObject *); /*proto*/

/* Module declarations from 'h5py.h5s' */
static PyTypeObject *__pyx_ptype_4h5py_3h5s_SpaceID = 0;
static PyObject *__pyx_f_4h5py_3h5s_lockid(hid_t); /*proto*/
#define __Pyx_MODULE_NAME "h5py.h5s"
int __pyx_module_is_main_h5py__h5s = 0;

/* Implementation of 'h5py.h5s' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_pf_4h5py_3h5s_create(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_class_code); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_2create_simple(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dims_tpl, PyObject *__pyx_v_max_dims_tpl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_4decode(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_buf); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_5shape___get__(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID__close(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_2copy(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_4encode(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_6__reduce__(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_8__setstate__(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_state); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_10is_simple(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_12offset_simple(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_14get_simple_extent_ndims(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_16get_simple_extent_dims(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, int __pyx_v_maxdims); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_18get_simple_extent_npoints(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_20get_simple_extent_type(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_22extent_copy(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_source); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_24set_extent_simple(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_dims_tpl, PyObject *__pyx_v_max_dims_tpl); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_26set_extent_none(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_28get_select_type(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_30get_select_npoints(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_32get_select_bounds(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_34select_all(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_36select_none(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_38select_valid(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_40get_select_elem_npoints(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_42get_select_elem_pointlist(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_44select_elements(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_coords, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_46get_select_hyper_nblocks(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_48get_select_hyper_blocklist(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_50select_hyperslab(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_count, PyObject *__pyx_v_stride, PyObject *__pyx_v_block, int __pyx_v_op); /* proto */
static char __pyx_k_1[] = "get_simple_extent_dims";
static char __pyx_k_4[] = "%d is not a simple dataspace";
static char __pyx_k_6[] = "Coordinate array must have shape (<npoints>, %d)";
static char __pyx_k_7[] = "get_simple_extent_ndims";
static char __pyx_k_9[] = " Numpy-style shape tuple representing dimensions.  () == scalar.\n        ";
static char __pyx_k_10[] = "\n    Low-level interface to the \"H5S\" family of data-space functions.\n";
static char __pyx_k_13[] = "/home/computer/h5py/h5py/h5s.pyx";
static char __pyx_k_14[] = "h5py.h5s";
static char __pyx_k_19[] = "SpaceID.select_elements (line 440)";
static char __pyx_k_20[] = "(SEQUENCE coords, INT op=SELECT_SET)\n\n        Select elements by specifying coordinates points.  The argument\n        \"coords\" may be an ndarray or any nested sequence which can be\n        converted to an array of uints with the shape::\n\n            (<npoints>, <space rank>)\n\n        Examples::\n\n            >>> obj.shape\n            (10, 10)\n            >>> obj.select_elements([(1,2), (3,4), (5,9)])\n\n        A zero-length selection (i.e. shape ``(0, <rank>)``) is not allowed\n        by the HDF5 library.\n        ";
static char __pyx_k__op[] = "op";
static char __pyx_k__ALL[] = "ALL";
static char __pyx_k__buf[] = "buf";
static char __pyx_k__buf_[] = "buf_";
static char __pyx_k__dims[] = "dims";
static char __pyx_k__h5py[] = "h5py";
static char __pyx_k__lock[] = "lock";
static char __pyx_k__open[] = "open";
static char __pyx_k__rank[] = "rank";
static char __pyx_k___NULL[] = "_NULL";
static char __pyx_k__block[] = "block";
static char __pyx_k__count[] = "count";
static char __pyx_k__start[] = "start";
static char __pyx_k__valid[] = "valid";
static char __pyx_k__SCALAR[] = "SCALAR";
static char __pyx_k__SIMPLE[] = "SIMPLE";
static char __pyx_k__coords[] = "coords";
static char __pyx_k__create[] = "create";
static char __pyx_k__decode[] = "decode";
static char __pyx_k__encode[] = "encode";
static char __pyx_k__offset[] = "offset";
static char __pyx_k__stride[] = "stride";
static char __pyx_k__SEL_ALL[] = "SEL_ALL";
static char __pyx_k__maxdims[] = "maxdims";
static char __pyx_k__NO_CLASS[] = "NO_CLASS";
static char __pyx_k__SEL_NONE[] = "SEL_NONE";
static char __pyx_k____exit__[] = "__exit__";
static char __pyx_k____main__[] = "__main__";
static char __pyx_k____test__[] = "__test__";
static char __pyx_k___objects[] = "_objects";
static char __pyx_k__dims_tpl[] = "dims_tpl";
static char __pyx_k__max_dims[] = "max_dims";
static char __pyx_k__registry[] = "registry";
static char __pyx_k__SELECT_OR[] = "SELECT_OR";
static char __pyx_k__SEL_ERROR[] = "SEL_ERROR";
static char __pyx_k__UNLIMITED[] = "UNLIMITED";
static char __pyx_k____enter__[] = "__enter__";
static char __pyx_k__SELECT_AND[] = "SELECT_AND";
static char __pyx_k__SELECT_SET[] = "SELECT_SET";
static char __pyx_k__SELECT_XOR[] = "SELECT_XOR";
static char __pyx_k__SEL_POINTS[] = "SEL_POINTS";
static char __pyx_k__ValueError[] = "ValueError";
static char __pyx_k__class_code[] = "class_code";
static char __pyx_k__SELECT_NOOP[] = "SELECT_NOOP";
static char __pyx_k__SELECT_NOTA[] = "SELECT_NOTA";
static char __pyx_k__SELECT_NOTB[] = "SELECT_NOTB";
static char __pyx_k__max_dims_tpl[] = "max_dims_tpl";
static char __pyx_k__SELECT_APPEND[] = "SELECT_APPEND";
static char __pyx_k__create_simple[] = "create_simple";
static char __pyx_k__SELECT_INVALID[] = "SELECT_INVALID";
static char __pyx_k__SELECT_PREPEND[] = "SELECT_PREPEND";
static char __pyx_k__SEL_HYPERSLABS[] = "SEL_HYPERSLABS";
static PyObject *__pyx_n_s_1;
static PyObject *__pyx_kp_s_13;
static PyObject *__pyx_n_s_14;
static PyObject *__pyx_kp_u_19;
static PyObject *__pyx_kp_u_20;
static PyObject *__pyx_kp_s_4;
static PyObject *__pyx_kp_s_6;
static PyObject *__pyx_n_s_7;
static PyObject *__pyx_n_s__ALL;
static PyObject *__pyx_n_s__NO_CLASS;
static PyObject *__pyx_n_s__SCALAR;
static PyObject *__pyx_n_s__SELECT_AND;
static PyObject *__pyx_n_s__SELECT_APPEND;
static PyObject *__pyx_n_s__SELECT_INVALID;
static PyObject *__pyx_n_s__SELECT_NOOP;
static PyObject *__pyx_n_s__SELECT_NOTA;
static PyObject *__pyx_n_s__SELECT_NOTB;
static PyObject *__pyx_n_s__SELECT_OR;
static PyObject *__pyx_n_s__SELECT_PREPEND;
static PyObject *__pyx_n_s__SELECT_SET;
static PyObject *__pyx_n_s__SELECT_XOR;
static PyObject *__pyx_n_s__SEL_ALL;
static PyObject *__pyx_n_s__SEL_ERROR;
static PyObject *__pyx_n_s__SEL_HYPERSLABS;
static PyObject *__pyx_n_s__SEL_NONE;
static PyObject *__pyx_n_s__SEL_POINTS;
static PyObject *__pyx_n_s__SIMPLE;
static PyObject *__pyx_n_s__UNLIMITED;
static PyObject *__pyx_n_s__ValueError;
static PyObject *__pyx_n_s___NULL;
static PyObject *__pyx_n_s____enter__;
static PyObject *__pyx_n_s____exit__;
static PyObject *__pyx_n_s____main__;
static PyObject *__pyx_n_s____test__;
static PyObject *__pyx_n_s___objects;
static PyObject *__pyx_n_s__block;
static PyObject *__pyx_n_s__buf;
static PyObject *__pyx_n_s__buf_;
static PyObject *__pyx_n_s__class_code;
static PyObject *__pyx_n_s__coords;
static PyObject *__pyx_n_s__count;
static PyObject *__pyx_n_s__create;
static PyObject *__pyx_n_s__create_simple;
static PyObject *__pyx_n_s__decode;
static PyObject *__pyx_n_s__dims;
static PyObject *__pyx_n_s__dims_tpl;
static PyObject *__pyx_n_s__encode;
static PyObject *__pyx_n_s__h5py;
static PyObject *__pyx_n_s__lock;
static PyObject *__pyx_n_s__max_dims;
static PyObject *__pyx_n_s__max_dims_tpl;
static PyObject *__pyx_n_s__maxdims;
static PyObject *__pyx_n_s__offset;
static PyObject *__pyx_n_s__op;
static PyObject *__pyx_n_s__open;
static PyObject *__pyx_n_s__rank;
static PyObject *__pyx_n_s__registry;
static PyObject *__pyx_n_s__start;
static PyObject *__pyx_n_s__stride;
static PyObject *__pyx_n_s__valid;
static PyObject *__pyx_int_neg_1;
static int __pyx_k_5;
static int __pyx_k_8;
static PyObject *__pyx_k_tuple_2;
static PyObject *__pyx_k_tuple_3;
static PyObject *__pyx_k_tuple_11;
static PyObject *__pyx_k_tuple_15;
static PyObject *__pyx_k_tuple_17;
static PyObject *__pyx_k_codeobj_12;
static PyObject *__pyx_k_codeobj_16;
static PyObject *__pyx_k_codeobj_18;

/* "h5py/h5s.pyx":22
 * 
 * 
 * cdef object lockid(hid_t id_):             # <<<<<<<<<<<<<<
 *     cdef SpaceID space
 *     space = SpaceID.open(id_)
 */

static PyObject *__pyx_f_4h5py_3h5s_lockid(hid_t __pyx_v_id_) {
  struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_space = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lockid", 0);

  /* "h5py/h5s.pyx":24
 * cdef object lockid(hid_t id_):
 *     cdef SpaceID space
 *     space = SpaceID.open(id_)             # <<<<<<<<<<<<<<
 *     space.locked = 1
 *     return space
 */
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5s_SpaceID)), __pyx_n_s__open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_to_py_hid_t(__pyx_v_id_); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_4h5py_3h5s_SpaceID))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_space = ((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "h5py/h5s.pyx":25
 *     cdef SpaceID space
 *     space = SpaceID.open(id_)
 *     space.locked = 1             # <<<<<<<<<<<<<<
 *     return space
 * 
 */
  __pyx_v_space->__pyx_base.locked = 1;

  /* "h5py/h5s.pyx":26
 *     space = SpaceID.open(id_)
 *     space.locked = 1
 *     return space             # <<<<<<<<<<<<<<
 * 
 * # === Public constants and data structures ====================================
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_space));
  __pyx_r = ((PyObject *)__pyx_v_space);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5s.lockid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_space);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_1create(PyObject *__pyx_self, PyObject *__pyx_arg_class_code); /*proto*/
static char __pyx_doc_4h5py_3h5s_create[] = "(INT class_code) => SpaceID\n\n    Create a new HDF5 dataspace object, of the given class.\n    Legal values are SCALAR and SIMPLE.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5s_1create = {__Pyx_NAMESTR("create"), (PyCFunction)__pyx_pw_4h5py_3h5s_1create, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_create)};
static PyObject *__pyx_pw_4h5py_3h5s_1create(PyObject *__pyx_self, PyObject *__pyx_arg_class_code) {
  int __pyx_v_class_code;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create (wrapper)", 0);
  assert(__pyx_arg_class_code); {
    __pyx_v_class_code = __Pyx_PyInt_AsInt(__pyx_arg_class_code); if (unlikely((__pyx_v_class_code == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_create(__pyx_self, ((int)__pyx_v_class_code));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":62
 * 
 * 
 * def create(int class_code):             # <<<<<<<<<<<<<<
 *     """(INT class_code) => SpaceID
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_create(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_class_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create", 0);

  /* "h5py/h5s.pyx":68
 *     Legal values are SCALAR and SIMPLE.
 *     """
 *     return SpaceID.open(H5Screate(<H5S_class_t>class_code))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5s_SpaceID)), __pyx_n_s__open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Screate(((H5S_class_t)__pyx_v_class_code)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyInt_to_py_hid_t(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_4), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5s.create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_3create_simple(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5s_2create_simple[] = "(TUPLE dims_tpl, TUPLE max_dims_tpl) => SpaceID\n\n    Create a simple (slab) dataspace from a tuple of dimensions.\n    Every element of dims_tpl must be a positive integer.\n\n    You can optionally specify the maximum dataspace size. The\n    special value UNLIMITED, as an element of max_dims, indicates\n    an unlimited dimension.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5s_3create_simple = {__Pyx_NAMESTR("create_simple"), (PyCFunction)__pyx_pw_4h5py_3h5s_3create_simple, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_2create_simple)};
static PyObject *__pyx_pw_4h5py_3h5s_3create_simple(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_dims_tpl = 0;
  PyObject *__pyx_v_max_dims_tpl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_simple (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__dims_tpl,&__pyx_n_s__max_dims_tpl,0};
    PyObject* values[2] = {0,0};

    /* "h5py/h5s.pyx":71
 * 
 * 
 * def create_simple(object dims_tpl, object max_dims_tpl=None):             # <<<<<<<<<<<<<<
 *     """(TUPLE dims_tpl, TUPLE max_dims_tpl) => SpaceID
 * 
 */
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__dims_tpl)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__max_dims_tpl);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_simple") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_dims_tpl = values[0];
    __pyx_v_max_dims_tpl = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_simple", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.create_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_2create_simple(__pyx_self, __pyx_v_dims_tpl, __pyx_v_max_dims_tpl);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5s_2create_simple(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dims_tpl, PyObject *__pyx_v_max_dims_tpl) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_dims;
  hsize_t *__pyx_v_max_dims;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  void *__pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  hid_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_simple", 0);

  /* "h5py/h5s.pyx":82
 *     """
 *     cdef int rank
 *     cdef hsize_t* dims = NULL             # <<<<<<<<<<<<<<
 *     cdef hsize_t* max_dims = NULL
 * 
 */
  __pyx_v_dims = NULL;

  /* "h5py/h5s.pyx":83
 *     cdef int rank
 *     cdef hsize_t* dims = NULL
 *     cdef hsize_t* max_dims = NULL             # <<<<<<<<<<<<<<
 * 
 *     require_tuple(dims_tpl, 0, -1, "dims_tpl")
 */
  __pyx_v_max_dims = NULL;

  /* "h5py/h5s.pyx":85
 *     cdef hsize_t* max_dims = NULL
 * 
 *     require_tuple(dims_tpl, 0, -1, "dims_tpl")             # <<<<<<<<<<<<<<
 *     rank = len(dims_tpl)
 *     require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_dims_tpl, 0, -1, __pyx_k__dims_tpl); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":86
 * 
 *     require_tuple(dims_tpl, 0, -1, "dims_tpl")
 *     rank = len(dims_tpl)             # <<<<<<<<<<<<<<
 *     require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")
 * 
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_dims_tpl); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 86; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_2;

  /* "h5py/h5s.pyx":87
 *     require_tuple(dims_tpl, 0, -1, "dims_tpl")
 *     rank = len(dims_tpl)
 *     require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_max_dims_tpl, 1, __pyx_v_rank, __pyx_k__max_dims_tpl); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":89
 *     require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         convert_tuple(dims_tpl, dims, rank)
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":90
 * 
 *     try:
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *         convert_tuple(dims_tpl, dims, rank)
 * 
 */
    __pyx_t_3 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_3 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_dims = ((hsize_t *)__pyx_t_3);

    /* "h5py/h5s.pyx":91
 *     try:
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         convert_tuple(dims_tpl, dims, rank)             # <<<<<<<<<<<<<<
 * 
 *         if max_dims_tpl is not None:
 */
    __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_dims_tpl, __pyx_v_dims, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 91; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5s.pyx":93
 *         convert_tuple(dims_tpl, dims, rank)
 * 
 *         if max_dims_tpl is not None:             # <<<<<<<<<<<<<<
 *             max_dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(max_dims_tpl, max_dims, rank)
 */
    __pyx_t_4 = (__pyx_v_max_dims_tpl != Py_None);
    if (__pyx_t_4) {

      /* "h5py/h5s.pyx":94
 * 
 *         if max_dims_tpl is not None:
 *             max_dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *             convert_tuple(max_dims_tpl, max_dims, rank)
 * 
 */
      __pyx_t_3 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_3 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 94; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_max_dims = ((hsize_t *)__pyx_t_3);

      /* "h5py/h5s.pyx":95
 *         if max_dims_tpl is not None:
 *             max_dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(max_dims_tpl, max_dims, rank)             # <<<<<<<<<<<<<<
 * 
 *         return SpaceID.open(H5Screate_simple(rank, dims, max_dims))
 */
      __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_max_dims_tpl, __pyx_v_max_dims, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 95; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L6;
    }
    __pyx_L6:;

    /* "h5py/h5s.pyx":97
 *             convert_tuple(max_dims_tpl, max_dims, rank)
 * 
 *         return SpaceID.open(H5Screate_simple(rank, dims, max_dims))             # <<<<<<<<<<<<<<
 * 
 *     finally:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5s_SpaceID)), __pyx_n_s__open); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __pyx_f_4h5py_4defs_H5Screate_simple(__pyx_v_rank, __pyx_v_dims, __pyx_v_max_dims); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_t_7 = __Pyx_PyInt_to_py_hid_t(__pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = PyObject_Call(__pyx_t_5, ((PyObject *)__pyx_t_8), NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(((PyObject *)__pyx_t_8)); __pyx_t_8 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L3;
  }

  /* "h5py/h5s.pyx":100
 * 
 *     finally:
 *         efree(dims)             # <<<<<<<<<<<<<<
 *         efree(max_dims)
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_dims);

    /* "h5py/h5s.pyx":101
 *     finally:
 *         efree(dims)
 *         efree(max_dims)             # <<<<<<<<<<<<<<
 * 
 * def decode(buf):
 */
    __pyx_f_4h5py_5utils_efree(__pyx_v_max_dims);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("h5py.h5s.create_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_5decode(PyObject *__pyx_self, PyObject *__pyx_v_buf); /*proto*/
static char __pyx_doc_4h5py_3h5s_4decode[] = "(STRING buf) => SpaceID\n\n    Unserialize a dataspace.  Bear in mind you can also use the native\n    Python pickling machinery to do this.\n    ";
static PyMethodDef __pyx_mdef_4h5py_3h5s_5decode = {__Pyx_NAMESTR("decode"), (PyCFunction)__pyx_pw_4h5py_3h5s_5decode, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_4decode)};
static PyObject *__pyx_pw_4h5py_3h5s_5decode(PyObject *__pyx_self, PyObject *__pyx_v_buf) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decode (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_4decode(__pyx_self, ((PyObject *)__pyx_v_buf));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":103
 *         efree(max_dims)
 * 
 * def decode(buf):             # <<<<<<<<<<<<<<
 *     """(STRING buf) => SpaceID
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_4decode(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_buf) {
  char *__pyx_v_buf_;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  hid_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("decode", 0);

  /* "h5py/h5s.pyx":109
 *     Python pickling machinery to do this.
 *     """
 *     cdef char* buf_ = buf             # <<<<<<<<<<<<<<
 *     return SpaceID.open(H5Sdecode(buf_))
 * 
 */
  __pyx_t_1 = PyBytes_AsString(__pyx_v_buf); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf_ = __pyx_t_1;

  /* "h5py/h5s.pyx":110
 *     """
 *     cdef char* buf_ = buf
 *     return SpaceID.open(H5Sdecode(buf_))             # <<<<<<<<<<<<<<
 * 
 * # === H5S class API ===========================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5s_SpaceID)), __pyx_n_s__open); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4h5py_4defs_H5Sdecode(__pyx_v_buf_); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyInt_to_py_hid_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_t_5), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_5)); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("h5py.h5s.decode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_5shape_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_5shape_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_5shape___get__(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":133
 *         """ Numpy-style shape tuple representing dimensions.  () == scalar.
 *         """
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.get_simple_extent_dims()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_5shape___get__(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "h5py/h5s.pyx":134
 *         """
 *         def __get__(self):
 *             return self.get_simple_extent_dims()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_1_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID__close[] = "()\n\n        Terminate access through this identifier.  You shouldn't have to\n        call this manually; dataspace objects are automatically destroyed\n        when their Python wrappers are freed.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_1_close(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_close (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID__close(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":137
 * 
 * 
 *     def _close(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID__close(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  herr_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_close", 0);

  /* "h5py/h5s.pyx":144
 *         when their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Sclose(self.id)
 *             if not self.valid:
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_GetName(__pyx_m, __pyx_n_s___objects); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s__registry); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_GetAttr(__pyx_t_2, __pyx_n_s__lock); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_3 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s____exit__); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s____enter__); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyObject_Call(__pyx_t_2, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "h5py/h5s.pyx":145
 *         """
 *         with _objects.registry.lock:
 *             H5Sclose(self.id)             # <<<<<<<<<<<<<<
 *             if not self.valid:
 *                 del _objects.registry[self.id]
 */
          __pyx_t_8 = __pyx_f_4h5py_4defs_H5Sclose(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L7_error;}

          /* "h5py/h5s.pyx":146
 *         with _objects.registry.lock:
 *             H5Sclose(self.id)
 *             if not self.valid:             # <<<<<<<<<<<<<<
 *                 del _objects.registry[self.id]
 * 
 */
          __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__valid); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_10 = (!__pyx_t_9);
          if (__pyx_t_10) {

            /* "h5py/h5s.pyx":147
 *             H5Sclose(self.id)
 *             if not self.valid:
 *                 del _objects.registry[self.id]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_1 = __Pyx_GetName(__pyx_m, __pyx_n_s___objects); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_4 = PyObject_GetAttr(__pyx_t_1, __pyx_n_s__registry); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (__Pyx_DelItemInt(__pyx_t_4, __pyx_v_self->__pyx_base.id, sizeof(hid_t), __Pyx_PyInt_to_py_hid_t) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            goto __pyx_L15;
          }
          __pyx_L15:;
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L14_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "h5py/h5s.pyx":144
 *         when their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Sclose(self.id)
 *             if not self.valid:
 */
        /*except:*/ {
          __Pyx_AddTraceback("h5py.h5s.SpaceID._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_2);
          __Pyx_GIVEREF(__pyx_t_2);
          __pyx_t_12 = PyObject_Call(__pyx_t_3, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
          __pyx_t_9 = (!__pyx_t_10);
          if (__pyx_t_9) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_ErrRestore(__pyx_t_4, __pyx_t_1, __pyx_t_2);
            __pyx_t_4 = 0; __pyx_t_1 = 0; __pyx_t_2 = 0; 
            {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L9_except_error;}
            goto __pyx_L18;
          }
          __pyx_L18:;
          __Pyx_DECREF(((PyObject *)__pyx_t_11)); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L14_try_end:;
      }
    }
    /*finally:*/ {
      if (__pyx_t_3) {
        __pyx_t_7 = PyObject_Call(__pyx_t_3, __pyx_k_tuple_2, NULL);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
    }
    goto __pyx_L19;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L1_error;
    __pyx_L19:;
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("h5py.h5s.SpaceID._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_3copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_2copy[] = "() => SpaceID\n\n        Create a new copy of this dataspace.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_3copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_2copy(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":150
 * 
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         """() => SpaceID
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_2copy(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  hid_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "h5py/h5s.pyx":155
 *         Create a new copy of this dataspace.
 *         """
 *         return SpaceID.open(H5Scopy(self.id))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)((PyObject*)__pyx_ptype_4h5py_3h5s_SpaceID)), __pyx_n_s__open); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Scopy(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyInt_to_py_hid_t(__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_t_4), NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_5encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_4encode[] = "() => STRING\n\n        Serialize a dataspace, including its selection.  Bear in mind you\n        can also use the native Python pickling machinery to do this.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_5encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("encode (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_4encode(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":158
 * 
 * 
 *     def encode(self):             # <<<<<<<<<<<<<<
 *         """() => STRING
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_4encode(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  void *__pyx_v_buf;
  size_t __pyx_v_nalloc;
  PyObject *__pyx_v_pystr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode", 0);

  /* "h5py/h5s.pyx":164
 *         can also use the native Python pickling machinery to do this.
 *         """
 *         cdef void* buf = NULL             # <<<<<<<<<<<<<<
 *         cdef size_t nalloc = 0
 * 
 */
  __pyx_v_buf = NULL;

  /* "h5py/h5s.pyx":165
 *         """
 *         cdef void* buf = NULL
 *         cdef size_t nalloc = 0             # <<<<<<<<<<<<<<
 * 
 *         H5Sencode(self.id, NULL, &nalloc)
 */
  __pyx_v_nalloc = 0;

  /* "h5py/h5s.pyx":167
 *         cdef size_t nalloc = 0
 * 
 *         H5Sencode(self.id, NULL, &nalloc)             # <<<<<<<<<<<<<<
 *         buf = emalloc(nalloc)
 *         try:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sencode(__pyx_v_self->__pyx_base.id, NULL, (&__pyx_v_nalloc)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 167; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":168
 * 
 *         H5Sencode(self.id, NULL, &nalloc)
 *         buf = emalloc(nalloc)             # <<<<<<<<<<<<<<
 *         try:
 *             H5Sencode(self.id, buf, &nalloc)
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(__pyx_v_nalloc); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = __pyx_t_2;

  /* "h5py/h5s.pyx":169
 *         H5Sencode(self.id, NULL, &nalloc)
 *         buf = emalloc(nalloc)
 *         try:             # <<<<<<<<<<<<<<
 *             H5Sencode(self.id, buf, &nalloc)
 *             pystr = PyBytes_FromStringAndSize(<char*>buf, nalloc)
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":170
 *         buf = emalloc(nalloc)
 *         try:
 *             H5Sencode(self.id, buf, &nalloc)             # <<<<<<<<<<<<<<
 *             pystr = PyBytes_FromStringAndSize(<char*>buf, nalloc)
 *         finally:
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sencode(__pyx_v_self->__pyx_base.id, __pyx_v_buf, (&__pyx_v_nalloc)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 170; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5s.pyx":171
 *         try:
 *             H5Sencode(self.id, buf, &nalloc)
 *             pystr = PyBytes_FromStringAndSize(<char*>buf, nalloc)             # <<<<<<<<<<<<<<
 *         finally:
 *             efree(buf)
 */
    __pyx_t_3 = PyBytes_FromStringAndSize(((char *)__pyx_v_buf), __pyx_v_nalloc); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_pystr = __pyx_t_3;
    __pyx_t_3 = 0;
  }

  /* "h5py/h5s.pyx":173
 *             pystr = PyBytes_FromStringAndSize(<char*>buf, nalloc)
 *         finally:
 *             efree(buf)             # <<<<<<<<<<<<<<
 * 
 *         return pystr
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_buf);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  /* "h5py/h5s.pyx":175
 *             efree(buf)
 * 
 *         return pystr             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pystr);
  __pyx_r = __pyx_v_pystr;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pystr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_6__reduce__(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":178
 * 
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return (type(self), (-1,), self.encode())
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_6__reduce__(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "h5py/h5s.pyx":179
 * 
 *     def __reduce__(self):
 *         return (type(self), (-1,), self.encode())             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s__encode); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
  __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
  __Pyx_INCREF(((PyObject *)__pyx_k_tuple_3));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_k_tuple_3));
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_3));
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_state); /*proto*/
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_9__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_8__setstate__(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), ((PyObject *)__pyx_v_state));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":182
 * 
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         cdef char* buf = state
 *         self.id = H5Sdecode(buf)
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_8__setstate__(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_state) {
  char *__pyx_v_buf;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  hid_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "h5py/h5s.pyx":183
 * 
 *     def __setstate__(self, state):
 *         cdef char* buf = state             # <<<<<<<<<<<<<<
 *         self.id = H5Sdecode(buf)
 * 
 */
  __pyx_t_1 = PyBytes_AsString(__pyx_v_state); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = __pyx_t_1;

  /* "h5py/h5s.pyx":184
 *     def __setstate__(self, state):
 *         cdef char* buf = state
 *         self.id = H5Sdecode(buf)             # <<<<<<<<<<<<<<
 * 
 *     # === Simple dataspaces ===================================================
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Sdecode(__pyx_v_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 184; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->__pyx_base.id = __pyx_t_2;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.__setstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_11is_simple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_10is_simple[] = "() => BOOL is_simple\n\n        Determine if an existing dataspace is \"simple\" (including scalar\n        dataspaces). Currently all HDF5 dataspaces are simple.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_11is_simple(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_simple (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_10is_simple(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":189
 * 
 * 
 *     def is_simple(self):             # <<<<<<<<<<<<<<
 *         """() => BOOL is_simple
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_10is_simple(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_simple", 0);

  /* "h5py/h5s.pyx":195
 *         dataspaces). Currently all HDF5 dataspaces are simple.
 *         """
 *         return <bint>(H5Sis_simple(self.id))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sis_simple(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 195; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyBool_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 195; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.is_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_13offset_simple(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_12offset_simple[] = "(TUPLE offset=None)\n\n        Set the offset of a dataspace.  The length of the given tuple must\n        match the rank of the dataspace. If None is provided (default),\n        the offsets on all axes will be set to 0.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_13offset_simple(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_offset = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("offset_simple (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__offset,0};
    PyObject* values[1] = {0};

    /* "h5py/h5s.pyx":198
 * 
 * 
 *     def offset_simple(self, object offset=None):             # <<<<<<<<<<<<<<
 *         """(TUPLE offset=None)
 * 
 */
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__offset);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "offset_simple") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("offset_simple", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.offset_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_12offset_simple(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), __pyx_v_offset);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_12offset_simple(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_offset) {
  int __pyx_v_rank;
  int __pyx_v_i;
  hssize_t *__pyx_v_dims;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  void *__pyx_t_6;
  herr_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("offset_simple", 0);

  /* "h5py/h5s.pyx":207
 *         cdef int rank
 *         cdef int i
 *         cdef hssize_t *dims = NULL             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_v_dims = NULL;

  /* "h5py/h5s.pyx":209
 *         cdef hssize_t *dims = NULL
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if not H5Sis_simple(self.id):
 *                 raise ValueError("%d is not a simple dataspace" % self.id)
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":210
 * 
 *         try:
 *             if not H5Sis_simple(self.id):             # <<<<<<<<<<<<<<
 *                 raise ValueError("%d is not a simple dataspace" % self.id)
 * 
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sis_simple(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 210; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_t_2 = (!__pyx_t_1);
    if (__pyx_t_2) {

      /* "h5py/h5s.pyx":211
 *         try:
 *             if not H5Sis_simple(self.id):
 *                 raise ValueError("%d is not a simple dataspace" % self.id)             # <<<<<<<<<<<<<<
 * 
 *             rank = H5Sget_simple_extent_ndims(self.id)
 */
      __pyx_t_3 = __Pyx_PyInt_to_py_hid_t(__pyx_v_self->__pyx_base.id); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyNumber_Remainder(((PyObject *)__pyx_kp_s_4), __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(((PyObject *)__pyx_t_4));
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_t_4));
      __Pyx_GIVEREF(((PyObject *)__pyx_t_4));
      __pyx_t_4 = 0;
      __pyx_t_4 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_t_3), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L6;
    }
    __pyx_L6:;

    /* "h5py/h5s.pyx":213
 *                 raise ValueError("%d is not a simple dataspace" % self.id)
 * 
 *             rank = H5Sget_simple_extent_ndims(self.id)             # <<<<<<<<<<<<<<
 * 
 *             require_tuple(offset, 1, rank, "offset")
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 213; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_rank = __pyx_t_5;

    /* "h5py/h5s.pyx":215
 *             rank = H5Sget_simple_extent_ndims(self.id)
 * 
 *             require_tuple(offset, 1, rank, "offset")             # <<<<<<<<<<<<<<
 *             dims = <hssize_t*>emalloc(sizeof(hssize_t)*rank)
 *             if(offset is not None):
 */
    __pyx_t_5 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_offset, 1, __pyx_v_rank, __pyx_k__offset); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5s.pyx":216
 * 
 *             require_tuple(offset, 1, rank, "offset")
 *             dims = <hssize_t*>emalloc(sizeof(hssize_t)*rank)             # <<<<<<<<<<<<<<
 *             if(offset is not None):
 *                 convert_tuple(offset, <hsize_t*>dims, rank)
 */
    __pyx_t_6 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hssize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_6 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 216; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_dims = ((hssize_t *)__pyx_t_6);

    /* "h5py/h5s.pyx":217
 *             require_tuple(offset, 1, rank, "offset")
 *             dims = <hssize_t*>emalloc(sizeof(hssize_t)*rank)
 *             if(offset is not None):             # <<<<<<<<<<<<<<
 *                 convert_tuple(offset, <hsize_t*>dims, rank)
 *             else:
 */
    __pyx_t_2 = (__pyx_v_offset != Py_None);
    if (__pyx_t_2) {

      /* "h5py/h5s.pyx":218
 *             dims = <hssize_t*>emalloc(sizeof(hssize_t)*rank)
 *             if(offset is not None):
 *                 convert_tuple(offset, <hsize_t*>dims, rank)             # <<<<<<<<<<<<<<
 *             else:
 *                 # The HDF5 docs say passing in NULL resets the offset to 0.
 */
      __pyx_t_5 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_offset, ((hsize_t *)__pyx_v_dims), __pyx_v_rank); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L7;
    }
    /*else*/ {

      /* "h5py/h5s.pyx":223
 *                 # Instead it raises an exception.  Imagine my surprise. We'll
 *                 # do this manually.
 *                 for i from 0<=i<rank:             # <<<<<<<<<<<<<<
 *                     dims[i] = 0
 * 
 */
      __pyx_t_5 = __pyx_v_rank;
      for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_5; __pyx_v_i++) {

        /* "h5py/h5s.pyx":224
 *                 # do this manually.
 *                 for i from 0<=i<rank:
 *                     dims[i] = 0             # <<<<<<<<<<<<<<
 * 
 *             H5Soffset_simple(self.id, dims)
 */
        (__pyx_v_dims[__pyx_v_i]) = 0;
      }
    }
    __pyx_L7:;

    /* "h5py/h5s.pyx":226
 *                     dims[i] = 0
 * 
 *             H5Soffset_simple(self.id, dims)             # <<<<<<<<<<<<<<
 * 
 *         finally:
 */
    __pyx_t_7 = __pyx_f_4h5py_4defs_H5Soffset_simple(__pyx_v_self->__pyx_base.id, __pyx_v_dims); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L4;}
  }

  /* "h5py/h5s.pyx":229
 * 
 *         finally:
 *             efree(dims)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_dims);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.offset_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_15get_simple_extent_ndims(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_14get_simple_extent_ndims[] = "() => INT rank\n\n        Determine the rank of a \"simple\" (slab) dataspace.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_15get_simple_extent_ndims(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_simple_extent_ndims (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_14get_simple_extent_ndims(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":232
 * 
 * 
 *     def get_simple_extent_ndims(self):             # <<<<<<<<<<<<<<
 *         """() => INT rank
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_14get_simple_extent_ndims(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_simple_extent_ndims", 0);

  /* "h5py/h5s.pyx":237
 *         Determine the rank of a "simple" (slab) dataspace.
 *         """
 *         return H5Sget_simple_extent_ndims(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_simple_extent_ndims", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_17get_simple_extent_dims(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_16get_simple_extent_dims[] = "(BOOL maxdims=False) => TUPLE shape\n\n        Determine the shape of a \"simple\" (slab) dataspace.  If \"maxdims\"\n        is True, retrieve the maximum dataspace size instead.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_17get_simple_extent_dims(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_maxdims;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_simple_extent_dims (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__maxdims,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__maxdims);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_simple_extent_dims") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_maxdims = __Pyx_PyInt_AsInt(values[0]); if (unlikely((__pyx_v_maxdims == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_maxdims = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_simple_extent_dims", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_simple_extent_dims", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_16get_simple_extent_dims(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), __pyx_v_maxdims);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":240
 * 
 * 
 *     def get_simple_extent_dims(self, int maxdims=0):             # <<<<<<<<<<<<<<
 *         """(BOOL maxdims=False) => TUPLE shape
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_16get_simple_extent_dims(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, int __pyx_v_maxdims) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_dims;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_simple_extent_dims", 0);

  /* "h5py/h5s.pyx":247
 *         """
 *         cdef int rank
 *         cdef hsize_t* dims = NULL             # <<<<<<<<<<<<<<
 * 
 *         rank = H5Sget_simple_extent_dims(self.id, NULL, NULL)
 */
  __pyx_v_dims = NULL;

  /* "h5py/h5s.pyx":249
 *         cdef hsize_t* dims = NULL
 * 
 *         rank = H5Sget_simple_extent_dims(self.id, NULL, NULL)             # <<<<<<<<<<<<<<
 * 
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_dims(__pyx_v_self->__pyx_base.id, NULL, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 249; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_1;

  /* "h5py/h5s.pyx":251
 *         rank = H5Sget_simple_extent_dims(self.id, NULL, NULL)
 * 
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *         try:
 *             if maxdims:
 */
  __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dims = ((hsize_t *)__pyx_t_2);

  /* "h5py/h5s.pyx":252
 * 
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         try:             # <<<<<<<<<<<<<<
 *             if maxdims:
 *                 H5Sget_simple_extent_dims(self.id, NULL, dims)
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":253
 *         dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         try:
 *             if maxdims:             # <<<<<<<<<<<<<<
 *                 H5Sget_simple_extent_dims(self.id, NULL, dims)
 *             else:
 */
    if (__pyx_v_maxdims) {

      /* "h5py/h5s.pyx":254
 *         try:
 *             if maxdims:
 *                 H5Sget_simple_extent_dims(self.id, NULL, dims)             # <<<<<<<<<<<<<<
 *             else:
 *                 H5Sget_simple_extent_dims(self.id, dims, NULL)
 */
      __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_dims(__pyx_v_self->__pyx_base.id, NULL, __pyx_v_dims); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/h5s.pyx":256
 *                 H5Sget_simple_extent_dims(self.id, NULL, dims)
 *             else:
 *                 H5Sget_simple_extent_dims(self.id, dims, NULL)             # <<<<<<<<<<<<<<
 * 
 *             return convert_dims(dims, rank)
 */
      __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_dims(__pyx_v_self->__pyx_base.id, __pyx_v_dims, NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 256; __pyx_clineno = __LINE__; goto __pyx_L4;}
    }
    __pyx_L6:;

    /* "h5py/h5s.pyx":258
 *                 H5Sget_simple_extent_dims(self.id, dims, NULL)
 * 
 *             return convert_dims(dims, rank)             # <<<<<<<<<<<<<<
 * 
 *         finally:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_4h5py_5utils_convert_dims(__pyx_v_dims, __pyx_v_rank); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L3;
  }

  /* "h5py/h5s.pyx":261
 * 
 *         finally:
 *             efree(dims)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L3: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_dims);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_simple_extent_dims", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_19get_simple_extent_npoints(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_18get_simple_extent_npoints[] = "() => LONG npoints\n\n        Determine the total number of elements in a dataspace.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_19get_simple_extent_npoints(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_simple_extent_npoints (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_18get_simple_extent_npoints(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":264
 * 
 * 
 *     def get_simple_extent_npoints(self):             # <<<<<<<<<<<<<<
 *         """() => LONG npoints
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_18get_simple_extent_npoints(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_simple_extent_npoints", 0);

  /* "h5py/h5s.pyx":269
 *         Determine the total number of elements in a dataspace.
 *         """
 *         return H5Sget_simple_extent_npoints(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_npoints(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_to_py_hssize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_simple_extent_npoints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_21get_simple_extent_type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_20get_simple_extent_type[] = "() => INT class_code\n\n        Class code is either SCALAR or SIMPLE.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_21get_simple_extent_type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_simple_extent_type (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_20get_simple_extent_type(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":272
 * 
 * 
 *     def get_simple_extent_type(self):             # <<<<<<<<<<<<<<
 *         """() => INT class_code
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_20get_simple_extent_type(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  H5S_class_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_simple_extent_type", 0);

  /* "h5py/h5s.pyx":277
 *         Class code is either SCALAR or SIMPLE.
 *         """
 *         return <int>H5Sget_simple_extent_type(self.id)             # <<<<<<<<<<<<<<
 * 
 *     # === Extents =============================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_type(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 277; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_simple_extent_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_23extent_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_source); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_22extent_copy[] = "(SpaceID source)\n\n        Replace this dataspace's extent with another's, changing its\n        typecode if necessary.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_23extent_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_source) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("extent_copy (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_source), __pyx_ptype_4h5py_3h5s_SpaceID, 0, "source", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_22extent_copy(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), ((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_source));
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":282
 * 
 * 
 *     def extent_copy(self, SpaceID source not None):             # <<<<<<<<<<<<<<
 *         """(SpaceID source)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_22extent_copy(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_source) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extent_copy", 0);

  /* "h5py/h5s.pyx":288
 *         typecode if necessary.
 *         """
 *         H5Sextent_copy(self.id, source.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sextent_copy(__pyx_v_self->__pyx_base.id, __pyx_v_source->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.extent_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_25set_extent_simple(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_24set_extent_simple[] = "(TUPLE dims_tpl, TUPLE max_dims_tpl=None)\n\n        Reset the dataspace extent via a tuple of dimensions.\n        Every element of dims_tpl must be a positive integer.\n\n        You can optionally specify the maximum dataspace size. The\n        special value UNLIMITED, as an element of max_dims, indicates\n        an unlimited dimension.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_25set_extent_simple(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_dims_tpl = 0;
  PyObject *__pyx_v_max_dims_tpl = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_extent_simple (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__dims_tpl,&__pyx_n_s__max_dims_tpl,0};
    PyObject* values[2] = {0,0};

    /* "h5py/h5s.pyx":291
 * 
 * 
 *     def set_extent_simple(self, object dims_tpl, object max_dims_tpl=None):             # <<<<<<<<<<<<<<
 *         """(TUPLE dims_tpl, TUPLE max_dims_tpl=None)
 * 
 */
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__dims_tpl)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__max_dims_tpl);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_extent_simple") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_dims_tpl = values[0];
    __pyx_v_max_dims_tpl = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_extent_simple", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 291; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.set_extent_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_24set_extent_simple(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), __pyx_v_dims_tpl, __pyx_v_max_dims_tpl);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_24set_extent_simple(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_dims_tpl, PyObject *__pyx_v_max_dims_tpl) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_dims;
  hsize_t *__pyx_v_max_dims;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  void *__pyx_t_3;
  int __pyx_t_4;
  herr_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_extent_simple", 0);

  /* "h5py/h5s.pyx":302
 *         """
 *         cdef int rank
 *         cdef hsize_t* dims = NULL             # <<<<<<<<<<<<<<
 *         cdef hsize_t* max_dims = NULL
 * 
 */
  __pyx_v_dims = NULL;

  /* "h5py/h5s.pyx":303
 *         cdef int rank
 *         cdef hsize_t* dims = NULL
 *         cdef hsize_t* max_dims = NULL             # <<<<<<<<<<<<<<
 * 
 *         require_tuple(dims_tpl, 0, -1, "dims_tpl")
 */
  __pyx_v_max_dims = NULL;

  /* "h5py/h5s.pyx":305
 *         cdef hsize_t* max_dims = NULL
 * 
 *         require_tuple(dims_tpl, 0, -1, "dims_tpl")             # <<<<<<<<<<<<<<
 *         rank = len(dims_tpl)
 *         require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_dims_tpl, 0, -1, __pyx_k__dims_tpl); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":306
 * 
 *         require_tuple(dims_tpl, 0, -1, "dims_tpl")
 *         rank = len(dims_tpl)             # <<<<<<<<<<<<<<
 *         require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")
 * 
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_dims_tpl); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_2;

  /* "h5py/h5s.pyx":307
 *         require_tuple(dims_tpl, 0, -1, "dims_tpl")
 *         rank = len(dims_tpl)
 *         require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_max_dims_tpl, 1, __pyx_v_rank, __pyx_k__max_dims_tpl); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 307; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":309
 *         require_tuple(max_dims_tpl, 1, rank, "max_dims_tpl")
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(dims_tpl, dims, rank)
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":310
 * 
 *         try:
 *             dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *             convert_tuple(dims_tpl, dims, rank)
 * 
 */
    __pyx_t_3 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_3 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 310; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_dims = ((hsize_t *)__pyx_t_3);

    /* "h5py/h5s.pyx":311
 *         try:
 *             dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(dims_tpl, dims, rank)             # <<<<<<<<<<<<<<
 * 
 *             if max_dims_tpl is not None:
 */
    __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_dims_tpl, __pyx_v_dims, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 311; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5s.pyx":313
 *             convert_tuple(dims_tpl, dims, rank)
 * 
 *             if max_dims_tpl is not None:             # <<<<<<<<<<<<<<
 *                 max_dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(max_dims_tpl, max_dims, rank)
 */
    __pyx_t_4 = (__pyx_v_max_dims_tpl != Py_None);
    if (__pyx_t_4) {

      /* "h5py/h5s.pyx":314
 * 
 *             if max_dims_tpl is not None:
 *                 max_dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *                 convert_tuple(max_dims_tpl, max_dims, rank)
 * 
 */
      __pyx_t_3 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_3 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 314; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_max_dims = ((hsize_t *)__pyx_t_3);

      /* "h5py/h5s.pyx":315
 *             if max_dims_tpl is not None:
 *                 max_dims = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(max_dims_tpl, max_dims, rank)             # <<<<<<<<<<<<<<
 * 
 *             H5Sset_extent_simple(self.id, rank, dims, max_dims)
 */
      __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_max_dims_tpl, __pyx_v_max_dims, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 315; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L6;
    }
    __pyx_L6:;

    /* "h5py/h5s.pyx":317
 *                 convert_tuple(max_dims_tpl, max_dims, rank)
 * 
 *             H5Sset_extent_simple(self.id, rank, dims, max_dims)             # <<<<<<<<<<<<<<
 * 
 *         finally:
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Sset_extent_simple(__pyx_v_self->__pyx_base.id, __pyx_v_rank, __pyx_v_dims, __pyx_v_max_dims); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 317; __pyx_clineno = __LINE__; goto __pyx_L4;}
  }

  /* "h5py/h5s.pyx":320
 * 
 *         finally:
 *             efree(dims)             # <<<<<<<<<<<<<<
 *             efree(max_dims)
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_dims);

    /* "h5py/h5s.pyx":321
 *         finally:
 *             efree(dims)
 *             efree(max_dims)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_f_4h5py_5utils_efree(__pyx_v_max_dims);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.set_extent_simple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_27set_extent_none(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_26set_extent_none[] = "()\n\n        Remove the dataspace extent; typecode changes to NO_CLASS.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_27set_extent_none(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_extent_none (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_26set_extent_none(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":324
 * 
 * 
 *     def set_extent_none(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_26set_extent_none(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_extent_none", 0);

  /* "h5py/h5s.pyx":329
 *         Remove the dataspace extent; typecode changes to NO_CLASS.
 *         """
 *         H5Sset_extent_none(self.id)             # <<<<<<<<<<<<<<
 * 
 *     # === General selection operations ========================================
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sset_extent_none(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 329; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.set_extent_none", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_29get_select_type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_28get_select_type[] = " () => INT select_code\n\n            Determine selection type.  Return values are:\n\n            - SEL_NONE\n            - SEL_ALL\n            - SEL_POINTS\n            - SEL_HYPERSLABS\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_29get_select_type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_type (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_28get_select_type(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":334
 * 
 * 
 *     def get_select_type(self):             # <<<<<<<<<<<<<<
 *         """ () => INT select_code
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_28get_select_type(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  H5S_sel_type __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_type", 0);

  /* "h5py/h5s.pyx":344
 *             - SEL_HYPERSLABS
 *         """
 *         return <int>H5Sget_select_type(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_select_type(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 344; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = PyInt_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 344; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_31get_select_npoints(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_30get_select_npoints[] = "() => LONG npoints\n\n        Determine the total number of points currently selected.\n        Works for all selection techniques.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_31get_select_npoints(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_npoints (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_30get_select_npoints(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":347
 * 
 * 
 *     def get_select_npoints(self):             # <<<<<<<<<<<<<<
 *         """() => LONG npoints
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_30get_select_npoints(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_npoints", 0);

  /* "h5py/h5s.pyx":353
 *         Works for all selection techniques.
 *         """
 *         return H5Sget_select_npoints(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_select_npoints(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 353; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_to_py_hssize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 353; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_npoints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_33get_select_bounds(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_32get_select_bounds[] = "() => (TUPLE start, TUPLE end)\n\n        Determine the bounding box which exactly contains\n        the current selection.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_33get_select_bounds(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_bounds (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_32get_select_bounds(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":356
 * 
 * 
 *     def get_select_bounds(self):             # <<<<<<<<<<<<<<
 *         """() => (TUPLE start, TUPLE end)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_32get_select_bounds(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_start;
  hsize_t *__pyx_v_end;
  PyObject *__pyx_v_start_tpl = NULL;
  PyObject *__pyx_v_end_tpl = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  hssize_t __pyx_t_2;
  int __pyx_t_3;
  void *__pyx_t_4;
  herr_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_bounds", 0);

  /* "h5py/h5s.pyx":363
 *         """
 *         cdef int rank
 *         cdef hsize_t *start = NULL             # <<<<<<<<<<<<<<
 *         cdef hsize_t *end = NULL
 * 
 */
  __pyx_v_start = NULL;

  /* "h5py/h5s.pyx":364
 *         cdef int rank
 *         cdef hsize_t *start = NULL
 *         cdef hsize_t *end = NULL             # <<<<<<<<<<<<<<
 * 
 *         rank = H5Sget_simple_extent_ndims(self.id)
 */
  __pyx_v_end = NULL;

  /* "h5py/h5s.pyx":366
 *         cdef hsize_t *end = NULL
 * 
 *         rank = H5Sget_simple_extent_ndims(self.id)             # <<<<<<<<<<<<<<
 * 
 *         if H5Sget_select_npoints(self.id) == 0:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_1;

  /* "h5py/h5s.pyx":368
 *         rank = H5Sget_simple_extent_ndims(self.id)
 * 
 *         if H5Sget_select_npoints(self.id) == 0:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Sget_select_npoints(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = (__pyx_t_2 == 0);
  if (__pyx_t_3) {

    /* "h5py/h5s.pyx":369
 * 
 *         if H5Sget_select_npoints(self.id) == 0:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         start = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5s.pyx":371
 *             return None
 * 
 *         start = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *         end = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 * 
 */
  __pyx_t_4 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_4 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 371; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_start = ((hsize_t *)__pyx_t_4);

  /* "h5py/h5s.pyx":372
 * 
 *         start = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *         end = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_4 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_4 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_end = ((hsize_t *)__pyx_t_4);

  /* "h5py/h5s.pyx":374
 *         end = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             H5Sget_select_bounds(self.id, start, end)
 * 
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":375
 * 
 *         try:
 *             H5Sget_select_bounds(self.id, start, end)             # <<<<<<<<<<<<<<
 * 
 *             start_tpl = convert_dims(start, rank)
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Sget_select_bounds(__pyx_v_self->__pyx_base.id, __pyx_v_start, __pyx_v_end); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 375; __pyx_clineno = __LINE__; goto __pyx_L5;}

    /* "h5py/h5s.pyx":377
 *             H5Sget_select_bounds(self.id, start, end)
 * 
 *             start_tpl = convert_dims(start, rank)             # <<<<<<<<<<<<<<
 *             end_tpl = convert_dims(end, rank)
 *             return (start_tpl, end_tpl)
 */
    __pyx_t_6 = __pyx_f_4h5py_5utils_convert_dims(__pyx_v_start, __pyx_v_rank); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_start_tpl = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "h5py/h5s.pyx":378
 * 
 *             start_tpl = convert_dims(start, rank)
 *             end_tpl = convert_dims(end, rank)             # <<<<<<<<<<<<<<
 *             return (start_tpl, end_tpl)
 * 
 */
    __pyx_t_6 = __pyx_f_4h5py_5utils_convert_dims(__pyx_v_end, __pyx_v_rank); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 378; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_end_tpl = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "h5py/h5s.pyx":379
 *             start_tpl = convert_dims(start, rank)
 *             end_tpl = convert_dims(end, rank)
 *             return (start_tpl, end_tpl)             # <<<<<<<<<<<<<<
 * 
 *         finally:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L5;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_start_tpl);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_start_tpl);
    __Pyx_GIVEREF(__pyx_v_start_tpl);
    __Pyx_INCREF(__pyx_v_end_tpl);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_end_tpl);
    __Pyx_GIVEREF(__pyx_v_end_tpl);
    __pyx_r = ((PyObject *)__pyx_t_6);
    __pyx_t_6 = 0;
    goto __pyx_L4;
  }

  /* "h5py/h5s.pyx":382
 * 
 *         finally:
 *             efree(start)             # <<<<<<<<<<<<<<
 *             efree(end)
 * 
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L6;
    __pyx_L4: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 3; goto __pyx_L6;
    __pyx_L5: {
      __pyx_why = 4;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L6;
    }
    __pyx_L6:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_start);

    /* "h5py/h5s.pyx":383
 *         finally:
 *             efree(start)
 *             efree(end)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_f_4h5py_5utils_efree(__pyx_v_end);
    switch (__pyx_why) {
      case 3: goto __pyx_L0;
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start_tpl);
  __Pyx_XDECREF(__pyx_v_end_tpl);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_35select_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_34select_all[] = "()\n\n        Select all points in the dataspace.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_35select_all(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_all (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_34select_all(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":386
 * 
 * 
 *     def select_all(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_34select_all(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select_all", 0);

  /* "h5py/h5s.pyx":391
 *         Select all points in the dataspace.
 *         """
 *         H5Sselect_all(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sselect_all(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 391; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_all", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_37select_none(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_36select_none[] = "()\n\n        Deselect entire dataspace.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_37select_none(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_none (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_36select_none(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":394
 * 
 * 
 *     def select_none(self):             # <<<<<<<<<<<<<<
 *         """()
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_36select_none(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select_none", 0);

  /* "h5py/h5s.pyx":399
 *         Deselect entire dataspace.
 *         """
 *         H5Sselect_none(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sselect_none(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_none", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_39select_valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_38select_valid[] = "() => BOOL\n\n        Determine if the current selection falls within\n        the dataspace extent.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_39select_valid(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_valid (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_38select_valid(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":402
 * 
 * 
 *     def select_valid(self):             # <<<<<<<<<<<<<<
 *         """() => BOOL
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_38select_valid(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select_valid", 0);

  /* "h5py/h5s.pyx":408
 *         the dataspace extent.
 *         """
 *         return <bint>(H5Sselect_valid(self.id))             # <<<<<<<<<<<<<<
 * 
 *     # === Point selection functions ===========================================
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sselect_valid(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyBool_FromLong(((int)__pyx_t_1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 408; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_valid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_41get_select_elem_npoints(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_40get_select_elem_npoints[] = "() => LONG npoints\n\n        Determine the number of elements selected in point-selection mode.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_41get_select_elem_npoints(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_elem_npoints (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_40get_select_elem_npoints(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":413
 * 
 * 
 *     def get_select_elem_npoints(self):             # <<<<<<<<<<<<<<
 *         """() => LONG npoints
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_40get_select_elem_npoints(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_elem_npoints", 0);

  /* "h5py/h5s.pyx":418
 *         Determine the number of elements selected in point-selection mode.
 *         """
 *         return H5Sget_select_elem_npoints(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_select_elem_npoints(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_to_py_hssize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_elem_npoints", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_43get_select_elem_pointlist(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_42get_select_elem_pointlist[] = "() => NDARRAY\n\n        Get a list of all selected elements.  Return is a Numpy array of\n        unsigned ints, with shape ``(<npoints>, <space rank)``.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_43get_select_elem_pointlist(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_elem_pointlist (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_42get_select_elem_pointlist(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":421
 * 
 * 
 *     def get_select_elem_pointlist(self):             # <<<<<<<<<<<<<<
 *         """() => NDARRAY
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_42get_select_elem_pointlist(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  hsize_t __pyx_v_dims[2];
  PyArrayObject *__pyx_v_buf = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  herr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_elem_pointlist", 0);

  /* "h5py/h5s.pyx":430
 *         cdef ndarray buf
 * 
 *         dims[0] = H5Sget_select_elem_npoints(self.id)             # <<<<<<<<<<<<<<
 *         dims[1] = H5Sget_simple_extent_ndims(self.id)
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_select_elem_npoints(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 430; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_dims[0]) = __pyx_t_1;

  /* "h5py/h5s.pyx":431
 * 
 *         dims[0] = H5Sget_select_elem_npoints(self.id)
 *         dims[1] = H5Sget_simple_extent_ndims(self.id)             # <<<<<<<<<<<<<<
 * 
 *         buf = create_numpy_hsize(2, dims)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 431; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_dims[1]) = __pyx_t_2;

  /* "h5py/h5s.pyx":433
 *         dims[1] = H5Sget_simple_extent_ndims(self.id)
 * 
 *         buf = create_numpy_hsize(2, dims)             # <<<<<<<<<<<<<<
 * 
 *         H5Sget_select_elem_pointlist(self.id, 0, dims[0], <hsize_t*>buf.data)
 */
  __pyx_t_3 = __pyx_f_4h5py_5utils_create_numpy_hsize(2, __pyx_v_dims); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_4h5py_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 433; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "h5py/h5s.pyx":435
 *         buf = create_numpy_hsize(2, dims)
 * 
 *         H5Sget_select_elem_pointlist(self.id, 0, dims[0], <hsize_t*>buf.data)             # <<<<<<<<<<<<<<
 * 
 *         return buf
 */
  __pyx_t_4 = __pyx_f_4h5py_4defs_H5Sget_select_elem_pointlist(__pyx_v_self->__pyx_base.id, 0, (__pyx_v_dims[0]), ((hsize_t *)__pyx_v_buf->data)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":437
 *         H5Sget_select_elem_pointlist(self.id, 0, dims[0], <hsize_t*>buf.data)
 * 
 *         return buf             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_buf));
  __pyx_r = ((PyObject *)__pyx_v_buf);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_elem_pointlist", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_45select_elements(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_44select_elements[] = "(SEQUENCE coords, INT op=SELECT_SET)\n\n        Select elements by specifying coordinates points.  The argument\n        \"coords\" may be an ndarray or any nested sequence which can be\n        converted to an array of uints with the shape::\n\n            (<npoints>, <space rank>)\n\n        Examples::\n\n            >>> obj.shape\n            (10, 10)\n            >>> obj.select_elements([(1,2), (3,4), (5,9)])\n\n        A zero-length selection (i.e. shape ``(0, <rank>)``) is not allowed\n        by the HDF5 library.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_45select_elements(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_coords = 0;
  int __pyx_v_op;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_elements (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__coords,&__pyx_n_s__op,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__coords)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__op);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "select_elements") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_coords = values[0];
    if (values[1]) {
      __pyx_v_op = __Pyx_PyInt_AsInt(values[1]); if (unlikely((__pyx_v_op == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_op = __pyx_k_5;
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("select_elements", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_elements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_44select_elements(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), __pyx_v_coords, __pyx_v_op);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":440
 * 
 * 
 *     def select_elements(self, object coords, int op=H5S_SELECT_SET):             # <<<<<<<<<<<<<<
 *         """(SEQUENCE coords, INT op=SELECT_SET)
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_44select_elements(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_coords, int __pyx_v_op) {
  PyArrayObject *__pyx_v_hcoords = 0;
  size_t __pyx_v_nelements;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  herr_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select_elements", 0);

  /* "h5py/h5s.pyx":466
 *         # a compatible type and initializing it to the input.
 * 
 *         hcoords = create_hsize_array(coords)             # <<<<<<<<<<<<<<
 * 
 *         if hcoords.nd != 2 or hcoords.dimensions[1] != H5Sget_simple_extent_ndims(self.id):
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_create_hsize_array(__pyx_v_coords); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_4h5py_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_hcoords = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/h5s.pyx":468
 *         hcoords = create_hsize_array(coords)
 * 
 *         if hcoords.nd != 2 or hcoords.dimensions[1] != H5Sget_simple_extent_ndims(self.id):             # <<<<<<<<<<<<<<
 *             raise ValueError("Coordinate array must have shape (<npoints>, %d)" % self.get_simple_extent_ndims())
 * 
 */
  __pyx_t_2 = (__pyx_v_hcoords->nd != 2);
  if (!__pyx_t_2) {
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_4 = ((__pyx_v_hcoords->dimensions[1]) != __pyx_t_3);
    __pyx_t_5 = __pyx_t_4;
  } else {
    __pyx_t_5 = __pyx_t_2;
  }
  if (__pyx_t_5) {

    /* "h5py/h5s.pyx":469
 * 
 *         if hcoords.nd != 2 or hcoords.dimensions[1] != H5Sget_simple_extent_ndims(self.id):
 *             raise ValueError("Coordinate array must have shape (<npoints>, %d)" % self.get_simple_extent_ndims())             # <<<<<<<<<<<<<<
 * 
 *         nelements = hcoords.dimensions[0]
 */
    __pyx_t_1 = PyObject_GetAttr(((PyObject *)__pyx_v_self), __pyx_n_s_7); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyObject_Call(__pyx_t_1, ((PyObject *)__pyx_empty_tuple), NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Remainder(((PyObject *)__pyx_kp_s_6), __pyx_t_6); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_1));
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)__pyx_t_1));
    __Pyx_GIVEREF(((PyObject *)__pyx_t_1));
    __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_Call(__pyx_builtin_ValueError, ((PyObject *)__pyx_t_6), NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(((PyObject *)__pyx_t_6)); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 469; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/h5s.pyx":471
 *             raise ValueError("Coordinate array must have shape (<npoints>, %d)" % self.get_simple_extent_ndims())
 * 
 *         nelements = hcoords.dimensions[0]             # <<<<<<<<<<<<<<
 * 
 *         H5Sselect_elements(self.id, <H5S_seloper_t>op, nelements, <hsize_t**>hcoords.data)
 */
  __pyx_v_nelements = (__pyx_v_hcoords->dimensions[0]);

  /* "h5py/h5s.pyx":473
 *         nelements = hcoords.dimensions[0]
 * 
 *         H5Sselect_elements(self.id, <H5S_seloper_t>op, nelements, <hsize_t**>hcoords.data)             # <<<<<<<<<<<<<<
 * 
 *     # === Hyperslab selection functions =======================================
 */
  __pyx_t_7 = __pyx_f_4h5py_4defs_H5Sselect_elements(__pyx_v_self->__pyx_base.id, ((H5S_seloper_t)__pyx_v_op), __pyx_v_nelements, ((hsize_t **)__pyx_v_hcoords->data)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_elements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_hcoords);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_47get_select_hyper_nblocks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_46get_select_hyper_nblocks[] = "() => LONG nblocks\n\n        Get the number of hyperslab blocks currently selected.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_47get_select_hyper_nblocks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_hyper_nblocks (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_46get_select_hyper_nblocks(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":478
 * 
 * 
 *     def get_select_hyper_nblocks(self):             # <<<<<<<<<<<<<<
 *         """() => LONG nblocks
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_46get_select_hyper_nblocks(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_hyper_nblocks", 0);

  /* "h5py/h5s.pyx":483
 *         Get the number of hyperslab blocks currently selected.
 *         """
 *         return H5Sget_select_hyper_nblocks(self.id)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_select_hyper_nblocks(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 483; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyInt_to_py_hssize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 483; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_hyper_nblocks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_49get_select_hyper_blocklist(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_48get_select_hyper_blocklist[] = "() => NDARRAY\n\n        Get the current hyperslab selection.  The returned array has shape::\n\n            (<npoints>, 2, <rank>)\n\n        and can be interpreted as a nested sequence::\n\n            [ (corner_coordinate_1, opposite_coordinate_1), ... ]\n\n        with length equal to the total number of blocks.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_49get_select_hyper_blocklist(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_select_hyper_blocklist (wrapper)", 0);
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_48get_select_hyper_blocklist(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":486
 * 
 * 
 *     def get_select_hyper_blocklist(self):             # <<<<<<<<<<<<<<
 *         """() => NDARRAY
 * 
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_48get_select_hyper_blocklist(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self) {
  hsize_t __pyx_v_dims[3];
  PyArrayObject *__pyx_v_buf = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  hssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  herr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_select_hyper_blocklist", 0);

  /* "h5py/h5s.pyx":502
 *         cdef ndarray buf
 * 
 *         dims[0] = H5Sget_select_hyper_nblocks(self.id)             # <<<<<<<<<<<<<<
 *         dims[1] = 2
 *         dims[2] = H5Sget_simple_extent_ndims(self.id)
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_select_hyper_nblocks(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 502; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_dims[0]) = __pyx_t_1;

  /* "h5py/h5s.pyx":503
 * 
 *         dims[0] = H5Sget_select_hyper_nblocks(self.id)
 *         dims[1] = 2             # <<<<<<<<<<<<<<
 *         dims[2] = H5Sget_simple_extent_ndims(self.id)
 * 
 */
  (__pyx_v_dims[1]) = 2;

  /* "h5py/h5s.pyx":504
 *         dims[0] = H5Sget_select_hyper_nblocks(self.id)
 *         dims[1] = 2
 *         dims[2] = H5Sget_simple_extent_ndims(self.id)             # <<<<<<<<<<<<<<
 * 
 *         buf = create_numpy_hsize(3, dims)
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 504; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  (__pyx_v_dims[2]) = __pyx_t_2;

  /* "h5py/h5s.pyx":506
 *         dims[2] = H5Sget_simple_extent_ndims(self.id)
 * 
 *         buf = create_numpy_hsize(3, dims)             # <<<<<<<<<<<<<<
 * 
 *         H5Sget_select_hyper_blocklist(self.id, 0, dims[0], <hsize_t*>buf.data)
 */
  __pyx_t_3 = __pyx_f_4h5py_5utils_create_numpy_hsize(3, __pyx_v_dims); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 506; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_4h5py_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 506; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_buf = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "h5py/h5s.pyx":508
 *         buf = create_numpy_hsize(3, dims)
 * 
 *         H5Sget_select_hyper_blocklist(self.id, 0, dims[0], <hsize_t*>buf.data)             # <<<<<<<<<<<<<<
 * 
 *         return buf
 */
  __pyx_t_4 = __pyx_f_4h5py_4defs_H5Sget_select_hyper_blocklist(__pyx_v_self->__pyx_base.id, 0, (__pyx_v_dims[0]), ((hsize_t *)__pyx_v_buf->data)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 508; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":510
 *         H5Sget_select_hyper_blocklist(self.id, 0, dims[0], <hsize_t*>buf.data)
 * 
 *         return buf             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_buf));
  __pyx_r = ((PyObject *)__pyx_v_buf);
  goto __pyx_L0;

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("h5py.h5s.SpaceID.get_select_hyper_blocklist", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_51select_hyperslab(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4h5py_3h5s_7SpaceID_50select_hyperslab[] = "(TUPLE start, TUPLE count, TUPLE stride=None, TUPLE block=None,\n             INT op=SELECT_SET)\n\n        Select a block region from an existing dataspace.  See the HDF5\n        documentation for the meaning of the \"block\" and \"op\" keywords.\n        ";
static PyObject *__pyx_pw_4h5py_3h5s_7SpaceID_51select_hyperslab(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_block = 0;
  int __pyx_v_op;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("select_hyperslab (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s__start,&__pyx_n_s__count,&__pyx_n_s__stride,&__pyx_n_s__block,&__pyx_n_s__op,0};
    PyObject* values[5] = {0,0,0,0,0};

    /* "h5py/h5s.pyx":513
 * 
 * 
 *     def select_hyperslab(self, object start, object count, object stride=None,             # <<<<<<<<<<<<<<
 *                          object block=None, int op=H5S_SELECT_SET):
 *         """(TUPLE start, TUPLE count, TUPLE stride=None, TUPLE block=None,
 */
    values[2] = ((PyObject *)Py_None);

    /* "h5py/h5s.pyx":514
 * 
 *     def select_hyperslab(self, object start, object count, object stride=None,
 *                          object block=None, int op=H5S_SELECT_SET):             # <<<<<<<<<<<<<<
 *         """(TUPLE start, TUPLE count, TUPLE stride=None, TUPLE block=None,
 *              INT op=SELECT_SET)
 */
    values[3] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__start)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s__count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("select_hyperslab", 0, 2, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 513; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__stride);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__block);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s__op);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "select_hyperslab") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 513; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_start = values[0];
    __pyx_v_count = values[1];
    __pyx_v_stride = values[2];
    __pyx_v_block = values[3];
    if (values[4]) {
      __pyx_v_op = __Pyx_PyInt_AsInt(values[4]); if (unlikely((__pyx_v_op == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 514; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_op = __pyx_k_8;
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("select_hyperslab", 0, 2, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 513; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_hyperslab", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4h5py_3h5s_7SpaceID_50select_hyperslab(((struct __pyx_obj_4h5py_3h5s_SpaceID *)__pyx_v_self), __pyx_v_start, __pyx_v_count, __pyx_v_stride, __pyx_v_block, __pyx_v_op);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/h5s.pyx":513
 * 
 * 
 *     def select_hyperslab(self, object start, object count, object stride=None,             # <<<<<<<<<<<<<<
 *                          object block=None, int op=H5S_SELECT_SET):
 *         """(TUPLE start, TUPLE count, TUPLE stride=None, TUPLE block=None,
 */

static PyObject *__pyx_pf_4h5py_3h5s_7SpaceID_50select_hyperslab(struct __pyx_obj_4h5py_3h5s_SpaceID *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_count, PyObject *__pyx_v_stride, PyObject *__pyx_v_block, int __pyx_v_op) {
  int __pyx_v_rank;
  hsize_t *__pyx_v_start_array;
  hsize_t *__pyx_v_count_array;
  hsize_t *__pyx_v_stride_array;
  hsize_t *__pyx_v_block_array;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  int __pyx_t_3;
  herr_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("select_hyperslab", 0);

  /* "h5py/h5s.pyx":522
 *         """
 *         cdef int rank
 *         cdef hsize_t* start_array = NULL             # <<<<<<<<<<<<<<
 *         cdef hsize_t* count_array = NULL
 *         cdef hsize_t* stride_array = NULL
 */
  __pyx_v_start_array = NULL;

  /* "h5py/h5s.pyx":523
 *         cdef int rank
 *         cdef hsize_t* start_array = NULL
 *         cdef hsize_t* count_array = NULL             # <<<<<<<<<<<<<<
 *         cdef hsize_t* stride_array = NULL
 *         cdef hsize_t* block_array = NULL
 */
  __pyx_v_count_array = NULL;

  /* "h5py/h5s.pyx":524
 *         cdef hsize_t* start_array = NULL
 *         cdef hsize_t* count_array = NULL
 *         cdef hsize_t* stride_array = NULL             # <<<<<<<<<<<<<<
 *         cdef hsize_t* block_array = NULL
 * 
 */
  __pyx_v_stride_array = NULL;

  /* "h5py/h5s.pyx":525
 *         cdef hsize_t* count_array = NULL
 *         cdef hsize_t* stride_array = NULL
 *         cdef hsize_t* block_array = NULL             # <<<<<<<<<<<<<<
 * 
 *         # Dataspace rank.  All provided tuples must match this.
 */
  __pyx_v_block_array = NULL;

  /* "h5py/h5s.pyx":528
 * 
 *         # Dataspace rank.  All provided tuples must match this.
 *         rank = H5Sget_simple_extent_ndims(self.id)             # <<<<<<<<<<<<<<
 * 
 *         require_tuple(start, 0, rank, "start")
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims(__pyx_v_self->__pyx_base.id); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_rank = __pyx_t_1;

  /* "h5py/h5s.pyx":530
 *         rank = H5Sget_simple_extent_ndims(self.id)
 * 
 *         require_tuple(start, 0, rank, "start")             # <<<<<<<<<<<<<<
 *         require_tuple(count, 0, rank, "count")
 *         require_tuple(stride, 1, rank, "stride")
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_start, 0, __pyx_v_rank, __pyx_k__start); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":531
 * 
 *         require_tuple(start, 0, rank, "start")
 *         require_tuple(count, 0, rank, "count")             # <<<<<<<<<<<<<<
 *         require_tuple(stride, 1, rank, "stride")
 *         require_tuple(block, 1, rank, "block")
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_count, 0, __pyx_v_rank, __pyx_k__count); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":532
 *         require_tuple(start, 0, rank, "start")
 *         require_tuple(count, 0, rank, "count")
 *         require_tuple(stride, 1, rank, "stride")             # <<<<<<<<<<<<<<
 *         require_tuple(block, 1, rank, "block")
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_stride, 1, __pyx_v_rank, __pyx_k__stride); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":533
 *         require_tuple(count, 0, rank, "count")
 *         require_tuple(stride, 1, rank, "stride")
 *         require_tuple(block, 1, rank, "block")             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_1 = __pyx_f_4h5py_5utils_require_tuple(__pyx_v_block, 1, __pyx_v_rank, __pyx_k__block); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":535
 *         require_tuple(block, 1, rank, "block")
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             start_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             count_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 */
  /*try:*/ {

    /* "h5py/h5s.pyx":536
 * 
 *         try:
 *             start_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *             count_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(start, start_array, rank)
 */
    __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 536; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_start_array = ((hsize_t *)__pyx_t_2);

    /* "h5py/h5s.pyx":537
 *         try:
 *             start_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             count_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *             convert_tuple(start, start_array, rank)
 *             convert_tuple(count, count_array, rank)
 */
    __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 537; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_count_array = ((hsize_t *)__pyx_t_2);

    /* "h5py/h5s.pyx":538
 *             start_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             count_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(start, start_array, rank)             # <<<<<<<<<<<<<<
 *             convert_tuple(count, count_array, rank)
 * 
 */
    __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_start, __pyx_v_start_array, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 538; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5s.pyx":539
 *             count_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *             convert_tuple(start, start_array, rank)
 *             convert_tuple(count, count_array, rank)             # <<<<<<<<<<<<<<
 * 
 *             if stride is not None:
 */
    __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_count, __pyx_v_count_array, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 539; __pyx_clineno = __LINE__; goto __pyx_L4;}

    /* "h5py/h5s.pyx":541
 *             convert_tuple(count, count_array, rank)
 * 
 *             if stride is not None:             # <<<<<<<<<<<<<<
 *                 stride_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(stride, stride_array, rank)
 */
    __pyx_t_3 = (__pyx_v_stride != Py_None);
    if (__pyx_t_3) {

      /* "h5py/h5s.pyx":542
 * 
 *             if stride is not None:
 *                 stride_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *                 convert_tuple(stride, stride_array, rank)
 *             if block is not None:
 */
      __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 542; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_stride_array = ((hsize_t *)__pyx_t_2);

      /* "h5py/h5s.pyx":543
 *             if stride is not None:
 *                 stride_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(stride, stride_array, rank)             # <<<<<<<<<<<<<<
 *             if block is not None:
 *                 block_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 */
      __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_stride, __pyx_v_stride_array, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L6;
    }
    __pyx_L6:;

    /* "h5py/h5s.pyx":544
 *                 stride_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(stride, stride_array, rank)
 *             if block is not None:             # <<<<<<<<<<<<<<
 *                 block_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(block, block_array, rank)
 */
    __pyx_t_3 = (__pyx_v_block != Py_None);
    if (__pyx_t_3) {

      /* "h5py/h5s.pyx":545
 *                 convert_tuple(stride, stride_array, rank)
 *             if block is not None:
 *                 block_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)             # <<<<<<<<<<<<<<
 *                 convert_tuple(block, block_array, rank)
 * 
 */
      __pyx_t_2 = __pyx_f_4h5py_5utils_emalloc(((sizeof(hsize_t)) * __pyx_v_rank)); if (unlikely(__pyx_t_2 == NULL && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 545; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_block_array = ((hsize_t *)__pyx_t_2);

      /* "h5py/h5s.pyx":546
 *             if block is not None:
 *                 block_array = <hsize_t*>emalloc(sizeof(hsize_t)*rank)
 *                 convert_tuple(block, block_array, rank)             # <<<<<<<<<<<<<<
 * 
 *             H5Sselect_hyperslab(self.id, <H5S_seloper_t>op, start_array,
 */
      __pyx_t_1 = __pyx_f_4h5py_5utils_convert_tuple(__pyx_v_block, __pyx_v_block_array, __pyx_v_rank); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 546; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L7;
    }
    __pyx_L7:;

    /* "h5py/h5s.pyx":549
 * 
 *             H5Sselect_hyperslab(self.id, <H5S_seloper_t>op, start_array,
 *                                          stride_array, count_array, block_array)             # <<<<<<<<<<<<<<
 * 
 *         finally:
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Sselect_hyperslab(__pyx_v_self->__pyx_base.id, ((H5S_seloper_t)__pyx_v_op), __pyx_v_start_array, __pyx_v_stride_array, __pyx_v_count_array, __pyx_v_block_array); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 548; __pyx_clineno = __LINE__; goto __pyx_L4;}
  }

  /* "h5py/h5s.pyx":552
 * 
 *         finally:
 *             efree(start_array)             # <<<<<<<<<<<<<<
 *             efree(count_array)
 *             efree(stride_array)
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    __pyx_f_4h5py_5utils_efree(__pyx_v_start_array);

    /* "h5py/h5s.pyx":553
 *         finally:
 *             efree(start_array)
 *             efree(count_array)             # <<<<<<<<<<<<<<
 *             efree(stride_array)
 *             efree(block_array)
 */
    __pyx_f_4h5py_5utils_efree(__pyx_v_count_array);

    /* "h5py/h5s.pyx":554
 *             efree(start_array)
 *             efree(count_array)
 *             efree(stride_array)             # <<<<<<<<<<<<<<
 *             efree(block_array)
 * 
 */
    __pyx_f_4h5py_5utils_efree(__pyx_v_stride_array);

    /* "h5py/h5s.pyx":555
 *             efree(count_array)
 *             efree(stride_array)
 *             efree(block_array)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_f_4h5py_5utils_efree(__pyx_v_block_array);
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py.h5s.SpaceID.select_hyperslab", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_4h5py_3h5s_SpaceID(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o = __pyx_ptype_4h5py_8_objects_ObjectID->tp_new(t, a, k);
  if (!o) return 0;
  return o;
}

static void __pyx_tp_dealloc_4h5py_3h5s_SpaceID(PyObject *o) {
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_4h5py_8_objects_ObjectID)) __pyx_ptype_4h5py_8_objects_ObjectID->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_4h5py_3h5s_SpaceID);
}

static int __pyx_tp_traverse_4h5py_3h5s_SpaceID(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_4h5py_8_objects_ObjectID)) ? ((__pyx_ptype_4h5py_8_objects_ObjectID->tp_traverse) ? __pyx_ptype_4h5py_8_objects_ObjectID->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4h5py_3h5s_SpaceID)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_4h5py_3h5s_SpaceID(PyObject *o) {
  if (likely(__pyx_ptype_4h5py_8_objects_ObjectID)) { if (__pyx_ptype_4h5py_8_objects_ObjectID->tp_clear) __pyx_ptype_4h5py_8_objects_ObjectID->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4h5py_3h5s_SpaceID);
  return 0;
}

static PyObject *__pyx_getprop_4h5py_3h5s_7SpaceID_shape(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4h5py_3h5s_7SpaceID_5shape_1__get__(o);
}

static PyMethodDef __pyx_methods_4h5py_3h5s_SpaceID[] = {
  {__Pyx_NAMESTR("_close"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_1_close, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID__close)},
  {__Pyx_NAMESTR("copy"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_3copy, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_2copy)},
  {__Pyx_NAMESTR("encode"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_5encode, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_4encode)},
  {__Pyx_NAMESTR("__reduce__"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_7__reduce__, METH_NOARGS, __Pyx_DOCSTR(0)},
  {__Pyx_NAMESTR("__setstate__"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_9__setstate__, METH_O, __Pyx_DOCSTR(0)},
  {__Pyx_NAMESTR("is_simple"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_11is_simple, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_10is_simple)},
  {__Pyx_NAMESTR("offset_simple"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_13offset_simple, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_12offset_simple)},
  {__Pyx_NAMESTR("get_simple_extent_ndims"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_15get_simple_extent_ndims, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_14get_simple_extent_ndims)},
  {__Pyx_NAMESTR("get_simple_extent_dims"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_17get_simple_extent_dims, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_16get_simple_extent_dims)},
  {__Pyx_NAMESTR("get_simple_extent_npoints"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_19get_simple_extent_npoints, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_18get_simple_extent_npoints)},
  {__Pyx_NAMESTR("get_simple_extent_type"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_21get_simple_extent_type, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_20get_simple_extent_type)},
  {__Pyx_NAMESTR("extent_copy"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_23extent_copy, METH_O, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_22extent_copy)},
  {__Pyx_NAMESTR("set_extent_simple"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_25set_extent_simple, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_24set_extent_simple)},
  {__Pyx_NAMESTR("set_extent_none"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_27set_extent_none, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_26set_extent_none)},
  {__Pyx_NAMESTR("get_select_type"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_29get_select_type, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_28get_select_type)},
  {__Pyx_NAMESTR("get_select_npoints"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_31get_select_npoints, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_30get_select_npoints)},
  {__Pyx_NAMESTR("get_select_bounds"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_33get_select_bounds, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_32get_select_bounds)},
  {__Pyx_NAMESTR("select_all"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_35select_all, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_34select_all)},
  {__Pyx_NAMESTR("select_none"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_37select_none, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_36select_none)},
  {__Pyx_NAMESTR("select_valid"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_39select_valid, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_38select_valid)},
  {__Pyx_NAMESTR("get_select_elem_npoints"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_41get_select_elem_npoints, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_40get_select_elem_npoints)},
  {__Pyx_NAMESTR("get_select_elem_pointlist"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_43get_select_elem_pointlist, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_42get_select_elem_pointlist)},
  {__Pyx_NAMESTR("select_elements"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_45select_elements, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_44select_elements)},
  {__Pyx_NAMESTR("get_select_hyper_nblocks"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_47get_select_hyper_nblocks, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_46get_select_hyper_nblocks)},
  {__Pyx_NAMESTR("get_select_hyper_blocklist"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_49get_select_hyper_blocklist, METH_NOARGS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_48get_select_hyper_blocklist)},
  {__Pyx_NAMESTR("select_hyperslab"), (PyCFunction)__pyx_pw_4h5py_3h5s_7SpaceID_51select_hyperslab, METH_VARARGS|METH_KEYWORDS, __Pyx_DOCSTR(__pyx_doc_4h5py_3h5s_7SpaceID_50select_hyperslab)},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4h5py_3h5s_SpaceID[] = {
  {(char *)"shape", __pyx_getprop_4h5py_3h5s_7SpaceID_shape, 0, __Pyx_DOCSTR(__pyx_k_9), 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_SpaceID = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  #if PY_VERSION_HEX >= 0x02050000
  0, /*nb_index*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_SpaceID = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_SpaceID = {
  0, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_SpaceID = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_getbuffer*/
  #endif
  #if PY_VERSION_HEX >= 0x02060000
  0, /*bf_releasebuffer*/
  #endif
};

static PyTypeObject __pyx_type_4h5py_3h5s_SpaceID = {
  PyVarObject_HEAD_INIT(0, 0)
  __Pyx_NAMESTR("h5py.h5s.SpaceID"), /*tp_name*/
  sizeof(struct __pyx_obj_4h5py_3h5s_SpaceID), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4h5py_3h5s_SpaceID, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #else
  0, /*reserved*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_SpaceID, /*tp_as_number*/
  &__pyx_tp_as_sequence_SpaceID, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_SpaceID, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_SpaceID, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  __Pyx_DOCSTR("\n        Represents a dataspace identifier.\n\n        Properties:\n\n        shape\n            Numpy-style shape tuple with dimensions.\n\n        * Hashable: No\n        * Equality: Unimplemented\n\n        Can be pickled if HDF5 1.8 is available.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4h5py_3h5s_SpaceID, /*tp_traverse*/
  __pyx_tp_clear_4h5py_3h5s_SpaceID, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4h5py_3h5s_SpaceID, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4h5py_3h5s_SpaceID, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4h5py_3h5s_SpaceID, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  #if PY_VERSION_HEX >= 0x02060000
  0, /*tp_version_tag*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    __Pyx_NAMESTR("h5s"),
    __Pyx_DOCSTR(__pyx_k_10), /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 1, 1},
  {&__pyx_kp_s_13, __pyx_k_13, sizeof(__pyx_k_13), 0, 0, 1, 0},
  {&__pyx_n_s_14, __pyx_k_14, sizeof(__pyx_k_14), 0, 0, 1, 1},
  {&__pyx_kp_u_19, __pyx_k_19, sizeof(__pyx_k_19), 0, 1, 0, 0},
  {&__pyx_kp_u_20, __pyx_k_20, sizeof(__pyx_k_20), 0, 1, 0, 0},
  {&__pyx_kp_s_4, __pyx_k_4, sizeof(__pyx_k_4), 0, 0, 1, 0},
  {&__pyx_kp_s_6, __pyx_k_6, sizeof(__pyx_k_6), 0, 0, 1, 0},
  {&__pyx_n_s_7, __pyx_k_7, sizeof(__pyx_k_7), 0, 0, 1, 1},
  {&__pyx_n_s__ALL, __pyx_k__ALL, sizeof(__pyx_k__ALL), 0, 0, 1, 1},
  {&__pyx_n_s__NO_CLASS, __pyx_k__NO_CLASS, sizeof(__pyx_k__NO_CLASS), 0, 0, 1, 1},
  {&__pyx_n_s__SCALAR, __pyx_k__SCALAR, sizeof(__pyx_k__SCALAR), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_AND, __pyx_k__SELECT_AND, sizeof(__pyx_k__SELECT_AND), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_APPEND, __pyx_k__SELECT_APPEND, sizeof(__pyx_k__SELECT_APPEND), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_INVALID, __pyx_k__SELECT_INVALID, sizeof(__pyx_k__SELECT_INVALID), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_NOOP, __pyx_k__SELECT_NOOP, sizeof(__pyx_k__SELECT_NOOP), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_NOTA, __pyx_k__SELECT_NOTA, sizeof(__pyx_k__SELECT_NOTA), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_NOTB, __pyx_k__SELECT_NOTB, sizeof(__pyx_k__SELECT_NOTB), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_OR, __pyx_k__SELECT_OR, sizeof(__pyx_k__SELECT_OR), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_PREPEND, __pyx_k__SELECT_PREPEND, sizeof(__pyx_k__SELECT_PREPEND), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_SET, __pyx_k__SELECT_SET, sizeof(__pyx_k__SELECT_SET), 0, 0, 1, 1},
  {&__pyx_n_s__SELECT_XOR, __pyx_k__SELECT_XOR, sizeof(__pyx_k__SELECT_XOR), 0, 0, 1, 1},
  {&__pyx_n_s__SEL_ALL, __pyx_k__SEL_ALL, sizeof(__pyx_k__SEL_ALL), 0, 0, 1, 1},
  {&__pyx_n_s__SEL_ERROR, __pyx_k__SEL_ERROR, sizeof(__pyx_k__SEL_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s__SEL_HYPERSLABS, __pyx_k__SEL_HYPERSLABS, sizeof(__pyx_k__SEL_HYPERSLABS), 0, 0, 1, 1},
  {&__pyx_n_s__SEL_NONE, __pyx_k__SEL_NONE, sizeof(__pyx_k__SEL_NONE), 0, 0, 1, 1},
  {&__pyx_n_s__SEL_POINTS, __pyx_k__SEL_POINTS, sizeof(__pyx_k__SEL_POINTS), 0, 0, 1, 1},
  {&__pyx_n_s__SIMPLE, __pyx_k__SIMPLE, sizeof(__pyx_k__SIMPLE), 0, 0, 1, 1},
  {&__pyx_n_s__UNLIMITED, __pyx_k__UNLIMITED, sizeof(__pyx_k__UNLIMITED), 0, 0, 1, 1},
  {&__pyx_n_s__ValueError, __pyx_k__ValueError, sizeof(__pyx_k__ValueError), 0, 0, 1, 1},
  {&__pyx_n_s___NULL, __pyx_k___NULL, sizeof(__pyx_k___NULL), 0, 0, 1, 1},
  {&__pyx_n_s____enter__, __pyx_k____enter__, sizeof(__pyx_k____enter__), 0, 0, 1, 1},
  {&__pyx_n_s____exit__, __pyx_k____exit__, sizeof(__pyx_k____exit__), 0, 0, 1, 1},
  {&__pyx_n_s____main__, __pyx_k____main__, sizeof(__pyx_k____main__), 0, 0, 1, 1},
  {&__pyx_n_s____test__, __pyx_k____test__, sizeof(__pyx_k____test__), 0, 0, 1, 1},
  {&__pyx_n_s___objects, __pyx_k___objects, sizeof(__pyx_k___objects), 0, 0, 1, 1},
  {&__pyx_n_s__block, __pyx_k__block, sizeof(__pyx_k__block), 0, 0, 1, 1},
  {&__pyx_n_s__buf, __pyx_k__buf, sizeof(__pyx_k__buf), 0, 0, 1, 1},
  {&__pyx_n_s__buf_, __pyx_k__buf_, sizeof(__pyx_k__buf_), 0, 0, 1, 1},
  {&__pyx_n_s__class_code, __pyx_k__class_code, sizeof(__pyx_k__class_code), 0, 0, 1, 1},
  {&__pyx_n_s__coords, __pyx_k__coords, sizeof(__pyx_k__coords), 0, 0, 1, 1},
  {&__pyx_n_s__count, __pyx_k__count, sizeof(__pyx_k__count), 0, 0, 1, 1},
  {&__pyx_n_s__create, __pyx_k__create, sizeof(__pyx_k__create), 0, 0, 1, 1},
  {&__pyx_n_s__create_simple, __pyx_k__create_simple, sizeof(__pyx_k__create_simple), 0, 0, 1, 1},
  {&__pyx_n_s__decode, __pyx_k__decode, sizeof(__pyx_k__decode), 0, 0, 1, 1},
  {&__pyx_n_s__dims, __pyx_k__dims, sizeof(__pyx_k__dims), 0, 0, 1, 1},
  {&__pyx_n_s__dims_tpl, __pyx_k__dims_tpl, sizeof(__pyx_k__dims_tpl), 0, 0, 1, 1},
  {&__pyx_n_s__encode, __pyx_k__encode, sizeof(__pyx_k__encode), 0, 0, 1, 1},
  {&__pyx_n_s__h5py, __pyx_k__h5py, sizeof(__pyx_k__h5py), 0, 0, 1, 1},
  {&__pyx_n_s__lock, __pyx_k__lock, sizeof(__pyx_k__lock), 0, 0, 1, 1},
  {&__pyx_n_s__max_dims, __pyx_k__max_dims, sizeof(__pyx_k__max_dims), 0, 0, 1, 1},
  {&__pyx_n_s__max_dims_tpl, __pyx_k__max_dims_tpl, sizeof(__pyx_k__max_dims_tpl), 0, 0, 1, 1},
  {&__pyx_n_s__maxdims, __pyx_k__maxdims, sizeof(__pyx_k__maxdims), 0, 0, 1, 1},
  {&__pyx_n_s__offset, __pyx_k__offset, sizeof(__pyx_k__offset), 0, 0, 1, 1},
  {&__pyx_n_s__op, __pyx_k__op, sizeof(__pyx_k__op), 0, 0, 1, 1},
  {&__pyx_n_s__open, __pyx_k__open, sizeof(__pyx_k__open), 0, 0, 1, 1},
  {&__pyx_n_s__rank, __pyx_k__rank, sizeof(__pyx_k__rank), 0, 0, 1, 1},
  {&__pyx_n_s__registry, __pyx_k__registry, sizeof(__pyx_k__registry), 0, 0, 1, 1},
  {&__pyx_n_s__start, __pyx_k__start, sizeof(__pyx_k__start), 0, 0, 1, 1},
  {&__pyx_n_s__stride, __pyx_k__stride, sizeof(__pyx_k__stride), 0, 0, 1, 1},
  {&__pyx_n_s__valid, __pyx_k__valid, sizeof(__pyx_k__valid), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetName(__pyx_b, __pyx_n_s__ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "h5py/h5s.pyx":144
 *         when their Python wrappers are freed.
 *         """
 *         with _objects.registry.lock:             # <<<<<<<<<<<<<<
 *             H5Sclose(self.id)
 *             if not self.valid:
 */
  __pyx_k_tuple_2 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_k_tuple_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_2);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_2));

  /* "h5py/h5s.pyx":179
 * 
 *     def __reduce__(self):
 *         return (type(self), (-1,), self.encode())             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_k_tuple_3 = PyTuple_Pack(1, __pyx_int_neg_1); if (unlikely(!__pyx_k_tuple_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_3);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_3));

  /* "h5py/h5s.pyx":62
 * 
 * 
 * def create(int class_code):             # <<<<<<<<<<<<<<
 *     """(INT class_code) => SpaceID
 * 
 */
  __pyx_k_tuple_11 = PyTuple_Pack(2, ((PyObject *)__pyx_n_s__class_code), ((PyObject *)__pyx_n_s__class_code)); if (unlikely(!__pyx_k_tuple_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_11);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_11));
  __pyx_k_codeobj_12 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_13, __pyx_n_s__create, 62, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":71
 * 
 * 
 * def create_simple(object dims_tpl, object max_dims_tpl=None):             # <<<<<<<<<<<<<<
 *     """(TUPLE dims_tpl, TUPLE max_dims_tpl) => SpaceID
 * 
 */
  __pyx_k_tuple_15 = PyTuple_Pack(5, ((PyObject *)__pyx_n_s__dims_tpl), ((PyObject *)__pyx_n_s__max_dims_tpl), ((PyObject *)__pyx_n_s__rank), ((PyObject *)__pyx_n_s__dims), ((PyObject *)__pyx_n_s__max_dims)); if (unlikely(!__pyx_k_tuple_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_15);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_15));
  __pyx_k_codeobj_16 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_13, __pyx_n_s__create_simple, 71, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "h5py/h5s.pyx":103
 *         efree(max_dims)
 * 
 * def decode(buf):             # <<<<<<<<<<<<<<
 *     """(STRING buf) => SpaceID
 * 
 */
  __pyx_k_tuple_17 = PyTuple_Pack(2, ((PyObject *)__pyx_n_s__buf), ((PyObject *)__pyx_n_s__buf_)); if (unlikely(!__pyx_k_tuple_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_17);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_17));
  __pyx_k_codeobj_18 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_k_tuple_17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_13, __pyx_n_s__decode, 103, __pyx_empty_bytes); if (unlikely(!__pyx_k_codeobj_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC inith5s(void); /*proto*/
PyMODINIT_FUNC inith5s(void)
#else
PyMODINIT_FUNC PyInit_h5s(void); /*proto*/
PyMODINIT_FUNC PyInit_h5s(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_h5s(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4(__Pyx_NAMESTR("h5s"), __pyx_methods, __Pyx_DOCSTR(__pyx_k_10), 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "h5py.h5s")) {
      if (unlikely(PyDict_SetItemString(modules, "h5py.h5s", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  __pyx_b = PyImport_AddModule(__Pyx_NAMESTR(__Pyx_BUILTIN_MODULE_NAME)); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_module_is_main_h5py__h5s) {
    if (__Pyx_SetAttrString(__pyx_m, "__name__", __pyx_n_s____main__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_ptype_4h5py_8_objects_ObjectID = __Pyx_ImportType("h5py._objects", "ObjectID", sizeof(struct __pyx_obj_4h5py_8_objects_ObjectID), 1); if (unlikely(!__pyx_ptype_4h5py_8_objects_ObjectID)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_4h5py_3h5s_SpaceID.tp_base = __pyx_ptype_4h5py_8_objects_ObjectID;
  if (PyType_Ready(&__pyx_type_4h5py_3h5s_SpaceID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_SetAttrString(__pyx_m, "SpaceID", (PyObject *)&__pyx_type_4h5py_3h5s_SpaceID) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_type_4h5py_3h5s_SpaceID.tp_weaklistoffset == 0) __pyx_type_4h5py_3h5s_SpaceID.tp_weaklistoffset = offsetof(struct __pyx_obj_4h5py_3h5s_SpaceID, __pyx_base.__weakref__);
  __pyx_ptype_4h5py_3h5s_SpaceID = &__pyx_type_4h5py_3h5s_SpaceID;
  /*--- Type import code ---*/
  __pyx_ptype_4h5py_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_dtype)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_4h5py_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_4h5py_5numpy_ndarray)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("h5py.defs"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Screate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Screate, "hid_t (H5S_class_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Scopy", (void (**)(void))&__pyx_f_4h5py_4defs_H5Scopy, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sclose", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sclose, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Screate_simple", (void (**)(void))&__pyx_f_4h5py_4defs_H5Screate_simple, "hid_t (int, hsize_t *, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sis_simple", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sis_simple, "htri_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Soffset_simple", (void (**)(void))&__pyx_f_4h5py_4defs_H5Soffset_simple, "herr_t (hid_t, hssize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_simple_extent_ndims", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_simple_extent_ndims, "int (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_simple_extent_dims", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_simple_extent_dims, "int (hid_t, hsize_t *, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_simple_extent_npoints", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_simple_extent_npoints, "hssize_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_simple_extent_type", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_simple_extent_type, "H5S_class_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sextent_copy", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sextent_copy, "herr_t (hid_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sset_extent_simple", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sset_extent_simple, "herr_t (hid_t, int, hsize_t *, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sset_extent_none", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sset_extent_none, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_type", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_type, "H5S_sel_type (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_npoints", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_npoints, "hssize_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_bounds", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_bounds, "herr_t (hid_t, hsize_t *, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sselect_all", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sselect_all, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sselect_none", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sselect_none, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sselect_valid", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sselect_valid, "htri_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_elem_npoints", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_elem_npoints, "hssize_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_elem_pointlist", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_elem_pointlist, "herr_t (hid_t, hsize_t, hsize_t, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sselect_elements", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sselect_elements, "herr_t (hid_t, H5S_seloper_t, size_t, hsize_t **)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_hyper_nblocks", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_hyper_nblocks, "hssize_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_hyper_blocklist", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_hyper_blocklist, "herr_t (hid_t, hsize_t, hsize_t, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sselect_hyperslab", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sselect_hyperslab, "herr_t (hid_t, H5S_seloper_t, hsize_t *, hsize_t *, hsize_t *, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sencode", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sencode, "herr_t (hid_t, void *, size_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sdecode", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sdecode, "hid_t (void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_ImportModule("h5py.utils"); if (!__pyx_t_2) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "emalloc", (void (**)(void))&__pyx_f_4h5py_5utils_emalloc, "void *(size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "efree", (void (**)(void))&__pyx_f_4h5py_5utils_efree, "void (void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "convert_tuple", (void (**)(void))&__pyx_f_4h5py_5utils_convert_tuple, "int (PyObject *, hsize_t *, hsize_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "convert_dims", (void (**)(void))&__pyx_f_4h5py_5utils_convert_dims, "PyObject *(hsize_t *, hsize_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "require_tuple", (void (**)(void))&__pyx_f_4h5py_5utils_require_tuple, "int (PyObject *, int, int, char *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "create_numpy_hsize", (void (**)(void))&__pyx_f_4h5py_5utils_create_numpy_hsize, "PyObject *(int, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_2, "create_hsize_array", (void (**)(void))&__pyx_f_4h5py_5utils_create_hsize_array, "PyObject *(PyObject *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  /*--- Execution code ---*/

  /* "h5py/h5s.pyx":19
 * from numpy cimport ndarray
 * 
 * from h5py import _objects             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_n_s___objects));
  PyList_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_n_s___objects));
  __Pyx_GIVEREF(((PyObject *)__pyx_n_s___objects));
  __pyx_t_4 = __Pyx_Import(((PyObject *)__pyx_n_s__h5py), ((PyObject *)__pyx_t_3), -1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(((PyObject *)__pyx_t_3)); __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_GetAttr(__pyx_t_4, __pyx_n_s___objects);
  if (__pyx_t_3 == NULL) {
    if (PyErr_ExceptionMatches(PyExc_AttributeError)) __Pyx_RaiseImportError(__pyx_n_s___objects);
    if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_GOTREF(__pyx_t_3);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s___objects, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":31
 * 
 * #enum H5S_seloper_t:
 * SELECT_NOOP     = H5S_SELECT_NOOP             # <<<<<<<<<<<<<<
 * SELECT_SET      = H5S_SELECT_SET
 * SELECT_OR       = H5S_SELECT_OR
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_NOOP); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_NOOP, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 31; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":32
 * #enum H5S_seloper_t:
 * SELECT_NOOP     = H5S_SELECT_NOOP
 * SELECT_SET      = H5S_SELECT_SET             # <<<<<<<<<<<<<<
 * SELECT_OR       = H5S_SELECT_OR
 * SELECT_AND      = H5S_SELECT_AND
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_SET); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_SET, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":33
 * SELECT_NOOP     = H5S_SELECT_NOOP
 * SELECT_SET      = H5S_SELECT_SET
 * SELECT_OR       = H5S_SELECT_OR             # <<<<<<<<<<<<<<
 * SELECT_AND      = H5S_SELECT_AND
 * SELECT_XOR      = H5S_SELECT_XOR
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_OR); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_OR, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 33; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":34
 * SELECT_SET      = H5S_SELECT_SET
 * SELECT_OR       = H5S_SELECT_OR
 * SELECT_AND      = H5S_SELECT_AND             # <<<<<<<<<<<<<<
 * SELECT_XOR      = H5S_SELECT_XOR
 * SELECT_NOTB     = H5S_SELECT_NOTB
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_AND); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_AND, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":35
 * SELECT_OR       = H5S_SELECT_OR
 * SELECT_AND      = H5S_SELECT_AND
 * SELECT_XOR      = H5S_SELECT_XOR             # <<<<<<<<<<<<<<
 * SELECT_NOTB     = H5S_SELECT_NOTB
 * SELECT_NOTA     = H5S_SELECT_NOTA
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_XOR); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_XOR, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":36
 * SELECT_AND      = H5S_SELECT_AND
 * SELECT_XOR      = H5S_SELECT_XOR
 * SELECT_NOTB     = H5S_SELECT_NOTB             # <<<<<<<<<<<<<<
 * SELECT_NOTA     = H5S_SELECT_NOTA
 * SELECT_APPEND   = H5S_SELECT_APPEND
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_NOTB); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_NOTB, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":37
 * SELECT_XOR      = H5S_SELECT_XOR
 * SELECT_NOTB     = H5S_SELECT_NOTB
 * SELECT_NOTA     = H5S_SELECT_NOTA             # <<<<<<<<<<<<<<
 * SELECT_APPEND   = H5S_SELECT_APPEND
 * SELECT_PREPEND  = H5S_SELECT_PREPEND
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_NOTA); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_NOTA, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":38
 * SELECT_NOTB     = H5S_SELECT_NOTB
 * SELECT_NOTA     = H5S_SELECT_NOTA
 * SELECT_APPEND   = H5S_SELECT_APPEND             # <<<<<<<<<<<<<<
 * SELECT_PREPEND  = H5S_SELECT_PREPEND
 * SELECT_INVALID  = H5S_SELECT_INVALID
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_APPEND); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 38; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_APPEND, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 38; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":39
 * SELECT_NOTA     = H5S_SELECT_NOTA
 * SELECT_APPEND   = H5S_SELECT_APPEND
 * SELECT_PREPEND  = H5S_SELECT_PREPEND             # <<<<<<<<<<<<<<
 * SELECT_INVALID  = H5S_SELECT_INVALID
 * 
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_PREPEND); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_PREPEND, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":40
 * SELECT_APPEND   = H5S_SELECT_APPEND
 * SELECT_PREPEND  = H5S_SELECT_PREPEND
 * SELECT_INVALID  = H5S_SELECT_INVALID             # <<<<<<<<<<<<<<
 * 
 * ALL = lockid(H5S_ALL)   # This is accepted in lieu of an actual identifier
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SELECT_INVALID); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SELECT_INVALID, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 40; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":42
 * SELECT_INVALID  = H5S_SELECT_INVALID
 * 
 * ALL = lockid(H5S_ALL)   # This is accepted in lieu of an actual identifier             # <<<<<<<<<<<<<<
 *                         # in functions like H5Dread, so wrap it.
 * UNLIMITED = H5S_UNLIMITED
 */
  __pyx_t_4 = __pyx_f_4h5py_3h5s_lockid(H5S_ALL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__ALL, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":44
 * ALL = lockid(H5S_ALL)   # This is accepted in lieu of an actual identifier
 *                         # in functions like H5Dread, so wrap it.
 * UNLIMITED = H5S_UNLIMITED             # <<<<<<<<<<<<<<
 * 
 * #enum H5S_class_t
 */
  __pyx_t_4 = __Pyx_PyInt_to_py_hsize_t(H5S_UNLIMITED); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__UNLIMITED, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":47
 * 
 * #enum H5S_class_t
 * NO_CLASS = H5S_NO_CLASS             # <<<<<<<<<<<<<<
 * SCALAR   = H5S_SCALAR
 * SIMPLE   = H5S_SIMPLE
 */
  __pyx_t_4 = PyInt_FromLong(H5S_NO_CLASS); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__NO_CLASS, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":48
 * #enum H5S_class_t
 * NO_CLASS = H5S_NO_CLASS
 * SCALAR   = H5S_SCALAR             # <<<<<<<<<<<<<<
 * SIMPLE   = H5S_SIMPLE
 * _NULL = H5S_NULL
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SCALAR); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SCALAR, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":49
 * NO_CLASS = H5S_NO_CLASS
 * SCALAR   = H5S_SCALAR
 * SIMPLE   = H5S_SIMPLE             # <<<<<<<<<<<<<<
 * _NULL = H5S_NULL
 * 
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SIMPLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SIMPLE, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":50
 * SCALAR   = H5S_SCALAR
 * SIMPLE   = H5S_SIMPLE
 * _NULL = H5S_NULL             # <<<<<<<<<<<<<<
 * 
 * #enum H5S_sel_type
 */
  __pyx_t_4 = PyInt_FromLong(H5S_NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s___NULL, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":53
 * 
 * #enum H5S_sel_type
 * SEL_ERROR       = H5S_SEL_ERROR             # <<<<<<<<<<<<<<
 * SEL_NONE        = H5S_SEL_NONE
 * SEL_POINTS      = H5S_SEL_POINTS
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SEL_ERROR); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SEL_ERROR, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":54
 * #enum H5S_sel_type
 * SEL_ERROR       = H5S_SEL_ERROR
 * SEL_NONE        = H5S_SEL_NONE             # <<<<<<<<<<<<<<
 * SEL_POINTS      = H5S_SEL_POINTS
 * SEL_HYPERSLABS  = H5S_SEL_HYPERSLABS
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SEL_NONE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SEL_NONE, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":55
 * SEL_ERROR       = H5S_SEL_ERROR
 * SEL_NONE        = H5S_SEL_NONE
 * SEL_POINTS      = H5S_SEL_POINTS             # <<<<<<<<<<<<<<
 * SEL_HYPERSLABS  = H5S_SEL_HYPERSLABS
 * SEL_ALL         = H5S_SEL_ALL
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SEL_POINTS); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 55; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SEL_POINTS, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 55; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":56
 * SEL_NONE        = H5S_SEL_NONE
 * SEL_POINTS      = H5S_SEL_POINTS
 * SEL_HYPERSLABS  = H5S_SEL_HYPERSLABS             # <<<<<<<<<<<<<<
 * SEL_ALL         = H5S_SEL_ALL
 * 
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SEL_HYPERSLABS); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SEL_HYPERSLABS, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":57
 * SEL_POINTS      = H5S_SEL_POINTS
 * SEL_HYPERSLABS  = H5S_SEL_HYPERSLABS
 * SEL_ALL         = H5S_SEL_ALL             # <<<<<<<<<<<<<<
 * 
 * # === Basic dataspace operations ==============================================
 */
  __pyx_t_4 = PyInt_FromLong(H5S_SEL_ALL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 57; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__SEL_ALL, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 57; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":62
 * 
 * 
 * def create(int class_code):             # <<<<<<<<<<<<<<
 *     """(INT class_code) => SpaceID
 * 
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5s_1create, NULL, __pyx_n_s_14); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__create, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 62; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":71
 * 
 * 
 * def create_simple(object dims_tpl, object max_dims_tpl=None):             # <<<<<<<<<<<<<<
 *     """(TUPLE dims_tpl, TUPLE max_dims_tpl) => SpaceID
 * 
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5s_3create_simple, NULL, __pyx_n_s_14); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__create_simple, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":103
 *         efree(max_dims)
 * 
 * def decode(buf):             # <<<<<<<<<<<<<<
 *     """(STRING buf) => SpaceID
 * 
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4h5py_3h5s_5decode, NULL, __pyx_n_s_14); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s__decode, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "h5py/h5s.pyx":440
 * 
 * 
 *     def select_elements(self, object coords, int op=H5S_SELECT_SET):             # <<<<<<<<<<<<<<
 *         """(SEQUENCE coords, INT op=SELECT_SET)
 * 
 */
  __pyx_k_5 = H5S_SELECT_SET;

  /* "h5py/h5s.pyx":514
 * 
 *     def select_hyperslab(self, object start, object count, object stride=None,
 *                          object block=None, int op=H5S_SELECT_SET):             # <<<<<<<<<<<<<<
 *         """(TUPLE start, TUPLE count, TUPLE stride=None, TUPLE block=None,
 *              INT op=SELECT_SET)
 */
  __pyx_k_8 = H5S_SELECT_SET;

  /* "h5py/h5s.pyx":1
 * # This file is part of h5py, a Python interface to the HDF5 library.             # <<<<<<<<<<<<<<
 * #
 * # http://www.h5py.org
 */
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_4));
  if (PyDict_SetItem(__pyx_t_4, ((PyObject *)__pyx_kp_u_19), ((PyObject *)__pyx_kp_u_20)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s____test__, ((PyObject *)__pyx_t_4)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_4)); __pyx_t_4 = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    __Pyx_AddTraceback("init h5py.h5s", __pyx_clineno, __pyx_lineno, __pyx_filename);
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init h5py.h5s");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif /* CYTHON_REFNANNY */

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name) {
    PyObject *result;
    result = PyObject_GetAttr(dict, name);
    if (!result) {
        if (dict != __pyx_b) {
            PyErr_Clear();
            result = PyObject_GetAttr(__pyx_b, name);
        }
        if (!result) {
            PyErr_SetObject(PyExc_NameError, name);
        }
    }
    return result;
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%s() got an unexpected keyword argument '%s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%s() takes %s %" CYTHON_FORMAT_SSIZE_T "d positional argument%s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
        goto bad;
    #endif
    Py_INCREF(local_type);
    Py_INCREF(local_value);
    Py_INCREF(local_tb);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    /* Make sure tstate is in a consistent state when we XDECREF
       these objects (DECREF may run arbitrary code). */
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    #if PY_VERSION_HEX < 0x02050000
    if (PyClass_Check(type)) {
    #else
    if (PyType_Check(type)) {
    #endif
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        #if PY_VERSION_HEX < 0x02050000
            if (PyInstance_Check(type)) {
                type = (PyObject*) ((PyInstanceObject*)type)->in_class;
                Py_INCREF(type);
            }
            else {
                type = 0;
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception must be an old-style class or instance");
                goto raise_error;
            }
        #else
            type = (PyObject*) Py_TYPE(type);
            Py_INCREF(type);
            if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception class must be a subclass of BaseException");
                goto raise_error;
            }
        #endif
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else /* Python 3+ */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *args;
        if (!value)
            args = PyTuple_New(0);
        else if (PyTuple_Check(value)) {
            Py_INCREF(value);
            args = value;
        }
        else
            args = PyTuple_Pack(1, value);
        if (!args)
            goto bad;
        owned_instance = PyEval_CallObject(type, args);
        Py_DECREF(args);
        if (!owned_instance)
            goto bad;
        value = owned_instance;
        if (!PyExceptionInstance_Check(value)) {
            PyErr_Format(PyExc_TypeError,
                         "calling %R should have returned an instance of "
                         "BaseException, not %R",
                         type, Py_TYPE(value));
            goto bad;
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause && cause != Py_None) {
        PyObject *fixed_cause;
        if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        }
        else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        }
        else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (!type) {
        PyErr_Format(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (Py_TYPE(obj) == type) return 1;
    }
    else {
        if (PyObject_TypeCheck(obj, type)) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%s' has incorrect type (expected %s, got %s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import = 0;
    py_import = __Pyx_GetAttrString(__pyx_b, "__import__");
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    #if PY_VERSION_HEX >= 0x02050000
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0; /* try absolute import on failure */
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
    #else
    if (level>0) {
        PyErr_SetString(PyExc_RuntimeError, "Relative import is not supported for Python <=2.4.");
        goto bad;
    }
    module = PyObject_CallFunctionObjArgs(py_import,
        name, global_dict, empty_dict, list, NULL);
    #endif
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static CYTHON_INLINE void __Pyx_RaiseImportError(PyObject *name) {
#if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_ImportError, "cannot import name %.230s",
                 PyString_AsString(name));
#else
    PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
#endif
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hsize_t(hsize_t val) {
    const hsize_t neg_one = (hsize_t)-1, const_zero = (hsize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hsize_t) == sizeof(char))  ||
        (sizeof(hsize_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hsize_t) == sizeof(int)) ||
               (sizeof(hsize_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hsize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hsize_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hid_t(hid_t val) {
    const hid_t neg_one = (hid_t)-1, const_zero = (hid_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hid_t) == sizeof(char))  ||
        (sizeof(hid_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hid_t) == sizeof(int)) ||
               (sizeof(hid_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hid_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hid_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject *__Pyx_PyInt_to_py_hssize_t(hssize_t val) {
    const hssize_t neg_one = (hssize_t)-1, const_zero = (hssize_t)0;
    const int is_unsigned = const_zero < neg_one;
    if ((sizeof(hssize_t) == sizeof(char))  ||
        (sizeof(hssize_t) == sizeof(short))) {
        return PyInt_FromLong((long)val);
    } else if ((sizeof(hssize_t) == sizeof(int)) ||
               (sizeof(hssize_t) == sizeof(long))) {
        if (is_unsigned)
            return PyLong_FromUnsignedLong((unsigned long)val);
        else
            return PyInt_FromLong((long)val);
    } else if (sizeof(hssize_t) == sizeof(PY_LONG_LONG)) {
        if (is_unsigned)
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG)val);
        else
            return PyLong_FromLongLong((PY_LONG_LONG)val);
    } else {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        return _PyLong_FromByteArray(bytes, sizeof(hssize_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject* x) {
    const unsigned char neg_one = (unsigned char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned char" :
                    "value too large to convert to unsigned char");
            }
            return (unsigned char)-1;
        }
        return (unsigned char)val;
    }
    return (unsigned char)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject* x) {
    const unsigned short neg_one = (unsigned short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned short" :
                    "value too large to convert to unsigned short");
            }
            return (unsigned short)-1;
        }
        return (unsigned short)val;
    }
    return (unsigned short)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject* x) {
    const unsigned int neg_one = (unsigned int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned int" :
                    "value too large to convert to unsigned int");
            }
            return (unsigned int)-1;
        }
        return (unsigned int)val;
    }
    return (unsigned int)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject* x) {
    const char neg_one = (char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to char" :
                    "value too large to convert to char");
            }
            return (char)-1;
        }
        return (char)val;
    }
    return (char)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject* x) {
    const short neg_one = (short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to short" :
                    "value too large to convert to short");
            }
            return (short)-1;
        }
        return (short)val;
    }
    return (short)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject* x) {
    const signed char neg_one = (signed char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed char" :
                    "value too large to convert to signed char");
            }
            return (signed char)-1;
        }
        return (signed char)val;
    }
    return (signed char)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject* x) {
    const signed short neg_one = (signed short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed short" :
                    "value too large to convert to signed short");
            }
            return (signed short)-1;
        }
        return (signed short)val;
    }
    return (signed short)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject* x) {
    const signed int neg_one = (signed int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed int" :
                    "value too large to convert to signed int");
            }
            return (signed int)-1;
        }
        return (signed int)val;
    }
    return (signed int)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject* x) {
    const unsigned long neg_one = (unsigned long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned long");
            return (unsigned long)-1;
        }
        return (unsigned long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned long");
                return (unsigned long)-1;
            }
            return (unsigned long)PyLong_AsUnsignedLong(x);
        } else {
            return (unsigned long)PyLong_AsLong(x);
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned long)-1;
        val = __Pyx_PyInt_AsUnsignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject* x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned PY_LONG_LONG");
            return (unsigned PY_LONG_LONG)-1;
        }
        return (unsigned PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned PY_LONG_LONG");
                return (unsigned PY_LONG_LONG)-1;
            }
            return (unsigned PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (unsigned PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsUnsignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject* x) {
    const long neg_one = (long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to long");
            return (long)-1;
        }
        return (long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to long");
                return (long)-1;
            }
            return (long)PyLong_AsUnsignedLong(x);
        } else {
            return (long)PyLong_AsLong(x);
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long)-1;
        val = __Pyx_PyInt_AsLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject* x) {
    const PY_LONG_LONG neg_one = (PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to PY_LONG_LONG");
            return (PY_LONG_LONG)-1;
        }
        return (PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to PY_LONG_LONG");
                return (PY_LONG_LONG)-1;
            }
            return (PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject* x) {
    const signed long neg_one = (signed long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed long");
            return (signed long)-1;
        }
        return (signed long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed long");
                return (signed long)-1;
            }
            return (signed long)PyLong_AsUnsignedLong(x);
        } else {
            return (signed long)PyLong_AsLong(x);
        }
    } else {
        signed long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed long)-1;
        val = __Pyx_PyInt_AsSignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject* x) {
    const signed PY_LONG_LONG neg_one = (signed PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed PY_LONG_LONG");
            return (signed PY_LONG_LONG)-1;
        }
        return (signed PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed PY_LONG_LONG");
                return (signed PY_LONG_LONG)-1;
            }
            return (signed PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (signed PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        signed PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsSignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        #if PY_VERSION_HEX < 0x02050000
        return PyErr_Warn(NULL, message);
        #else
        return PyErr_WarnEx(NULL, message, 1);
        #endif
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%s.%s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        #if PY_VERSION_HEX < 0x02050000
        if (PyErr_Warn(NULL, warning) < 0) goto bad;
        #else
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
        #endif
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%s.%s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%s does not export expected C function %s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3 && PY_MINOR_VERSION==0)
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,            /*int argcount,*/
        0,            /*int kwonlyargcount,*/
        0,            /*int nlocals,*/
        0,            /*int stacksize,*/
        0,            /*int flags,*/
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,      /*int firstlineno,*/
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_globals = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_globals = PyModule_GetDict(__pyx_m);
    if (!py_globals) goto bad;
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        py_globals,          /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else  /* Python 3+ has unicode identifiers */
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_VERSION_HEX < 0x03000000
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_VERSION_HEX < 0x03000000
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%s__ returned non-%s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject* x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
#if PY_VERSION_HEX < 0x02050000
   if (ival <= LONG_MAX)
       return PyInt_FromLong((long)ival);
   else {
       unsigned char *bytes = (unsigned char *) &ival;
       int one = 1; int little = (int)*(unsigned char*)&one;
       return _PyLong_FromByteArray(bytes, sizeof(size_t), little, 0);
   }
#else
   return PyInt_FromSize_t(ival);
#endif
}
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject* x) {
   unsigned PY_LONG_LONG val = __Pyx_PyInt_AsUnsignedLongLong(x);
   if (unlikely(val == (unsigned PY_LONG_LONG)-1 && PyErr_Occurred())) {
       return (size_t)-1;
   } else if (unlikely(val != (unsigned PY_LONG_LONG)(size_t)val)) {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to size_t");
       return (size_t)-1;
   }
   return (size_t)val;
}


#endif /* Py_PYTHON_H */
